-- $Source: /home/dsd/d/aa/monoBANK/AA/TSP_ada.real_time.spc,v $
-- $Revision: 1.1 $ $Date: 96/02/09 13:22:50 $ $Author: akira $
--        Copyright (C) 1995 by Intermetrics, Inc.   

pragma ada_child;
package Ada.Real_Time is 

    pragma Suppress( Elaboration_Check );

--* OVERVIEW
-- Implements the language-defined package System.Real_Time.

-- The type Time is target dependent.

    type Time is private;
    Time_First: constant Time;
    Time_Last: constant Time;
    Time_Unit: constant := 1.0E-7;	-- Target dependent

    type Time_Span is private;
    Time_Span_First: constant Time_Span;
    Time_Span_Last: constant Time_Span;
    Time_Span_Zero: constant Time_Span;
    Time_Span_Unit: constant Time_Span;

    Tick: constant Time_Span;
    function Clock return Time;

    function "+"(Left: Time; Right: Time_Span) return Time;
    function "+"(Left: Time_Span; Right: Time) return Time;
    function "-"(Left: Time; Right: Time_Span) return Time;
    function "-"(Left: Time; Right: Time) return Time_Span;

    function "<"(Left, Right: Time) return Boolean;
    function "<="(Left, Right: Time) return Boolean;
    function ">"(Left, Right: Time) return Boolean;
    function ">="(Left, Right: Time) return Boolean;

    function "+"(Left, Right: Time_Span) return Time_Span;
    function "-"(Left, Right: Time_Span) return Time_Span;
    function "-"(Right: Time_Span) return Time_Span;
    function "/"(Left: Time_Span; Right: Integer) return Time_Span;
    function "/"(Left, Right: Time_Span) return Integer;
    function "*"(Left: Time_Span; Right: Integer) return Time_Span;
    function "*"(Left: Integer; Right: Time_Span) return Time_Span;

    function "abs"(Right : Time_Span) return Time_Span;

    function "<"(Left, Right: Time_Span) return Boolean;
    function "<="(Left, Right: Time_Span) return Boolean;
    function ">"(Left, Right: Time_Span) return Boolean;
    function ">="(Left, Right: Time_Span) return Boolean;

    function To_Duration(TS: Time_Span) return Duration;
    --* EFFECTS
    -- Converts Time_Span to Duration
    -- Could raise Constraint_Error

    function To_Time_Span(D: Duration) return Time_Span;
    --* EFFECTS
    -- Converts Duration to Time_Span
    -- Could raise Constraint_Error

    function Nanoseconds(NS: Integer) return Time_Span;
    --* OVERVIEW
    -- Returns NS within the accuracy of Time_Span.

    function Microseconds(US: Integer) return Time_Span;
    --* OVERVIEW
    -- Returns US within the accuracy of Time_Span.

    function Milliseconds(MS: Integer) return Time_Span;
    -- Returns MS within the accuracy of Time_Span.

    type Seconds_Count is range	-- Target dependent
	Integer'First .. Integer'Last;

    procedure Split(
      T : in Time;
      SC: out Seconds_Count;
      TS: out Time_Span
    );
    --* EFFECTS
    -- Splits Time into seconds and time_span

    function Time_Of(SC: Seconds_Count; TS: Time_Span) return Time;
    --* EFFECTS
    -- Converts a number of seconds and Time_Span into Time.

private

    -- Private part is target dependent.

    type Unsigned is mod 2**32;

    type Time is record
      LSW : Unsigned;
      MSW : Integer;  -- So can represent times before system startup
    end record;

    type Time_Span is record
      LSW : Unsigned;
      MSW : Integer;
    end record;

    Time_First: constant Time :=
	(MSW => Integer'First,  -- So can represent times before startup
	 LSW => 0);
    Time_Last: constant Time :=
	(MSW => Natural'Last, LSW => Unsigned'Last);

    Time_Span_First: constant Time_Span := 
	(MSW => Integer'First, LSW => 0);
    Time_Span_Last: constant Time_Span :=
        (MSW => Integer'Last, LSW => Unsigned'Last);

    Time_Span_Zero: constant Time_Span :=
        (MSW => 0, LSW => 0);
    Time_Span_Unit: constant Time_Span :=
        (MSW => 0, LSW => 1);

    Tick: constant Time_Span := Time_Span_Unit;

end Ada.Real_Time; 
