-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.strings.maps.bdy,v $ 
-- $Revision: 1.6 $ $Date: 96/08/20 15:14:22 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.

package body Ada.Strings.Maps is
pragma Suppress( Elaboration_Check );

    function To_Set (Ranges : in Character_Ranges) return Character_Set is
        Result : Character_Set := Null_Set;
    begin
        for Inx in Ranges'first .. Ranges'last loop
	    Result(Ranges(Inx).Low .. Ranges(Inx).High) := (others => true);
        end loop;
        return Result;
    end To_Set;

    function To_Set (Span : in Character_Range) return Character_Set is
        Result : Character_Set := Null_Set;
    begin
	Result(Span.Low .. Span.High) := (others => true);
        return Result;
    end To_Set;

    function To_Ranges (Set : in Character_Set) return Character_Ranges is
        Ranges : Character_Ranges (1 .. (Character_Set'Length + 1) / 2);
        From   : Character := Character'First;
	Char   : Character;
	Index  : Integer := 1;
    begin
	for Inx1 in 1 .. (Character_Set'Length + 1) / 2 loop
	    Index := Inx1;
            for Inx2 in From .. Character'Last loop
		Char := Inx2;
	        exit when Set(Inx2);
	    end loop;
	    exit when Char = Character'Last;
	    Ranges(Inx1).Low := Char;
	    From := Char;
            for Inx2 in From .. Character'Last loop
		Char := Inx2;
	        exit when not Set(Inx2);
	    end loop;
	    if Set(Char) then
		Ranges(Inx1).High := Character'Last;
            else
	        Ranges(Inx1).High := Character'Pred(Char);
            end if;
	    From := Char;
        end loop;
        return Ranges(1 .. Index - 1);
    end To_Ranges;

    function "="   (Left, Right : Character_Set) return Boolean is
    begin
        for Inx in Character_Set'Range loop
           if Left(Inx) /= Right(Inx) then
               return false;
           end if;
        end loop;
        return true;
    end "=";

    function "not" (Right : Character_Set) return Character_Set is
        Not_Right : Character_Set;
    begin
        for Inx in Character_Set'Range loop
           Not_Right(Inx) := not Right(Inx);
        end loop;
        return Not_Right;
    end "not";

    function "and" (Left, Right : Character_Set) return Character_Set is
        Left_And_Right : Character_Set;
    begin
        for Inx in Character_Set'Range loop
            Left_And_Right(Inx) := Left(Inx) and Right(Inx);
        end loop;
        return Left_And_Right;
    end "and";

    function "or"  (Left, Right : Character_Set) return Character_Set is
        Left_Or_Right : Character_Set;
    begin
        for Inx in Character_Set'Range loop
            Left_Or_Right(Inx) := Left(Inx) or Right(Inx);
        end loop;
        return Left_Or_Right;
    end "or";

    function "xor" (Left, Right : Character_Set) return Character_Set is
        Left_Xor_Right : Character_Set;
    begin
        for Inx in Character_Set'Range loop
            Left_Xor_Right(Inx) := Left(Inx) xor Right(Inx);
        end loop;
        return Left_Xor_Right;
    end "xor";

    function "-" (Left, Right : Character_Set) return Character_Set is
    begin
       return Left and not Right;
    end "-";

    function Is_In (Element : Character;
                    Set     : Character_Set) return Boolean is
    begin
        return Set(Element);
    end Is_In;

    function Is_Subset (Elements : Character_Set;
                        Set      : Character_Set) return Boolean is
    begin
        return (Elements = (Elements and Set));
    end Is_Subset;

    function To_Set (Sequence : Character_Sequence) return Character_Set is
        Result : Character_Set := Null_Set;
    begin
        for Inx in Sequence'first .. Sequence'last loop
            Result(Sequence(Inx)) := true;
        end loop;
        return Result;
    end To_Set;

    function To_Set (Singleton : Character) return Character_Set is
        Result : Character_Set := Null_Set;
    begin
        Result(Singleton) := true;
        return Result;
    end To_Set;

    function To_Sequence (Set : Character_Set) return Character_Sequence is
        Result : string (Character'Pos(Character'First) + 1 .. 
                         Character'Pos(Character'Last) + 1);
        Index  : integer := Character'Pos(Character'First);
    begin
        for Inx in Set'first .. Set'last loop
            if Set(Inx) then
                Index := Index + 1;
                Result(Index) := Inx;
            end if;
        end loop;
        return Result (Result'first .. Index);
    end To_Sequence;

    function Value (Map : in Character_Mapping; Element : in Character)
        return Character is
    begin
        return Map(Element);
    end Value;

    function To_Mapping (From, To : in Character_Sequence) 
      return Character_Mapping is
        Result    : Character_Mapping := Identity;
        Duplicate : Character_Set     := Null_Set;
    begin
        if From'Length /= To'Length then
            raise Translation_Error;
        end if;
        for Inx in From'first .. From'last loop
            if Duplicate(From(Inx)) then
                raise Translation_Error;
            end if;
            Duplicate(From(Inx)) := True;
            Result(From(Inx)) := To(To'first + (Inx - From'first));
        end loop;
        return Result;
    end To_Mapping;

    function To_Domain (Map : in Character_Mapping) 
      return Character_Sequence is
        Result : String(1 .. Map'Length);
        Index  : Integer := 0;
    begin
        for Inx in Map'First .. Map'Last loop
            if Map(Inx) /= Inx then
                Index := Index + 1;
                Result(Index) := Inx;
            end if;
        end loop;
        return Result(1 .. Index);
    end To_Domain;

    function To_Range (Map : in Character_Mapping) 
      return Character_Sequence is
        Result : String (1 .. Map'Length);
        Index  : Integer := 0;
    begin
        for Inx in Map'First .. Map'Last loop
            if Map (Inx) /= Inx then
                Index := Index + 1;
                Result(Index) := Map(Inx);
            end if;
        end loop;
        return Result (1 .. Index);
    end To_Range;

end Ada.Strings.Maps;
