-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.lexer.bdy,v $ 
-- $Revision: 1.3 $ $Date: 96/05/13 11:41:07 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

--dbg with Temp_IO;               -- for debugging ONLY
with Ada.IO_EXCEPTIONS;     -- raised by RTS_STRING_LEXER 

package body System.RTS.Lexer is

pragma Suppress( Elaboration_Check );

    --dbg package TIO renames Temp_IO;

    --dbg dbg : constant boolean := false;

    -------------------------------------------------------------
    --   LEXER
    -------------------------------------------------------------

    function  LEXER (State_Table: State_Transitions;
            Char_To_Token_Table: Char_To_Token_Mapping
    ) return State
      is

        Current_State: State := Start;      -- initial state
        Current_Token: Token ;
        Ch: character;      -- Character being parsed
    begin
            -- A poorly defined state transition table will cause
            -- infinite loops here:
        while Current_State /= Accepted and then Current_State /= Error
        loop
            Ch := Next;  -- Get next character; next() might raise End_Error
            Current_token := Char_To_Token_Table(Ch);       -- interpret
            Current_State := State_Table(Current_State)(Current_Token);

--dbg       if dbg then --dbg_on
--dbg         TIO.put_line("Lexer: current char:" & ch &
--dbg           ":token, and new state:");
--dbg         TIO.put_line(token'image(current_token));
--dbg         TIO.put_line(state'image(current_state));
--dbg       end if;     --dbg_off
        end loop;
--dbg   if dbg then --dbg_on
--dbg     TIO.put_line("Lexer: returning with state:");
--dbg     TIO.put_line(state'image(current_state));
--dbg   end if;     --dbg_off
        return Current_State;   -- Either Accepted or Error
    end Lexer;

    --------------------------------------------------------------
    -- LEX_STRING
    --------------------------------------------------------------

    procedure RTS_LEX_STRING (FROM : STRING;
                        STATE_TABLE: STATE_TRANSITIONS;
                        CHAR_TO_TOKEN_TABLE: CHAR_TO_TOKEN_MAPPING;
                        LAST  : out NATURAL
                            -- resulting lexical unit withOUT
                            -- trailing blanks is in
                            --     FROM (FROM'first .. LAST)
                        )
    is
        --+-----------------------------------------------------
        --| Overview
        --|   Lexes the next token in FROM according to STATE_TABLE and
        --|   CHAR_TO_TOKEN_TABLE.  If the token is Accepted according to the
        --|   tables, then then token without trailing blanks
        --|   is returned in FROM(FROM'first .. LAST).  Otherwise,
        --|   DATA_ERROR or END_ERROR are raised.
        --|
        --| Effect
        --|  The effect is as specified for GET(string) procedures in enum_io,
        --|   integer_io, fixed_io, and float_io: as to when
        --|   DATA_ERROR and END_ERROR are raised, and which character is at
        --|   FROM(LAST + 1).  LAST is not defined if an exception is raised.
        --|
        --| Algorithm
        --|   1. SKIP_BLANKS is called to skip spaces and tabs
        --|      such that a call to NEXT will return
        --|      the first non-blank character.
        --|   2. LEXER is called to "lex" the desired token, using NEXT.
        --|   3. NEXT returns a single "lookahead" character to LEXER
        --|      each time it
        --|      is called, and also "consumes" the previous lookahead character
        --|      by incrementing POS.
        --|   4. DATA_ERROR is raised if the final state isn't Accepted;
        --|      else LAST is given its appropriate value.
        --|
        --| Requires
        --|    The FSA specified by TABLEs are such that all accept transitions
        --|    are made on reading a space.  (NEXT returns a space when it
        --|    reaches the end of the string.)


        Pos : Integer := FROM'first;  -- FROM may be null, so Pos has to
                                      -- be an integer
        Result_State : state;         -- result of MY_LEXER

        function Next return character;

        function MY_LEXER is new LEXER (next => Next);

        function Next return character is
            begin
                Pos := Pos + 1;
                if Pos > From'Last then
--dbg               if dbg then --dbg_on
--dbg                 TIO.put_line("NEXT: Past end of string, return SPACE.");
--dbg               end if;     --dbg_off
                    return ' ';
                end if;
--dbg           if dbg then --dbg_on
--dbg             TIO.put_line("NEXT: Returning char :" & From(pos) & ':' );
--dbg           end if;     --dbg_off
                return From (Pos);
            end Next;

    begin -- Ignore spaces
        loop -- Scan for the first non-blank
            if Pos > From'Last then
               raise ada.io_exceptions.END_ERROR;  
            end if;
            exit when From(Pos) /= ' ' and then From(Pos) /= ascii.HT;
            Pos := Pos + 1;
        end loop;

--dbg   if dbg then --dbg_on
--dbg     TIO.put_line("lex_str: finished skipping blanks at :" & from(pos) &
--dbg     ": at position:");
--dbg     TIO.put_line(integer'image(pos));
--dbg   end if;     --dbg_off
        Pos := Pos - 1;  -- back up since NEXT increments pos

        Result_State := MY_LEXER (state_table, char_to_token_table);
--dbg   if dbg then --dbg_on
--dbg     TIO.new_line;
--dbg     TIO.put_line("lex_string: after LEXER with pos, result_state," &
--dbg       " from(..last)");
--dbg     TIO.put_line(integer'image(pos));
--dbg     TIO.put_line(state'image(result_state));
--dbg     TIO.put_line(from(from'first .. pos-1));
--dbg   end if;     --dbg_off

        if Result_State = Accepted then
            Last := Pos - 1;  -- Last char "READ" was previous position.
        else
           raise ada.io_exceptions.DATA_ERROR; 
        end if;
    end RTS_LEX_STRING;

end System.RTS.Lexer;
