-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.strings.bounded.bdy,v $ 
-- $Revision: 1.4 $ $Date: 96/04/16 13:49:09 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.

pragma Suppress( Elaboration_Check );

with System.RTS.Strings_Search;
with Ada.Strings.Fixed;

package body Ada.Strings.Bounded is
pragma Suppress( Elaboration_Check );

    package body Generic_Bounded_Length is

        package ASM renames Ada.Strings.Maps; 
        package ASF renames Ada.Strings.Fixed;
        package RSS renames System.RTS.Strings_Search;
 
        function Length (Source : in Bounded_String) return Length_Range is
        begin
            return Source.Length;
        end Length;

        function To_Bounded_String (Source : in String;
                                    Drop   : in Truncation := Error)
           return   Bounded_String is
            SLen   : constant integer := Source'Length;
            Result : Bounded_String;
        begin
            if Slen <= Max_Length then
                Result.Length := Slen;
                Result.Data (1 .. Slen) := Source;
            else
                case Drop is
                    when Left =>
                        Result.Length := Max_Length;
                        Result.Data := Source (Source'Last - (Max_Length - 1) ..
				               Source'Last);
                    when Right =>
                        Result.Length := Max_Length;
                        Result.Data := Source (Source'First ..
				               Source'First + (Max_Length - 1));
                    when others =>
                        raise Length_Error;
                end case;
            end if;
            return Result;
        end To_Bounded_String;

        function To_String (Source : in Bounded_String) return String is
        begin
           return Source.Data (1 .. Source.Length);
        end To_String;

        function Append (Left, Right : in Bounded_String;
                         Drop        : in Truncation  := Error)
           return Bounded_String is
        begin
	    return To_Bounded_String(To_String(Left) & To_String(Right), Drop);
        end Append;

        function Append (Left  : in Bounded_String;
                         Right : in String;
                         Drop  : in Truncation := Error)
           return  Bounded_String is
        begin
	    return To_Bounded_String(To_String(Left) & Right, Drop);
	end Append;

        function Append (Left  : in String;
                         Right : in Bounded_String;
                         Drop  : in Truncation := Error)
	   return Bounded_String is
        begin
	    return To_Bounded_String(Left & To_String(Right), Drop);
        end Append;

        function Append (Left  : in Bounded_String;
                         Right : in Character;
                         Drop  : in Truncation := Error)
           return  Bounded_String is
        begin
	    return To_Bounded_String(To_String(Left) & Right, Drop);
        end Append;

        function Append (Left  : in Character;
                         Right : in Bounded_String;
                         Drop  : in Truncation := Error)
           return  Bounded_String is
        begin
	    return To_Bounded_String(Left & To_String(Right), Drop);
        end Append;

        procedure Append (Source   : in out Bounded_String;
                          New_Item : in Bounded_String;
                          Drop     : in Truncation  := Error) is
        begin
	    Source := Append(Source, New_Item, Drop);
        end Append;

        procedure Append (Source   : in out Bounded_String;
                          New_Item : in String;
                          Drop     : in Truncation  := Error) is
        begin
	    Source := Append(Source, New_Item, Drop);
        end Append;

        procedure Append (Source   : in out Bounded_String;
                          New_Item : in Character;
                          Drop     : in Truncation  := Error) is
        begin
	    Source := Append(Source, New_Item, Drop);
        end Append;

        function "&" (Left, Right : in Bounded_String)
           return Bounded_String is
        begin
	    return Append(Left, Right, Error);
        end "&";

        function "&" (Left : in Bounded_String; Right : in String)
           return Bounded_String is
        begin
	    return Append(Left, Right, Error);
        end "&";

        function "&" (Left : in String; Right : in Bounded_String)
           return Bounded_String is
        begin
            return Append(Left, Right, Error);
        end "&";

        function "&" (Left  : in Bounded_String;
                      Right : in Character) return  Bounded_String is
        begin
            return Append(Left, Right, Error);
        end "&";

        function "&" (Left : in Character; Right : in Bounded_String)
           return Bounded_String is
        begin
            return Append(Left, Right, Error);
        end "&";

        function Element (Source : in Bounded_String;
                          Index  : in Positive) return Character is
        begin
            if Index <= Length(Source) then
                return Source.Data(Index);
            else
                raise Index_Error;
            end if;
        end Element;

        procedure Replace_Element (Source : in out Bounded_String;
                                   Index  : in Positive;
                                   By     : in Character) is
        begin
            if Index <= Length(Source) then
                Source.Data(Index) := By;
            else
                raise Index_Error;
            end if;
        end Replace_Element;

        function Slice (Source : in Bounded_String;
                        Low    : in Positive;
                        High   : in Natural)
           return String is
        begin
            if Low > Length(Source) + 1 or else High > Length(Source) then
                raise Index_Error;
            else
                return Source.Data (Low .. High);
            end if;
        end Slice;

        function "=" (Left, Right : in Bounded_String) 
           return Boolean is
        begin
            return (To_String(Left) = To_String(Right));
        end "=";

        function "="  (Left : in Bounded_String; Right : in String)
           return Boolean is
        begin
            return (To_String(Left) = Right);
        end "=";

        function "="  (Left : in String; Right : in Bounded_String)
           return Boolean is
        begin
            return (Left = To_String(Right));
        end "=";

        function "<" (Left, Right : in Bounded_String)
           return Boolean is
        begin
            return (To_String(Left) < To_String(Right));
        end "<";

        function "<"  (Left : in Bounded_String; Right : in String)
           return Boolean is
        begin
            return (To_String(Left) < Right);
        end "<";

        function "<"  (Left : in String; Right : in Bounded_String)
           return Boolean is
        begin
            return (Left < To_String(Right));
        end "<";

        function "<=" (Left, Right : in Bounded_String)
           return Boolean is
        begin
            return (To_String(Left) <= To_String(Right));
        end "<=";

        function "<="  (Left : in Bounded_String; Right : in String)
           return Boolean is
        begin
            return (To_String(Left) <= Right);
        end "<=";

        function "<="  (Left : in String; Right : in Bounded_String)
           return Boolean is
        begin
            return (Left <= To_String(Right));
        end "<=";

        function ">" (Left, Right : in Bounded_String)
           return Boolean is
        begin
            return (To_String(Left) > To_String(Right));
        end ">";

        function ">"  (Left : in Bounded_String; Right : in String)
           return Boolean is
        begin
            return (To_String(Left) > Right);
        end ">";

        function ">"  (Left : in String; Right : in Bounded_String)
           return Boolean is
        begin
            return (Left > To_String(Right));
        end ">";

        function ">=" (Left, Right : in Bounded_String)
           return Boolean is
        begin
            return (To_String(Left) >= To_String(Right));
        end ">=";

        function ">="  (Left : in Bounded_String; Right : in String)
           return Boolean is
        begin
            return (To_String(Left) >= Right);
        end ">=";

        function ">="  (Left : in String; Right : in Bounded_String)
           return Boolean is
        begin
            return (Left >= To_String(Right));
        end ">=";

        function Index (Source   : in Bounded_String;
                        Pattern  : in String;
                        Going    : in Strings.Direction := Strings.Forward;
                        Mapping  : in Maps.Character_Mapping := Maps.Identity)
           return Natural is
        begin
            return RSS.Index (Source.Data (1 .. Source.Length),
                              Pattern,
                              Going, 
                              Mapping);
        end Index;

        function Index (Source   : in Bounded_String;
                        Pattern  : in String;
                        Going    : in Direction := Forward;
                        Mapping  : in Maps.Character_Mapping_Function)
           return Natural is
        begin
            return RSS.Index (Source.Data (1 .. Source.Length),
                              Pattern,
                              Going,
                              Mapping);
        end Index;

        function Index (Source : in Bounded_String;
                        Set    : in Maps.Character_Set;
                        Test   : in Strings.Membership := Strings.Inside;
                        Going  : in Strings.Direction  := Strings.Forward)
           return Natural is
        begin
            return RSS.Index (Source.Data (1 .. Source.Length),
                              Set,
                              Test,
                              Going);
        end Index;

        function Index_Non_Blank (Source : in Bounded_String;
                                  Going  : in Direction := Forward)
           return Natural is
        begin
            return RSS.Index_Non_Blank (Source.Data (1 .. Source.Length),
                                        Going);
        end Index_Non_Blank;

        function Count (Source   : in Bounded_String;
                        Pattern  : in String;
                        Mapping  : in Maps.Character_Mapping := Maps.Identity)
           return Natural is
        begin
            return RSS.Count (Source.Data (1 .. Source.Length),
                              Pattern,
                              Mapping);
        end Count;

        function Count (Source   : in Bounded_String;
                        Pattern  : in String;
                        Mapping  : in Maps.Character_Mapping_Function)
           return Natural is
        begin
            return RSS.Count (Source.Data (1 .. Source.Length),
                              Pattern,
                              Mapping);
        end Count;

        function Count (Source : in Bounded_String;
                        Set    : in Maps.Character_Set)
           return   Natural is
        begin
            return RSS.Count (Source.Data (1 .. Source.Length), Set);
        end Count;

        procedure Find_Token (Source : in Bounded_String;
                              Set    : in Maps.Character_Set;
                              Test   : in Strings.Membership;
                              First  : out Positive;
                              Last   : out Natural) is
        begin
            RSS.Find_Token (Source.Data (1 .. Source.Length),
                            Set,
                            Test,
                            First,
                            Last);
        end Find_Token;

        function Translate (Source  : in Bounded_String;
                            Mapping : in Maps.Character_Mapping)
           return Bounded_String is
        begin
            return To_Bounded_String(ASF.Translate(To_String(Source), 
						   Mapping));
        end Translate;

        procedure Translate (Source : in out Bounded_String;
                             Mapping  : in Maps.Character_Mapping) is
        begin
            Source := To_Bounded_String(ASF.Translate(To_String(Source),
                                                      Mapping));
        end Translate;

        function Translate (Source  : in Bounded_String;
                            Mapping : in Maps.Character_Mapping_Function)
           return Bounded_String is
        begin
            return To_Bounded_String(ASF.Translate(To_String(Source),
						   Mapping));
        end Translate;

        procedure Translate (Source  : in out Bounded_String;
                             Mapping : in Maps.Character_Mapping_Function) is
        begin
            Source := To_Bounded_String(ASF.Translate(To_String(Source),
                                                      Mapping));
        end Translate;

        function Replace_Slice (Source   : in Bounded_String;
                                Low      : in Positive;
                                High     : in Natural;
                                By       : in String;
                                Drop     : in Truncation := Error)
           return Bounded_String is
        begin
            return To_Bounded_String(ASF.Replace_Slice(To_String(Source),
				                       Low,
				                       High,
				                       By),
				     Drop);
        end Replace_Slice;

        procedure Replace_Slice (Source   : in out Bounded_String;
                                 Low      : in Positive;
                                 High     : in Natural;
                                 By       : in String;
                                 Drop     : in Truncation := Error) is
        begin
            Source := To_Bounded_String(ASF.Replace_Slice(To_String(Source),
				                          Low,
						          High,
						          By),
					Drop);
        end Replace_Slice;

        function Insert (Source   : in Bounded_String;
                         Before   : in Positive;
                         New_Item : in String;
                         Drop     : in Truncation := Error)
           return     Bounded_String is
        begin
            return To_Bounded_String(ASF.Insert(To_String(Source),
			                        Before,
				                New_Item),
				     Drop);
        end Insert;

        procedure Insert (Source   : in out Bounded_String;
                          Before   : in Positive;
                          New_Item : in String;
                          Drop     : in Truncation := Error) is
        begin
            Source := To_Bounded_String(ASF.Insert(To_String(Source),
			                           Before,
				                   New_Item),
				        Drop);
        end Insert;

        function Overwrite (Source    : in Bounded_String;
                            Position  : in Positive;
                            New_Item  : in String;
                            Drop      : in Strings.Truncation := Strings.Error)
           return      Bounded_String is
        begin
	    return To_Bounded_String(ASF.Overwrite(To_String(Source),
						   Position,
					           New_Item),
				     Drop);
        end Overwrite;

        procedure Overwrite (Source    : in out Bounded_String;
                             Position  : in Positive;
                             New_Item  : in String;
                             Drop      : in Truncation := Error) is
        begin
	    Source := To_Bounded_String(ASF.Overwrite(To_String(Source),
						      Position,
						      New_Item),
				        Drop);
        end Overwrite;

        function Delete (Source  : in Bounded_String;
                         From    : in Positive;
                         Through : in Natural)
           return    Bounded_String is
        begin
	    return To_Bounded_String(ASF.Delete(To_String(Source),
						From,
						Through));
        end Delete;

        procedure Delete (Source  : in out Bounded_String;
                          From    : in Positive;
                          Through : in Natural) is
        begin
	    Source := To_Bounded_String(ASF.Delete(To_String(Source),
						   From,
						   Through));
        end Delete;

        function Trim (Source : in Bounded_String; Side : in Trim_End)
           return Bounded_String is
        begin
	    return To_Bounded_String(ASF.Trim(To_String(Source), Side));
        end Trim;

        procedure Trim (Source : in out Bounded_String;
                        Side   : in Trim_End) is
        begin
	    Source := To_Bounded_String(ASF.Trim(To_String(Source), Side));
        end Trim;

        function Trim (Source : in Bounded_String;
                       Left   : in Maps.Character_Set;
                       Right  : in Maps.Character_Set)
           return   Bounded_String is
        begin
	    return To_Bounded_String(ASF.Trim(To_String(Source), Left, Right));
        end Trim;

        procedure Trim (Source : in out Bounded_String;
                        Left   : in Maps.Character_Set;
                        Right  : in Maps.Character_Set) is
        begin
	    Source := To_Bounded_String(ASF.Trim(To_String(Source),
						 Left,
						 Right));

        end Trim;

        function Head (Source : in Bounded_String;
                       Count  : in Natural;
                       Pad    : in Character := Space;
                       Drop   : in Truncation := Error)
           return Bounded_String is
        begin
	    return To_Bounded_String(ASF.Head(To_String(Source),
				              Count,
				              Pad),
		                     Drop);
        end Head;

        procedure Head (Source : in out Bounded_String;
                        Count  : in Natural;
                        Pad    : in Character  := Space;
                        Drop   : in Truncation := Error) is
        begin
	    Source := To_Bounded_String(ASF.Head(To_String(Source),
						 Count,
					         Pad),
				        Drop);
        end Head;

        function Tail (Source : in Bounded_String;
                       Count  : in Natural;
                       Pad    : in Character := Space;
                       Drop   : in Truncation := Error)
           return   Bounded_String is
        begin
	    return To_Bounded_String(ASF.Tail(To_String(Source),
					      Count,
					      Pad),
				     Drop);
        end Tail;

        procedure Tail (Source : in out Bounded_String;
                        Count  : in Natural;
                        Pad    : in Character  := Space;
                        Drop   : in Truncation := Error) is
        begin
	    Source := To_Bounded_String(ASF.Tail(To_String(Source),
					         Count,
					         Pad),
					Drop);
        end Tail;

        function "*" (Left  : in Natural;
                      Right : in Character)
           return  Bounded_String is
        begin
	    return To_Bounded_String(ASF."*"(Left, Right), Error);
        end "*";

        function "*" (Left  : in Natural;
                      Right : in String)
           return  Bounded_String is
        begin
	    return To_Bounded_String(ASF."*"(Left, Right), Error);
        end "*";

        function "*" (Left  : in Natural;
                      Right : in Bounded_String)
           return  Bounded_String is
        begin
	    return To_Bounded_String(ASF."*"(Left, To_String(Right)), Error);
        end "*";

        function Replicate (Count : in Natural;
                            Item  : in Character;
                            Drop  : in Truncation := Error)
           return  Bounded_String is
        begin
	    return To_Bounded_String(ASF."*"(Count, Item), Drop);
	end Replicate;

        function Replicate (Count : in Natural;
                            Item  : in String;
                            Drop  : in Truncation := Error)
           return  Bounded_String is
        begin
	    return To_Bounded_String(ASF."*"(Count, Item), Drop);
	end Replicate;

        function Replicate (Count : in Natural;
                            Item  : in Bounded_String;
                            Drop  : in Truncation := Error)
           return  Bounded_String is
        begin
	    return To_Bounded_String(ASF."*"(Count, To_String(Item)), Drop);
        end Replicate;

   end Generic_Bounded_Length;

end Ada.Strings.Bounded;
