-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.delay_interface.spc,v $ 
-- $Revision: 1.8 $ $Date: 96/04/24 15:49:09 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Interface_Types;

package System.RTS.Delay_Interface is

    pragma Suppress( Elaboration_Check );

--* OVERVIEW
-- This package provides interfaces to support all delay functions:
-- delay, delay until, and asynchronous select with a delay as a trigger
-- alternative.  Delay expressions passed as
-- parameters are of either an absolute time type or Duration.
-- Absolute time types include Calendar.Time, Real_Time.Time, and
-- possibly user-defined time types wrapped by the compiler into
-- a type derived from System.RTS.TGT.Kernel.Timers.Timer_Record;
-- Expiration times that are stored in queue elements (ECDs) are contained
-- in a Timer_Record, so that subsequent processing can use
-- class-wide programming.
--

    procedure RTS_Delay_Relative( D : Duration );
    pragma Export (ada,
		   RTS_Delay_Relative,
                   "rts_delay_relative" );
    pragma Global (RTS_Delay_Relative);
    --* OVERVIEW
    -- Called for a delay_relative statement.
    --* EFFECTS
    -- Checks for <= 0.0, if so does a yield operation.  Otherwise, calls
    -- TGT.Kernel.Timers.Relative.Suspend_Until_True_Or_Timeout

    procedure RTS_Delay_Until(
      Expiration      : Interface_Types.Absolute_Delay_Handle;
      Which_Time_Type : Interface_Types.Time_Type_Enum );
    pragma Export (ada, RTS_Delay_Until, "rts_delay_until" );
    pragma Global (RTS_Delay_Until);
    --* OVERVIEW
    -- Called for a delay_until_statement.
    --* EFFECTS
    -- Suspends caller and resumes when expiration time is reached.

    function RTS_Async_Rel_Delay(
      ECD             : Interface_Types.ECD_Handle;
      Setjmp_Buffer   : Interface_Types.Data_Address;
      Master          : Interface_Types.Simple_Handle;
      D               : Duration ) return Boolean;
    pragma Export (ada,
		   RTS_Async_Rel_Delay,
                   "rts_async_rel_delay" );
    pragma Global (RTS_Async_Rel_Delay);
    --* OVERVIEW
    -- Called for an asynchronous select with delay_relative as trigger
    -- alternative.
    --* EFFECTS
    -- Returns True if D <= 0.0.
    -- Otherwise, sets a timer to cause an abort signal to be sent
    -- and returns False immediately.
    -- Also, sets the triggering event in the master.

    function RTS_Async_Abs_Delay(
      ECD             : Interface_Types.ECD_Handle;
      Setjmp_Buffer   : Interface_Types.Data_Address;
      Master          : Interface_Types.Simple_Handle;
      Expiration      : Interface_Types.Absolute_Delay_Handle;
      Which_Time_Type : Interface_Types.Time_Type_Enum ) return Boolean;
    pragma Export (ada,
		   RTS_Async_Abs_Delay,
                   "rts_async_abs_delay" );
    pragma Global (RTS_Async_Abs_Delay);
    --* OVERVIEW
    -- Called for an asynchronous select with delay_until
    -- as trigger alternative.
    --* EFFECTS
    -- Sets a timer to cause an abort signal to be sent
    -- and returns False.
    -- If the time is already passed, either causes an immediate abort signal,
    -- or returns True, depending on the implementation and kind of time type.
    -- Also, sets the triggering event in the master.

end System.RTS.Delay_Interface;
