-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.ecd_queue_pkg.spc,v $ 
-- $Revision: 1.3 $ $Date: 96/06/26 18:09:57 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Protected_Types_Pkg;    use System.RTS.Protected_Types_Pkg;

package System.RTS.ECD_Queue_Pkg is  

    pragma Suppress( Elaboration_Check );

--* OVERVIEW
-- The queue structure is visible outside of this package because of 
-- the way that ECDs are threaded together to form queues.  
-- Therefore, this package contains no data hiding. It contains 
-- procedures for modifying queues by the addition and removal of 
-- ECDs.  The formal parameter Queue_First points to the first element
-- of the queue (before and after the operation).  It may be treated as 
-- a queue head but not necessarily. This is up to the clients (the small
-- and large cases differ here).

-- This package has two alternate bodies (in two different files): one for
-- FIFO queuing and one for priority queuing.

-- See if we need, in addition, a Top function that merely returns the top
-- element in a queue without removing it.  Currently, clients of this 
-- package achieve the same by dereferencing the queue header.

    procedure Add (
        Queue_First  : in out ECD_Access;
        ECD : ECD_Access
    );
    --* OVERVIEW 
    -- Add to the specified queue using the ECD's pred/succ links

    procedure Remove (
        Queue_First : in out ECD_Access; -- Null if queue becomes empty
        ECD : ECD_Access
    );
    --* OVERVIEW
    -- Remove an arbirary ECD from the specified queue using the ECD's
    -- pred/succ links.  Assume that ECD is on this queue (i.e. the queue 
    -- in not empty)

    procedure Remove_First (
        Queue_First : in out ECD_Access; -- Null if queue becomes empty
        First_ECD : out ECD_Access       -- Null if queue is originally empty
    );
    --* OVERVIEW
    -- Remove and return the first ECD (if any) from the specified queue.  
    -- Queue may be empty.

--DBG--     procedure Print_Queue ( Queue_First : ECD_Access );
--DBG--     --* IMPLEMENTATION
--DBG--     -- Prints the queue specified by Queue_First
--DBG--     -- this is only used for debugging purposes

end System.RTS.ECD_Queue_Pkg;
