-- == % filter "-- "
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ************************************************************************
--
-- PACKAGE SPECIFICATION System.RTS.TGT.Exceptions.GLUE
-- -----------------------------------
--
-- OUTLINE
-- -------
-- This module contains the specifications of the interface routines which
-- are used to connect the exception manager to the runtime environment.
--
-- NOTE
-- ----
-- The specifications are called by the common code and must not be changed.
-- However the implementation of the bodies is entirely target-dependent and
-- is generally in non-Ada code.
--
-- ************************************************************************

with System.RTS.TGT.Exceptions.TDEP;
with System.Rts.Types; use System.Rts.Types;
with SYSTEM;   use SYSTEM;
with System.RTS.Exception_Types;

package System.RTS.TGT.Exceptions.GLUE is

   pragma SUPPRESS (ELABORATION_CHECK);

   --	    *********************
   --	    * TABLE OF CONTENTS *
   --	    *********************

   --	      1. EXCEPTION_HANDLE
   --	      2. CODE_TABLE Pointer
   --	      3. Revision History

   -- ------------------------------------------
   -- Package renamings to remove prefix:
   -- ------------------------------------------
   package EM_TDEP renames System.RTS.TGT.Exceptions.TDEP;
   package RT_TYPES renames System.Rts.Types;
   -- ------------------------------------------


   --	    ***********************
   --	    * 1. EXCEPTION_HANDLE *
   --	    ***********************

   -- Routine to pass control to exception handler,
   -- restoring the machine state from the supplied context.

   procedure EXCEPTION_HANDLE (CONTEXT	  : EM_TDEP.REF_EXCEPTION_CONTEXT;
                               OCCURRENCE : System.RTS.Exception_Types.Exception_Occurrence_Access);

   -- Note: the CONTEXT passed to this routine points to the same context
   -- record as was initially passed to the exception handler with its
   -- fields modified appropriately. In particular, the PC has been updated
   -- to hold the address of the handler.


private

--    == % if TARGET_NT
--    == % if TARGET_RIOS
--    pragma import (ADA, EXCEPTION_HANDLE, "ada__exception_handle");
--    == % elsif TARGET_I80386
   pragma import (ADA, EXCEPTION_HANDLE, "_rts_exception_handle");
--    == % end if
--    == % end if

end System.RTS.TGT.Exceptions.GLUE;

--	    ***********************
--	    * 3. REVISION HISTORY *
--	    ***********************

--  6.01  20 Dec 1990 (BJD)
--	  Initial introduction to CMS v6.
--	  Convert into a template file.
--	  Comment out target-dependent INTERFACE/INTERFACE_NAME pragmas.

--  6.02  20 Feb 1991 (BJD)
--	  Re-instate as common file instead of template file.
--	  Uncomment pragma INTERFACE stuff, move it to private part, and
--	  bracket it with TARGET_RIOS filtering

--  6.03  21 May 1991 (BJD)
--	  Add TARGET_SPARC filtering.

--  6.04  3rd November 1991 (BJD)
--	  Prefix all CRTS library unit names with ALSYS_

--  6.05  3rd June 1993 (BJD)
--	  EXCEPTION_HANDLE spec no longer needs explicit handler address,
--	  since this is now contained in the updated CONTEXT parameter.
