-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ***************************************************************************
--
-- PACKAGE SPECIFICATION System.RTS.TGT.Exceptions.MAIN
-- -----------------------------
--
-- OUTLINE
-- -------
-- This module contains the main modules of the exception manager
--
-- ***************************************************************************

with System.RTS.TGT.Exceptions.TDEP;
with System.Rts.Types; use System.Rts.Types;
with System.RTS.TGT.Exceptions.TYPES; use System.RTS.TGT.Exceptions.TYPES;
with System.RTS.Exception_Types;

package System.RTS.TGT.Exceptions.MAIN is

   pragma SUPPRESS (ELABORATION_CHECK);

   --	    *********************
   --	    * TABLE OF CONTENTS *
   --	    *********************

   --	      1. Case 1 Entry Point
   --	      2. Case 2 Entry Point
   --	      3. Case 3 Entry Point
   --	      4. Revision History

   -- ------------------------------------------
   -- Package renamings to remove prefix:
   -- ------------------------------------------
   package EM_TDEP renames System.RTS.TGT.Exceptions.TDEP;
   package RT_TYPES renames System.Rts.Types;
   -- ------------------------------------------


   --	    *************************
   --	    * 1. CASE 1 ENTRY POINT *
   --	    *************************

   -- For a call to the case 1 entry point, the caller supplies only the
   -- exception context and the exception code.

   procedure EXCEPTION_MANAGER_CASE_1
      (CONTEXT    : EM_TDEP.REF_EXCEPTION_CONTEXT;
       OCCURRENCE : System.RTS.Exception_Types.Exception_Occurrence_Access);


   --	    *************************
   --	    * 2. CASE 2 ENTRY POINT *
   --	    *************************

   -- For a call to the case 2 entry point, the caller supplies the entry
   -- table index, in addition to the exception context and exception code.

   procedure EXCEPTION_MANAGER_CASE_2
      (CONTEXT	  : EM_TDEP.REF_EXCEPTION_CONTEXT;
       OCCURRENCE : System.RTS.Exception_Types.Exception_Occurrence_Access;
       PROC_IND   : ENTRY_TABLE_SUBSCRIPT);


   --	    *************************
   --	    * 3. CASE 3 ENTRY POINT *
   --	    *************************

   -- For a call to the case 3 entry point, the caller supplies the offset
   -- from the current location to the start of the handler, in addition
   -- to the exception code and the exception context.

   procedure EXCEPTION_MANAGER_CASE_3
      (CONTEXT	  : EM_TDEP.REF_EXCEPTION_CONTEXT;
       OCCURRENCE : System.RTS.Exception_Types.Exception_Occurrence_Access;
       HAND_OFFS  : CODE_OFFSET);

private


   pragma export (Ada, EXCEPTION_MANAGER_CASE_1, "_rts_em_case_1");
   pragma export (Ada, EXCEPTION_MANAGER_CASE_2, "_rts_em_case_2");
   pragma export (Ada, EXCEPTION_MANAGER_CASE_3, "_rts_em_case_3");





   
end System.RTS.TGT.Exceptions.MAIN;


--	    ***********************
--	    * 4. REVISION HISTORY *
--	    ***********************

--  0.01  15 May 1990, 23:08 (RBKD)
--        Initial version

--  0.02  14 Oct 1990, 22:47 (RBKD)
--        Reduce dependence on use clauses

--  0.03  20 Dec 1990, 01:37 (RBKD)
--        Initial version for common runtime source checkin (no changes)

--  6.01  20 Dec 1990 (BJD)
--        Initial introduction to CMS v6.

--  6.02  24 Jan 1991 (BJD)
--        Update for integer types moved from EM_TYPES to RT_TYPES

-- 6.??   3rd November 1991 (BJD)
--        Prefix all CRTS library unit names with ALSYS_
