-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ***************************************************************************
--
-- PACKAGE SPECIFICATION System.RTS.TGT.Exceptions.PROBE
-- ------------------------------
--
-- OUTLINE
-- -------
-- This package contains the interface used by the common Exception Manager
-- for notifying significant events to AdaProbe. The subprograms defined in 
-- this specification are called by the common EM at the occurrence of events 
-- which are of interest to AdaProbe, allowing Probe to gain control of the 
-- program (by breakpointing the corresponding entry point).
-- ***************************************************************************

with System.RTS.TGT.Exceptions.TDEP;
with System.Rts.Types;
package System.RTS.TGT.Exceptions.PROBE is

   pragma SUPPRESS (ELABORATION_CHECK);

   -- ------------------------------------------
   -- Package renamings to remove prefix:
   -- ------------------------------------------
   package EM_TDEP renames System.RTS.TGT.Exceptions.TDEP;
   package RT_TYPES renames System.Rts.Types;
   -- ------------------------------------------


   Is_Active : constant Boolean := False;
   -- True when AdaProbe is active.
   

   procedure EXCEPTION_OCCURS
      (ECODE          : RT_TYPES.EXCEPTION_CODE;
       HANDLER_ADDR   : RT_TYPES.CODE_ADDRESS;
       CONTEXT_ORIG   : EM_TDEP.REF_EXCEPTION_CONTEXT;
       IS_DEAD        : BOOLEAN;
       IS_ENV         : BOOLEAN;
       OTHERS_HANDLED : BOOLEAN);
   ---------------------------------------------------------------------------
   -- OUTLINE:
   -- --------
   --    This procedure is called by the common Exception Manager when the
   -- program is under Probe control when the EM finds a handler, but before
   -- transferring control to the handler.
   --
   --    All the data required by Probe is contained in the actual parameters:
   -- ECODE is the code of the raised exception;
   -- HANDLER_ADDR is the address of the handler;
   -- CONTEXT_ORIG is the exception context at the point of raise;
   -- IS_DEAD indicates whether the exception is handled by the program or not
   -- IS_ENV is only meaningful if IS_DEAD is TRUE. IS_ENV indicates whether
   --    it is the environment task (or a user task) which has the unhandled
   --    exception;
   -- OTHERS_HANDLED indicates whether the handler is a "when others" handler
   --    or not.
   --
   -- Probe can retrieve this data by breakpointing the entry point of this 
   -- subprogram and reading the actual parameters.
   --
   -- NOTE:
   -- ----
   --    There are no semantics for the body of this routine. It merely acts
   -- as a global code address. As such, it is important that it is NOT 
   -- inlined. Typically, it will be implemented as interfaced to non-Ada, in
   -- order to make a global symbol available to Probe.
   ---------------------------------------------------------------------------

private 
   -- pragma INTERFACE definition for EXCEPTION_OCCURS
end System.RTS.TGT.Exceptions.PROBE;

-- Modification History
--  6.01 20th June 1991 (BJD)
--       Initial version

-- 6.??   3rd November 1991 (BJD)
--        Prefix all CRTS library unit names with ALSYS_
