-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ***************************************************************************
--
-- PACKAGE SPECIFICATION System.RTS.TGT.Exceptions.PROP
-- -----------------------------
--
-- OUTLINE
-- -------
-- This module contains the routine which propagates an exception.
--
-- ***************************************************************************

with System.RTS.TGT.Exceptions.TDEP;

with System.Rts.Types; use System.Rts.Types;
with System.RTS.TGT.Exceptions.TYPES; use System.RTS.TGT.Exceptions.TYPES;
with System.RTS.Exception_Types;

package System.RTS.TGT.Exceptions.PROP is

   pragma SUPPRESS (ELABORATION_CHECK);

   --	    *********************
   --	    * TABLE OF CONTENTS *
   --	    *********************

   --	      1. PROPAGATE_EXCEPTION
   --	      2. FIND_ADA_CODE_TABLE
   --	      3. FIND_PROCEDURE
   --	      4. Revision History

   -- ------------------------------------------
   -- Package renamings to remove prefix:
   -- ------------------------------------------
   package EM_TDEP renames System.RTS.TGT.Exceptions.TDEP;
   package RT_TYPES renames System.Rts.Types;
   package EM_TYPES renames System.RTS.TGT.Exceptions.TYPES;
   -- ------------------------------------------


   --	    **************************
   --	    * 1. PROPAGATE EXCEPTION *
   --	    **************************

   -- This is the routine which propagates an exception after the caller
   -- has located the Ada code table and determined the entry table
   -- subscript for the procedure from which the exception was raised.


   procedure PROPAGATE_EXCEPTION
      (CONTEXT		 : EM_TDEP.REF_EXCEPTION_CONTEXT;
       OCCURRENCE        : System.RTS.Exception_Types.Exception_Occurrence_Access;
       INIT_ADA_CODE_TBL : REF_ADA_CODE_TABLE;
       INIT_ENTRY_SUB	 : ENTRY_TABLE_SUBSCRIPT);


   --	    **************************
   --	    * 2. FIND_ADA_CODE_TABLE *
   --	    **************************

   -- This procedure locates the Ada code table entry corresponding to the
   -- current exception location in the supplied exception context

   function FIND_ADA_CODE_TABLE (CONTEXT : EM_TDEP.REF_EXCEPTION_CONTEXT)
      return REF_ADA_CODE_TABLE;


   --	    *********************
   --	    * 3. FIND_PROCEDURE *
   --	    *********************

   -- This is an auxiliary routine which performs the binary search on
   -- an entry table to locate the entry for the current procedure.

   function FIND_PROCEDURE (ENTRY_TBL   : REF_ENTRY_TABLE;
			    ENTRY_LAST  : ENTRY_TABLE_SUBSCRIPT;
			    CODE_ADRS   : CODE_ADDRESS)
      return ENTRY_TABLE_SUBSCRIPT;

end System.RTS.TGT.Exceptions.PROP; 

--	    ***********************
--	    * 4. REVISION HISTORY *
--	    ***********************

--  0.01  15 May 1990, 23:45 (RBKD)
--        Initial version

--  0.02  14 Oct 1990, 22:39 (RBKD)
--        Reduce dependence on use clauses

--  0.03  20 Dec 1990, 01:37 (RBKD)
--        Initial version for common runtime source checkin (no changes)

--  6.01  20 Dec 1990 (BJD)
--        Initial introduction to CMS v6.

--  6.02  24 Jan 1991 (BJD)
--        Update for integer types moved from EM_TYPES to RT_TYPES

-- 6.??   3rd November 1991 (BJD)
--        Prefix all CRTS library unit names with ALSYS_
