-- == % filter "-- "
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ***************************************************************************
--
-- PACKAGE BODY System.RTS.TGT.Exceptions.TYPES
-- ---------------------
--
-- OUTLINE
-- -------
-- This module contains the dump routines for the exception manager.
--
-- ***************************************************************************

with System.RTS.TGT.Exceptions.DEBUG;
with System.RTS.TGT.Exceptions.GLUE;
with System.Rts.Error_IO;
with SYSTEM;

with UNCHECKED_CONVERSION;

package body System.RTS.TGT.Exceptions.TYPES is

   pragma SUPPRESS (ELABORATION_CHECK);
   pragma SUPPRESS (STORAGE_CHECK);

   --	    *********************
   --	    * TABLE OF CONTENTS *
   --	    *********************

   --	      1. Dump Routines
   --	      2. Elaboration
   --	      3. Revision History

   -- ------------------------------------------
   -- Package renamings to remove prefix:
   -- ------------------------------------------
   package ERROR_IO renames System.Rts.Error_IO;
   package EM_DEBUG renames System.RTS.TGT.Exceptions.DEBUG;
   package EM_GLUE renames System.RTS.TGT.Exceptions.GLUE;
   -- ------------------------------------------

   function TO_ADR is new UNCHECKED_CONVERSION (REF_SH_RECORD,  DATA_ADDRESS);



   function GET_SCOPE_TABLE_LENGTH (SH_RECORD : in REF_SH_RECORD) 
            return SCOPE_TABLE_SUBSCRIPT is
      --------------------------------------------------------------------------------
      -- Return the length of the scope table for this SH record.
      --------------------------------------------------------------------------------
   begin
      return SH_RECORD.SCOPE_TABLE_LENGTH;
   end GET_SCOPE_TABLE_LENGTH;



   function GET_SCOPE_TABLE (SH_RECORD : in REF_SH_RECORD) return REF_SCOPE_TABLE is
      --------------------------------------------------------------------------------
      -- Return a pointer to the scope table for this SH record.
      --------------------------------------------------------------------------------
      function TO_REF_SCOPE_TABLE is 
         new UNCHECKED_CONVERSION (DATA_ADDRESS, REF_SCOPE_TABLE);
      use SYSTEM; -- for operators
   begin
      -- The scope table is at an offset of 4 bytes from the SH_RECORD address.
      return TO_REF_SCOPE_TABLE (TO_ADR (SH_RECORD) + 4);
   end GET_SCOPE_TABLE;



   function GET_HANDLER_TABLE (SH_RECORD : in REF_SH_RECORD) return REF_HANDLER_TABLE is
      --------------------------------------------------------------------------------
      -- Return a pointer to the scope table for this SH record.
      --------------------------------------------------------------------------------
      function TO_REF_HANDLER_TABLE is 
         new UNCHECKED_CONVERSION (DATA_ADDRESS, REF_HANDLER_TABLE);
      use SYSTEM; -- for operators
   begin
      -- The handler table immediately follows the scope table.
      return TO_REF_HANDLER_TABLE (
               TO_ADR (SH_RECORD) + 4 + SYSTEM.OFFSET (
                  GET_SCOPE_TABLE_LENGTH (SH_RECORD) * (SCOPE_TABLE_RECORD'SIZE/8)));
   end GET_HANDLER_TABLE;



   --	    ********************
   --	    * 1. DUMP ROUTINES *
   --	    ********************

   procedure DUMP_TABLES is

      ACT : REF_ADA_CODE_TABLE := CODE_TABLE_HEAD;

      function TO_ADR is new UNCHECKED_CONVERSION (REF_ADA_CODE_TABLE,
						   SYSTEM.ADDRESS);
      function TO_ADR is new UNCHECKED_CONVERSION (REF_ENTRY_TABLE,
						   SYSTEM.ADDRESS);
      function TO_ADR is new UNCHECKED_CONVERSION (REF_SCOPE_TABLE,
						   SYSTEM.ADDRESS);
      function TO_ADR is new UNCHECKED_CONVERSION (REF_SH_RECORD,
						   SYSTEM.ADDRESS);
      function TO_ADR is new UNCHECKED_CONVERSION (REF_HANDLER_TABLE,
						   SYSTEM.ADDRESS);
      function TO_ADR is new UNCHECKED_CONVERSION (CODE_ADDRESS,
						   SYSTEM.ADDRESS);
      function TO_ADR is new UNCHECKED_CONVERSION (CODE_OFFSET,
						   SYSTEM.ADDRESS);
      function TO_ADR is new UNCHECKED_CONVERSION (PROC_OFFSET,
						   SYSTEM.ADDRESS);

   begin
      ERROR_IO.PUT ("Dump of ADA CODE TABLE Entries");

      ERROR_IO.NEW_LINE;
      ERROR_IO.NEW_LINE;

      while ACT /= null loop
         ERROR_IO.PUT ("   CODE TABLE Entry at ");
	 ERROR_IO.PUT (TO_ADR (ACT));
	 ERROR_IO.NEW_LINE;

         ERROR_IO.PUT ("      NEXT = ");
	 ERROR_IO.PUT (TO_ADR (ACT.NEXT));
	 ERROR_IO.NEW_LINE;

         ERROR_IO.PUT ("      CODE_START = ");
	 ERROR_IO.PUT (TO_ADR (ACT.CODE_START));
	 ERROR_IO.NEW_LINE;

         ERROR_IO.PUT ("      CODE_END = ");
	 ERROR_IO.PUT (TO_ADR (ACT.CODE_END));
	 ERROR_IO.NEW_LINE;

         ERROR_IO.PUT ("      ENTRY_TABLE_PTR = ");
	 ERROR_IO.PUT (TO_ADR (ACT.ENTRY_TABLE_PTR));
	 ERROR_IO.NEW_LINE;

	 ERROR_IO.NEW_LINE;

         ERROR_IO.PUT ("  Dump of ENTRY_TABLE at ");
	 ERROR_IO.PUT (TO_ADR (ACT.ENTRY_TABLE_PTR));
	 ERROR_IO.NEW_LINE;

	 for I in ENTRY_TABLE_SUBSCRIPT range 0 .. ACT.ENTRY_TABLE_LAST loop
            ERROR_IO.PUT ("    Entry (");
	    ERROR_IO.PUT (RT_TYPES.LONGEST_INTEGER (I));
            ERROR_IO.PUT (")  Entry Point = ");
	    ERROR_IO.PUT (TO_ADR (ACT.ENTRY_TABLE_PTR (I).ENTRY_POINT));
            ERROR_IO.PUT (")  SH Record = ");
	    ERROR_IO.PUT (TO_ADR (ACT.ENTRY_TABLE_PTR (I).SH_RECORD));
            ERROR_IO.PUT ("  PIR Record = ");
            EM_TDEP.DUMP_PROCEDURE_INFO_RECORD (ACT.ENTRY_TABLE_PTR (I).PROCEDURE_INFORMATION);
	    ERROR_IO.NEW_LINE;
	 end loop;

	 ERROR_IO.NEW_LINE;

	 for I in ENTRY_TABLE_SUBSCRIPT range 0 .. ACT.ENTRY_TABLE_LAST loop
            declare
               SH : REF_SH_RECORD := ACT.ENTRY_TABLE_PTR (I).SH_RECORD;
            begin
               if SH /= null then
                  ERROR_IO.NEW_LINE;
                  ERROR_IO.PUT ("  Dump of SH Record at ");
                  ERROR_IO.PUT (TO_ADR (SH));
                  ERROR_IO.NEW_LINE;
                  ERROR_IO.PUT ("    Scope Table Length = ");
                  ERROR_IO.PUT (RT_TYPES.LONGEST_INTEGER (GET_SCOPE_TABLE_LENGTH (SH)));
                  ERROR_IO.NEW_LINE;
   
                  for I in 0 .. GET_SCOPE_TABLE_LENGTH (SH)-1 loop
                     declare 
                        SCOPES : REF_SCOPE_TABLE := GET_SCOPE_TABLE (SH);
                     begin
                        ERROR_IO.PUT ("      SCOPE_TABLE_RECORD (");
                        ERROR_IO.PUT (RT_TYPES.LONGEST_INTEGER (I));
                        ERROR_IO.PUT (")");
	                ERROR_IO.NEW_LINE;
   
                        ERROR_IO.PUT ("         STARTING_PROC_OFFSET = ");
                        ERROR_IO.PUT (TO_ADR (SCOPES (I).STARTING_PROC_OFFSET));
                        ERROR_IO.NEW_LINE;
   
                        ERROR_IO.PUT ("         ENDING_PROC_OFFSET = ");
                        ERROR_IO.PUT (TO_ADR (SCOPES (I).ENDING_PROC_OFFSET));
                        ERROR_IO.NEW_LINE;
   
                        ERROR_IO.PUT ("         HANDLER_TABLE_ENTRY = ");
	                ERROR_IO.PUT (RT_TYPES.LONGEST_INTEGER
	                   (SCOPES (I).HANDLER_TABLE_ENTRY));
                        ERROR_IO.NEW_LINE;
                     end;
                  end loop;

                  declare
                     HANDLERS : REF_HANDLER_TABLE       := GET_HANDLER_TABLE (SH);
                     I        : HANDLER_TABLE_SUBSCRIPT := 0;
                  begin
                     loop
                        ERROR_IO.PUT ("      HANDLER_TABLE_RECORD (");
	                ERROR_IO.PUT (RT_TYPES.LONGEST_INTEGER (I));
                        ERROR_IO.PUT (")");
                        ERROR_IO.NEW_LINE;
   
                        case HANDLERS (I).ECODE is
   
                           when HTE_OTHERS =>
                              ERROR_IO.PUT ("         ECODE = HTE_OTHERS");
                              ERROR_IO.NEW_LINE;
   
                              ERROR_IO.PUT ("         OTHERS_HANDLER = ");
                              ERROR_IO.PUT (RT_TYPES.LONGEST_INTEGER
                               (HANDLERS (I).OTHERS_HANDLER));
                              ERROR_IO.NEW_LINE;
   
         	          when HTE_END_OF_SEARCH =>
                              ERROR_IO.PUT ("         ECODE = HTE_END_OF_SEARCH");
		              ERROR_IO.NEW_LINE;
                              exit;
   
         	          when HTE_GOTO_ENCLOSING_SCOPE =>
		              ERROR_IO.PUT
                                 ("         ECODE = HTE_GOTO_ENCLOSING_SCOPE");
		              ERROR_IO.NEW_LINE;
   
                              ERROR_IO.PUT ("         NEXT_ENTRY = ");
		              ERROR_IO.PUT (RT_TYPES.LONGEST_INTEGER
         			    (HANDLERS (I).NEXT_ENTRY));
		              ERROR_IO.NEW_LINE;
   
         	          when HTE_SET_MARKER =>
                              ERROR_IO.PUT ("         ECODE = HTE_SET_MARKER");
		              ERROR_IO.NEW_LINE;
   
                              ERROR_IO.PUT ("         MARKER_OFFSET = ");
		              ERROR_IO.PUT (RT_TYPES.LONGEST_INTEGER
         		        (HANDLERS (I).MARKER_OFFSET));
		              ERROR_IO.NEW_LINE;
   
         	          when others =>
                              ERROR_IO.PUT ("         ECODE = ");
		              ERROR_IO.PUT
         		        (RT_TYPES.LONGEST_INTEGER
         			   (HANDLERS (I).ECODE));
		              ERROR_IO.NEW_LINE;
   
                              ERROR_IO.PUT ("         HANDLER = ");
		              ERROR_IO.PUT
         		        (RT_TYPES.LONGEST_INTEGER
		         	   (HANDLERS (I).HANDLER));
         		     ERROR_IO.NEW_LINE;
   
                        end case;
                        I := I + 1;
                     end loop;
                  end;
               end if;

            end;
	 end loop;

	 ERROR_IO.NEW_LINE;
	 ERROR_IO.NEW_LINE;

	 ACT := ACT.NEXT;
      end loop;
   end DUMP_TABLES;
   
   procedure INIT_EM_TYPES is
      -- Dump tables if needed
   begin
--       == % if DEBUG
--       if EM_DEBUG.DUMP_TABLE_SWITCH then
--         DUMP_TABLES;
--       end if;
--       == % else
      null;
--       == % end if
   end INIT_EM_TYPES;
   
end System.RTS.TGT.Exceptions.TYPES;

--	    ***********************
--	    * 3. REVISION HISTORY *
--	    ***********************

--  0.01  28 Sep 1990, 15:00 (PC+RBKD)
--        Initial version of this module

--  0.02  28 Sep 1990, 16:12 (PC)
--        Fix compilation errors

--  0.03  01 Oct 1990, 10:42 (PC)
--        Fix infinite loop in the DUMP_TABLES routine

--  0.04  14 Oct 1990, 23:25 (RBKD)
--        Use definitions in EM_DEBUG to control debug output
--        Reduce dependence on use clauses

--  0.05  20 Dec 1990, 01:33 (RBKD)
--        Initial version for common runtime source checkin
--        Uses new version of RT_TYPES, ERROR_IO

--  6.01  20 Dec 1990 (BJD)
--        Initial introduction to CMS v6.

--  6.02  7 Jun 1991 (BJD)
--        Correct initialisation of code table to use the routine in EM_GLUE.
--        Make debug output dependent on filtering instead of DEBUG Boolean.

-- 6.??   3rd November 1991 (BJD)
--        Prefix all CRTS library unit names with ALSYS_
