-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.image_and_value.bdy,v $
-- $Revision: 1.4 $ -- $Date: 95/11/20 13:57:18 $ -- $Author: akira $
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Lexer;
with System.RTS.Integer_State_Tables;
with Ada.IO_Exceptions; use Ada.IO_Exceptions;
with System.RTS.String_Utilities; use System.RTS.String_Utilities;

-- A note for any young and enthusiastic S.E. that reads this:

-- The lexing stuff was originally part of the IO. The packages are there
-- the exceptions are IO exceptions and the behavior is tailored
-- accordingly. We've enlisted the integer lexing for integer'value in
-- order not to duplicate the code (issues such as the ilegality of HT's
-- and the check for the tail of the image are solved locally here).
-- It is still slightly kluggy. The entire lexing stuff should be moved
-- higher and be used identicaly by IO and 'value, including exceptions
-- usage, catalog location etc. But, as I said, we need someone more
-- energetic for it. Offer

pragma elaborate(System.RTS.String_Utilities);
package body System.RTS.Image_And_Value is

    pragma Suppress( Elaboration_Check );

    RTS_ERROR : exception;

    subtype LegalRange is integer range -1 .. 15 ;
    type CharValueArray is array (character) of LegalRange ;
    CharValue : constant CharValueArray := CharValueArray'
      ('0' => 0, '1' => 1, '2' => 2, '3' => 3, '4' => 4,
       '5' => 5, '6' => 6, '7' => 7, '8' => 8, '9' => 9,
       'a'|'A'=> 10, 'b'|'B' => 11, 'c'|'C'=>12, 'd'|'D'=>13,
       'e'|'E' => 14, 'f'|'F' => 15, others => -1) ;
         -- Put this here so it is only done once (since
         --  static aggregates are still a ways off)


    ----------------------------------------------------------
    -- Scalar routines.
    ----------------------------------------------------------
    
    function RTS_Ivalue(
      The_Image : String
    ) return TGT.Longest_Integer
    is
	use type TGT.Longest_Integer;
        Result : TGT.Longest_Integer ;
        Base : TGT.Longest_Integer := 10 ;
	    -- Starts as 10 for integer or base of based literal
        Exponent_Base : TGT.Longest_Integer := Base;
    
        Curr_Int  : TGT.Longest_Integer 
			range TGT.Longest_Integer'first .. 0 := 0;
            --  Accumulates the negative of the "current integer",
            -- which can be the base, integer, or exponent.
            -- The value is always accumulated as a non-POSITIVE number,
            -- because there is one more negative number than positive numbers
            -- so it is possible to do value("<integer'first>").
            -- THEREFORE you must negate curr_int to get its "TRUE" value
            -- before use as base or exponent. Note that vbl Sign (below)
            -- has positive== -1  negative== +1 so the conversion of curr_int
            -- to its TRUE signed value happens via the multiply by sign.
    
        Sharp_Encountered : Boolean := false;
        subtype LegalBaseRange is TGT.Longest_Integer 
				      range 2 .. 16; IntValue : LegalRange ;
        Sign : TGT.Longest_Integer range -1 .. +1 := -1 ;
          -- NOTE WELL: -1 == '+'; +1 == '-'; not the usual way!
    
        E_is_exponent : boolean := true ; --| consider 'e' to be the
                                          --| beginning of exponent.
        Exponent_Seen : boolean := false; -- Has there been an exponent?
    begin
        --| Integer_literal ::= (integer | based_integer) exponent
        --| integer ::= digit < (:_:) digit >
        --| based_integer ::= base # extended_integer #
        --| extended_integer ::= extended_digit < (:_:) extended_digit >
        --| Requires
        --| The input string must be syntactically legal.
        --| Follows the rules in LRM Chapter 2.
    
        -- Note that if we are processing the digits between # and #
        -- that an e should be interpreted as a digit, not the
        -- indicator that an exponent is following.
    
        for i in The_Image'range loop
            case The_Image(i) is
            when '#' | ':' =>
                if Sharp_Encountered then
                    E_is_exponent := true ;
                    Sharp_encountered := false;
                else
                    Sharp_encountered := true;
                    E_is_exponent := false ;
                    Base := - Curr_Int ;
                    Exponent_Base := Base;
                    Curr_Int := 0; -- get ready for based number
                    if Base not in LegalBaseRange then
                        raise CONSTRAINT_ERROR ;
                    end if ;
                end if;
            when '_' =>
                null;
            when ' ' | ASCII.HT =>
                -- Should not really get here, but...
                exit;
            when '+' =>
                Sign := -1 ;  -- Note: opposite usual value since curr_int is
                              -- negative.
            when '-' =>
                Sign := +1 ;
            when others =>
                -- Process the digit.
                --| Should be either 0..9 a..f A..F
                IntValue := CharValue(The_Image(i)) ;
                case IntValue is
                when -1 =>
                    raise CONSTRAINT_ERROR ;
                when 14 =>  -- 'e' or 'E'
                    if E_is_exponent then
                        if Exponent_Seen then
                            -- Multiple Exponents are Bad!
                            raise CONSTRAINT_ERROR;
                        end if;
                        Result := Sign * Curr_Int; -- what we've got so far
                        Exponent_Seen := true;
                        -- Reset things for accumulating exponent
                        Sign := -1;
                        Curr_Int := 0;
                        Base := 10;
                        -- set IntValue to zero, so when we fall through to
                        -- the intvalue test and multiply updating Curr_Int,
                        -- that the effect will be to set Curr_Int to zero
                        -- again.  Alternative is to use GOTO, or to introduce
                        -- a boolean...
                        IntValue := 0;
                    end if ;
                    -- E is just a digit in the number (between # and #)
                when others =>
                    null ;
                end case ;
                if TGT.Longest_Integer(IntValue) not in 0 .. Base-1 then
                    raise CONSTRAINT_ERROR ;
                end if ;
                Curr_Int := Curr_Int * Base - TGT.Longest_Integer(IntValue);
                     -- Subtract IntValue to keep Curr_Int non-positive
            end case ;
        end loop ;
    
        if Exponent_Seen then
    
            if Sign = +1 then
                -- Negative exponent is illegal for integer'value
                raise CONSTRAINT_ERROR ;
            end if;
            Result := Result * Exponent_Base**Integer(-Curr_Int) ;
    
        else
            Result := Sign * Curr_Int;
        end if;
    
        return Result;
    
    exception
        when NUMERIC_ERROR =>
            -- Can be raised by overflows.  But we're only supposed to
            -- raise constraint error in this routine, so convert it.
            raise CONSTRAINT_ERROR;
    
    end RTS_Ivalue;
    
    
    function RTS_Integer_Image(
      The_Integer : TGT.Longest_Integer
    )
    return String
    is
    --+-------------------------------------------------------
    --| Overview :
    --|   Computes Integer'Image (The_Integer) LRM 3.3.5
    --|     1. Decimal
    --|     2. Leading '-' or blank
    --|     3. return string'first == 1
    --|
    --| Algorithm
    --|   1. Check sign of result, and make Int NEGATIVE (you can represent
    --|      one more negative number than positive one, i.e., min_int).
    --|   2. Take rem of Int, convert to character by adding pos of zero
    --|      (rem is negative, since Neg rem Pos == Negative)
    --|      (Nb. MOD gives different, WRONG result)
    --|   3. Put char into Work_String, from back to front
    --|   4. Div Int by 10, if it's now zero, we're done, else loop
    --|   5. Done: copy from work_String to Result so that 'first == 1
    --|   Note that the Done test is performed after the char is put
    --|   into string; this makes the case of Zero work. (i.e., a while
    --|   loop won't work).
    --+-------------------------------------------------------
	use type TGT.Longest_Integer;
        Base : constant := 10;
        Pos_Zero : constant := Character'pos ('0');
        Subtype String_Range is positive range 1 .. 30;
                -- 30 chars should be enough to hold any int!
                -- 'first MUST be 1
                -- Efficiency note: By declaring everything to have range
                --   String_Range, constraint checks will be eliminated.
    
        Work_String,      -- digits get put in from 'last toward 'first
        Result :          -- return result must start at 1
                string (String_Range);
        Last_Result_Index : String_Range := String_Range'first;
                -- last char of Result to return
        Int : TGT.Longest_Integer
		  range TGT.Longest_Integer'first .. 0;
		-- work space for integer
    begin
        if The_Integer >= 0 then
            Int := - The_Integer;
            Result (1) := ' ';
        else
            Int :=  The_Integer;
            Result (1) := '-';
        end if;
    
        for i in reverse String_Range loop
            Work_String (i) := Character'val (Pos_Zero - (Int rem Base));
                -- (Int rem Base is <= 0 )
            Int := Int / Base;
            if Int >= 0 then
                -- We're done! Compute where last char should be moved,
                -- Make slice assignment, then return the slice.
                -- Last_index := length_int + first_int_pos<2> - 1
                --  == (w_s'last - i + 1) + 2 - 1
                --      Cancel         $        $
                Last_Result_Index := (Work_String'last - i) + 2;
                Result (2 .. Last_Result_Index) :=
                  Work_String (i .. Work_String'last);
                return Result (1 .. Last_Result_Index);
            end if;
        end loop;
        -- Can't get here unless the work_string is too short!
        -- Program_Error will be raised.
    end RTS_Integer_Image;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
    function RTS_Integer_Value(
      The_Image : String
    )
    return TGT.Longest_Integer
    is
    --+-------------------------------------------------------
    --| Overview :
    --|   returns integer'value of the string.
    --|
    --| Algorithm :
    --|  Skips all leading blanks, HT's are illegal, then calls RTS_Ivalue
    --|  to return an integer value. Scan the remainder of the string (if any)
    --|  only blank are allowed.
    --|
    --+-------------------------------------------------------
    
        My_Last : natural := 0;
        My_First: natural := 0;
    
    begin
    
        for i in The_Image'range loop
            if The_Image(i) /= ' ' then
                if The_Image(i) = Ascii.HT then
                    -- Because of IO, HT is allowed in the lex tables but
                    -- not here.
                    raise Constraint_error;
                else
                    My_First := i;
                    exit;
                end if;
            end if;
        end loop;
    
        if My_First = 0 then
            -- a string with all blanks is not a valid integer
            raise Constraint_Error;
        end if;
    
        begin
            System.RTS.Lexer.RTS_lex_String (The_image(My_First..The_Image'last),
                                      System.RTS.Integer_State_Tables.Transitions,
                                      System.RTS.Integer_State_Tables.Mapping,
                                      My_Last);
        exception
            when Data_Error | End_Error =>
                -- Since the lexer is originally part of the IO stuff, it
                -- sometimes raises these exceptions, all we need is
                -- Constraint_error.
                raise Constraint_error;
        end;
    
        if My_Last < My_First then
            raise Constraint_error;
        end if;
    
        for i in My_Last+1..The_Image'last loop
            -- The only character allowed here is blank.
            if The_image(i) /= ' ' then
                raise Constraint_error;
            end if;
        end loop;
    
        return RTS_Ivalue(The_image(My_First..My_Last));
    
    end RTS_Integer_Value;
    
    --
    ----------------------------------------------------------
    
    function RTS_Integer_Width
    (
      First_Position   : TGT.Longest_Integer;
      Last_Position    : TGT.Longest_Integer
    )
    return Integer
    is
    --+-------------------------------------------------------
    --| Overview :
    --| Return maximum length of the image of the integers
    --| between First_Position and Last_Position.
    --| Only called if width is not statically known.
    --|  Note that as defined in LRM 3.5.5, if range is null, answer is zero.
    --|
    --| Algorithm
    --|   check for null range and return zero, else
    --|   find the one with the largest number of digits, i.e.,
    --|      max Log<10> (value) + 1 <for sign>.
    --|            (define Log<10>(0) == 1)
    --+-------------------------------------------------------

    use type TGT.Longest_Integer;

        function Log_10 (i:TGT.Longest_Integer) return TGT.Longest_Integer is
        --+-----------------------------------------------------
        --| Overview
        --|  Returns Log10(i)+1; where log10(0)==1.
        --|  and Log10(-i)==Log10(i).
        --|  Safely handles case where i = integer'first
            Numerator : TGT.Longest_Integer := i;  -- Starts as i,
                              -- gets divided by 10 in loop, counting
                              -- number of trips around the loop
        begin
            for Result in 2 .. 100 loop
                Numerator := Numerator / 10;
                if Numerator = 0 then
                    return TGT.Longest_Integer(Result);
                end if;
            end loop;
        end Log_10;
    
        function max (a, b: TGT.Longest_Integer) return Integer;
    
        PRAGMA INLINE (max);
    
        function max (a, b: TGT.Longest_Integer) return Integer is
        begin
            if a >= b then
                return Integer(a);
            else
                return Integer(b);
            end if;
        end max;
    begin
        if First_Position > Last_Position then
            return 0;
        else
            return max (log_10(First_Position), log_10(Last_Position));
        end if;
    end RTS_Integer_Width;
    --
    
    ----------------------------------------------------------
    -- Enumeration type routines.
    ----------------------------------------------------------
    
    function RTS_Enum_Position_To_Image(
      Enum_Position    : Integer;
      Enum_Index_Table : Enum_Integer_Array_Type;
      Enum_Image_Table : Enum_Character_Array_Type
    )
    return String
    is
    --+-------------------------------------------------------
    --| Overview :
    --|
    --| Given the position number of an enumeral, return its
    --| normalized spelling (i.e. upper case unless char enumeral).
    --+-------------------------------------------------------
    
        First_Char, Last_Char : Integer;
    
    begin
        --| Algorithm
        --| Determine appropriate slice of enum-image-table
        --| using enum-index-table, as follows:
        --| 'first is enum-index-table(enum-pos - 1) + 1
        --| 'last  is enum-index-table(enum-pos)
        --| Except if enum-pos is = 0 --> 'first = enum-image-table'first
        --|
        --| According to LRM 3.5.5 (depending on how you read it)
        --| low bound should be 1, hence we must convert or assign
        --| slice to get appropriate bounds.
        --|
        --| Character enumerals are a special case because only
        --| the first "tick" appears in the image-table.
    
        if Enum_Position = 0 then
            First_Char := Enum_Image_Table'First;
        else
            First_Char := Enum_Index_Table(Enum_Position - 1) + 1;
        end if;
    
        if Enum_Image_Table(First_Char) = ''' then
            -- Handle char literal as special case
            declare
                subtype str3 is String(1..3);
                retval : str3;
            begin
                retval(1) := ''';
                retval(2) := Enum_Image_Table(First_Char+1);
                retval(3) := ''';
                return retval;
            end; -- declare
    
        else
            -- Normal case, just fetch appropriate slice
            Last_Char  := Enum_Index_Table(Enum_Position);
    
            declare
                Length : constant Integer := Last_Char - First_Char + 1;
                subtype str_l is String (1 .. Length);
                retval : str_l;
            begin
                -- can't handle the slice assignment, either, so do it
                -- the hard way...
                for i in retval'range loop
                    retval (i) := Enum_Image_Table (First_Char + i - 1);
                end loop;
                return retval;
            end;  -- declare
    
        end if;  -- character special case
    
    
    end RTS_Enum_Position_To_Image;
    
    ----------------------------------------------------------
    --
    
    
    
    function RTS_Enum_Image_To_Position(
      Enum_Image       : String;
      Enum_Index_Table : Enum_Integer_Array_Type;
      Enum_Image_Table : Enum_Character_Array_Type
    )
    return Integer
    is
    --+-------------------------------------------------------
    --| Overview :
    --|
    --| Convert image of enumeral to its position number.
    --| Handle special case of char enumeral (only first "tick" in image tab)
    --+-------------------------------------------------------
    
        --| Algorithm
        --|
        --| Strip leading and trailing blanks
        --| Convert to upper case unless char enumeral
        --| remove trailing "tick" if char enumeral.
        --| scan image table for match, using last char and length as
        --| quick check.
    
        First_Index, Last_Index : Integer;  --| Indexes for slicing
        First_Character, Last_Character : Character;  -- Used for quick check
    --  Stripped_Image : constant string := Strip (Enum_Image);
            --| have to remove leading and trailing blanks
    
        first_nonblank : integer := Enum_Image'First;
        last_nonblank  : integer := Enum_Image'Last ;
            -- If have a null string, will have first_non > last_non
            --   and will raise constraint_error below
    
        First_Last_Diff : integer; -- length on non-blank portion of
    
            -- enum_imag, -1. Used to find candidate enum string
    begin
    
        for i in Enum_Image'range loop
            first_nonblank := i;
            exit when Enum_Image(i) /= ' ';
        end loop;
    
        for i in reverse Enum_Image'range loop
            last_nonblank := i;
            exit when Enum_Image(i) /= ' ';
        end loop;
    
            -- Check for bad string.
            -- Do > check instead of doing subtraction in case the string
            --    is empty and has bounds: (int'last .. int'first), which
            --    would raise numeric error.
        if first_nonblank > last_nonblank then
            -- In order for first and last to be crossed, either
            --   a. we had an empty string, in which case they started
            --      crossed and neither loop was entered, or
            -- b. The string was all blanks.
            raise CONSTRAINT_ERROR;
        end if;
    
        First_Last_Diff :=  last_nonblank  - first_nonblank;
        --| ASSERT First_Last_Diff > 0 if we reach here w/o Constraint_Err
    
        Last_Character := Upper_Case(Enum_Image(last_nonblank));
        First_Character := Enum_Image(first_nonblank);
        if Last_Character = ''' or First_Character = ''' then
            -- Handle character enumeral special case
            --  by stripping of trailing "tick"
            if First_Last_Diff /= 2 or else Last_Character /= First_Character
              -- Must be 'x' (I think).
              then
                raise CONSTRAINT_ERROR;  -- Can't match
            end if;
            last_nonblank := last_nonblank - 1;
            Last_Character := Enum_Image(last_nonblank); -- Not Upper Case!
            First_Last_Diff := 1;
        end if;
    
        First_Index := Enum_Image_Table'first;
    
        -- Now scan the image table, looking for a match (in upper-case)
    
        for Current_Position in Enum_Index_Table'Range loop
    
            Last_Index  := Enum_Index_Table(Current_Position);
            if First_Last_Diff = Last_Index - First_Index and then
              Last_Character = Enum_Image_Table(Last_Index) then
                -- last char & lengths match, check the rest
    
                for Enum_Index in first_nonblank .. last_nonblank-1 loop
                    exit when Upper_Case(Enum_Image (Enum_Index)) /=
                              Enum_Image_Table (First_Index);
                    -- the prefixes match, try another character
                    First_Index := First_Index + 1;
                end loop;
    
                if First_Index = Last_Index then
                    -- All of the chars matched, so we've found the enum
                    return Current_Position;
                end if;
            end if;
            -- NOPE! Try the next enum's string
            First_Index := Last_Index + 1;
        end loop;
    
           -- Nothing matches, so raise constraint error.
        raise CONSTRAINT_ERROR;
    
    end RTS_Enum_Image_To_Position;
    
    ----------------------------------------------------------
    --
    function RTS_Enum_Width(
      First_Position   : Integer;
      Last_Position    : Integer;
      Enum_Index_Table : Enum_Integer_Array_Type
    )
    return Integer
    is
    --+-------------------------------------------------------
    --| Overview :
    --|
    --| Calculate width, by scanning image table from first to last.
    --| Return maximum found.
    --?stt? Unable to properly handle char enumeral special case
    --?stt? without image table !!
    --?stt? It should be added as another parameter.
    --|
    --+-------------------------------------------------------
    
        Current_Width : Integer;
        Max_Width : Integer := 0;
        Previous_Last : Integer ;
          --  This is char position of slice corresponding
          -- to enum previous to current.
        Current_Last : Integer;
    
    begin
        if first_position <= Enum_Index_Table'first then
            Previous_Last := 0;
              -- first char of first enum starts at 1, so "last" char of
              -- "previous" enum ends at 0
        else
            Previous_Last := Enum_Index_Table (First_Position - 1);
              -- get last position of previous position
        end if;
    
        for Current_Position in First_Position .. Last_Position loop
            Current_Last := Enum_Index_Table(Current_Position);
            Current_Width := Current_Last - Previous_Last;
            if Current_Width = 2 then
                --?stt? Trouble, can't see if it is a char literal!
                --?stt? Assume it is for now, because bigger is safer
                Current_Width := 3;
            end if;
            if Current_Width > Max_Width then
                Max_Width := Current_Width;
            end if;
            Previous_Last := Current_Last;
        end loop;
        return Max_Width;
    
    end RTS_Enum_Width;
    
    
    function RTS_Enum_Rep_To_Pos(
    Rep_Array : Int_Array;
    Rep_Value : Integer
    ) return Integer is
    --| Algorithm
    --| Search slices of the Rep_Array by cutting in half successively,
    --| switching focus to the left or right half depending
    --| on the relation between the wanted value and the
    --| value at the mid point of the slice.
    
    --| Notes
    --| Rep_Array is in increasing order because "the integer codes
    --| must satisfy the predefined ordering relation" -- LRM 13.3:4.
    
    --| Pos of first rep is always zero -- LRM 3.5.1:4.
    
    --| Worst case performance is O(log2(Rep_Array'length)) thanks
    --| to binary search algorithm.
    
    Low : Int_Array_Index := Rep_Array'First;
       --| Low index in slice of interest
    High : Int_Array_Index := Rep_Array'Last;
       --| High index in slice of interest
    Mid : Int_Array_Index;
       --| Mid point in slice of interest
    Mid_Point_Rep_Value : Integer;
       --| Rep stored at mid point.
    begin
    while Low <= High loop
        -- Find mid point between Low and High
        Mid := (Low + High)/2;
        Mid_Point_Rep_Value := Rep_Array(Mid);
        if Rep_Value = Mid_Point_Rep_Value then
    
            -- Found the element of interest;
            -- Return its Pos
            -- (Pos of first rep is always zero LRM 3.5.1:4)
            return Integer(Mid - Rep_Array'First);
    
        elsif Rep_Value < Mid_Point_Rep_Value then
    
            -- Desired value is in left half of slice
            High := Mid - 1;
        else
    
            -- Desired value is in right half of slice
            Low := Mid + 1;
        end if;
    end loop;
    -- Not found in table.  Something is erroneous out there
    raise PROGRAM_ERROR;
    exception
    when others =>
        -- Something is erroneous
        raise PROGRAM_ERROR;
    end RTS_Enum_Rep_To_Pos;

end System.RTS.Image_And_Value;
