-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.image_and_value.spc,v $ 
-- $Revision: 1.4 $ -- $Date: 95/11/20 13:57:16 $ -- $Author: akira $
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.TGT;

package System.RTS.Image_And_Value is

    pragma Suppress( Elaboration_Check );

    package TGT renames System.RTS.TGT;

    ----------------------------------------------------------
    -- Scalar type routines.
    ----------------------------------------------------------

    function RTS_Integer_Image
    (
      The_Integer : TGT.Longest_Integer
    )
    return String;
    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the image attribute ('IMAGE) of an integer type.
    --| See LRM 3.5.5.
    --+-------------------------------------------------------

    function RTS_Integer_Value
    (
      The_Image : String
    )
    return TGT.Longest_Integer;
    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Given a string representing an integer, returns the value of
    --| the corresponding integer type ('VALUE). Raises
    --| CONSTRAINT_ERROR if the string is not a syntactically legal
    --| integer literal.
    --+-------------------------------------------------------

    function RTS_Integer_Width
    (
      First_Position   : TGT.Longest_Integer;
      Last_Position    : TGT.Longest_Integer
    )
    return Integer;
    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Return maximum length of the image of the integers
    --| between First_Position and Last_Position.
    --| Only called if width is not statically known.
    --+-------------------------------------------------------

-- New Page


    ----------------------------------------------------------
    -- Enumeration type routines.
    ----------------------------------------------------------

    type Enum_Character_Array_Type is
        Array (positive range <>) of Character;

    type Enum_Integer_Array_Type is array (Natural range <>) of Positive;
       -- index of last char position for i'th enumeral

    function RTS_Enum_Position_To_Image
    (
      Enum_Position    : Integer;
      Enum_Index_Table : Enum_Integer_Array_Type;
      Enum_Image_Table : Enum_Character_Array_Type
    )
    return String;
    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the image ('IMAGE) of an enumeration value in upper
    --| case or the corresponding character literal (including the
    --| two apostrophes).
    --|
    --+-------------------------------------------------------

    function RTS_Enum_Image_To_Position
    (
      Enum_Image       : String;
      Enum_Index_Table : Enum_Integer_Array_Type;
      Enum_Image_Table : Enum_Character_Array_Type
    )
    return Integer;
    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| If the sequence of characters has the syntax of an enumeration
    --| literal and if this literal exists for the base type of T,
    --| returns the corresponding enumeration value.
    --| Otherwise, CONSTRAINT_ERROR is raised.
    --+-------------------------------------------------------

    function RTS_Enum_Width
    (
      First_Position   : Integer;
      Last_Position    : Integer;
      Enum_Index_Table : Enum_Integer_Array_Type
    )
    return Integer;
    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Return maximum length of the image of the enumerations
    --| between First_Position and Last_Position.
    --| Only called if width is not statically known.
    --+-------------------------------------------------------
    type Int_Array_Index is new Integer;
    type Int_Array is array (Int_Array_Index range <>) of Integer;

    function RTS_Enum_Rep_To_Pos(
        Rep_Array : Int_Array;
        Rep_Value : Integer
    ) return Integer;

    --| EFFECTS
    --|
    --| Search Rep_Array and return actual Pos of Rep.
    --| This is only used for enumeration rep-specs.

    function RTS_Ivalue (
       The_image : String
    ) return TGT.Longest_Integer;

    --| EFFECTS
    --|
    --| Returns an integer value of the string. ASSUMES A LEGAL ONE !!
    --| (This is a semi private function used by RTS_Integer_Value and the
    --| Integer_IO_Support.

end System.RTS.Image_And_Value;
