-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.integer_image_and_value_pkg.bdy,v $ 
-- $Revision: 1.8 $ $Date: 96/08/22 18:00:20 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Lexer;
with System.RTS.Integer_State_Tables;
with Ada.IO_Exceptions; use Ada.IO_Exceptions;
with Ada.Characters.Handling;
with Ada.Characters.Latin_1;
with System.RTS.Unsigned;
with Ada.Unchecked_Conversion;

package body System.RTS.Integer_Image_And_Value_Pkg is

-- WARNING: the subprogram declaration format is important, since this
-- file is read used with the rts calls from the emitter.
-- see intrintic_rts.c

    pragma Suppress( Elaboration_Check );

-- Note: The basis of this code is taken from the aie host_rts _type
-- subdirectory:
-- Source: /home/inmet/m/ada/monoBANK/RTS/RTS_IV.bdy,v 
-- Revision: 1.37  -- Date: 88/06/13 16:34:04  -- Author: callen 

    subtype LegalRange is System.RTS.Unsigned.Unsigned_Integer range 0 .. 16 ;
    type CharValueArray is array (character) of LegalRange ;
    CharValue : constant CharValueArray := CharValueArray'
      ('0' => 0, '1' => 1, '2' => 2, '3' => 3, '4' => 4,
       '5' => 5, '6' => 6, '7' => 7, '8' => 8, '9' => 9,
       'a'|'A'=> 10, 'b'|'B' => 11, 'c'|'C'=>12, 'd'|'D'=>13,
       'e'|'E' => 14, 'f'|'F' => 15, others => 16) ;
         -- Put this here so it is only done once (since
	 --  static aggregates are still a ways off)
     
    function conv is new Ada.Unchecked_Conversion
                            (System.RTS.Unsigned.Unsigned_Integer, Integer);

    ----------------------------------------------------------
    -- Local Scalar routines.
    ----------------------------------------------------------
   
    procedure RTS_Ivalue(
      The_Image : in  String;
      The_Value : out System.RTS.Unsigned.Unsigned_Integer;
      The_Sign  : out Boolean
    )
    is
	use System.RTS.Unsigned;
        Result : System.RTS.Unsigned.Unsigned_Integer ;
        E_Result : System.RTS.Unsigned.Unsigned_Integer := 1 ;
	-- Starts as 10 for integer or base of based literal
        Base : System.RTS.Unsigned.Unsigned_Integer := 10 ;
        Exponent_Base : System.RTS.Unsigned.Unsigned_Integer := Base;
        
        Curr_Int  : System.RTS.Unsigned.Unsigned_Integer := 0;
    	--  Accumulates the negative of the "current integer",
    	-- which can be the base, integer, or exponent.
    	-- The value is always accumulated as a non-POSITIVE number,
    	-- because there is one more negative number than positive numbers
    	-- so it is possible to do value("<integer'first>").
    	-- THEREFORE you must negate curr_int to get its "TRUE" value
    	-- before use as base or exponent. Note that vbl Sign (below)
    	-- has positive== -1  negative== +1 so the conversion of curr_int
    	-- to its TRUE signed value happens via the multiply by sign.
        Prev_Int  : System.RTS.Unsigned.Unsigned_Integer;

        Sharp_Encountered : Boolean := false;
        subtype LegalBaseRange is 
	            System.RTS.Unsigned.Unsigned_Integer range 2 .. 16;
        IntValue : LegalRange ;
        Plus : Boolean := True;
        E_Plus : Boolean := True;
        
        E_is_exponent : boolean := true ; --| consider 'e' to be the
    				      --| beginning of exponent.
        Exponent_Seen : boolean := false; -- Has there been an exponent?
    begin
        --| Integer_literal ::= (integer | based_integer) exponent
        --| integer ::= digit < (:_:) digit >
        --| based_integer ::= base # extended_integer #
        --| extended_integer ::= extended_digit < (:_:) extended_digit >
        --| Requires
        --| The input string must be syntactically legal.
        --| Follows the rules in LRM Chapter 2.
        
        -- Note that if we are processing the digits between # and #
        -- that an e should be interpreted as a digit, not the
        -- indicator that an exponent is following.

        for i in The_Image'range loop
    	case The_Image(i) is
    	when '#' | ':' =>
    	    if Sharp_Encountered then
    		E_is_exponent := true ;
    		Sharp_encountered := false;
    	    else
    		Sharp_encountered := true;
    		E_is_exponent := false ;
    		Base := Curr_Int;
    		Exponent_Base := Base;
    		Curr_Int := 0; -- get ready for based number
    		if Base not in LegalBaseRange then
    		    raise CONSTRAINT_ERROR ;
    		end if ;
    	    end if;
    	when '_' =>
    	    null;
        when ' ' | Ada.Characters.Latin_1.HT =>
    	    -- Should not really get here, but...
    	    exit;
    	when '+' =>
	    if Exponent_Seen then
    	        E_Plus := True ;
            else
    	        Plus := True ;
	    end if;
    	when '-' =>
	    if Exponent_Seen then
    	        E_Plus := False ;
            else
    	        Plus := False ;
	    end if;
    	when others =>
    	    -- Process the digit.
    	    --| Should be either 0..9 a..f A..F
    	    IntValue := CharValue(The_Image(i)) ;
    	    case IntValue is
    	    when 16 =>
    		raise CONSTRAINT_ERROR ;
    	    when 14 =>  -- 'e' or 'E'
    		if E_is_exponent then
    		    if Exponent_Seen then
    			-- Multiple Exponents are Bad!
    			raise CONSTRAINT_ERROR;
    		    end if;
    		    Result := Curr_Int; 
    		    Exponent_Seen := true;
    		    -- Reset things for accumulating exponent
    		    E_Plus := True;
    		    Curr_Int := 0;
    		    Base := 10;
    		    -- set IntValue to zero, so when we fall through to
    		    -- the intvalue test and multiply updating Curr_Int,
    		    -- that the effect will be to set Curr_Int to zero
    		    -- again.  Alternative is to use GOTO, or to introduce
    		    -- a boolean...
    		    IntValue := 0;
    		end if ;
    		-- E is just a digit in the number (between # and #)
    	    when others =>
    		null ;
    	    end case ;
    	    if IntValue not in 0 .. Base-1 then
    		raise CONSTRAINT_ERROR ;
    	    end if ;
	    Prev_Int := Curr_Int;
    	    Curr_Int := (Curr_Int*System.RTS.Unsigned.Unsigned_Integer(Base)) + 
                System.RTS.Unsigned.Unsigned_Integer(IntValue) ;
            if Prev_Int > Curr_Int then
		raise CONSTRAINT_ERROR;
	    end if;
    	end case ;
        end loop ;
        
        if Exponent_Seen then
    	    if not E_Plus then
    	        -- Negative exponent is illegal for integer'value
    	        raise CONSTRAINT_ERROR ;
    	    end if;
	    for i in 1 .. Curr_Int loop
	        E_Result := E_Result * 
		    System.RTS.Unsigned.Unsigned_Integer(Exponent_Base);
            end loop;
    	    Result := Result * E_Result;
        else
    	    Result := Curr_Int;
        end if;

        The_Value := Result;
	The_Sign  := Plus;
    exception
        when NUMERIC_ERROR =>
    	-- Can be raised by overflows.  But we're only supposed to
    	-- raise constraint error in this routine, so convert it.
    	raise CONSTRAINT_ERROR;
    
    end RTS_Ivalue;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
        ----------------------------------------------------------
        -- Integer type routines.
        ----------------------------------------------------------
    
    function RTS_Integer_Image(
      The_Integer : Integer
    )
    return String
    is
    --+-------------------------------------------------------
    --| Overview :
    --|   Computes Integer'Image (The_Integer) 
    --|     1. Decimal
    --|     2. Leading '-' or blank
    --|     3. return string'first == 1
    --| 
    --| Algorithm
    --|   1. Check sign of result, and make Int NEGATIVE (you can represent
    --|      one more negative number than positive one, i.e., min_int).
    --|   2. Take rem of Int, convert to character by adding pos of zero
    --|      (rem is negative, since Neg rem Pos == Negative)
    --|      (Nb. MOD gives different, WRONG result)
    --|   3. Put char into Work_String, from back to front
    --|   4. Div Int by 10, if it's now zero, we're done, else loop
    --|   5. Done: copy from work_String to Result so that 'first == 1
    --|   Note that the Done test is performed after the char is put
    --|   into string; this makes the case of Zero work. (i.e., a while
    --|   loop won't work).
    --+-------------------------------------------------------
        Base : constant := 10;
        Pos_Zero : constant := Character'pos ('0');
        Subtype String_Range is positive range 1 .. 20;
                -- 30 chars should be enough to hold any int!
    	    -- 'first MUST be 1
    	    -- Efficiency note: By declaring everything to have range
    	    --   String_Range, constraint checks will be eliminated.
        
        Work_String,      -- digits get put in from 'last toward 'first
        Result :          -- return result must start at 1
                string (String_Range);
        Last_Result_Index : String_Range := String_Range'first;
                -- last char of Result to return
        Int : Integer range Integer'first .. 0;    -- work space for integer
    begin
        if The_Integer >= 0 then
    	Int := - The_Integer;
    	Result (1) := ' ';
        else
    	Int :=  The_Integer;
    	Result (1) := '-';
        end if;
    
        for i in reverse String_Range loop
    	Work_String (i) := Character'val (Pos_Zero - (Int rem Base));
    	    -- (Int rem Base is <= 0 )
    	Int := Int / Base;
    	if Int >= 0 then
    	    -- We're done! Compute where last char should be moved,
    	    -- Make slice assignment, then return the slice.
    	    -- Last_index := length_int + first_int_pos<2> - 1
    	    --  == (w_s'last - i + 1) + 2 - 1
    	    --      Cancel         $        $
    	    Last_Result_Index := (Work_String'last - i) + 2;
    	    Result (2 .. Last_Result_Index) :=
    	      Work_String (i .. Work_String'last);
    	    return Result (1 .. Last_Result_Index);
    	end if;
        end loop;
        -- Can't get here unless the work_string is too short!
        -- Program_Error will be raised.
    end RTS_Integer_Image;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
    function RTS_Integer_Wide_Image (
      The_Integer : Integer
        ) return Wide_String 
    is
    --+-------------------------------------------------------
    --| Overview:
    --|
    --|    Call the RTS_Integer_Image and then convert to wide_image.
    --+-------------------------------------------------------
    begin
        return Ada.Characters.Handling.To_Wide_String (
       	RTS_Integer_Image (The_Integer));
    end RTS_Integer_Wide_Image;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
    function RTS_Integer_Value (
      The_Image : String
    ) return Integer
    is
        --| EFFECTS
        --|
        --| Given a string representing an integer, returns the value of
        --| the corresponding integer type ('VALUE). Raises
        --| CONSTRAINT_ERROR if the string is not a syntactically legal
        --| integer literal.
    --+-------------------------------------------------------
    --| Overview :
    --|   returns integer'value of the string.
    --|
    --| Algorithm :
    --|  Skips all leading blanks, HT's are illegal, then calls RTS_Ivalue
    --|  to return an integer value. Scan the remainder of the string (if any)
    --|  only blank are allowed.
    --|
    --+-------------------------------------------------------
	use System.RTS.Unsigned;

        My_Last : natural := 0;
        My_First: natural := 0;
	Value : Integer;
	U_Value : System.RTS.Unsigned.Unsigned_Integer;
	Plus    : Boolean;
        
    begin
        
        for i in The_Image'range loop
    	if The_Image(i) /= ' ' then
        if The_Image(i) = Ada.Characters.Latin_1.HT then
    		-- Because of IO, HT is allowed in the lex tables but
    		-- not here.
    		raise Constraint_error;
    	    else
    		My_First := i;
    		exit;
    	    end if;
    	end if;
        end loop;
    	
        if My_First = 0 then
    	return 0;
        end if;
    	
        begin
            System.RTS.Lexer.RTS_lex_String (
				  The_image(My_First..The_Image'last),
    				  System.RTS.Integer_State_Tables.Transitions,
    				  System.RTS.Integer_State_Tables.Mapping,
    				  My_Last);
        exception
    	when Data_Error | End_Error =>
    	    -- Since the lexer is originally part of the IO stuff, it
    	    -- sometimes raises these exceptions, all we need is
    	    -- Constraint_error.
    	    raise Constraint_error;
        end;
    				  
        if My_Last < My_First then
    	raise Constraint_error;
        end if;
    	
        for i in My_Last+1..The_Image'last loop
    	-- The only character allowed here is blank.
    	if The_image(i) /= ' ' then
    	    raise Constraint_error;
    	end if;
        end loop;
       
        RTS_Ivalue(The_image(My_First..My_Last),U_Value,Plus);
	if U_Value <= System.Max_Int then
	    Value := conv(U_Value);
	    if not Plus then
		Value := -Value;
            end if;
        elsif not Plus and then U_Value = System.Max_Int + 1 then
	    Value := conv(U_Value);
        else
            raise Constraint_error;
	end if;

        return Value;
        
    end RTS_Integer_Value;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
    function RTS_Integer_Wide_Value (
        The_Image : Wide_String
    ) return Integer
    is
        --+-------------------------------------------------------
        --| EFFECTS
        --|
        --| Given a string representing an integer, returns the value of
        --| the corresponding integer type ('WIDE_VALUE). Raises
        --| CONSTRAINT_ERROR if the string is not a syntactically legal
        --| integer literal.
        --+-------------------------------------------------------
    begin
        return RTS_Integer_Value(Ada.Characters.Handling.To_String(The_Image));
    end RTS_Integer_Wide_Value;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
   
    function RTS_Unsigned_Image (
      The_Unsigned : System.RTS.Unsigned.Unsigned_Integer
    ) return String is
	use System.RTS.Unsigned;
        Base : constant System.RTS.Unsigned.Unsigned_Integer := 10;
        Pos_Zero : constant := Character'pos ('0');
        Subtype String_Range is positive range 1 .. 20;
        Work_String, Result : string (String_Range);
        Last_Result_Index : String_Range := String_Range'first;
                -- last char of Result to return
        Uns: System.RTS.Unsigned.Unsigned_Integer := The_Unsigned;
    begin
	Result(1) := ' ';
        for i in reverse String_Range loop
            Work_String (i) := 
		Character'val (Pos_Zero + (Uns rem Base));
            Uns := Uns / Base;
            if Uns = 0 then
                -- We're done! Compute where last char should be moved,
                -- Make slice assignment, then return the slice.
                -- Last_index := length_int + first_int_pos<2> - 1
                --  == (w_s'last - i + 1) + 2 - 1
                --      Cancel         $        $
                Last_Result_Index := (Work_String'last - i) + 2;
                Result (2 .. Last_Result_Index) :=
                  Work_String (i .. Work_String'last);
                return Result (1 .. Last_Result_Index);
            end if;
        end loop;
    end RTS_Unsigned_Image;
 
    function RTS_Unsigned_Wide_Image (
      The_Unsigned : System.RTS.Unsigned.Unsigned_Integer
    ) return Wide_String is
    begin
        return Ada.Characters.Handling.To_Wide_String (
		    RTS_Unsigned_Image (The_Unsigned));
    end RTS_Unsigned_Wide_Image;

    function RTS_Unsigned_Value (
      The_Image : String
    ) return System.RTS.Unsigned.Unsigned_Integer is
	use System.RTS.Unsigned;
        LL : natural := 0;
        FF : natural := 0;
        Result : Unsigned_Integer;
	Plus : Boolean;
    begin

        for i in The_Image'range loop
            if The_Image(i) /= ' ' then
                if The_Image(i) = Ada.Characters.Latin_1.HT then
                -- Because of IO, HT is allowed in the lex tables but
                -- not here.
                    raise Constraint_error;
                else
                    FF := i;
                    exit;
                end if;
            end if;
        end loop;

        if FF = 0 then
            return 0;
        end if;

        begin
            System.RTS.Lexer.RTS_lex_String (
				The_image(FF..The_Image'last),
                                System.RTS.Integer_State_Tables.Transitions,
                                System.RTS.Integer_State_Tables.Unsigned_Mapping,
                                LL);
        exception
            when Data_Error | End_Error =>
            -- Since the lexer is originally part of the IO stuff, it
            -- sometimes raises these exceptions, all we need is
            -- Constraint_error.
            raise Constraint_error;
        end;

        if LL < FF then
            raise Constraint_error;
        end if;

        for i in LL+1..The_Image'last loop
            -- The only character allowed here is blank.
            if The_image(i) /= ' ' then
                raise Constraint_error;
            end if;
        end loop;

	RTS_Ivalue(The_Image(FF..LL),Result,Plus);

        return Result;

    end RTS_Unsigned_Value;

    function RTS_Unsigned_Wide_Value (
      The_Image : Wide_String
    ) return System.RTS.Unsigned.Unsigned_Integer is
    begin
        return RTS_Unsigned_Value(Ada.Characters.Handling.To_String(The_Image));
    end RTS_Unsigned_Wide_Value;

    ----------------------------------------------------------
    ----------------------------------------------------------
    function RTS_Integer_Width
    (
      First_Position   : Integer;
      Last_Position    : Integer
    )
    return Integer
    is
    --+-------------------------------------------------------
    --| Overview :
    --| Return maximum length of the image of the integers
    --| between First_Position and Last_Position.
    --| Only called if width is not statically known.
    --|  Note that as defined in LRM 3.5.5, if range is null, answer is zero.
    --|  
    --| Algorithm
    --|   check for null range and return zero, else
    --|   find the one with the largest number of digits, i.e.,
    --|      max Log<10> (value) + 1 <for sign>.
    --|            (define Log<10>(0) == 1)
    --+-------------------------------------------------------

      function Log_10 (i:integer)
        return integer
      --+-----------------------------------------------------
      --| Overview
      --|  Returns Log10(i)+1; where log10(0)==1.
      --|  and Log10(-i)==Log10(i).
      --|  Safely handles case where i = integer'first
      is
          Numerator : integer := i;  -- Starts as i,
    			             -- gets divided by 10 in loop, counting
    			             -- number of trips around the loop
      begin
        for Result in 2 .. 100 loop
    	  Numerator := Numerator / 10;
    	  if Numerator = 0 then
    	    return Result;
    	  end if;
        end loop;
      end Log_10;
    			    
      function max (a, b: integer) return integer;
      PRAGMA INLINE (max);
    
      function max (a, b: integer)
        return integer
      is
      begin
    	if a >= b then
    	  return a;
    	else
    	  return b;
    	end if;
      end max;
        
    begin
      if First_Position > Last_Position then
    	return 0;
      else
    	return max (log_10(First_Position), log_10(Last_Position));
      end if;
    end RTS_Integer_Width;

    function RTS_Unsigned_Width (
      Position  : System.RTS.Unsigned.Unsigned_Integer
    )
    return Integer
    is
      Numerator : System.RTS.Unsigned.Unsigned_Integer := Position;
    begin
      for Result in 2 .. 100 loop
	Numerator := System.RTS.Unsigned."/"(Numerator, 10);
        if System.RTS.Unsigned."="(Numerator, 0) then
	  return Result;
        end if;
      end loop;
    end RTS_Unsigned_Width;


end System.RTS.Integer_Image_And_Value_Pkg;
