-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.master_pkg.spc,v $ 
-- $Revision: 1.14 $ $Date: 96/12/17 17:09:36 $ $Author: vdelvecc $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Tasking_Types_Pkg;
with System.RTS.Protected_Types_Pkg;

package System.RTS.Master_Pkg is

    pragma Suppress( Elaboration_Check );

    subtype ECD_Access is Protected_Types_Pkg.ECD_Access;

    procedure RTS_Init_Master( Master : Tasking_Types_Pkg.Master_Access;
	                       Task_Body_Master : Boolean );
    --* OVERVIEW
    --  This procedure is called at the top of a declarative
    --  part to create an empty master object.  As declarations are
    --  elaborated, they are attached to the finalization list
    --  if necessary.  As task declarations are elaborated, they are
    --  added to the master's list of dependents.
    pragma Export( ada,
                   RTS_Init_Master,
                   "rts_init_master" );
    pragma Global( RTS_Init_Master );

    procedure RTS_Finalize_And_Keep( Master : Tasking_Types_Pkg.Master_Access );
    pragma Export( ada,
                   RTS_Finalize_And_Keep, 
                   "rts_finalize_and_keep" );
    pragma Global( RTS_Finalize_And_Keep );
    --* OVERVIEW
    -- Called from generated code for reuse of empty master.
    --* EFFECTS
    -- Walk the finalization list backwards, up to an including
    -- designated master.  If there are dependent tasks, they will
    -- be awaited before returning.

    procedure RTS_Finalize_And_Pop( Master : Tasking_Types_Pkg.Master_Access );
    pragma Export( ada,
                   RTS_Finalize_And_Pop,
                   "rts_finalize_and_pop" );
    pragma Global( RTS_Finalize_And_Pop );
    --* OVERVIEW
    -- Same as above except that Master is removed from master
    -- list (the task-wide finalization list).

    procedure Finalize_Default_Master;
    --* OVERVIEW
    -- Called from startup to finalize application.

    -- Locking Operations (uses global lock)

    procedure Add_Dependent( Master    : Tasking_Types_Pkg.Master_Access;
	    	    	     Dependent : Tasking_Types_Pkg.TCB_Access );
    --* OVERVIEW
    -- Called when a task is created.  Dependents are added
    -- here so that if the declarative part does not complete,
    -- the dependents can be terminated.  They won't
    -- have been activated at this time.  Dependents must be
    -- tracked so that TASKING_ERROR
    -- can be raised in them.  Also, stack space can be released.

    procedure Remove_Terminated_Dependent(
	Master    : Tasking_Types_Pkg.Master_Access;
	Dependent : Tasking_Types_Pkg.TCB_Access
    );
    --* OVERVIEW
    -- When a task terminates, it is removed from the list of dependents
    -- and the masters' terminable counts are decremented.

    procedure Set_Acceptor_Terminable(
	TCB : Tasking_Types_Pkg.TCB_Access
    );
    --* OVERVIEW
    -- Called when a selective accept has an open terminate alternative
    -- (ARM 9.3(6))
    -- Effects reversed by Cancel_Terminable.

    procedure Cancel_Terminable( Master : Tasking_Types_Pkg.Master_Access );
    --* OVERVIEW
    -- Called when an open terminate alternative is bypassed.

    procedure Abort_Self(Level : Natural);
    --* OVERVIEW
    --  Aborts current task

    procedure Abort_Task( TCB : Tasking_Types_Pkg.TCB_Access );
    --* OVERVIEW
    --  Abort specified task and all dependents.

    -- Control blocks for the envirnment task.
    -- (rest of initialization takes place in "Init_Environment_TCB"
    Envr_TCB : aliased Tasking_Types_Pkg.TCB_Type(
	Num_Entries => 0 );	-- Really zero entries
 
end System.RTS.Master_Pkg;
