-- $Source: /home/dsd/d/aa/monoBANK/AA/TSP_system.rts.memory_pkg.spc,v $
-- $Revision: 1.2 $ $Date: 96/06/21 11:17:08 $ $Author: stt $
--        Copyright (C) 1995 by Intermetrics, Inc.   

package System.RTS.Memory_Pkg is

    pragma Suppress( Elaboration_Check );


    procedure RTS_Memmove_Left_to_Right( Src       : in Address;
		          		 Dest      : in Address;
		                         Size      : in Natural;
			                 Alignment : in Natural := 0 );
    pragma Import (Intrinsic, RTS_Memmove_Left_to_Right,
                   External_Name => "rts_memmove_left_to_right" );
    pragma Global (RTS_Memmove_Left_to_Right);
    --*OVERVIEW
    -- RTS_Memmove_Left_to_Right should be called when it can be determined
    -- staticly by the compiler that...
    --    	1) Src >= Dest
    --	  	2) Size <= 1
    -- 		3) Src..Src+Size-1 AND Dest..Dest+Size-1 are disjoint
    --
    --*EFFECTS
    -- RTS_Memmove_Left_to_Right will copy the contents of memory from
    -- Src to Src+Size-1 to the memory from Dest to Dest+Size-1.  Copying
    -- will begin with the Src location


    procedure RTS_Memmove_Right_to_Left( Src       : in Address;
		          		 Dest      : in Address;
		                         Size      : in Natural;
			                 Alignment : in Natural := 0 );
    pragma Import (Intrinsic, RTS_Memmove_Right_to_Left,
                   External_Name => "rts_memmove_right_to_left" );
    pragma Global (RTS_Memmove_Right_to_Left);
    --*OVERVIEW
    -- RTS_Memmove_Right_to_Left should be called when it can be determined
    -- by the compiler that...
    --    	1) Src < Dest
    --
    --*EFFECTS
    -- RTS_Memmove_Right_to_Left will copy the contents of memory from
    -- Src to Src+Size-1 to the memory from Dest to Dest+Size-1.  Copying
    -- will begin with the Src+Size -1 location


    procedure RTS_Memmove( Src       : in Address;
		           Dest      : in Address;
		           Size      : in Natural;
			   Alignment : in Natural := 0 );
    pragma Import (Intrinsic, RTS_Memmove, External_Name => "rts_memmove" );
    pragma Global (RTS_Memmove);
    --*OVERVIEW
    -- RTS_Memmove should be called when Src,Dest, and Size can not be
    -- determined at compiler time.  This routine incorporates the logic
    -- of the previous 2 routines.
    --
    --*EFFECTS
    -- RTS_Memmove acts as the ANSI C memmove, except that there is no
    -- return value

    function RTS_Memcmp( Left  : in Address;
		         Right : in Address;
		         Size  : in Natural ) return Integer;
    pragma Import (Intrinsic, RTS_Memcmp, External_Name => "rts_memcmp" );
    pragma Global (RTS_Memcmp);
    --*OVERVIEW
    -- RTS_Memeq is used to compare raw blocks of memory.  It should
    -- only be used when the number of words to compare is either
    -- large or not compile-time known.  Small comparisons should be
    -- performed in-line.
    --
    --*EFFECTS
    -- RTS_Memeq compares the two blocks of memory, returning 1 if
    -- Left > Right, 0 if the two are equal, and -1 if Left < Right.
    -- The comparison is logical (i.e.: unsigned).

end System.RTS.Memory_Pkg;
