-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ***************************************************************************
-- PACKAGE SPECIFICATION : System.RTS.Shared_Messages
--
-- OUTLINE: 
--   User error messages may be written in one of several natural languages.
-- The text of these error messages is encapsulated in this package whose body
-- is natural language dependent. Each error message is referenced by an error
-- code of type MESSAGE. Some natural languages require extended ascii to
-- provide the entire character set.
-- ***************************************************************************

with System.Rts.Error_IO; use System.Rts.Error_IO;
package System.RTS.Shared_Messages is

   pragma SUPPRESS (ELABORATION_CHECK);

   -- ------------------------------------------
   -- Package renamings to remove ALSYS_ prefix:
   -- ------------------------------------------
   package ERROR_IO renames System.Rts.Error_IO;
   -- ------------------------------------------

   procedure MESSAGE_LOOKUP (MESS   : in MESSAGE;
	       	             TEXT   : in out MESSAGE_STRING;
	                     LENGTH : out NATURAL);
   -- -------------------------------------------------------------------
   -- OUTLINE:
   --   Looks up a language-dependent message in the message table and
   -- the extended ascii string which is that message is written into
   -- TEXT. The length of the string is returned in LENGTH.
   -- -------------------------------------------------------------------

end System.RTS.Shared_Messages;

-- Modification History:

--  6.01 22nd March 1991 (BJD)
--       Initial introduction to CMS.

-- 6.??   3rd November 1991 (BJD)
--        Prefix all CRTS library unit names with ALSYS_
