-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

------------------------------------------------------------------------------
-- PACKAGE SPECIFICATION : System.RTS.Tgt_INTEGERS   (TEMPLATE FILE)
--
-- This module is a TEMPLATE file. It is entirely target-dependent and each
-- target is expected to have its own version. It serves two purposes:
--
-- (a) a central definition of the integers supported on the target for use
--     by target-dependent runtime code;
--
-- Within the target-dependent parts of the runtime, these types should be
-- used only when the exact representation is important (e.g. inside
-- records which interface to external routines or other external data
-- structures). In all other cases, local type declarations should be
-- used to map onto the "best" underlying type.
--
-- (b) the provider of the parent types for the derived type definitions in
--     RT_TYPES, which is the interface to the common runtime code.
--
-- The common portion of the runtime does not 'with' this package. All
-- representation-sensitive types are acquired from RT_TYPES. These types
-- are derived from types declared in this package.
--
-- For example, one target might define type RT_TYPES.EXCEPTION_CODE as
-- TARGET_INTEGERS.INTEGER_16, whilst another might use 
-- TARGET_INTEGERS.INTEGER_32, or even TARGET_INTEGERS.INTEGER_36 if it has
-- a 36-bit word.
--
-- The version provided here is an illustration of a target supporting three
-- predefined integer types on 8, 16 and 32-bits.
------------------------------------------------------------------------------

package System.RTS.Tgt_INTEGERS is

   pragma SUPPRESS (ELABORATION_CHECK);

   --       *********************
   --       * TABLE OF CONTENTS *
   --       *********************

   --       1. Target-Dependent Integer Types
   --       2. Target-Dependent Access Types

   --       *************************************
   --       * 1. Target-Dependent Integer Types *
   --       *************************************

   type INTEGER_32 is range - (2 ** 31) .. (2 ** 31) - 1;
   type INTEGER_16 is range -32768 .. +32767;
   type INTEGER_8  is range -128 .. +127;

   for INTEGER_32'SIZE use 32;
   for INTEGER_16'SIZE use 16;
   for INTEGER_8'SIZE  use 8;

   -- Note that due to limitations in the current Ada rules (and our
   -- consequent implementation), the UNSIGNED_32 type cannot be properly
   -- defined, nevertheless, we define it here for documentation purposes.

   type UNSIGNED_32 is mod 2**32;
   type UNSIGNED_16 is mod 2**16;
   type UNSIGNED_8  is mod 2**8;

   for UNSIGNED_32'SIZE use 32;
   for UNSIGNED_16'SIZE use 16;
   for UNSIGNED_8'SIZE  use 8;

   --       ************************************
   --       * 2. Target-Dependent Access Types *
   --       ************************************

   type REF_INTEGER_32 is access INTEGER_32;
   type REF_INTEGER_16 is access INTEGER_16;
   type REF_INTEGER_8  is access INTEGER_8;

   type REF_UNSIGNED_32 is access UNSIGNED_32;
   type REF_UNSIGNED_16 is access UNSIGNED_16;
   type REF_UNSIGNED_8  is access UNSIGNED_8;

   -- Note that we do not need to suppress the Heap calls to Init_Collection
   -- for these access types since this spec is always compiled before
   -- ALSYS_ADA_RUNTIME spec.

end System.RTS.Tgt_INTEGERS;


-- Modification History:

--  6.01 9th January 1991 (BJD)
--       Initial introduction to Shared RTS libraries

--  6.02 1st February 1991 (APC)
--       Removed GET and PUT, instances of FETCH_FROM/ASSIGN_TO_ADDRESS
--       which aren't used and couldn't have been inlined if they were.

--  Moved to TEMPLATE library 12th March 1991 (BJD)

--  6.02 13th March 1991 (BJD)
--       Add more comments to template file.

--  6.03 3rd November 1991 (BJD)
--       Add null body as per coding standards.
--       Prefix all CRTS library unit names with ALSYS_
--  DEC/CMS REPLACEMENT HISTORY, Element TARGINTS.ADS
--  *3     6-NOV-1991 23:08:44 CMS_DAEMON "ltd_daemon:dobbing:Add ALSYS_ prefix"
--  *2    19-MAR-1991 23:24:47 CMS_DAEMON "ltd_daemon:DOBBING:Add more comments"
--  *1    19-MAR-1991 23:24:31 CMS_DAEMON "Mimic From France"
--  DEC/CMS REPLACEMENT HISTORY, Element TARGINTS.ADS
-- VAX/DEC CMS REPLACEMENT HISTORY, Element TARGINTS.ADS
-- *1    12-MAY-1992 11:26:52 KEHS "INITIAL VERSION FOR 386-UNIX"
-- VAX/DEC CMS REPLACEMENT HISTORY, Element TARGINTS.ADS
