-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.tasking_types_pkg.bdy,v $ 
-- $Revision: 1.26 $ $Date: 1997/02/25 12:22:51 $ $Author: bethc $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with Ada.Unchecked_Conversion;
with Ada.Unchecked_Deallocation;
with Ada.Exceptions.RTS_Utils;
with System.RTS.TGT.Kernel.Threads;
with System.RTS.TGT.Kernel.Mutexes;
with System.RTS.TGT.Kernel.Abort_Signals;
with System.RTS.Interface_Types;
with System.RTS.Protected_Operations_Pkg;
with System.RTS.Protected_Interface;
with System.RTS.Master_Pkg; 

package body System.RTS.Tasking_Types_Pkg is                                   

    pragma Suppress( Elaboration_Check );
    pragma Suppress( Storage_Check); -- Release_Thread is called
    				     -- after the stack limits are deallocated

    package EXU	renames Ada.Exceptions.RTS_Utils;    
    package PRO	renames Protected_Operations_Pkg;    
    package PRI	renames Protected_Interface;

    subtype Kernel_Priority is TGT.Kernel.Kernel_Priority;
    subtype Thread_Ptr is TGT.Kernel.Threads.Thread_Ptr;

    Held_Priority : constant Kernel_Priority := Kernel_Priority'First;

    -- Constants used as special values of TCB.Storage_Size
    Not_Released : constant := -1;
    Released : constant := 0;

    -- Free list of TCBS and associated protection
    Free_TCBs : TCB_Access;
    Free_TCBs_Mutex : aliased TGT.Kernel.Mutexes.Mutex_Descriptor;

    Next_Attribute_ID : Attribute_ID := Attribute_ID'First;

    -- Global table of attribute finalization routines
    Attribute_Cleanup_Op_Table : array(Attribute_ID
      range 1..Last_Attribute_ID) of Attribute_Cleanup_Op := 
	(others => null);

    function To_ECD_Access is new Ada.Unchecked_Conversion (
      INT.Data_Address, PRT.ECD_Access);
		    
    function To_TCB_Access is new Ada.Unchecked_Conversion (
      INT.Data_Address, TCB_Access);

    function Large_Queue_Count(
	PO : PRT.PO_Access;
	Name : PRT.Entry_Name_Index;
	Member : PRT.Entry_Member_Index
    ) return Natural is
	ECD : PRT.ECD_Access := PO.Large_Queue;
	use PRT;
    begin
        while ECD /= null loop
	    if (ECD.Name_Index = Name and then ECD.Member_Index = Member) then
		return ECD.Count;
	    end if;
	    ECD := ECD.Next_Queue;
	end loop;
	return 0;    
    end Large_Queue_Count;

    procedure Free is new Ada.Unchecked_Deallocation(
      Attribute_Index_Extension, Attribute_Index_Extension_Access);

    protected body Pr_TCB_Type is  

	-- ***TDB*** 
	-- We have an unsolved race condition regarding the check for
	-- termination.  The tasking outer layer does a gen# check, and
	-- if the check succeeds, continues with a protected call 
        -- to the Pr_TCB.  The TCB could become obsolete between the check
	-- and the call. One solution is to move the TCB's gen# into
        -- the Pr_TCB and alos pass in the gen# from GC to most
        -- protected operations.  There might be a better way to do
	-- this by considering the contexts of the gen# check.

        function max(
	   Left, Right : System.Any_Priority'Base
	) return System.Any_Priority'Base is
	begin
            if Left > Right then
	        return Left;
	    else
	        return Right;
	    end if;
        end max;

        function Get_Attribute_Handle(ID : Attribute_ID)
          return Attribute_Handle;

        procedure Default_Reinitialize(Enclosing_TCB : TCB_Access) is
	begin
	     -- Initialize important components to same values as default.
            Current_Nesting_Level := 0;
	    Current_Call_Info := null;
	    Current_Params := null;
	    Current_Select_Info := null;
	    Callable := True;    
	    Aborting_Level := Natural'Last; 
	    Aborting_ECD := null;
	    Pr_TCB_Type.Enclosing_TCB := Enclosing_TCB;
	    Active_Priority := System.Any_Priority'First;
	    Base_Priority := System.Any_Priority'First;
            Attribute := (others => null);
            Attribute_Extension := null;
	    Contributing_Base_Priority := System.Any_Priority'First;
	end Default_Reinitialize;

        entry Task_Entry(for Ent_Index in 1..Num_Entries) (
	    Params : PRT.Param_Block_Ptr
   	) when (Wait_For_Caller(Current_Nesting_Level)'Count > 0 and then 
	  Is_Open(Ent_Index)) or else not Callable is
	    This_ECD : PRT.ECD_Access;
	begin  --
	    if not Callable then  
		raise Tasking_Error;    
	    end if;  
	    -- check if task was terminable
    	    if Enclosing_TCB.Status = Acceptor_Terminable then   
		Master_Pkg.Cancel_Terminable(
                  Enclosing_TCB.Innermost_Submaster);
	        Enclosing_TCB.Status := Runnable;
	    end if;  
            -- Indicate that this is the current caller
	    -- use implementation defined attribute to get current ECD
	    Current_Params := Params;  

	    Current_Nesting_Level := Current_Nesting_Level + 1;
	    -- save current rendezvous information in ECD
	    This_ECD := To_ECD_Access(Task_Entry(Ent_Index)'ECD);  
	    This_ECD.Old_Caller_Info := Current_Call_Info;

	    Current_Call_Info := This_ECD;
            This_ECD.Old_Acceptor_Priority := Active_Priority;	

 	    -- close all entries
            Current_Select_Info.Entry_List := (others => 0); 

	    -- Wait until rendezvous is complete
            requeue Caller_In_Rendezvous(Current_Nesting_Level - 1);
	end Task_Entry; 
	
	entry Simple_Accept(
	    Select_Info : Select_Access;  
	    Alt_Index_Chosen : out Alternative_Index;  
	    Params : out PRT.Param_Block_Ptr;
	    ECD : out PRT.ECD_Access
	) when True is  
	begin  --
	    Current_Select_Info := Select_Info;
	    Select_Info.Alt_Index_Chosen := Alternative_Index'First;
	    requeue Wait_For_Caller(Current_Nesting_Level) with abort; 
	      -- abortable requeue  
        end Simple_Accept;

	entry Selective_Accept(  
	    Select_Info : Select_Access;  
            Alt_Index_Chosen : out Alternative_Index;  
            Params : out PRT.Param_Block_Ptr;
            Call_Info : out PRT.ECD_Access
	) when True is  
	    Highest_Prio : System.Any_Priority := System.Any_Priority'First; 
            Best_Alt_So_Far   : Alternative_Index'Base := -1;  
            E : Task_Entry_Index;  
	    My_ECD : constant PRT.ECD_Access 
	      := To_ECD_Access(Selective_Accept'ECD);
	    PO : PRT.PO_Access := My_ECD.Called_PO;
	    Task_Entry_Name_Index : constant PRT.Entry_Name_Index := 
		PRT.Entry_Name_Index'First;		
	begin  --
	    Current_Select_Info := Select_Info;

            -- Loop to see if there is already an eligible caller.
            for I in Select_Info.Entry_List'Range loop  
                E := Select_Info.Entry_List(I);  
		if Task_Entry(E)'Count /= 0 then  
		    -- Found an eligible caller, see if is best so far
		    -- Assume queueing policy is priority.  If FIFO, 
		    -- we've done more work than required. 
		    if Best_Alt_So_Far < 0 or else 
		      Task_Entry(E)'Caller_Priority > Highest_Prio  then
			-- This is the best caller so far
			Best_Alt_So_Far := I;  
			-- This is an impl-defined attribute which
			-- gives prio of highest-prio caller of entry
			Highest_Prio := Task_Entry(E)'Caller_Priority; 
		    end if;  
		end if;
            end loop;  

	    -- If found a caller, make only its entry open.
	    Select_Info.Alt_Index_Chosen := Best_Alt_So_Far;

            if Best_Alt_So_Far < 0 then  
             -- If terminate alternative is open, then task becomes
             -- terminable.
             -- Wait for callers anyway.
		if Select_Info.Has_Terminate then   
		    Master_Pkg.Set_Acceptor_Terminable(
			TCB_Access(Enclosing_TCB)
		    );
		end if;  
		-- No eligible callers now, open entries and requeue
	    end if;

	    requeue Wait_For_Caller(Current_Nesting_Level) with abort;
        end Selective_Accept;  

	function Is_Callable return Boolean is  
	begin  
	    return Callable;   
	end Is_Callable;  

       	procedure Set_Uncallable is  
       	begin  
            Callable := False;   
      	end Set_Uncallable;  

	function Get_Base_Priority return System.Any_Priority is  
	begin  
	    return Base_Priority;   
	end Get_Base_Priority;  

	function Get_Active_Priority return System.Any_Priority'Base is  
	begin  
	    return Active_Priority;   
	end Get_Active_Priority;  

        procedure Recompute_Priorities is

	    New_Active_Priority : System.Any_Priority'Base;
	    Caller_In_Rendezvous_Index : PRT.Entry_Name_Index := 3;
	      -- use this to determine whether we're blocked on a 
	      -- requeued task entry call and, therefore, have
	      -- contributed a priority
	    Activator : TCB_Access;  
            use type ECD_Access;
            use type PRT.Pr_Type_Desc_Access;
	    use type PRT.Entry_Name_Index;
	    function Recompute_Call_Priority(
		ECD : ECD_Access;
		Base_Priority : System.Any_Priority'Base
	    ) return System.Any_Priority'Base is
	    -- Recurses along ECD chain representing a nested rendezvous.
	    -- Returns virtual active priority of acceptor at each level.
	    begin
	        if ECD = null then
		    return Base_Priority;
	        else
		  ECD.Old_Acceptor_Priority := 
		    Recompute_Call_Priority(ECD.Old_Caller_Info, Base_Priority);
	        end if;
	        return max(ECD.Old_Acceptor_Priority, ECD.Caller_Priority);
	    end Recompute_Call_Priority;

	begin
	    Activator := Enclosing_TCB.Activator;
	    if Activator /= null then 
                New_Active_Priority := max(Contributing_Base_Priority,
	          Activator.Pr_TCB.Get_Active_Priority);
	    elsif
		Current_Call_Info /= null then 
                New_Active_Priority := Recompute_Call_Priority(
		  Current_Call_Info, 
		  Contributing_Base_Priority);
	    else
		New_Active_Priority := Contributing_Base_Priority;
	    end if;

	    if New_Active_Priority /= Active_Priority then
		Active_Priority := New_Active_Priority;
                -- update priority in kernel
	        TGT.Kernel.Threads.Set_Thread_Priority(
		    Thread => Thread_Ptr(Enclosing_TCB),
		    Priority => Active_Priority,
		    Yielding => False
	        );

		-- If contributing a priority through activation or 
		-- rendezvous, recompute those tasks which are inheriting.
		if Enclosing_TCB.Activation_Sync /= null then
		    declare
			Activatee : TCB_Access := 
			  Enclosing_TCB.Activation_Sync.To_Be_Activated.First;
		    begin
			while Activatee /= null loop
			    if Activatee.Activator /= null then
				-- task is not done activating
				Activatee.Pr_TCB.Recompute_Priorities;
			    end if;
		            Activatee := Activatee.Act_Next;
			end loop;
		    end;
		elsif (Enclosing_TCB.Blocking_ECD /= null) and then
		  (Enclosing_TCB.Blocking_ECD.Called_PO.Type_Info = 
		    TCB_Pr_Type_Desc_Access) and then
		      (Enclosing_TCB.Blocking_ECD.Name_Index = 
			Caller_In_Rendezvous_Index) then
		    -- Caller in rendezvous
		    declare
			function cnv is new Ada.Unchecked_Conversion(
			  Source => PRT.PO_Access, Target => Pr_TCB_Type_Access);
			-- NOTE: This conversion is run-time model dependent.
			-- Depends on PO_Rec (RTS portion of protected
			-- object) preceding private data.
			Acceptor : Pr_TCB_Type_Access := 
			  cnv(Enclosing_TCB.Blocking_ECD.Called_PO);
		    begin
			Enclosing_TCB.Blocking_ECD.Caller_Priority := 
			  Active_Priority;
			Acceptor.Recompute_Priorities;
		    end;
		else -- not contributing
		    null;
	        end if;
	    end if;
	end Recompute_Priorities;

	procedure Set_Contributing_Base_Priority(
	    Priority : System.Any_Priority'Base
	) is  
	begin  
	    if Priority /= Contributing_Base_Priority then
		Contributing_Base_Priority := Priority;
		Recompute_Priorities;
	    end if;
	
	end Set_Contributing_Base_Priority;


	procedure Set_Base_Priority(  Priority : System.Any_Priority  ) is  
	begin  
	    Base_Priority := Priority;   
	    -- Set kernel base priority if new base priority is greater   
	    -- than the current active priority.  Note that base   
	    -- priority in the kernel correponds to active priority  
	    -- except for the consideration of inheritance from locks.  

	    -- NOTE: based on LRM D5, Note 22 when a task priority is 
            -- changed it "always causes the task to move to the end
            -- of the ready queue".
	    -- 
	    -- This procedure is called for task intialization and 
  	    -- only for ADP.Set_Priority thereafter.  Yielding => True
	    -- means put this task on the end of the queue for it's
  	    -- active priority.

	    if not Is_Held then
		Set_Contributing_Base_Priority(Base_Priority);
	    end if;
	end Set_Base_Priority;  

	procedure Inherit_Priority(  
	    Priority : System.Any_Priority  
	) is  
	begin  
	    -- account for possibility of held task
	    if Active_Priority < Priority then
	        Active_Priority := Priority;   
		TGT.Kernel.Threads.Set_Thread_Priority(
		    Thread => Thread_Ptr(Enclosing_TCB),
		    Priority => Active_Priority,
		    Yielding => True
		);
	    end if;  
	end Inherit_Priority;  

	procedure Disinherit_Priority(  
	    Priority : System.Any_Priority  
	) is  
        -- This procedure is used when activation completes.
        -- Disinheritance is more complex in the cases of rendezvous
        -- and protected actions.  These cases are handled separately.
	begin  
            Active_Priority := Contributing_Base_Priority;   
	    TGT.Kernel.Threads.Set_Thread_Priority(
		Thread => Thread_Ptr(Enclosing_TCB),
		Priority => Active_Priority,
		Yielding => False
	    );
	end Disinherit_Priority;  

	function Get_Aborting_ECD return ECD_Access is 
	begin 
	    return Aborting_ECD; 
	end Get_Aborting_ECD; 


	function Get_Aborting_Level return Natural is 
	begin 
	    return Aborting_Level; 
	end Get_Aborting_Level; 

        procedure Reset_Aborting_Level is
	begin
	    Aborting_Level := Natural'Last;
	    -- need to do target dependent reset as well
	    TGT.Kernel.Abort_Signals.Reset_Abort; 
	end Reset_Aborting_Level;

	procedure Abort_Task is
	begin
	    if Callable then
		Aborting_ECD := null;  
		Aborting_Level := 0;
		Callable := False;
		TGT.Kernel.Abort_Signals.Signal_Abort(
		    Thread => Thread_Ptr(Enclosing_TCB),
		    Level => 0
		);
	    end if;
	end Abort_Task;

        procedure Abort_To_Level( 
	    ECD : ECD_Access
	) is
	begin  
	    if Callable and then ECD.Async_Level < Aborting_Level then   
		Aborting_ECD := ECD;  
		Aborting_Level := ECD.Async_Level;  
		TGT.Kernel.Abort_Signals.Signal_Abort(
		    Thread => Thread_Ptr(To_TCB_Access(ECD.Calling_TCB)),
		    Level => Aborting_Level
		);
	    end if;  
	end Abort_To_Level;  

	procedure End_Of_Rendezvous (ECD : out PRT.ECD_Access) is  
	    use type PRT.ECD_Access;
	begin  
	    -- Return ECD that represents requeued call so that
            -- calller can "move" the call via Do_Or_Queue.
	    -- Can't be done here because our active priority may be too 
	    -- high due to holding lock on rendezvous object.
	    if PRT."="(Current_Call_Info.State, PRT.Requeue_Task_To_Protected)
	      then
	        ECD := Current_Call_Info;
		-- Cancel call, now requeued on Caller_In_Rendezvous, so
                -- that original caller will not be released.
	        PRO.Remove_From_Queue(ECD);
            else
	        ECD := null;
            end if;
	        
	    Active_Priority := Current_Call_Info.Old_Acceptor_Priority;
	    -- if task completing rendezvous is held, it's priority goes
	    -- back to Held_Priority in kernel and we must Yield to
	    -- put it behind the idle task.
-- Don't lower our priority until after the 
--		protected operation finished.  Finishing the protected
--		operation is what makes the caller start executing.
--		This is a fix for CXD2003.
--	    TGT.Kernel.Threads.Set_Thread_Priority(
--		Thread => Thread_Ptr(Enclosing_TCB),
--		Priority => Active_Priority,
--		Yielding => Active_Priority = Held_Priority
--	    );
	    Current_Nesting_Level := Current_Nesting_Level - 1;
	    Current_Call_Info := Current_Call_Info.Old_Caller_Info;
	end End_Of_Rendezvous;  

	procedure Except_End_Of_Rendezvous(
		 Exc : Ada.Exceptions.Exception_Occurrence ) is
	    ECD : PRT.ECD_Access;
	    use EXU;
	begin 
	    Current_Call_Info.Exception_To_Be_Raised := True; 
            Current_Call_Info.Exception_Occurrence := 
                +EXU.Temporary_Save_Occurrence(Exc);
	    End_Of_Rendezvous ( ECD );
	end Except_End_Of_Rendezvous;  

	entry Caller_In_Rendezvous(
	  for Nesting_Level in Accept_Nesting_Level'range) (
	    Params : PRT.Param_Block_Ptr  -- work-around for requeue bug
	) when ( Nesting_Level = Current_Nesting_Level or else 
            not Callable) is  
	begin  
  	    if not Callable then   
	        -- raise Tasking_Error in caller
		PRO.Record_Exception(
		    To_ECD_Access(Caller_In_Rendezvous(Nesting_Level)'ECD),
		    Standard.Tasking_Error'Identity);
	    end if; 
	end Caller_In_Rendezvous;  

	entry Wait_For_Caller(  
	  for Nesting_Level in Accept_Nesting_Level'range) (
	    Select_Info : Select_Access;  
            Alt_Index_Chosen : out Alternative_Index;  
            Params : out PRT.Param_Block_Ptr;
            Call_Info : out PRT.ECD_Access
	) when (Caller_In_Rendezvous(Nesting_Level)'Count > 0) is  
		Callers_Priority : System.Any_Priority;
	begin  
	    -- Caller has arrived, pass back their info.
	    Alt_Index_Chosen := Select_Info.Alt_Index_Chosen;  
            Params := Current_Params;  
	    Call_Info := Current_Call_Info;
	    -- execute rendezvous on priority of call, if higher
	    if PRI."="( PRI.Queuing_Policy,PRI.FIFO) then
	       Callers_Priority := 
		 To_TCB_Access(Current_Call_Info.Calling_TCB).Pr_Tcb.Get_Active_Priority;
	    else
		    Callers_Priority :=Current_Call_Info.Priority;
	    end if;


            Inherit_Priority( Callers_Priority );
	end Wait_For_Caller;  

	function Is_Open(
	    Ent : Task_Entry_Index
	) return Boolean is
	begin
	    if Current_Select_Info.Alt_Index_Chosen >= 0 then 
		--  acceptor has selected one particular entry
	        return Current_Select_Info.Entry_List(
		    Current_Select_Info.Alt_Index_Chosen) = Ent;
	    else
		--  acceptor is waiting for multiple entries
		for I in Current_Select_Info.Entry_List'Range loop
		    if Current_Select_Info.Entry_List(I) = Ent then
			-- Remember alt index so it can be returned to callee
			Current_Select_Info.Alt_Index_Chosen := I;
			return True;
		    end if;
		end loop;
		return False;
	    end if;
	end Is_Open;

	function Is_Held return Boolean is
	begin
	    if not Callable then  
		raise Tasking_Error;    
	    end if;  
	    return (Contributing_Base_Priority = Held_Priority);
	end Is_Held;

        procedure Hold is
	begin
	    if not Callable then  
		raise Tasking_Error;    
	    end if;  
	    if not Is_Held then
		Set_Contributing_Base_Priority(Held_Priority);
	    end if;
        end Hold;    

	procedure Continue is
      	begin
	    if not Callable then  
		raise Tasking_Error;    
	    end if;  
	    if Is_Held then
	        Set_Contributing_Base_Priority(Base_Priority);
	    end if;
    	end Continue;

        -- operations for Ada.Task_Attributes;
        function Value(ID : Attribute_ID) return Attribute_Handle is
        begin
            if not Callable then
                raise Tasking_Error;
            end if;
            return Get_Attribute_Handle(ID);
        end Value;

        function Reference(ID : Attribute_ID) return Attribute_Handle is
        begin
            return Value(ID);
        end Reference;

        procedure Set_Value(
            Val : in Attribute_Handle;
            ID : Attribute_ID;
            Old_Val : out Attribute_Handle
        ) is
        begin

            if not Callable then
                raise Tasking_Error;
            end if;
            -- return pointer to current value, so
            -- storage can be reused.
            Old_Val := Get_Attribute_Handle(ID);

            if ID <= Last_Attribute_ID_Preallocated then
                Attribute(ID) := Val;
            elsif Attribute_Extension = null then
                -- no extension has been allocated yet.  Do it now, making
                -- the index only as big as needed for current ID.
                declare
                    First_Ext_ID : Attribute_ID :=
                      Last_Attribute_ID_Preallocated + 1;
                begin
                    Attribute_Extension := new Attribute_Index_Extension'(
                      First_Ext_ID..ID => null);
                    Attribute_Extension(ID) := Val;
                end;
            elsif ID <= Attribute_Extension.all'Last then
                -- The ID has a place in the current extension index.
                Attribute_Extension.all(ID) := Val;
            else
                -- An extension index has been allocated.   However, it
                -- is too small. Deallocate the current extension index and
                -- allocate a larger one.
                declare
                    First_Ext_ID : Attribute_ID :=
                      Last_Attribute_ID_Preallocated + 1;
                    Temp : Attribute_Index_Extension_Access :=
                      new Attribute_Index_Extension'(
                        First_Ext_ID..ID => null);
                begin
                    for i in First_Ext_ID..Attribute_Extension.all'Last loop
                        Temp.all(i) := Attribute_Extension.all(i);
                    end loop;
                    Temp.all(ID) := Val;
                    Free(Attribute_Extension);
                    Attribute_Extension := Temp;
                end;
            end if;

        end Set_Value;

	procedure Finalize_Attribute(
	    ID : Attribute_ID
	) is
	    Old_Val : Attribute_Handle;
	begin
	    
	    if not Callable then  
		raise Tasking_Error;    
	    end if;  

	    Set_Value(null, ID, Old_Val);
	    if Old_Val /= null then 
	        Attribute_Cleanup_Op_Table(ID).all(Old_Val);
	    end if;

	end Finalize_Attribute;

        function Get_Attribute_Handle(ID : Attribute_ID)
          return Attribute_Handle is
        begin
            if ID <= Last_Attribute_ID_Preallocated then
                return Attribute(ID);
            elsif Attribute_Extension = null then
                return null;
            elsif ID <= Attribute_Extension.all'Last then
                return Attribute_Extension.all(ID);
            else
                return null;
            end if;
        end Get_Attribute_Handle;

    end Pr_TCB_Type;  

    function Self return TCB_Access is
	pragma Suppress(Tag_Check);
	-- Note: this Suppress and the pragma Inline will cause
	-- calls to Self to be processed like calls to Current_Thread.
	-- Thus Current_Thread may be defined as an Intrinsic and 
	-- recognized at sites of calls to Self.
    begin
        return TCB_Access(TGT.Kernel.Threads.Current_Thread);
    end Self;

    procedure Default_Reinitialize_TCB (TCB : TCB_Access) is
	Null_Occurrence : Exception_Types.Minimal_Exception_Occurrence;
    begin
         -- Initialize important components to same values as default.
        TCB.Status	 	    := Unactivated;
        TCB.Completion_Status 	    := Not_Yet_Completed;
	TCB.Occurrence		    := Null_Occurrence;
        TCB.Blocking_ECD            := null;
        TCB.Activation_Sync	    := null;
        TCB.Async_Level   	    := 0;
        TCB.Select_Info             := null;
        TCB.Act_Prev                := null;
        TCB.Act_Next                := null;
        TCB.Dep_Prev                := null;
        TCB.Dep_Next                := null;
        -- Now reinitialize the private data of the protected object
        TCB.Pr_TCB.Default_Reinitialize(TCB);
    end Default_Reinitialize_TCB;

    function New_TCB return TCB_Access is
	TCB : TCB_Access;
	Last_TCB : TCB_Access := null;
    begin
	TGT.Kernel.Mutexes.Enter_Mutex(Free_TCBs_Mutex'access);
	TCB := Free_TCBs;
	while TCB /= null loop
	    if TCB.Storage_Size = Released then
		-- Got one we can use
		if Last_TCB = null then
		    Free_TCBs := TCB.Dep_Next;
		else
		    Last_TCB.Dep_Next := TCB.Dep_Next;
		end if;
		TGT.Kernel.Mutexes.Exit_Mutex(Free_TCBs_Mutex'access);
		Default_Reinitialize_TCB(TCB);
		    -- Re-initialize TCB for reuse.
		return TCB;
	    end if;
	    -- Here if freed but not released TCB. Try next one.
	    Last_TCB := TCB;
	    TCB := TCB.Dep_Next;
	end loop;
	TGT.Kernel.Mutexes.Exit_Mutex(Free_TCBs_Mutex'access);
	-- Note: we set the Num_ENtries discriminants of TCB and Pr_TCB in
	-- RTS_Initialize task, so its value now is not important,
	-- as long as they do not affect any variants. The only variant
	-- is in a PO_Rec, but all Pr_TCBs have the same variant.
	TCB := new TCB_Type(0);
        -- The compiler does default initialization here.
	Default_Reinitialize_TCB(TCB);
	return TCB;
    end New_TCB;

    procedure Free_TCB(TCB : TCB_Access) is
    begin
	-- This routine is to return a TCB for re-use.
	-- We leave the Pr_TCB as an active protected object.
	TCB.Gen_Num := NOT_IN_USE;	-- Change Gen_Num so refs are invalid.
	TCB.Storage_Size := Not_Released;
	    -- Set Storage_Size as released flag: see Release_Thread, New_TCB
	-- Place TCB on free list.
	TGT.Kernel.Mutexes.Enter_Mutex(Free_TCBs_Mutex'access);
	TCB.Dep_Next := Free_TCBs;
	Free_TCBs := TCB_Access(TCB);
	TGT.Kernel.Mutexes.Exit_Mutex(Free_TCBs_Mutex'access);
    end Free_TCB;

    procedure Release_Thread(TCB : access TCB_Type) is
    begin
	-- This routine is called as last action of a thread.
	TCB.Storage_Size := Released;
    end Release_Thread;

    function Assign_Attribute_ID(
        Cleanup_Op : Attribute_Cleanup_Op
    ) return Attribute_ID is
        ID : Attribute_ID;
        function Find_First_Available_ID return Attribute_ID is
            ID : Attribute_ID;
        begin
--            for ID in Attribute_ID'First..Last_Attribute_ID loop
--	        if Attribute_Cleanup_Op_Table(ID) = null then
--		    return ID; 
--		end if;
--		-- no attibute slot available
             if Next_Attribute_ID > Last_Attribute_ID then 
		raise Program_Error; --
	     else
	        Next_Attribute_ID := Next_Attribute_ID + 1;
		return Next_Attribute_ID - 1;
	     end if;
--            end loop;
        end Find_First_Available_ID;
    begin
        ID := Find_First_Available_ID;
        Attribute_Cleanup_Op_Table(ID) := Cleanup_Op;
        return (ID);
    end Assign_Attribute_ID;

    procedure Deassign_Attribute_ID(
        ID : Attribute_ID
    ) is
	use type Attribute_Handle;
    begin
        Attribute_Cleanup_Op_Table(ID) := null;
    end Deassign_Attribute_ID;

    procedure Finalize_Attribute_In_Task(
   	ID : Attribute_ID;
	This_Task : TCB_Access
    );

    procedure Finalize(Obj : in out System_Wide_Attribute_Cleanup_Type) is
        ID : Attribute_ID := Obj.ID;
    begin
        -- Visit each task in system, deallocating current attribute values 
	Finalize_Attribute_In_Task(ID, Master_Pkg.Envr_TCB'access);
	Deassign_Attribute_ID(ID);
    end Finalize; 

    procedure Finalize_Attribute_In_Task(
   	ID : Attribute_ID;
	This_Task : TCB_Access
    ) is
    --* OVERVIEW
    -- Walk master chain for this task recursing on all dependents
    -- found.
	Sub_Master : Master_Access;
	use type TCB_Access; use type Master_Access;
--	use type TKT.Master_Access;
--	use type TKT.TCB_Access;
    begin 
	-- Finalize attribute in all dependents of all submasters
--     	MUT.Enter_Mutex( Master_Lock'Access );
	Sub_Master := This_Task.Innermost_Submaster;
	while (Sub_Master /= This_Task.Master) and (Sub_Master /= null) loop
	    declare
        	Dependent : TCB_Access := Sub_Master.Dependents.First; 
	    begin
        	while Dependent /= null loop
                    Finalize_Attribute_In_Task(ID, Dependent);
	            Dependent := Dependent.Dep_Next;
		end loop;
            end;
	    Sub_Master := Sub_Master.Enclosing_Master;
	end loop;
--     	MUT.Exit_Mutex( Master_Lock'access );
	-- Call protected procedure to cleanup attribute 
        This_Task.Pr_TCB.Finalize_Attribute(ID);	
    end Finalize_Attribute_In_Task;

begin
    TGT.Kernel.Mutexes.Init_Mutex(
	Free_TCBs_Mutex'Access,
	Interrupt_Priority'Last);
    TGT.Kernel.Threads.Set_True(Default_Master.Dependents_Complete);
end System.RTS.Tasking_Types_Pkg;
