-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.task_activation_pkg.spc,v $ 
-- $Revision: 1.6 $ $Date: 95/10/10 14:04:33 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Tasking_Types_Pkg;

package System.RTS.Task_Activation_Pkg is

    pragma Suppress( Elaboration_Check );

--* OVERVIEW
-- This package contains operations on activation lists.  An
-- object of this type is allocated in contexts that perform
-- activation, either a declarative part or an object containing
-- components of task type.  Operations on the activation list are
-- performed by both the activator and tasks being activated.
-- 
-- The activation list is kept outside of the activation barrier

-- 
-- The sequence of operations called from the generated code
-- of the activator is:
--     Initialize_Activation_List
--     Add_To_Activation_List
--     Add_To_Activation_List
--     ...
--     Initialize_Barrier(list length) 
--     Do_Activations
--
-- 
    subtype Activation_Access is Tasking_Types_Pkg.Activation_Access;
    subtype TCB_Access is Tasking_Types_Pkg.TCB_Access;

    -- Links in TCB are used for constructing activation list.
    
    procedure Activation_Complete( 
	Unhandled_Exception : Boolean := False
    );
     --* OVERVIEW
     -- Called from the end of the elaboration of the 
     -- declarative_part of a task body.
     --* EFFECTS
     -- Decrement count of to-be-activated tasks
     -- Unhandled_Exception is true if task died during
     -- its activation due to an unhandled exception.
     -- It is false if task is aborted or completes activation
     -- normally.

    procedure Initialize_Activation_List( List : Activation_Access );
    --* EFFECTS
    -- initializes the contents of the activation list pointed
    -- to by List
    
    procedure Add_To_Activation_List( List :  Activation_Access;
	    	    	    	      TCB  :  TCB_Access );
    --* OVERVIEW
    -- Called when a component of task type is encountered when evaluating
    -- an allocator. Also called during the elaboration of the declarative
    -- part when elaborating a task object declaration.

    procedure Do_Activations( List: Activation_Access );
    --* OVERVIEW
    -- Called to activate a group of tasks in parallel, either the group
    -- associated with an allocation or a declarative_part.

end System.RTS.Task_Activation_Pkg;


