-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.task_attributes.bdy,v $ 
-- $Revision: 1.5 $ $Date: 96/12/31 15:40:02 $ $Author: bethc $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   
with Ada.Task_Identification; use Ada.Task_Identification; 
with System.RTS.Tasking_Types_Pkg;
with Ada.Unchecked_Conversion;
with Ada.Unchecked_Deallocation;

pragma ada_child;
package body Ada.Task_Attributes is

    pragma Suppress( Elaboration_Check );

    package RTT renames System.RTS.Tasking_Types_Pkg;
    -- when an instantiation is elaborated, the next available ID is taken
    -- from System.RTS.Tasking_Types_Pkg.

    This_Attribute_ID : RTT.Attribute_ID;

    This_Initial_Value : aliased Attribute := Initial_Value;

    type Attribute_Cleanup_Op is access protected procedure(
      Attr : in out Attribute_Handle);

    protected Cleanup_Obj is
        procedure Finalize_Attribute(Attr : in out Attribute_Handle);
    end;

    procedure Free is new
      Ada.Unchecked_Deallocation(Attribute, Attribute_Handle);

    protected body Cleanup_Obj is
        procedure Finalize_Attribute(Attr : in out Attribute_Handle) is
        begin
	    -- finalization will be performed if Attribute is Controlled.
	    Free(Attr);
	end Finalize_Attribute;
    end Cleanup_Obj;

    function To_Task_Obj_Header is new Ada.Unchecked_Conversion(
        Source => Ada.Task_Identification.Task_ID,
        Target => RTT.Task_Obj_Header);

    function To_Internal_Attribute_Handle is new Ada.Unchecked_Conversion(
        Source => Attribute_Handle,
        Target => RTT.Attribute_Handle);

    function To_Attribute_Handle is new Ada.Unchecked_Conversion(
        Source => RTT.Attribute_Handle,
        Target => Attribute_Handle);

    function To_Internal_Cleanup_Op is new Ada.Unchecked_Conversion(
        Source => Attribute_Cleanup_Op,
        Target => RTT.Attribute_Cleanup_Op);

    --------------------------------------------------------------
    -- Visible Routines
    --------------------------------------------------------------
    
    function Value( T : Task_ID := Current_Task ) return Attribute is
        Task_Obj : RTT.Task_Obj_Header := To_Task_Obj_Header(T);
        TCB : RTT.TCB_Access := Task_Obj.TCB;
        Task_Attr : RTT.Attribute_Handle;
        use type RTT.TCB_Gen_Num;
    begin
        if T = Null_Task_ID then
            raise Program_Error;
        end if;

        if Task_Obj.Gen_Num /= TCB.Task_Obj.Gen_Num then
            raise Tasking_Error;
        end if;

        Task_Attr := TCB.Pr_TCB.Reference(This_Attribute_ID);

        if not RTT."="(Task_Attr, null) then
            return To_Attribute_Handle(Task_Attr).all;
        else
            return This_Initial_Value;
        end if;
 
    end Value;
    
    function Reference( T : Task_ID := Current_Task )
    	    	    	    	    	    return Attribute_Handle is
        Task_Obj : RTT.Task_Obj_Header := To_Task_Obj_Header(T);
        TCB : RTT.TCB_Access := Task_Obj.TCB;
        Task_Attr : RTT.Attribute_Handle;
	Old_Val : RTT.Attribute_Handle;
        New_Val : Attribute_Handle;
        use type RTT.TCB_Gen_Num;
    begin
        if T = Null_Task_ID then
            raise Program_Error;
        end if;

        if Task_Obj.Gen_Num /= TCB.Task_Obj.Gen_Num then
            raise Tasking_Error;
        end if;

        Task_Attr := TCB.Pr_TCB.Reference(This_Attribute_ID);

        if not RTT."="(Task_Attr, null) then
            return To_Attribute_Handle(Task_Attr);
        else
            New_Val := new Attribute'(This_Initial_Value);
            TCB.Pr_TCB.Set_Value(
              To_Internal_Attribute_Handle(New_Val), This_Attribute_ID,
              Old_Val);
	    return New_Val;
        end if;
 
    end Reference;					    

    procedure Set_Value( Val : in Attribute;
    			 T   : in Task_ID := Current_Task ) is
        Task_Obj : RTT.Task_Obj_Header := To_Task_Obj_Header(T);
        TCB : RTT.TCB_Access := Task_Obj.TCB;
        New_Val, Temp_Val : Attribute_Handle;
        Old_Val : RTT.Attribute_Handle;
        use type RTT.TCB_Gen_Num;
        use type RTT.Attribute_Handle;
    begin

        if T = Null_Task_ID then
            raise Program_Error;
        end if;

        if Task_Obj.Gen_Num /= TCB.Task_Obj.Gen_Num then
            raise Tasking_Error;
        end if;

        New_Val := new Attribute'(Val);
        TCB.Pr_TCB.Set_Value(
          To_Internal_Attribute_Handle(New_Val), This_Attribute_ID,
            Old_Val);
        if Old_Val /= null then
            Temp_Val := To_Attribute_Handle(Old_Val);
            Free(Temp_Val);
        end if;

    end Set_Value;			 
			 
    procedure Reinitialize( T : in Task_ID := Current_Task ) is
        Task_Obj : RTT.Task_Obj_Header := To_Task_Obj_Header(T);
        TCB : RTT.TCB_Access := Task_Obj.TCB;
        --use type Ada.Task_Identification.Task_ID;
        Temp_Val : Attribute_Handle;
        Old_Val : RTT.Attribute_Handle;
        use type RTT.TCB_Gen_Num;
        use type RTT.Attribute_Handle;
    begin

        if T = Null_Task_ID then
            raise Program_Error;
        end if;

        if Task_Obj.Gen_Num /= TCB.Task_Obj.Gen_Num then
            raise Tasking_Error;
        end if;

        TCB.Pr_TCB.Set_Value(null, This_Attribute_ID, Old_Val);
        if Old_Val /= null then
            Temp_Val := To_Attribute_Handle(Old_Val);
            Free(Temp_Val);
        end if;

    end Reinitialize;
begin
    This_Attribute_ID := RTT.Assign_Attribute_ID(To_Internal_Cleanup_Op(
      Cleanup_Obj.Finalize_Attribute'Access));
end Ada.Task_Attributes;
