-- $Source: /home/dsd/d/aa/monoBANK/AA/TSP_system.rts.tgt.kernel.mutexes.spc,v $ 
-- $Revision: 1.4 $ $Date: 96/10/23 14:32:52 $ $Author: stt $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

-- Derived from:
--  Source: /home/dsd/d/aa/monoBANK/AA/system.rts.tgt.kernel.mutexes.spc,v   
--  Revision: 1.3    Date: 96/10/23 14:31:35    Author: stt   

with System.RTS.WIN32;
with System.RTS.TGT.Kernel.Threads;
with System.RTS.TGT.Kernel.Threads.NT;

package System.RTS.TGT.Kernel.Mutexes is
  -- This package provides for mutual exclusion.

  -- A mutex has a "ceiling priority" which is the maximum
  -- (active) priority allowed for any thread which tries to enter
  -- the mutex (Program_Error is raised if exceeded);
  -- the ceiling priority is inherited by the thread holding the mutex.
  -- In a non-real time implementation, ceiling priorities may be
  -- ignored.

  -- For a mutex with a given (non-interrupt-level) ceiling priority, 
  -- if there is at most one thread in existence at the given priority 
  -- or below, then no actual lock need be acquired when entering the 
  -- mutex.  Hence, in a "mono"-threaded system, no actual lock need 
  -- ever be acquired.

  -- For a mutex with a given interrupt-level ceiling priority,
  -- in addition to acquiring a lock (if necessary), one or more
  -- interrupts must be masked.  In particular, any interrupt associated
  -- with a mutex at the given priority or below, must be masked while
  -- "inside" a mutex at the given priority.

  -- It is recommended that the kernel maintain an array indexed by
  -- priority level which contains the interrupt mask for that priority
  -- level, as well as a count of the tasks at that level or below,
  -- allowing a quick check to determine whether any locking and/or masking
  -- is required upon Enter_Mutex.

  -- Read-only versions of Enter/Exit are provided to support
  -- protected functions (probably only worth making different from 
  -- normal Enter/Exit in multiprocessor environments).

	package THD renames System.RTS.TGT.Kernel.Threads;

    type Mutex_Descriptor is tagged limited private;
    -- A protected object is an extension of a Mutex_Descriptor

    type Mutex_Ptr is access all Mutex_Descriptor'Class;

    procedure Init_Mutex(
      Mutex : Mutex_Ptr; Priority : Kernel_Priority);

    procedure Enter_Mutex(Mutex : Mutex_Ptr);
    procedure Exit_Mutex(Mutex : Mutex_Ptr);

    -- "Read only" versions of enter/exit; multiple readers
    -- are permitted, or one writer, but not both.
    procedure Enter_Mutex_RO(Mutex : Mutex_Ptr);
    procedure Exit_Mutex_RO(Mutex : Mutex_Ptr);

    procedure Finish_Mutex(Mutex : Mutex_Ptr);
private

    for Mutex_Ptr'Storage_Size use 0;
      -- Disallow allocators, and suppress registration of collection

    type Mutex_Descriptor is tagged limited 
      record
        Flags    : System.RTS.WIN32.Dword;
        Lock     : aliased System.RTS.TGT.Kernel.Threads.NT.Lock_Type;
	Priority : Kernel_Priority;	-- ceiling priority for holder of mutex
	Previous_Priority : Kernel_Priority; -- old priority of thread that holds mutex
      end record;

end System.RTS.TGT.Kernel.Mutexes;
