-- == % filter "-- "
with System;
with System.Storage_Elements;
with Ada.Unchecked_Conversion; use Ada;
with Ada.Unchecked_Deallocation;
with SYSTEM.RTS.TCB;
with System.RTS.Win32; use System.RTS.Win32;
with System.RTS.TGT.Sec_Stack_Pkg;
with System.RTS.Traces;use System.RTS.Traces;
with System.Rts.Temp_IO; use System.Rts.Temp_IO;
with System.RTS.TGT.Kernel.Threads.NT;
use  System.RTS.TGT.Kernel.Threads.NT;
with System.RTS.Init_Pkg.NT.BuildOpts;
with SYSTEM.RTS.TCB_Tdep;
package body System.RTS.TGT.Kernel.Threads is
  -- This package defines the basic "thread" abstraction
  -- Threads can wait on at most one "suspension" object
  -- at a time, and at most one thread can wait on a given
  -- suspension object.

  -- Register_User_Thread and Thread_Main run in the context of a new thread, 
  -- on a new stack, before the stack limit at FS:4 has been established
    pragma suppress (Elaboration_Check);
    pragma suppress (Storage_Check);

    procedure Set_Exception_Handler;
    pragma Import (Assembler, Set_Exception_Handler, "_rts_set_exception");

    procedure Remove_Exception_Handler;
    pragma Import (Assembler, Remove_Exception_Handler, "_rts_remove_exception_no_pop");

    function Thrd2Int is new Unchecked_Conversion(Thread_Ptr, Integer);

    function Thrd2Addr is new Unchecked_Conversion(Thread_Ptr, System.Address);
    function Addr2Thrd is new Unchecked_Conversion(System.Address, Thread_Ptr);

    function Code2Addr is new Unchecked_Conversion(Thread_Code, System.Address);

-- == %if TARGET_I80X86
-- == %elsif TARGET_RIOS
--     procedure rts_load_tcb( tcb : Ref_TCB);
--     pragma import(C, rts_load_tcb, "rts_load_tcb" );
-- 
--     procedure rts_save_display( tcb : Ref_TCB );
--     pragma import(C, rts_save_display, "rts_save_display") ;
-- == %end if

    procedure Set_System_Thread_As_Current_Thread(
	Thread : Thread_Ptr
    ) is
    -- Set up so that Current_Thread may be called, even before
    -- thread structure is fully initialized, in case it is used
    -- to hold information such as current exception, secondary
    -- stack pointer, etc.
	This_TCB   : TCB.Ref_TCB := TCB.Curr_TCB; 
    begin
	This_TCB.Thread_Pointer := Thrd2Addr(Thread);
    end Set_System_Thread_As_Current_Thread;

    -- Hook into the user-specified main priority.
      -- See system.rts.main_priority_fallback.dat for the default definition
    Envr_Task_Priority : System.Any_Priority'Base;
    pragma Export( C, Envr_Task_Priority, "rts_envr_task_priority");

    function Init_Suspension return Win32_Handle is
        Event_Object : Win32_Handle;
    begin
      	Event_Object := System.RTS.Win32.CreateEvent(C_NULL,  -- default SECURITY_ATTRIBUTES
                                               False,    -- auto ResetEvent
                                               FALSE,   -- initially non-signalled
                                               C_NULL); -- unnamed event     

	Verify (Event_Object, NotEqualTo => 0);
        if trace(11) then
           put("Create event ");
           put_hex (Event_Object);
           new_line;
        end if;
        return Event_Object;
    end Init_Suspension;

    procedure Register_System_Thread(
      Thread : Thread_Ptr;
      Main_Priority : in out System.Priority)
    is
    -- The given Thread_Ptr will represent the thread that the system
    -- created to execute the program. Only a subset of the operations
    -- will be applied to the system thread. Operations that will not
    -- be used are Init_Thread, Activate_Thread, 
    -- and Remove_Unactivated_Thread. 
    -- The Main_Priority will be initialized to System.Default_Priority;
    -- on return it should contain the user-specified main priority, if any,
    -- as specified by the prelinker.
    -- This operation will be called only once and from the system thread.

	This_TCB   : SYSTEM.RTS.TCB.Ref_TCB := SYSTEM.RTS.TCB.Curr_TCB; 
	CurProcess,CurThread : WIN32_HANDLE;
	RC         : BOOL;
    begin
	This_TCB.Thread_Pointer := Thrd2Addr(Thread);
      
	-- Set the Thread_handle.  We must duplicate the handle returned
	-- by GetCurrentThread.
	CurProcess := GetCurrentProcess;
        CurThread  := GetCurrentThread;
        if rts_is_ets and CurThread=1024 then
           -- A pseudo-handle will [have to] do.
           Thread.Thread_handle := CurThread;
           EtsStandardEdition := True;           
           MonoThreaded := True;           
           Thread.Dbg_ID := 0;
        else
	   RC := Duplicatehandle (hSourceProcess => CurProcess,
                                hSource        => CurThread,
                                hTargetProcess => CurProcess,
                                lphTarget      => Thread.Thread_handle'address,
                                fwdAccess      => DUPLICATE_SAME_ACCESS,
                                fInherit       => TRUE,
                                fwdOptions     => DUPLICATE_SAME_ACCESS);
           Thread.Dbg_ID := GetCurrentThreadID;
        end if;
        Thread.TCB    := This_TCB;

	-- Initialize the abort level.  Note that this precedes secondary stack
	-- initialization since the sec. stack uses mutexes (indirectly) which
	-- defer and undefer abort handling.
	Thread.Pending_Abort_Level := Abort_Signal_Level'last;
	Thread.Defer_Count := 0;
	Thread.State := Running;
	    
	if Envr_Task_Priority in System.Any_Priority then
	    -- Pass back the user-specified main priority
	    Main_Priority := Envr_Task_Priority;
	end if;

	Thread.Base_Priority := Main_Priority;
	Thread.Active_Priority := Kernel_Priority'Last;
        if not EtsStandardEdition then
           NT.Set_Active_Priority( Thread, Thread.Base_Priority );
  	   Thread.Wake_Thread:=Init_Suspension;
        end if;

	-- Init the Win32 event we use to abort a wait within the thread

    end Register_System_Thread;


    procedure Thread_Main( 
	Thread : Thread_Ptr )
    is	
    begin	

-- == %if TARGET_I80X86
	declare
	   procedure rts_dump_stack_limits;
	   pragma import(C, rts_dump_stack_limits);
	begin
	    if Trace(10) then
	        rts_dump_stack_limits;
	    end if;
	end;
-- == %else
-- == %end if

	SYSTEM.RTS.TCB.Initialize_TCB(
            TCB                => Thread.TCB, -- set in Init_Thread
            Thread_Pointer     => Thrd2Addr(Thread),
            Saved_Integer_Data => Thread.TCB_Saved_Data'address);

        -- On PowerPC, we need to load the TCB into the TCB register
-- == %if TARGET_RIOS
--  	rts_load_tcb( Thread.TCB );
-- == %else
-- == %end if

	System.RTS.TGT.Sec_Stack_Pkg.Initialize (
           System.RTS.Init_Pkg.NT.BuildOpts.AdaBuildOpts.sec_stack_size);
	-- This pushes an exception block onto the stack; the stack
	-- must not be cut back after it returns.

-- == %if TARGET_I80X86
	declare
	   procedure rts_dump_tcb( TCB : System.RTS.TCB.Ref_TCB);
	   pragma import(C, rts_dump_tcb);
	   TCB : System.RTS.TCB.Ref_TCB := CURR_TCB;
	begin
	    if Trace(10) then
	        rts_dump_tcb(TCB);
	    end if;
	end;
-- == %else
-- == %end if

	Set_Exception_Handler;

	-- This calls Task_Envelope, which in turn calls Register_User_Thread
	Thread.Thread_Start( Thread ); -- never returns
    end Thread_Main;

    procedure Register_User_Thread(
      Thread : Thread_Ptr
    ) is
    -- Register a user thread;
    -- This performs anything that must be done in the context of the 
    -- thread.  It is called as the first thing in task_envelope.  For
    -- example, with a Posix-like OS, you could use this to set the thread_ptr 
    -- in the thread specific data area.
    -- For other real-time executives, each task has its own signal
    -- handlers, so these must be set up before the task body is called.
    begin
	null;  -- Everything already performed by Thread_Main
	  -- TBD: if we inlined Register_User_Thread, then we could
	  --      probably move all the calls from Thread_Main into
	  --      this routine and get rid of Thread_Main
    end Register_User_Thread;

    procedure Init_Thread(
	Thread : Thread_Ptr; 
	Storage_Size : Storage_Elements.Storage_Count;
	Code : Thread_Code;
	Priority : Kernel_Priority)
    is
	-- Init_Thread operates on an existing but uninitialized
	-- data structure, creating a thread that is initially inactive,
	-- represented by that data structure.
	-- Init_Thread allocates the thread's stack.

	-- Note: It is CRITICAL that this subprogram not modify the display!
      
	Thread_Object   : Win32_handle;
	Event_Object    : Win32_handle;
	Task_ID         : LPVOID;
	Thread_ID       : aliased Win32_thread_id;
	Thread_Start    : Thread_Code;
        Stack_Size      : DWORD;
        
        use type Storage_Elements.Storage_Count;
    begin
-- == %if TARGET_RIOS
-- 	-- For PowerPC, we need to spill display levels 1 and 2 to the display
-- 	-- area of the TCB.  Normally, they are kept in registers only.
-- 	rts_save_display( SYSTEM.RTS.TCB.Curr_TCB );
-- == %else
-- == %end if

	--
	-- Start the thread in our private routine, Thread_Main
	Thread_Start := Thread_Main'access;

	-- Allocate a new TCB and initialize its display.
	Thread.TCB := SYSTEM.RTS.TCB.ALLOCATE_TCB;
      
        if Storage_Size > 0 then
           -- A little hack to get around the 1024 in cd2c11b
           if Storage_Size < System.RTS.TCB_Tdep.EM_Stack_Slop then 
              Stack_Size := DWORD(Storage_Size + System.RTS.TCB_Tdep.EM_Stack_Slop);
           else
              Stack_Size := DWORD(Storage_Size);
           end if;
        else
           Stack_Size := DWORD (System.RTS.Init_Pkg.NT.BuildOpts.AdaBuildOpts.stack_size);
        end if;
	Thread_Object := System.RTS.Win32.CreateThread (C_NULL, 
                                    Stack_Size, 
                                    Code2Addr(Thread_Start),
                                    Thrd2Addr(Thread), 
                                    CREATE_SUSPENDED, 
                                    Thread_Id'access );
	if Thread_Object = 0 then
	    raise storage_error;
	end if;
    
	Thread.Thread_Start := Code;
	Thread.Thread_handle := Thread_Object;
        Thread.Dbg_ID := Thread_ID;

	Thread.Base_Priority := Priority;
	Thread.Active_Priority := Kernel_Priority'Last;
	NT.Set_Active_Priority( Thread, Priority );

	if trace(13) then
	    put("Init_Thread  ");put_hex(Thread_Id); 
	    put(" priority "); put_dec(Integer(Priority));
	    New_Line;
	end if;

	-- Init the suspension object we use to abort a wait within the thread
	Thread.Wake_Thread:=Init_Suspension;

	-- Initialize the abort level
	Thread.Pending_Abort_Level := Abort_Signal_Level'last;
 	Thread.Defer_Count := 0;
	Thread.State := Running;

        -- Force the heap to do mutex locking.

	if Trace(10) then
	    put("       Thread create "); put_hex(Thread_Id); 
	    put(" wake handle "); put_hex(Thread.Wake_Thread);
	    put(" thread_ptr = "); put_hex( Thrd2Int(Thread) );
	    New_Line;
	end if;

    end Init_Thread;

    procedure Activate_Thread(Thread : Thread_Ptr) is
    -- Activate_Thread is used only for the initial activation.
    -- Control after that is via Suspend_Until_True.
    begin
       Verify (ResumeThread (Thread.Thread_handle));
    end Activate_Thread;

    procedure Finish_Thread is
    -- Stops the current thread "dead in its tracks",
    -- releasing the thread's stack.
        Thread : Thread_ptr := Current_Thread;
    begin

	if Trace(10) then
	    put("       Thread finish ");
	    put(" wake handle "); put_hex(Thread.Wake_Thread);
	    New_Line;
	end if;

	Remove_Exception_Handler;
	
	Verify (CloseHandle(Thread.Wake_Thread));
--	Verify (CloseHandle(Thread.Suspension_Handle));
 	Verify (CloseHandle(Thread.Thread_handle));

	-- Release the secondary stack
	System.RTS.TGT.Sec_Stack_Pkg.Free;

	-- Restore the data we saved at the top of the stack
	-- Note that this has to be the last call to any Ada
	-- code that does stack limit checking.  The flag of
	-- 1 indicates this is not the environment TCB
	Restore_Object( Thread.TCB_Saved_Data'Address, 1 );

	-- Now mark the thread as being available for re-use.
	-- It's important that Release_Thread not do stack
	-- limit checking, nor otherwise attempt to use
	-- the current Thomson TCB, nor use the secondary
	-- stack.
	Release_Thread( Thread );

	System.RTS.Win32.ExitThread(0); -- used zero for lack of a better number.
    end Finish_Thread;

    procedure Remove_Unactivated_Thread(Thread : Thread_Ptr) is
    -- Cleans up a thread that was created by Init_Thread but never
    -- activated by Activate_Thread. Further reference to the thread
    -- is an error.
    -- This situation arises when a task is aborted prior to activation.
    begin

	if Trace(10) then
	    put("  remove unactivated - ");
	    put(" wake handle "); put_hex(Thread.Wake_Thread);
	    New_Line;
	end if;

	Verify (CloseHandle(Thread.Wake_Thread));
--	Verify (CloseHandle(Thread.Suspension_Handle));
        Verify (TerminateThread (Thread.Thread_handle, 0));
 	Verify (CloseHandle(Thread.Thread_handle));

    end Remove_Unactivated_Thread;

    procedure Yield is
    -- Causes the current thread to give up the processor to another
    -- equal priority ready but not running thread, allowing round-robin
    -- scheduling.
    -- Used by RTS when D <= 0.0 in a simple delay D;
    begin
	-- The Win32 API spec says that (Win32) Yield is obsolete and:
	--	"Threads that do not contain windows should use the Sleep 
	--	function with a sleep time of zero milliseconds to give up 
	--	the remainder of their current timeslice."

	System.RTS.Win32.Sleep(0);
    end Yield;

    procedure Set_Thread_Priority(
	Thread : Thread_Ptr;
	Priority : Kernel_Priority;
	Yielding : Boolean) is	-- If true, puts at tail of ready Q
    begin
	-- If the thread kernel hasn't started, ignore this call
	if Thread.Thread_handle = 0 then
	    return;
	end if;

	-- If we're changing the base priority, and do not
	-- own any mutexes, change the active priority of the thread
	if Thread.Base_Priority /= Priority then
	    Thread.Base_Priority := Priority;
	    Thread.Yielding := Yielding;
	    if Thread.Mutexes_Owned = 0 then
               NT.Set_Active_Priority( Thread, Priority, Yielding );
	    end if;
	end if;

    end Set_Thread_Priority;
    
    procedure Get_Thread_Priority(
	Thread : in Thread_Ptr; 
	Priority : out Kernel_Priority) is
    begin

	-- If the thread is not yet initialized, return
	-- a default priority.  This can only occur when
	-- this routine is called during elaboration of
	-- the run-time.
	if Thread.Thread_handle = 0 then
	   Priority:=System.Any_Priority'First;
        else
           Priority:=Thread.Base_Priority;
        end if;

    end Get_Thread_Priority;
             
    Main_Exception_Save_Area : aliased Exception_Types.Exception_Occurrence;
      -- Used in the absence of tasking, or before tasking is initialized

    function Exception_Occurrence_Save_Area 
      return Exception_Types.Exception_Occurrence_Access is
    -- Return pointer to save area in current thread 
	Thread : Thread_Ptr := Current_Thread;
    begin
	if Thread /= null then
	    return Thread.Save_Area'Unchecked_Access;
	else
	    -- Must be mono-threaded, or before system thread registration
	    return Main_Exception_Save_Area'Access;
	end if;
    end Exception_Occurrence_Save_Area;

--
-- The following section handles suspension objects
--
    procedure put_state (State:in Dword) is
    begin
       if State=State_False then
          put("False");
       elsif State=State_True then
          put("True");
       else
          put_hex(state);
       end if;
    end put_state;

    procedure Set_True(S : in out Suspension_Object) is
       State : Dword;
    begin

       loop

          -- If the prior state is False it is set to True
          -- else if it was a wait handle that handle is signalled
          State :=InterLockedCompareExchange (S.State'Address, State_True, State_False);

          if Trace(11) then
             put("Set_True ");
             put_hex(S'Address);
             put(" state=");
             put_state(State);
             new_line;
          end if;

          -- Job is perhaps already done?
          exit when State=State_False or State=State_True;

          -- It's an event handle.  Only a race with another Set_True can alter that,
          -- so allow only one Set_True to bring the waiting thread to life:
          if InterLockedCompareExchange (S.State'Address, State_False, State)=State then
             -- Note that PulseEvent is not an option because there is a window
             -- between the InterLockedCompareExchange and the Wait_for_Handle
             -- in Suspend_Until_True_Or_Timeout during which no one will
             -- be waiting.
             if Trace(11) then
                put("SetEvent ");
                put_hex(State);
                new_line;
             end if;
             Verify (SetEvent (State));
             exit;
          end if;

          if Trace(11) then
             put_line("Set_True looping to retry");
          end if;

       end loop;

    end Set_True;
       
    procedure Set_False(S : in out Suspension_Object) is
       -- If the state holds a wait handle it is left unchanged; otherwise
       -- the state is set to False:
       State : constant Dword:=InterLockedCompareExchange (S.State'Address, State_False, State_True);
    begin
       if Trace(11) then
          put("Set_False ");
          put_hex(S'Address);
          put(" state=");
          put_state(State);
          new_line;
       end if;
   end Set_False;

    function Current_State(S : Suspension_Object) return Boolean is
    begin
        return S.State=State_True;
    end Current_State;
    
    procedure Suspend_Until_True_Or_Timeout ( S : in out Suspension_Object;Wait_Time:in Dword;sts : out Dword) is
       Thread : constant Thread_Ptr := Current_Thread;
       MyWake : constant Dword := Thread.Wake_Thread;
       -- Store our wake handle if prior state is False
       State  : constant Dword := InterlockedCompareExchange (S.State'Address,MyWake,State_False);
    begin
       if Trace(11) then
          put("Suspend_Until_True_Or_Timeout ");
          put_hex(S'Address);
          put(" state=");
          put_state(State);
          if Wait_Time/=Infinite then
             put(" timeout=");
             put_dec(integer(wait_time));
          end if;
          new_line;
       end if;

       case State is
          when State_True   =>
             sts:=WAIT_OBJECT_0;
             Set_False (S);
          when State_False  =>
             -- The InterlockedCompareExchange has stored our wake handle
             Thread.Suspension_Waiting:=S.State'address;
             sts:=Wait_for_Handle (MyWake, Wait_Time);
             -- The Set_True that woke us has probably set the state back to False, but if we time out...
             InterLockedCompareExchange (S.State'Address, State_False, MyWake);
             Thread.Suspension_Waiting:=System.Null_Address;
             
          when others =>
             raise Program_Error;
       end case;

   end Suspend_Until_True_Or_Timeout;


   procedure Suspend_Until_True(S : in out Suspension_Object) is
       sts : Dword;
   begin
       Suspend_Until_True_Or_Timeout (S,Infinite,sts);
       Verify(sts,NotEqualTo => Wait_Failed);
   end Suspend_Until_True;

begin
    -- Note that this calls rts_allocate, which in turn enters/exits
    -- a mutex, which in turn calls current_thread, which will discover
    -- that the current thread is null (since the system thread hasn't
    -- yet been registered), which will cause the enter/exit-mutex
    -- to act as a no-op.
    System.RTS.TGT.Sec_Stack_Pkg.Initialize (
       System.RTS.Init_Pkg.NT.BuildOpts.AdaBuildOpts.sec_stack_size);

end System.RTS.TGT.Kernel.Threads;
