-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ************************************************************************
-- PACKAGE SPECIFICATION System.RTS.Traces
-- ----------------------------
--
-- OUTLINE :
-- -------
--    This package provides the facility to read and set trace numbers.
-- They control the generation of debug messages during execution of the 
-- Runtime.
-- ************************************************************************

package System.RTS.Traces is

   pragma SUPPRESS (ELABORATION_CHECK);
   pragma PreElaborate (System.RTS.Traces);
   
   -- ---------------------------------------------------------------------
   -- The definition of Trace number ranges is as follows:
   --
   --    Trace 10-29: task kernel
   --       10  Thread creation
   --       12  Suspension objects - Set_true/Set_False
   --       13  Priority changes
   --       14  Yield in signal_abort
   --       15  Mutexes
   --       16  Suspension lock
   --       17  Turn on debug_tasking
   --       20  Print ErrorReturn codes
   --       21  Yield in Suspend_Until_True_Or_Timeout
   --       22  Yield on exiting mutex
   --       23  Yield in Set_True
   --       26  No blocking in temp_io output
   --       29  Buffer output
   --    Traces 30-39: exception manager
   --       30  Min
   --       31  Max
   --       36  Table
   --       37  Dump table
   --
   -- ---------------------------------------------------------------------





   TRACE : constant array (INTEGER range 0..39) of BOOLEAN := (others => FALSE);
   DEBUG : constant BOOLEAN := FALSE;


   procedure INIT_TRACES;

end System.RTS.Traces;
