------------------------------------------------------------------------------
-- PACKAGE SPEC : System.RTS.Win32
--
-- This package is the interface to WIN32 API for the compiler and runtime.
--
------------------------------------------------------------------------------
with system.rts.temp_io;
with System.RTS.Traces;

package body System.RTS.Win32 is

   pragma suppress (Elaboration_Check);

   -- Error processing procedure for the case of WIN32 API call returning
   -- error.  It calls GetLastError and raises WIN32_ERROR exception.
   procedure Error_Return is
       use System.RTS.Temp_io;
   begin
       if System.RTS.Traces.Trace(20) then
          Put("Error_Return LastError=");
          Put_Dec(integer(GetLastError));
          Put(" (0x");
          Put_Hex(integer(GetLastError));
          Put_Line(")");
       end if;
       raise Win32_Error;
   end Error_Return;

   procedure Verify (Val:in BOOL) is
   begin
      if Val=FALSE then
         Error_Return;
      end if;
   end Verify;

   procedure Verify (Val:in DWORD; NotEqualTo : in DWORD:=16#ffffffff#) is
   begin
      if Val=NotEqualTo then
         Error_Return;
      end if;
   end Verify;

end System.RTS.Win32;
