with SYSTEM, UNCHECKED_CONVERSION;
package System.RTS.Win32 is

   pragma PreElaborate (System.RTS.Win32);

   MAX_PATH : constant := 260;

   type DWORD is mod 2**32;
   type BYTE is mod 2**8;

   subtype WIN32_HANDLE is DWORD;
   subtype LPVOID is SYSTEM.ADDRESS;
   subtype LPHANDLE is SYSTEM.ADDRESS;
   type WIN32_HANDLE_ARRAY is array (INTEGER range<>) OF WIN32_HANDLE;
   type BOOL is (FALSE, TRUE);
   for BOOL use (FALSE => 0, TRUE => 1);
   for BOOL'SIZE use 32;
   subtype WIN32_THREAD_ID is DWORD;

   type HMODULE is new DWORD;

   INVALID_HANDLE_VALUE : constant WIN32_HANDLE := 16#FFFFFFFF#;
   STD_INPUT_HANDLE     : constant WIN32_HANDLE := 16#FFFFFFF6#;
   STD_OUTPUT_HANDLE    : constant WIN32_HANDLE := 16#FFFFFFF5#;
   STD_ERROR_HANDLE     : constant WIN32_HANDLE := 16#FFFFFFF4#;

   function CONV is new UNCHECKED_CONVERSION (INTEGER, SYSTEM.ADDRESS);
   C_NULL	  : constant SYSTEM.ADDRESS := null;
   WIN32_ERR_CODE : constant DWORD := 16#FFFFFFFF#;
   WIN32_ERROR	  : exception;


   -- Error processing procedure for the case of WIN32 API call returning
   -- error. It calls GetLastError and raise WIN32_ERROR exception.
   procedure ERROR_RETURN;

   -- Call Error_Return if Val==FALSE
   procedure Verify (Val:in BOOL);

   -- Call Error_Return if Val==NotEqualTo
   procedure Verify (Val:in DWORD; NotEqualTo : in DWORD:=16#ffffffff#);
   pragma Inline(Verify);

   function CloseHandle (OBJECT : WIN32_HANDLE) return BOOL;

   function CreateEvent( 
	SECURITY_ATTRIBUTES : SYSTEM.ADDRESS;
	MANUAL_RESET : BOOL;
	INITIAL_STATE : BOOL;
	EVENT_NAME : SYSTEM.ADDRESS )
	return WIN32_HANDLE;

   function CreateMutex (
      SECURITY_ATTR	: SYSTEM.ADDRESS;
      INITIAL_OWNDER	: BOOL;
      MUTEX_NAME	: SYSTEM.ADDRESS)
      return WIN32_HANDLE;

   -- Parameter values for CreateThread
   CREATE_SUSPENDED : constant := 4;

   function CreateThread (
	SECURITY_ATTRIBUTES : SYSTEM.ADDRESS;
	STACK_SIZE : DWORD;
	THREAD_START_ROUTINE : SYSTEM.ADDRESS;
	THREAD_PARAM : LPVOID;
	CREATION_FLAGS : DWORD;
	THREAD_ID : access DWORD )
   	return WIN32_HANDLE;

   -- Value for duplicateHandle Access and Options parameters
   DUPLICATE_SAME_ACCESS         : constant  := 16#00000002#; 

   function Duplicatehandle (hSourceProcess : WIN32_HANDLE;
                             hSource        : WIN32_HANDLE;
                             hTargetProcess : WIN32_HANDLE;
                             lphTarget      : LPHANDLE; -- address of handle
                             fwdAccess      : DWORD;
                             fInherit       : BOOL;
                             fwdOptions     : DWORD) return BOOL;

   -- Full declaration of CRITICAL_SECTION object type
   type CRITICAL_SECTION is 
      record
         PRTL_CRITICAL_SECTION_DEBUG	: SYSTEM.ADDRESS;
	 LOCK_COUNT			: DWORD;
	 RECURSION_COUNT		: DWORD;
	 OWNING_THREAD			: WIN32_HANDLE;
	 LOCK_SEMAPHORE			: WIN32_HANDLE;
	 RESERVED			: DWORD;
      end record;

   for CRITICAL_SECTION use
      record
         PRTL_CRITICAL_SECTION_DEBUG	at  0 range  0 .. 31;
	 LOCK_COUNT			at  4 range  0 .. 31;
	 RECURSION_COUNT		at  8 range  0 .. 31;
	 OWNING_THREAD			at 12 range  0 .. 31;
	 LOCK_SEMAPHORE			at 16 range  0 .. 31;
	 RESERVED			at 20 range  0 .. 31;
      end record;
   type CRITICAL_SECTION_ACCESS is access all CRITICAL_SECTION;

   procedure EnterCriticalSection (CS_OBJECT : CRITICAL_SECTION_ACCESS);

   procedure DeleteCriticalSection( CS_OBJECT : CRITICAL_SECTION_ACCESS);

   procedure ExitThread (EXIT_CODE : DWORD);


   function GetCurrentProcess return WIN32_HANDLE;

   function GetCurrentThread return WIN32_HANDLE;

   function GetCurrentThreadId return DWORD;

   function GetFullPathName (SHORT_NAME : in String;
                             FULL_NAME_L: in Dword;
                             FULL_NAME  : in System.Address;
                             FILE_PART  : in SYSTEM.ADDRESS) return Dword;

   function GetLastError return DWORD;

   type SYSTEM_TIME is
      record
         YEAR,
	 MONTH,
	 DAY_OF_WEEK,
	 DAY,
	 HOUR,
	 MINUTE,
	 SECOND,
	 MILLISECOND : SHORT_INTEGER;
      end record;
   for SYSTEM_TIME use
      record
         YEAR		at  0 range 0 .. 15;
	 MONTH		at  2 range 0 .. 15;
	 DAY_OF_WEEK	at  4 range 0 .. 15;
	 DAY		at  6 range 0 .. 15;
	 HOUR		at  8 range 0 .. 15;
	 MINUTE		at 10 range 0 .. 15;
	 SECOND		at 12 range 0 .. 15;
	 MILLISECOND	at 14 range 0 .. 15;
      end record;

   procedure GetLocalTime (T : out SYSTEM_TIME);

   procedure GetSystemTime (T : out SYSTEM_TIME);


	-- WIN32 Context for GetThreadContext and SetThreadContext

	Context_i386 : constant DWORD := 16#00010000#;
	Context_i486 : constant DWORD := 16#00010000#; -- assumes i386 and i486 are the same
	Context_Control : constant DWORD := 16#00010001#;
	Context_Integer : constant DWORD := 16#00010002#;
	Context_Segments : constant DWORD := 16#00010004#;
	Context_Floating_Point : constant DWORD := 16#00010008#;
	Context_Debug_Registers : constant DWORD := 16#00010010#;
	Context_Full : constant DWORD := 16#00010007#;

	type Float_Register_Area is array(1..80) of BYTE;

	type Floating_Save_Area is record
		ControlWord : DWORD;
		StatusWord : DWORD;
		TagWord : DWORD;
		ErrorOffset : DWORD;
		ErrorSelector : DWORD;
		DataOffset : DWORD;
		DataSelector : DWORD;
		RegisterArea : Float_Register_Area;
		Cr0NpxState : DWORD;
	end record;

	type Context is record 
		ContextFlags : DWORD;
		Dr0 : DWORD;
		Dr1	: DWORD;
		Dr2 : DWORD;
		Dr3 : DWORD;
		Dr6 : DWORD;
		Dr7 : DWORD;
		FloatSave : Floating_Save_Area;
		SegGs : DWORD;
		SegFs : DWORD;
		SegEs : DWORD;
		SegDs : DWORD;
		Edi : DWORD;
		Esi : DWORD;
		Ebx : DWORD;
		Edx : DWORD;
		Ecd : DWORD;
		Eax : DWORD;
		Ebp : DWORD;
		Eip : System.Address;
		SegCS : DWORD;
		EFlags : DWORD;
		Esp : System.Address;
		SegSs : DWORD;
	end record;
































   function GetThreadContext(
   	  H_THREAD		: WIN32_HANDLE;
	  Thread_Context : access Context ) return Bool;

    type TIME_ZONE_INFORMATION is
       record
          Bias         : INTEGER;
          StandardName : STRING (1..32);
          StandardDate : SYSTEM_TIME;
          StandardBias : Integer;
          DaylightName : STRING (1..32);
          DaylightDate : System_time;
          DaylightBias : INTEGER;
       end record;
    for  TIME_ZONE_INFORMATION use
       record
          Bias         at 0   range 0 .. 31;
          StandardName at 4   range 0 .. 255;
          StandardDate at 68  range 0 .. 127;
          StandardBias at 84  range 0 .. 31;
          DaylightName at 88  range 0 .. 255;
          DaylightDate at 152 range 0 .. 127;
          DaylightBias at 168 range 0 .. 31;
       end record;

   procedure GetTimezoneInformation (T:out Time_zone_information);

   function GetModuleHandle( lpstrModule : in String ) return HMODULE;
   function GetProcAddress( module : in HMODULE; lpstrProcedure : in String) return Address;
   function LoadLibrary( lpstrLibname : in String) return HMODULE;

   function GlobalAlloc (Flags : in Natural;
                         Bytes : in Natural) return System.Address;

   procedure GlobalFree (Mem : in System.Address);

   procedure InitializeCriticalSection (CS_OBJECT : CRITICAL_SECTION_ACCESS);

   procedure LeaveCriticalSection (CS_OBJECT : CRITICAL_SECTION_ACCESS);

   function ReleaseMutex (H_MUTEX : WIN32_HANDLE) return BOOL;

   function PulseEvent (H_EVENT : WIN32_HANDLE) return BOOL;

   function ResetEvent (H_EVENT : WIN32_HANDLE) return BOOL;

   function ResumeThread (H_THREAD : WIN32_HANDLE) return DWORD;

   function SetEvent (H_EVENT : WIN32_HANDLE) return BOOL;

   function SetThreadContext(
   	  H_THREAD		: WIN32_HANDLE;
	  Thread_Context : access Context ) return Bool;

   -- Parameter values for SetThreadPriority 
   THREAD_PRIORITY_IDLE		: constant := -15;
   THREAD_PRIORITY_LOWEST	: constant := -2;
   THREAD_PRIORITY_BELOW_NORMAL	: constant := -1;
   THREAD_PRIORITY_NORMAL	: constant := 0;
   THREAD_PRIORITY_ABOVE_NORMAL	: constant := 1;
   THREAD_PRIORITY_HIGHEST	: constant := 2;
   THREAD_PRIORITY_TIME_CRITICAL: constant := 15;
   THREAD_PRIORITY_ERROR_RETURN : constant INTEGER := INTEGER'LAST;  -- Win32 MAXLONG

   function SetThreadPriority (
      H_THREAD		: WIN32_HANDLE;
      THREAD_PRIORITY	: Integer)
      return BOOL;
   function GetThreadPriority (H_THREAD : WIN32_HANDLE) return Integer;

   procedure Sleep (TIME : DWORD);

   function SuspendThread (H_THREAD : WIN32_HANDLE) return DWORD;

   type FILETIME is record
      dwLowDateTime  : DWORD;
      dwHighDateTime : DWORD;
   end record;

   for FILETIME use 
      record
         dwLowDateTime  at 0 range 0 .. 31;
         dwHighDateTime at 4 range 0 .. 31;
      end record;

   procedure SystemTimeToFileTime ( ST : in out System_Time; FT : out FileTime);
   procedure FileTimeToSystemTime ( FT : in out FileTime;    ST : out System_Time);

   function TerminateThread (H_THREAD : WIN32_HANDLE; ExitCode : DWORD) return BOOL;

   function WaitForMultipleObjects (
   	  cObjects		: DWORD; 				-- number of handles in handle array
--	  H_OBJECTS		: WIN32_HANDLE_ARRAY;  	-- addr of object handle array
	  H_OBJECTS		: System.Address; -- for now, can't have bounds passed
      fWaitAll		: BOOL;					-- wait flag
      TIMEOUT		: DWORD)				-- time-out interval in milliseconds 	
      return DWORD;

   -- Constants for WaitForSingleObject
   INFINITE		: constant DWORD := 16#FFFFFFFF#;
   WAIT_FAILED		: constant DWORD := 16#FFFFFFFF#;
   WAIT_OBJECT_0	: constant DWORD := 0;
   WAIT_TIMEOUT		: constant DWORD := 16#00000102#;
   WAIT_ABANDONED	: constant := 16#80#;

   function WaitForSingleObject (
      H_OBJECT		: WIN32_HANDLE;
      TIMEOUT		: DWORD)
      return DWORD;

   function ReadFile (hFile : WIN32_HANDLE;
      lpBuffer : LPVOID;
      nNumberOfBytesToRead : DWORD;
      lpNumberOfBytesRead : SYSTEM.ADDRESS;
      lpOverlapped : SYSTEM.ADDRESS)
   return BOOL;

   function WriteFile (hFile : WIN32_HANDLE;
      lpBuffer : LPVOID;
      nNumberOfBytesToWrite : DWORD;
      lpNumberOfBytesWritten : SYSTEM.ADDRESS;
      lpOverlapped : SYSTEM.ADDRESS)
   return BOOL;
   
   CREATE_NEW : constant := 1;
   CREATE_ALWAYS : constant := 2;
   OPEN_EXISTING : constant := 3;
   OPEN_ALWAYS : constant := 4;
   TRUNCATE_EXISTING : constant := 5;

   GENERIC_READ                    : constant := 16#80000000#;
   GENERIC_WRITE                   : constant := 16#40000000#;
   GENERIC_EXECUTE                 : constant := 16#20000000#;
   GENERIC_ALL                     : constant := 16#10000000#;

   FILE_SHARE_READ                 : constant := 16#00000001#;
   FILE_SHARE_WRITE                : constant := 16#00000002#;
   FILE_ATTRIBUTE_READONLY         : constant := 16#00000001#;
   FILE_ATTRIBUTE_HIDDEN           : constant := 16#00000002#;
   FILE_ATTRIBUTE_SYSTEM           : constant := 16#00000004#;
   FILE_ATTRIBUTE_DIRECTORY        : constant := 16#00000010#;
   FILE_ATTRIBUTE_ARCHIVE          : constant := 16#00000020#;
   FILE_ATTRIBUTE_NORMAL           : constant := 16#00000080#;
   FILE_ATTRIBUTE_TEMPORARY        : constant := 16#00000100#;
   FILE_ATTRIBUTE_COMPRESSED       : constant := 16#00000800#;

   FILE_FLAG_WRITE_THROUGH         : constant := 16#80000000#;
   FILE_FLAG_OVERLAPPED            : constant := 16#40000000#;
   FILE_FLAG_NO_BUFFERING          : constant := 16#20000000#;
   FILE_FLAG_RANDOM_ACCESS         : constant := 16#10000000#;
   FILE_FLAG_SEQUENTIAL_SCAN       : constant := 16#08000000#;
   FILE_FLAG_DELETE_ON_CLOSE       : constant := 16#04000000#;
   FILE_FLAG_BACKUP_SEMANTICS      : constant := 16#02000000#;
   FILE_FLAG_POSIX_SEMANTICS       : constant := 16#01000000#;

   function CreateFile (lpFileName : STRING;
      dwDesiredAccess : DWORD;
      dwShareMode : DWORD;
      lpSecurityAttributes : SYSTEM.ADDRESS;
      dwCreationDisposition : DWORD;
      dwFlagsAndAttributes : DWORD;
      hTemplateFile : WIN32_HANDLE)
   return WIN32_HANDLE;

   function DeleteFile (lpFileName : String) return BOOL;

   function GetFileSize (Handle : in Win32_Handle;
                         lpFileSizeHigh : in System.Address) return Dword;

   procedure ExitProcess (uExitCode : Integer);

   function GetStdHandle (nStdHandle : DWORD) return WIN32_HANDLE;

   FILE_BEGIN           : constant := 0;
   FILE_CURRENT         : constant := 1;
   FILE_END             : constant := 2;

   function SetEndOfFile(hFile : WIN32_HANDLE) return Boolean;
    
   function SetConsoleMode(
    hConsoleHandle : WIN32_HANDLE;
    dwMode         : Dword) return Bool;

   function SetFilePointer(
    hFile : WIN32_HANDLE;
    lDistanceToMove : DWORD;
    lpDistanceToMoveHigh:SYSTEM.ADDRESS;
    dwMoveMethod : DWORD
    ) return DWORD;

   function GetCommandLine return SYSTEM.ADDRESS;

   function GetModuleFileName (hMod      : HMODULE;
                               lpFilename: SYSTEM.ADDRESS;
                               nSize     : DWORD)
                                            return DWORD;

   function FlushFileBuffers (hFile : WIN32_HANDLE) return BOOL;

   type OSVERSIONINFO is
      record
         dwOSVersionInfoSize : DWORD:=148; 
         dwMajorVersion      : DWORD; 
         dwMinorVersion      : DWORD; 
         dwBuildNumber       : DWORD; 
         dwPlatformId        : DWORD; 
         szCSDVersion        : STRING(1..128); 
       end record;
   
   function GetVersionEx (lpVersionInformation : in System.Address) return BOOLEAN;

   function InterlockedIncrement (count : in system.address) return Integer;
   procedure InterlockedDecrement (count : in system.address);
   function InterlockedDecrement (count : in system.address) return Integer;
   procedure InterlockedIncrement (count : in system.address);
   function InterlockedExchange  (count : in system.address;Value:in Integer) return Integer;

   function GetEnvironmentVariable (Name:in String;Buffer:in System.Address;Size:in DWORD) return DWORD;
   function SetEnvironmentVariable (Name:in String;Value:in String) return BOOL;

   function InterlockedCompareExchange( 
       Destination:System.Address;
       Exchange,Comperand:DWORD) return DWORD;
   procedure InterlockedCompareExchange( 
       Destination:System.Address;
       Exchange,Comperand:DWORD);
       
   type LARGE_INTEGER is
      record
        LowPart : DWORD; 
        HighPart: Integer;
      end record;
   type LARGE_INTEGER_ACCESS is access all LARGE_INTEGER;
      
   function QueryPerformanceCounter (T : in Large_integer_Access) return Boolean;
   function QueryPerformanceFrequency (T : in Large_integer_Access) return Boolean;

   function AllocConsole return Boolean;

   pragma Import (Win32, AllocConsole, "AllocConsole");
   pragma Import (Win32, CloseHandle, "CloseHandle");
   pragma Import (Win32, CreateEvent, "CreateEventA");
   pragma Import (Win32, CreateFile, "CreateFileA");
   pragma Import (Win32, CreateMutex, "CreateMutexA");
   pragma Import (Win32, CreateThread, "CreateThread");
   pragma Import (Win32, DeleteCriticalSection, "DeleteCriticalSection");
   pragma Import (Win32, Deletefile, "DeleteFileA");
   pragma Import (Win32, DuplicateHandle, "DuplicateHandle" );
   pragma Import (Win32, EnterCriticalSection, "EnterCriticalSection");
   pragma Import (Win32, ExitProcess, "ExitProcess");
   pragma Import (Win32, ExitThread, "ExitThread");
   pragma Import (Win32, FileTimeToSystemTime, "FileTimeToSystemTime");
   pragma Import (Win32, FlushFileBuffers, "FlushFileBuffers");
   pragma Import (Win32, GetEnvironmentVariable, "GetEnvironmentVariableA");
   pragma Import (Win32, GetCommandLine, "GetCommandLineA");
   pragma Import (Win32, GetCurrentProcess, "GetCurrentProcess" );
   pragma Import (Win32, GetCurrentThread, "GetCurrentThread" );
   pragma Import (Win32, GetCurrentThreadId, "GetCurrentThreadId" );
   pragma Import (Win32, GetFileSize, "GetFileSize");
   pragma Import (Win32, GetFullPathName, "GetFullPathNameA");
   pragma Import (Win32, GetLastError, "GetLastError");
   pragma Import (Win32, GetLocalTime, "GetLocalTime");
   pragma Import (Win32, GetModuleFileName, "GetModuleFileNameA");
   pragma Import (Win32, GetModuleHandle, "GetModuleHandleA");
   pragma Import (Win32, GetProcAddress, "GetProcAddress");
   pragma Import (Win32, GetStdHandle, "GetStdHandle");
   pragma Import (Win32, GetSystemTime, "GetSystemTime");
   pragma Import (Win32, GetThreadContext, 	"GetThreadContext" );
   pragma Import (Win32, GetTimezoneInformation, "GetTimeZoneInformation");
   pragma Import (Win32, GetThreadPriority,"GetThreadPriority");
   pragma Import (Win32, GetVersionEx, "GetVersionExA");
   pragma import (Win32, GlobalAlloc, "GlobalAlloc");
   pragma import (Win32, GlobalFree, "GlobalFree");
   pragma Import (Win32, InitializeCriticalSection, "InitializeCriticalSection");
   pragma Import (Win32, InterlockedDecrement,"InterlockedDecrement");
   pragma Import (Win32, InterlockedIncrement,"InterlockedIncrement");
   pragma Import (Win32, InterlockedExchange,"InterlockedExchange");
   -- Should be Win32, but wouldn't load under Win95/NT3.5.1:
   pragma Import (Stdcall, InterlockedCompareExchange, "rts_ilex");
   pragma Import (Win32, LeaveCriticalSection, "LeaveCriticalSection");
   pragma Import (Win32, LoadLibrary, "LoadLibraryA");
   pragma Import (Win32, Readfile, "ReadFile");
   pragma Import (Win32, ReleaseMutex, "ReleaseMutex");
   pragma Import (Win32, PulseEvent, "PulseEvent");
   pragma Import (Win32, QueryPerformanceCounter, "QueryPerformanceCounter");
   pragma Import (Win32, QueryPerformanceFrequency, "QueryPerformanceFrequency");
   pragma Import (Win32, ResetEvent, "ResetEvent");
   pragma Import (Win32, ResumeThread, "ResumeThread");
   pragma Import (Win32, SetConsoleMode, "SetConsoleMode");
   pragma Import (Win32, SetEndOfFile, "SetEndOfFile");
   pragma Import (Win32, SetEnvironmentVariable, "SetEnvironmentVariableA");
   pragma Import (Win32, SetEvent, "SetEvent");
   pragma Import (Win32, SetFilePointer, "SetFilePointer");
   pragma Import (Win32, SetThreadCOntext, "SetThreadContext" );
   pragma Import (Win32, SetThreadPriority, "SetThreadPriority");
   pragma Import (Win32, Sleep, "Sleep");
   pragma Import (Win32, SuspendThread, "SuspendThread");
   pragma Import (Win32, SystemTimeToFileTime, "SystemTimeToFileTime");
   pragma Import (Win32, TerminateThread, "TerminateThread");
   pragma Import (Win32, WaitForMultipleObjects, "WaitForMultipleObjects");
   pragma Import (Win32, WaitForSingleObject, "WaitForSingleObject");
   pragma Import (Win32, Writefile, "WriteFile");

end System.RTS.Win32;
