--
-- BigCalc
--   An applet which implements a desk calculator (in a separate frame).
--
--   By Vince Del Vecchio, Intermetrics, Inc.
--      vdelvecc@inmet.com
--   Copyright 1995, Intermetrics, Inc.
--

with interfaces.java; use interfaces.java;
with java.applet.Applet; use java.applet.Applet;
with java.awt.Graphics; use java.awt.Graphics;
with java.lang.String; use java.lang.String;
package BigCalc is
    procedure main(argv : String_Array);

    type BigCalc_obj is new Applet_obj with null record;
    type BigCalc_ptr is access all BigCalc_obj'class;

    procedure paint(obj : access BigCalc_obj; g : Graphics_Ptr);

    procedure init(obj_ptr : access BigCalc_obj);

end BigCalc;


with java.lang.System; use java.lang.System;
with java.awt.Component; use java.awt.Component;
with java.awt.Container; use java.awt.Container;
with java.awt.LayoutManager; use java.awt.LayoutManager;
with java.awt.TextField; use java.awt.TextField;
with java.awt.Color; 
with image; use image;
with nonLayout; use nonLayout;
with Ind_Frame; use Ind_Frame;
with typeablePanel;
with calculator;
with cbbutton;
with java.awt.Frame; use java.awt.Frame;
with java.awt.Font; use java.awt.Font;
with java.io.PrintStream; use java.io.PrintStream;
with java.applet.Applet; use java.applet.Applet;


package body BigCalc is

    procedure key(ch : Character);
    procedure output(s : string);

    package calcworks is new calculator(output);

    package typeable is new typeablePanel(handleKey => key);

    procedure buttonPress(button : calcworks.ButtonOpPtr);

    package calcbutton is new cbbutton(calcworks.ButtonOpPtr, buttonPress);
    use calcbutton;

    subtype ButtonOp is calcworks.ButtonOp;

    calcFrame_ptr  : Ind_Frame_ptr;
    panel          : typeable.typeablePanel_Ptr;
    display_ptr    : TextField_Ptr;
    state          : calcworks.CalcState;
    mainlayout_ptr : nonLayout.nonLayout_ptr;
    standalone     : Boolean;


    procedure main(argv : String_Array) is
	dummy: boolean;
	s :  BigCalc_ptr;
    begin
	standalone := true;
	s := new BigCalc_obj;
	init(s);
	start(s);	
    end main;


    procedure buttonPress(button : calcworks.ButtonOpPtr)  
	is
    begin
	calcworks.buttonPress(state, button);
    end;


    procedure key(ch : Character) is
	use calcworks;
    begin
	case ch is
	    when 'q' | 'Q' =>
		dispose(calcFrame_ptr);
		if standalone then
		     exit_op(0);
		end if;
	    when '0' .. '9' =>
		buttonPress(
		  new ButtonOp'(Digit,
			      Character'Pos(ch) - Character'Pos('0')));
	    when '.' =>
		buttonPress(new ButtonOp'(Point, 0));
	    when '+' =>
		buttonPress(new ButtonOp'(Add, 0));
	    when '-' =>
		buttonPress(new ButtonOp'(Sub, 0));
	    when '*' =>
		buttonPress(new ButtonOp'(Mult, 0));
	    when '/' =>
		buttonPress(new ButtonOp'(Divide, 0));
	    when '=' =>
		buttonPress(new ButtonOp'(Equals, 0));
	    when 'c' | 'C' =>
		buttonPress(new ButtonOp'(Clear, 0));	
	    when others => null;
        end case;
    end;


    procedure output(s : String) is
    begin
	setText(display_ptr, Ada_To_Java_String(s));
    end;

    procedure paint(obj :access BigCalc_obj; g : Graphics_Ptr) is
    begin
	-- inherited from Component
	null;
    end paint;

    procedure init(obj_ptr : access BigCalc_obj) is
        b : Component_Ptr;
	s : String(1..1);

	height    : constant Integer := 45;
	width     : constant Integer := 60;
	margin    : constant Integer := 15;
	inter     : constant Integer :=  8;
	buttontop : constant Integer := 102;
	topmargin : constant Integer := 40;

	framewidth : constant Integer :=
	    width * 4 + inter * 3 + margin * 2 ;
	frameheight : constant Integer :=
	    (buttontop + height * 5 + inter * 4 ) * 11/10;
    

	use calcworks;
	use typeable;

    begin
	if not standalone then
	    resize(obj_ptr, 0, 0);
	end if;

	calcFrame_ptr := new_ind_Frame(+"ObjectAda(TM)");
	resize(calcFrame_ptr, framewidth, frameheight);

	mainlayout_ptr := new_nonLayout;
	setLayout(calcFrame_ptr, mainlayout_ptr.LayoutManager'Access);
	setFont(calcFrame_ptr,  new_Font(+"Helvetica", PLAIN, 24));

	panel := new_typeablePanel(calcframe_ptr,
				   java.awt.Color.lightGray,
				   framewidth,
				   frameheight);
	setLayout(panel, mainlayout_ptr.LayoutManager'access);	
				   
	display_ptr:=textfield_ptr(add(container_ptr(panel),
		         component_ptr(new_TextField( +""))));
	setEditable(display_Ptr,true);
        reshape(Component_ptr(display_ptr),
		margin,
		topmargin,		
		width * 4 + inter * 3, 
		48);


	for j in reverse 0..2 loop
            for i in 0..2 loop
		s(1) := digitimage(j * 3 + i + 1);
		b := New_CbButton(+s, 
				  container_ptr(panel),
				  new ButtonOp'(Digit, j * 3 + i + 1));

		reshape(b,
		       x => margin + i * (width + inter),
		       y => buttontop + (3 - j) * (height + inter),
		       width => width, height => height);
	    end loop;
	end loop;
	s(1) := '0';
	b := New_CbButton(+s, container_ptr(panel), 
			new ButtonOp'(Digit, 0));
	reshape(b, margin, buttontop + 4 * (height + inter),
	       width * 2 + inter, height);
	s(1) := '.';
	b := New_CbButton(+s,  container_ptr(panel), 
			new ButtonOp'(Point, 0));
	reshape(b,
	       margin + (width + inter) * 2,
	       buttontop + 4 * (height + inter),
	       width, height);

	b := New_CbButton(+"Clr", container_ptr(panel), 
			new ButtonOp'(Clear, 0));
	reshape(b, margin, buttontop, width, height);

	declare
	    type ops_t is array(Add .. Divide) of Character;
	    ops : ops_t;
	begin
	    ops := "+-*/";
	    for op in Add .. Divide loop
		s(1) := ops(op);
		b := New_CbButton(+s, 
			container_ptr(panel), 
			new ButtonOp'(op, 0));
		reshape(b,
		       margin + (width + inter) * 3,
		       buttontop +
			 (ButtonKind'Pos(Divide) - ButtonKind'Pos(op)) *
			 (height + inter),
		       width, height);
	    end loop;
	end;
	b := New_CbButton(+"=", container_ptr(panel), 
		new ButtonOp'(Equals, 0));
	reshape(b,
	       margin + (width + inter) * 3, buttontop + 4 * (height + inter),
	       width, height);

	clearAll(state);
	showMsg(state, " ObjectAda(TM) ");

	show(calcFrame_ptr); 

    end init;


end BigCalc;
