with Ada_Store.Log;
with Ada_Store.Station;
with Ada_Store.Trading.Department;

package Ada_Store.User is

   type Instance 	is tagged private;

   type Identifier 	is new Natural;

   type Security is ( Manager, Supervisor, Office, Cashier );

   function  Lookup
     (User_ID : in Identifier)
      return Instance;

   procedure Create_New
     (User_ID         : in Identifier;
      Name            : in Short_Description;
      In_Department   : in Trading.Department.Identifier;
      Level           : in Security := Cashier);

   procedure Remove
     (User_ID : in Identifier);

   procedure Sign_On
     (User     : in Instance;
      Password : in String);

   procedure Sign_Off
     (User : in out Instance);

   function  Name
     (User : in Instance)
      return Short_Description;

   function  Department
     (User : in Instance)
      return Trading.Department.Identifier;

   function  Level
     (User : in Instance)
      return Security;

   function  Signed_On
     (User : in Instance)
      return Boolean;

   Invalid_Identifier  	: exception;
   Sign_On_Error        : exception;

   type User_Action	is ( Sign_On, Sign_Off, Forced_Off );
   
   type User_Log (Action : User_Action) is new Ada_Store.Log.Element
      with record
	 User_ID		: Identifier;
      end record;

   function To_String(Element : User_Log) return String;

private
   type Instance is tagged 
      record
	 User_ID       	: Identifier := 0;
	 Name		: Short_Description := (others => ' ');
	 Department     : Trading.Department.Identifier := 0;
	 Level		: Security := Cashier;
      end record;
  
end Ada_Store.User;    
