package Ada_Store.Support.Screen is
  pragma Elaborate_Body(Screen);

  Max_Lines		: constant Positive := 66;
  Max_Columns		: constant Positive := 132;

  type Height		is new Positive range 1 .. Max_Lines;
  type Width		is new Positive range 1 .. Max_Columns;

  type Position		is
    record
      Line		: Height;
      Column		: Width;
    end record;

  type Rectangle	is
    record
      Top_Left		: Position;
      Bottom_Right	: Position;
    end record;

  type Movement		is ( Up, Down, Left, Right );

  function  Cursor	return Position;

  procedure Set_Cursor
    (To	: in Position);

  function  Size	
    return Rectangle;

  procedure Clear;

  procedure Clear
    (Lines : in Height);

  procedure Clear
    (Area : in Rectangle);

  procedure Scroll
    (Direction : in Movement;
     Number    : in Positive);

  procedure Scroll
    (Direction : in Movement;
     Number    : in Positive;
     Lines     : in Height);

  procedure Scroll
    (Direction : in Movement;
     Number    : in Positive;
     Area      : in Rectangle);

  procedure HLine
    (Line : in Height);

  procedure HLine
    (Line  : in Height;
     Start : in Width;
     Stop  : in Width);

  procedure VLine
    (Column : in Width);

  procedure VLine
    (Column : in Width;
     Start  : in Height;
     Stop   : in Height);

  procedure Box
    (Area : in Rectangle);

  procedure Put
    (C : in Character);

  procedure Put
    (Str : in String);

  function Get
     return Character;

  Bad_Position		: exception;

  procedure Lock_Exclusive;
  procedure Unlock;

end Ada_Store.Support.Screen;
