package Ada_Store.Trading.Department is
   pragma Elaborate_Body(Ada_Store.Trading.Department);

   type Instance 	is private;

   type Identifier 	is new Natural range 0 .. 50;
   
   function  Lookup
     (Department_Number	: in Identifier)
      return Instance;

   procedure Create_New
     (Department_Number	: in Identifier;
      Description	: in Short_Description;
      Min_Price		: in Currency;
      Max_Price		: in Currency;
      Tax_Rate          : in Percentage := 0.00);

   procedure Remove
     (Department_Number : in Identifier);

   function  Description
     (Department : in Instance)
      return Short_Description;

   function  Min_Price
     (Department : in Instance)
      return Currency;

   function  Max_Price
     (Department : in Instance)
      return Currency;

   function  Tax_Rate
     (Department : in Instance)
      return Percentage;

   Invalid_Identifier : exception;

private
   type Instance is
      record
	 Department_Number	: Identifier := 0;
	 Description            : Short_Description := (others => ' ');
	 Min_Price              : Currency   := 0.0;
	 Max_Price              : Currency   := 0.0;
	 Tax_Rate               : Percentage := 0.0;
      end record;

end Ada_Store.Trading.Department;    
