-- ***************************************************************************
-- *                         record_tests.adb 
-- * Copyright (c) Simon Johnston & Addison Wesley Longman 1996.
-- *
-- * Description: This shows some simple record types, starting with a record
-- *    containing simple types, then some compound types. It also shows use
-- *    of discriminants to declare enclosed types and to declare variant
-- *    records.
-- * Inputs:  None.
-- * Outputs: None.
-- ***************************************************************************

procedure Record_Tests is
begin

   pragma Page;
   Simple_Records:
      declare

         type Device is
            record
               Major_Number : Integer;
               Minor_Number : Integer;
               Name         : String(1 .. 20);
            end record;

         Device_1 : Device;
         Device_2 : Device := (1, 1, "lp1                 ");

         type Initialized_Device is
            record
               Major_Number : Integer         := 0;
               Minor_Number : Integer         := 0;
               Name         : String(1 .. 21) := (others => Character'Val(0));
            end record;
      begin
   
         Device_1 := Device_2;

      end Simple_Records;

   pragma Page;
   Nested_Records:
      declare

         type Device is
            record
               Major_Number : Integer;
               Minor_Number : Integer;
               Name         : String(1 .. 20);
            end record;

         type Device_Command is (Open, Close, Flush_Input, Flush_Output, Test);
         type Device_Command_Packet is
            record
               To_Device    : Device;
               The_Command  : Device_Command := Test;
            end record;

         Printer : Device := (1, 1, "lp1                 ");
         Printer_Flush_Command : Device_Command_Packet;

      begin
   
         Printer_Flush_Command := (To_Device   => Printer, 
                                   The_Command => Flush_Output);

      end Nested_Records;

   pragma Page;
   Discriminant_Records:
      declare

         type Event_Item is
            record
               Event_ID     : Integer;
               Event_Info   : String(1 .. 80);
            end record;
         type Event_Array is array (Positive range <>) of Event_Item;

         type Cyclic_Event_Log(Max_Size : Positive) is
            record
               First_Event  : Positive := 1;
               Events       : Event_Array(1 .. Max_Size);
            end record;

         My_Event_Log : Cyclic_Event_Log(Max_Size => 100);

      begin

         null;

      end Discriminant_Records;

   pragma Page;
   Variant_Records:
      declare

         type Payment_Type is (Cash, Cheque, Credit);
         type Payment(The_Type : Payment_Type) is
            record
               Transaction_ID  : Integer;
               Amount          : Float;
               case The_Type is
               when Cash =>
                  null;
               when Cheque =>
                  Cheque_Number      : Integer;
                  Cheque_Card_Number : String(1 .. 16);
               when Credit =>
                  Credit_Card_Number : String(1 .. 16);
                  Expiry_Date        : Integer;
               end case;
            end record;

         Sale_1 : Payment(Cash);

         subtype Cash_Payment is Payment(Cash);
         subtype Cheque_Payment is Payment(Cheque);
         subtype Credit_Payment is Payment(Cheque);

         Sale_2 : Cheque_Payment;
 
      begin

         Sale_1.Transaction_ID := 1;
         Sale_1.Amount := 15.99;
         Sale_1.Cheque_Number := 1; -- FAILS !!!

         if Sale_2.The_Type = Cheque then

            Sale_2.Cheque_Number := 1; -- OK
         end if;

      end Variant_Records;

end Record_Tests;
