-- ***************************************************************************
-- *                      System_Representation.adb 
-- * Copyright (c) Simon Johnston & Addison Wesley Longman 1996.
-- *
-- * Description: This example comprises examples of system representation
-- *    attributes and contains code from section 3.7.
-- * Inputs:  None.
-- * Outputs: None.
-- ***************************************************************************

with System;
with System.Storage_Elements;

procedure System_Representation is

   -- * Size representation.
   type BYTE is mod 256;
   for  BYTE'Size use 8;

   -- * Some useful system values.
   BYTE_Size         : constant Integer := BYTE'Size;
   Word_Size         : constant Integer := System.Word_Size;
   Storage_Unit_Size : constant Integer := System.Storage_Unit;

   -- * Specifying enumeration values.
   type Activity is  (Reading, Writing, Idle);

   Read_Active_Bit  : constant BYTE := 2#0000_0001#;
   Write_Active_Bit : constant BYTE := 2#0000_0010#;
   Idle_Bit         : constant BYTE := 2#0000_0100#;
   for  Activity use (Reading => Read_Active_Bit, 
                      Writing => Write_Active_Bit, 
                      Idle    => Idle_Bit);

   -- * Specifying storage locations.
   Status_Address : constant System.Address :=
      System.Storage_Elements.To_Address(16#0340#);

   Device_Status : Activity;
   for Device_Status'Address use Status_Address;

   -- * Specifying record layouts.
   type Bit_Flag is mod 2**1;

   type Device_Status_Word is
      record
         Status : Activity;
         Unused : Integer range 0 .. 7;
         Parity : Bit_Flag;
      end record;

   for  Device_Status_Word use
      record
         Status at 0 range 0 .. 3;
         Unused at 0 range 4 .. 6;
         Parity at 0 range 7 .. 7;
      end record;

   -- * Packed boolean arrays.
   type Error_Locus_Word is array(Natural range 0 .. Word_Size) of Boolean;
   pragma Pack(Error_Locus_Word);

   Error_Register : Error_Locus_Word := (others => false);
begin

   null;

end System_Representation;
