-- ***************************************************************************
-- *                         Variant_Tests.adb 
-- * Copyright (c) Simon Johnston & Addison Wesley Longman 1996.
-- *
-- * Description: This file contains three simple blocks demonstrating the
-- *    declaration and use of variant types.
-- * Inputs:  None.
-- * Outputs: None.
-- ***************************************************************************

with Ada.Text_IO;
use  Ada.Text_IO;

procedure Variant_Tests is

   type Transaction_ID is new Integer;
   type Currency       is delta 0.01 digits 9;
   type Card_ID        is new Integer;
   type Date           is new Positive;

   type Payment_Type is (Cash, Cheque, Credit);

   type Transaction(The_Type : Payment_Type) is
      record
         Transaction       : Transaction_ID;
         Amount            : Currency;
         case The_Type is
         when Cash =>
            null;
         when Cheque =>
            Cheque_Number      : Integer;
            Cheque_Card_Number : Card_ID; 
         when Credit =>
            Credit_Card_Number : Card_ID; 
            Expiry             : Date;
         end case;
      end record;

   subtype Cash_Transaction   is Transaction(Cash);
   subtype Cheque_Transaction is Transaction(The_Type => Cheque);
   subtype Credit_Transaction is Transaction(The_Type => Credit);

begin
   First_Test:
      declare
         A_Trans : Transaction(Cash);
      begin
         A_Trans.The_Type      := Cheque; -- illegal
         A_Trans.Transaction   := 0000;
         A_Trans.Amount        := 10.99;
         A_Trans.Cheque_Number := 10099;
      end First_Test;

   Second_Test:
      declare
         A_Trans : Transaction(Cash);
      begin
         case A_Trans.The_Type is
            when Cash =>
               Put_Line("Cash");
            when Cheque =>
               Put_Line("Cheque");
            when Credit =>
               Put_Line("Credit Card");
         end case;
      end Second_Test;

   Third_Test:
      declare
         A_Trans : Transaction(Cash);
      begin
         A_Trans := (The_Type      => Cheque,
                     Transaction   => 0000,
                     Amount        => 10.99,
                     Cheque_Number => 10099,
                     Cheque_Card_Number => 0);
         -- illegal
      end Third_Test;
end Variant_Tests;
