--[class_abstract_account.ads] Specification
PACKAGE Class_abstract_account04 IS

  TYPE Abstract_account IS ABSTRACT TAGGED NULL RECORD;
  SUBTYPE Money  IS Float;
  SUBTYPE PMoney IS Float RANGE 0.0 .. Float'Last;

  PROCEDURE statement( the:IN Abstract_account ) IS ABSTRACT;
  PROCEDURE deposit  ( the:IN OUT Abstract_account;
                      amount:IN PMoney ) IS ABSTRACT;
  PROCEDURE withdraw ( the:IN OUT Abstract_account;
                       amount:IN PMoney;
                       get:OUT PMoney ) IS ABSTRACT;
  FUNCTION  balance  ( the:IN Abstract_account )
                       RETURN Money IS ABSTRACT;
END Class_abstract_account04;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

-- FIX
-- TYPE Abstract_account  IS TAGGED PRIVATE;
--  MUST BE 
-- TYPE Abstract_account  IS ABSTRACT TAGGED PRIVATE;

--[class_abstract_account.ads] Specification
PACKAGE Class_abstract_account_other IS

  TYPE Abstract_account  IS ABSTRACT TAGGED PRIVATE;
  TYPE Abstract_account2 IS ABSTRACT TAGGED PRIVATE;
  SUBTYPE Money  IS Float;
  SUBTYPE PMoney IS Float RANGE 0.0 .. Float'Last;

  PROCEDURE statement( the:IN Abstract_account ) IS ABSTRACT;
  PROCEDURE deposit  ( the:IN OUT Abstract_account;
                      amount:IN PMoney ) IS ABSTRACT;
  PROCEDURE withdraw ( the:IN OUT Abstract_account;
                       amount:IN PMoney;
                       get:OUT PMoney ) IS ABSTRACT;
  FUNCTION  balance  ( the:IN Abstract_account )
                       RETURN Money IS ABSTRACT;
PRIVATE
  TYPE Abstract_account  IS ABSTRACT TAGGED RECORD NULL; END RECORD;
  TYPE Abstract_account2 IS ABSTRACT TAGGED NULL RECORD;
END Class_abstract_account_other;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

--[class_account.ads] Specification
WITH Class_abstract_account04;
USE  Class_abstract_account04;
PACKAGE Class_account030 IS

  TYPE Account IS NEW Abstract_account WITH PRIVATE;
  SUBTYPE Money  IS Class_abstract_account04.Money;
  SUBTYPE PMoney IS Class_abstract_account04.PMoney;

  PROCEDURE statement( the:IN Account );
  PROCEDURE deposit  ( the:IN OUT Account; amount:IN PMoney );
  PROCEDURE withdraw ( the:IN OUT Account; amount:IN PMoney;
                       get:OUT PMoney );
  FUNCTION  balance  ( the:IN Account ) RETURN Money;
PRIVATE
  TYPE Account IS NEW Abstract_account WITH RECORD
    balance_of : Money := 0.00;       -- Amount in account
  END RECORD;
END Class_account030;


--[class_account.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_account030 IS

  PROCEDURE statement( the:IN Account ) IS
  BEGIN
    put("Mini statement: The amount on deposit is $" );
    put( the.balance_of, aft=>2, exp=>0 );
    new_line(2);
  END statement;

  PROCEDURE deposit( the:IN OUT Account; amount:IN PMoney ) IS
  BEGIN
    the.balance_of := the.balance_of + amount;
  END deposit;

  PROCEDURE withdraw( the:IN OUT Account;
                      amount:IN PMoney; get:OUT PMoney ) IS
  BEGIN
    IF the.balance_of >= amount THEN
      the.balance_of := the.balance_of - amount;
      get := amount;
    ELSE
      get := 0.00;
    END IF;
  END withdraw;

  FUNCTION  balance( the:IN Account ) RETURN Money IS
  BEGIN
    RETURN the.balance_of;
  END balance;

END Class_account030;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

--[class_interest_account.ads] Specification
WITH Class_account030;
USE  Class_account030;
PACKAGE Class_interest_account030 IS

  TYPE Interest_account IS NEW Account WITH PRIVATE;

  PROCEDURE set_rate( rate:IN Float );
  PROCEDURE calc_interest( the:IN OUT Interest_account );
  PROCEDURE add_interest( the:IN OUT Interest_account );
PRIVATE
  DAILY_INTEREST_RATE: CONSTANT Float := 0.00026116; -- 10%
  TYPE Interest_account IS NEW Account WITH RECORD
    accumulated_interest : Money := 0.00;            -- To date
  END RECORD;
  the_interest_rate      : Float := DAILY_INTEREST_RATE;
END Class_interest_account030;


--[class_interest_account.adb] Implementation
PACKAGE BODY Class_interest_account030 IS

  PROCEDURE set_rate( rate:IN Float ) IS
  BEGIN
    the_interest_rate := rate;
  END set_rate;

  PROCEDURE calc_interest( the:IN OUT Interest_account ) IS
  BEGIN
    the.accumulated_interest := the.accumulated_interest +
      balance(the) * the_interest_rate;
  END calc_interest;

  PROCEDURE add_interest( the:IN OUT Interest_account ) IS
  BEGIN
    deposit( the, the.accumulated_interest );
    the.accumulated_interest := 0.00;
  END add_interest;

END Class_interest_account030;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---
--[class_account_ltd.ads] Specification
WITH Class_account030;
USE  Class_account030;
PACKAGE Class_account_ltd IS

  TYPE Account_ltd IS NEW Account WITH PRIVATE;

  PROCEDURE withdraw ( the:IN OUT Account_ltd;
                       amount:IN PMoney; get:OUT PMoney );
  PROCEDURE reset( the:IN OUT Account_ltd );
PRIVATE
  WITHDRAWALS_IN_A_WEEK : Natural := 3;
  TYPE Account_ltd IS NEW Account WITH RECORD
    withdrawals : Natural := WITHDRAWALS_IN_A_WEEK;
  END RECORD;
END Class_account_ltd;


--[class_account_ltd.adb] Implementation
PACKAGE BODY Class_account_ltd IS

  PROCEDURE withdraw ( the:IN OUT Account_ltd;
                       amount:IN PMoney; get:OUT PMoney ) IS
  BEGIN
    IF the.withdrawals > 0 THEN               -- Not limit
      the.withdrawals := the.withdrawals - 1;
      withdraw( Account(the), amount, get );  -- In Account
    ELSE
     get := 0.00;                             -- Sorry
    END IF;
  END withdraw;

  PROCEDURE reset( the:IN OUT Account_ltd ) IS
  BEGIN
    the.withdrawals := WITHDRAWALS_IN_A_WEEK;
  END reset;

END Class_account_ltd;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---
-- Can not have abstract account as now two definitions for PMoney

--[pack_procedures.ads] Specification
WITH Ada.Text_io, Class_account030, Class_interest_account030,
                  Class_account_ltd;
USE  Ada.Text_io, Class_account030, Class_interest_account030,
                  Class_account_ltd;
PACKAGE Pack_procedures030 IS
  PROCEDURE ex1;
  PROCEDURE ex2;
  PROCEDURE ex3;
  PROCEDURE ex4;
END Pack_procedures030;


--[pack_procedures.adb] Implementation
PACKAGE BODY Pack_procedures030 IS

PROCEDURE ex1 IS
  mike  : Interest_account;
  obtain: Money;
BEGIN
  statement( mike );
  withdraw( mike, 100.00, obtain ); -- Withdraw some money
  statement( mike );
  deposit( mike, 300.00 );          -- In credit
  statement( mike );
END ex1;

PROCEDURE ex2 IS
  mike  : Account_ltd;
  obtain: Money;
BEGIN
  deposit( mike, 300.00 );          -- In credit
  statement( mike );
  withdraw( mike, 100.00, obtain ); -- Withdraw some money
  withdraw( mike,  10.00, obtain ); -- Withdraw some money
  withdraw( mike,  10.00, obtain ); -- Withdraw some money
  withdraw( mike,  20.00, obtain ); -- Withdraw some money
  statement( mike );
END ex2;

PROCEDURE ex3 IS
  mike  : Account_ltd;
  obtain: Money;
BEGIN
  deposit( mike, 300.00 );          -- In credit
  statement( mike );
  withdraw( mike, 100.00, obtain ); -- Withdraw some money
  statement( mike );
  withdraw( mike,  10.00, obtain ); -- Withdraw some money
  statement( mike );
  withdraw( mike,  10.00, obtain ); -- Withdraw some money
  statement( mike );
  withdraw( Account(mike),  20.00, obtain ); -- Cheat
  statement( mike );
END ex3;

PROCEDURE ex4 IS
  MAX_ACCOUNTS : CONSTANT := 5;
  TYPE P_account IS ACCESS ALL Account'Class;

  TYPE    Bank_index IS RANGE 1 .. MAX_ACCOUNTS;
  SUBTYPE Bank_range IS Bank_index;
  TYPE    Bank_array IS ARRAY ( Bank_range ) OF P_account;

  PROCEDURE print_statement( no:IN Bank_range;
                              a:IN Account'Class ) IS
  BEGIN
    put("Account number #");
    put( Integer'Image(Integer(no)) ); new_line;
    statement( a );
  END print_statement;

  piggy    : Bank_array;
  obtained : Money;

BEGIN
  piggy(1) := NEW Account;
  piggy(2) := NEW Account_ltd;
  piggy(3) := NEW Interest_account;
  piggy(4) := NEW Interest_account;
  piggy(5) := NEW Account;


  deposit( piggy(1).ALL, 100.00 );        -- deposit $100 #1
  deposit( piggy(3).ALL, 100.00 );        -- deposit $100 #3

  withdraw( piggy(2).ALL, 50.00, obtained ); -- withdraw $50 #2
  withdraw( piggy(4).ALL, 50.00, obtained ); -- withdraw $50 #4

  FOR i IN Bank_range LOOP
    print_statement( i, piggy(i).ALL );
  END LOOP;
  NULL;
END ex4;

END Pack_procedures030;

--[main.adb] Procedure
WITH Simple_io, Pack_procedures030;
USE  Simple_io, Pack_procedures030;
PROCEDURE main030 IS
BEGIN
  put("Example 1"); new_line; ex1;
  put("Example 2"); new_line; ex2;
  put("Example 3"); new_line; ex3;
  put("Example 4"); new_line; ex4;
END main030;
