
--[class_abstract_account.ads] Specification
PACKAGE Class_abstract_account05 IS

  TYPE Abstract_account IS ABSTRACT TAGGED NULL RECORD;
  SUBTYPE Money  IS Float;
  SUBTYPE PMoney IS Float RANGE 0.0 .. Float'Last;

  PROCEDURE statement( the:IN Abstract_account ) IS ABSTRACT;
  PROCEDURE deposit  ( the:IN OUT Abstract_account;
                      amount:IN PMoney ) IS ABSTRACT;
  PROCEDURE withdraw ( the:IN OUT Abstract_account;
                       amount:IN PMoney;
                       get:OUT PMoney ) IS ABSTRACT;
  FUNCTION  balance  ( the:IN Abstract_account )
                       RETURN Money IS ABSTRACT;
END Class_abstract_account05;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

--[class_account.ads] Specification
WITH Class_abstract_account05;
USE  Class_abstract_account05;
PACKAGE Class_account IS

  TYPE Account IS NEW Abstract_account WITH PRIVATE;
  SUBTYPE Money  IS Class_abstract_account05.Money;
  SUBTYPE PMoney IS Class_abstract_account05.PMoney;

  PROCEDURE statement( the:IN Account );
  PROCEDURE deposit  ( the:IN OUT Account; amount:IN PMoney );
  PROCEDURE withdraw ( the:IN OUT Account; amount:IN PMoney;
                       get:OUT PMoney );
  FUNCTION  balance  ( the:IN Account ) RETURN Money;
PRIVATE
  TYPE Account IS NEW Abstract_account WITH RECORD
    balance_of : Money := 0.00;       -- Amount in account
  END RECORD;
END Class_account;


--[class_account.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_account IS

  PROCEDURE statement( the:IN Account ) IS
  BEGIN
    put("Mini statement: The amount on deposit is $" );
    put( the.balance_of, aft=>2, exp=>0 );
    new_line(2);
  END statement;

  PROCEDURE deposit( the:IN OUT Account; amount:IN PMoney ) IS
  BEGIN
    the.balance_of := the.balance_of + amount;
  END deposit;

  PROCEDURE withdraw( the:IN OUT Account;
                      amount:IN PMoney; get:OUT PMoney ) IS
  BEGIN
    IF the.balance_of >= amount THEN
      the.balance_of := the.balance_of - amount;
      get := amount;
    ELSE
      get := 0.00;
    END IF;
  END withdraw;

  FUNCTION  balance( the:IN Account ) RETURN Money IS
  BEGIN
    RETURN the.balance_of;
  END balance;

END Class_account;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

--[class_interest_account.ads] Specification
WITH Class_account;
USE  Class_account;
PACKAGE Class_interest_account IS

  TYPE Interest_account IS NEW Account WITH PRIVATE;

  PROCEDURE set_rate( rate:IN Float );
  PROCEDURE calc_interest( the:IN OUT Interest_account );
  PROCEDURE add_interest( the:IN OUT Interest_account );
PRIVATE
  DAILY_INTEREST_RATE: CONSTANT Float := 0.00026116; -- 10%
  TYPE Interest_account IS NEW Account WITH RECORD
    accumulated_interest : Money := 0.00;            -- To date
  END RECORD;
  the_interest_rate      : Float := DAILY_INTEREST_RATE;
END Class_interest_account;


--[class_interest_account.adb] Implementation
PACKAGE BODY Class_interest_account IS

  PROCEDURE set_rate( rate:IN Float ) IS
  BEGIN
    the_interest_rate := rate;
  END set_rate;

  PROCEDURE calc_interest( the:IN OUT Interest_account ) IS
  BEGIN
    the.accumulated_interest := the.accumulated_interest +
      balance(the) * the_interest_rate;
  END calc_interest;

  PROCEDURE add_interest( the:IN OUT Interest_account ) IS
  BEGIN
    deposit( the, the.accumulated_interest );
    the.accumulated_interest := 0.00;
  END add_interest;

END Class_interest_account;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

--[class_interest_account-inspect_interest.ads] Specification
PACKAGE Class_interest_account.inspect_interest IS
  FUNCTION interest_is( the:IN Interest_account )
                        RETURN Money;
END Class_interest_account.inspect_interest;

--[class_interest_account-inspect_interest.adb] Implementation
PACKAGE BODY Class_interest_account.inspect_interest IS

  FUNCTION interest_is( the:IN Interest_account )
                        RETURN Money IS
  BEGIN
    RETURN the.accumulated_interest;
  END interest_is;

END Class_interest_account.inspect_interest;

--[pack_procedures.ads] Specification
WITH Simple_io, Class_account, Class_interest_account,
                Class_interest_account.inspect_interest;
USE  Simple_io, Class_account, Class_interest_account,
                Class_interest_account.inspect_interest;
PACKAGE Pack_procedures050 IS
  PROCEDURE ex1;
  PROCEDURE ex2;
  PROCEDURE ex3;
  PROCEDURE ex4;
END Pack_procedures050;

--[pack_procedures.adb] Implementation
PACKAGE BODY Pack_procedures050 IS

PROCEDURE ex1 IS
  my_account: Interest_account;
  obtained  : Money;
BEGIN
  statement( my_account );
  put("Deposit 100.00 into account"); new_line;
  deposit( my_account, 100.00 );            -- Day 1
  calc_interest( my_account );              -- End of day 1
  calc_interest( my_account );              -- End of day 2
  statement( my_account );                  -- Day 3
  obtained := interest_is( my_account );    -- How much interest
  put("Interest accrued so far : $" );
  put( obtained, aft=>2, exp=>0 ); new_line;
END ex1;

PROCEDURE ex2 IS
BEGIN
  NULL;
END ex2;

PROCEDURE ex3 IS
BEGIN
  NULL;
END ex3;

PROCEDURE ex4 IS
BEGIN
  NULL;
END ex4;

END Pack_procedures050;

--[main.adb] Procedure
WITH Simple_io, Pack_procedures050;
USE  Simple_io, Pack_procedures050;
PROCEDURE main050 IS
BEGIN
  put("Example 1"); new_line; ex1;
  put("Example 2"); new_line; ex2;
  put("Example 3"); new_line; ex3;
  put("Example 4"); new_line; ex4;
END main050;
