--[class_description.ADS] Implementation Instantiation
WITH Ada.Strings.Bounded; USE Ada.Strings.Bounded;
  PACKAGE B_String IS
    NEW Generic_Bounded_Length( 80 );

--[class_room.ads] Specification
WITH B_String; USE B_String;
PACKAGE Class_room IS
  TYPE Room   IS TAGGED PRIVATE;

  PROCEDURE initialize( the:IN OUT Room; no:IN Positive;
                        mes:IN String );
  FUNCTION  where( the:IN Room ) RETURN Positive;
  FUNCTION  describe( the:IN Room ) RETURN String;
PRIVATE
  TYPE Room IS TAGGED RECORD
     desc  : Bounded_string;     -- Description of room
     number: Positive;           -- Room number
  END RECORD;
END Class_room;

--[class_room.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_room IS

  PROCEDURE initialize( the:IN OUT Room;
                        no:IN Positive; mes:IN String ) IS
  BEGIN
    the.desc := to_bounded_string( mes );
    the.number := no;
  END initialize;

  FUNCTION  where( the:IN Room ) RETURN Positive IS
  BEGIN
    RETURN the.number;
  END where;

  FUNCTION  describe( the:IN Room ) RETURN String IS
    num : String( 1 .. 4 );    -- Room number as string
  BEGIN
    put( num, the.number );
    RETURN num & " " & to_string(the.desc);
  END describe;

END Class_room;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---


--[class_office.ads] Specification
WITH Class_room; USE  Class_room;
PACKAGE Class_office IS
  TYPE Office IS NEW Room WITH PRIVATE;

  PROCEDURE initialize( the:IN OUT Office; no:IN Positive;
                        desc:IN String; people:IN Natural );
  FUNCTION  deliver_no_of_people(the:IN Office) RETURN Natural;
  FUNCTION  describe( the:IN Office ) RETURN String;
PRIVATE
  TYPE Office IS NEW Room WITH RECORD
    people : Natural := 0;            -- Occupants
  END RECORD;
END Class_office;


--[class_office.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_office IS

  PROCEDURE initialize( the:IN OUT Office; no:IN Positive;
                        desc:IN String; people:IN Natural ) IS
  BEGIN
    initialize( the, no, desc );
    the.people := people;
  END initialize;

  FUNCTION deliver_no_of_people( the:IN Office ) RETURN Natural IS
  BEGIN
    RETURN the.people;
  END deliver_no_of_people;

  FUNCTION  describe( the:IN Office ) RETURN String IS
    no : String( 1 .. 4 );    -- the.people as string
  BEGIN
    put(  no, the.people );
    RETURN describe( Room(the) ) &
           " occupied by" & no & " people";
  END describe;

END Class_office;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

PACKAGE Class_room.build IS
  TYPE P_Room IS ACCESS ALL Room'Class;

  FUNCTION  build_room( no:IN Positive;
                        desc:IN String ) RETURN P_Room;
END Class_room.build;

PACKAGE BODY Class_room.build IS

  FUNCTION  build_room( no:IN Positive;
                        desc:IN String ) RETURN P_Room IS
    p : P_Room;
  BEGIN
    p := NEW Room; initialize( p.ALL, no, desc );
    RETURN p;
  END build_room;

END Class_room.build;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

WITH Class_room, Class_room.build;
USE  Class_room, Class_room.build;
PACKAGE Class_office.build IS

  FUNCTION  build_office( no:IN Positive; desc:IN String;
                          people:IN Natural ) RETURN P_Room;
END Class_office.build;


PACKAGE BODY Class_office.build IS

  TYPE P_Office IS ACCESS ALL Office;

  FUNCTION  build_office( no:IN Positive; desc:IN String;
                          people:IN Natural ) RETURN P_Room IS
    p : P_Office;
  BEGIN
    p := NEW Office; initialize( p.ALL, no, desc, people );
    RETURN p.ALL'Access;
  END build_office;

END Class_office.build;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

--[class_building.ads] Specification
WITH Ada.Text_io, Class_room, Class_room.build;
USE  Ada.Text_io, Class_room, Class_room.build;
PACKAGE Class_building IS

  TYPE Building IS TAGGED PRIVATE;

  PROCEDURE add( the:IN OUT Building; desc:IN P_Room );
  FUNCTION about(the:IN Building; no:IN Positive) RETURN String;

PRIVATE
  MAX_ROOMS : CONSTANT := 15;
  TYPE    Rooms_index IS RANGE 0 .. MAX_ROOMS;
  SUBTYPE Rooms_range IS Rooms_index RANGE 1 .. MAX_ROOMS;
  TYPE    Rooms_array IS ARRAY (Rooms_range) OF P_Room;

  TYPE Building IS TAGGED RECORD
    last        : Rooms_index := 0;  -- Last slot allocated
    description : Rooms_array;       -- Rooms in building
  END RECORD;
END Class_building;


--[class_building.adb] Implementation
PACKAGE BODY Class_building IS

  PROCEDURE add( the:IN OUT Building; desc:IN P_Room ) IS
  BEGIN
    IF the.last < MAX_ROOMS THEN
      the.last := the.last + 1;
      the.description( the.last ) := desc;
    ELSE
      RAISE Constraint_error;
    END IF;
  END add;

  FUNCTION about(the:IN Building; no:IN Positive) RETURN String IS
  BEGIN
    FOR i IN 1 .. the.last LOOP
      IF where(the.description(i).ALL) = no THEN
        RETURN describe(the.description(i).ALL);
      END IF;
    END LOOP;
    RETURN "Sorry room not known";
  END about;
END Class_building;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

--[pack_procedures.ads] Specification
WITH Simple_io, Class_room, Class_room.build,
     Class_office, Class_office.build, Class_building, Ada.Tags;
USE  Simple_io, Class_room, Class_room.build,
     Class_office, Class_office.build, Class_building, Ada.Tags;
PACKAGE Pack_procedures060 IS
  PROCEDURE ex1;
  PROCEDURE ex2;
  PROCEDURE ex3;
  PROCEDURE ex4;
  PROCEDURE ex5;
  PROCEDURE ex6;
  PROCEDURE ex7;
  PROCEDURE ex8;
END Pack_procedures060;

--[pack_procedures.adb] Implementation
PACKAGE BODY Pack_procedures060 IS

PROCEDURE ex1 IS
  w422 : Room;
  w414 : Office;
  PROCEDURE about( place:IN Room'Class ) IS
  BEGIN
    put( "The place is" ); new_line;
    put( "  " & describe( place ) ) ;  -- Run time dispatch
    new_line;
  END about;
BEGIN
  initialize( w414, 414, "4th Floor west wing", 2 );
  initialize( w422, 422, "4th Floor east wing" );

  about( w422 );                       -- Call with a room
  about( w414 );                       -- Call with an Office

END ex1;

PROCEDURE ex2 IS
  TYPE P_Room IS ACCESS ALL Room'Class;
  w422   : ALIASED Room;
  w414   : ALIASED Office;
  p_w422 : P_Room := w422'Access;
  p_w414 : P_Room := w414'Access;
BEGIN
  initialize( w414, 414, "4th Floor west wing", 2 );
  initialize( w422, 422, "4th Floor east wing" );

  IF p_w422'Tag = p_w414'Tag THEN
    put("Areas are the same type of accommodation");
    new_line;
  END IF;
  IF p_w422'Tag /= p_w414'Tag THEN
    put("Areas are different type of accommodation");
    new_line;
  END IF;
END ex2;

PROCEDURE ex3 IS

PROCEDURE set_up( watts:IN OUT Building ) IS
BEGIN
  add( watts, build_office( 414, "4th Floor west wing", 2 ) );
  add( watts, build_room  ( 422, "4th Floor east wing" ) );
END set_up;

PROCEDURE main IS
  watts   : Building;                      -- Watts Building
  room_no : Positive;                      -- Queried room
BEGIN
  set_up( watts );                         -- Populate building
  LOOP
    BEGIN
      put( "Inquiry about room: " );       -- Ask
      EXIT WHEN end_of_file;
      get( room_no ); skip_line;           -- User response
      put( about( watts, room_no ) );
      new_line;                            -- Display answer
    EXCEPTION
      WHEN Data_error =>
        put("Please retype the number");   -- Ask again
        new_line; skip_line;
    END;
  END LOOP;
END main;

BEGIN
  main;
END ex3;

PROCEDURE ex4 IS
BEGIN
  put("Nothing"); new_line;
  NULL;
END ex4;

-- Downcasting

PROCEDURE ex5 IS
  MAX_ROOMS : CONSTANT := 3;
  TYPE    Rooms_index  IS RANGE 0 .. MAX_ROOMS;
  SUBTYPE Rooms_range  IS Rooms_index RANGE 1 .. MAX_ROOMS;
  TYPE    Rooms_array  IS ARRAY ( Rooms_range ) OF P_Room;
  TYPE    Office_array IS ARRAY ( Rooms_range ) OF Office;
  accommodation : Rooms_array;    -- Rooms and Offices
  offices       : Office_array;   -- Offices only
  no_offices    : Rooms_index;
BEGIN
  accommodation(1):=build_office(414, "4th Floor west wing", 2);
  accommodation(2):=build_room  (518, "5th Floor east wing");
  accommodation(3):=build_office(403, "4th Floor east wing", 1);

  no_offices := 0;
  FOR i IN Rooms_range LOOP
    IF accommodation(i).ALL'Tag = Office'Tag THEN
      no_offices := no_offices + 1;
      offices(no_offices) := Office(accommodation(i).all);  --
    END IF;
  END LOOP;

  put("The offices are:" ); new_line;
  FOR i IN 1 .. no_offices LOOP
    put( describe( offices(i) ) ); new_line;
  END LOOP;

END ex5;

PROCEDURE ex6 IS
  --MAX_ROOMS : CONSTANT := 2;
  TYPE P_Room IS ACCESS ALL Room'Class;
  w422 : ALIASED Room;
  w414 : ALIASED Office;
  accommodation : P_Room;
BEGIN
  initialize( w422, 422, "4th Floor east wing" );
  accommodation := w422'Access;
  put( Ada.Tags.external_tag( accommodation'Tag ) ); new_line;
  put( Ada.Tags.expanded_name( accommodation'Tag ) ); new_line;
  initialize( w414, 414, "4th Floor west wing", 2 );
  accommodation := w414'Access;
  put( Ada.Tags.external_tag( accommodation'Tag ) ); new_line;
  put( Ada.Tags.expanded_name( accommodation'Tag ) ); new_line;
END ex6;

PROCEDURE ex7 IS
  MAX_ROOMS : CONSTANT := 3;
  TYPE    Rooms_index IS RANGE 0 .. MAX_ROOMS;
  SUBTYPE Rooms_range IS Rooms_index RANGE 1 .. MAX_ROOMS;
  TYPE    Rooms_array IS ARRAY ( Rooms_range ) OF P_Room;
  accommodation : Rooms_array;
BEGIN
  accommodation(1):=build_office(414, "4th Floor west wing", 2);
  accommodation(2):=build_room  (518, "5th Floor east wing");
  accommodation(3):=build_office(403, "4th Floor east wing", 1);
END ex7;

-- If only one componant in a record aggrigate then must be named
-- If more than one componant WITH
--  withdraws => 2, xx => 3 etc.

PROCEDURE ex8 IS
  WITHDRAWALS_IN_A_WEEK : CONSTANT Natural := 3;
  SUBTYPE Money    IS Float;
  TYPE Account     IS TAGGED RECORD
     balance_of : Money := 0.00;      -- Amount in account
  END RECORD;
  TYPE Account_ltd IS NEW Account WITH RECORD
    withdrawals : Natural := WITHDRAWALS_IN_A_WEEK;
  END RECORD;
  TYPE Account_2 IS NEW Account WITH NULL RECORD;
  normal     : Account;
  restricted : Account_ltd;
  other      : Account_2;
BEGIN
  normal     := ( balance_of => 20.0 );
  restricted := ( normal WITH 4 );
  restricted := ( normal WITH withdrawals => 4 );
  other      := ( normal WITH NULL RECORD );
END ex8;

END Pack_procedures060;

--[main.adb] Procedure
WITH Ada.Text_io, Pack_procedures060;
USE  Ada.Text_io, Pack_procedures060;
PROCEDURE main060 IS
BEGIN
  put("Example 1"); new_line; ex1;
  put("Example 2"); new_line; ex2;
  put("Example 3"); new_line; ex3;
  put("Example 4"); new_line; ex4;
  put("Example 5"); new_line; ex5;
  put("Example 6"); new_line; ex6;
  put("Example 7"); new_line; ex7;
  put("Example 8"); new_line; ex8;
END main060;
