--[class_stack.ads] Specification
PACKAGE Class_stack IS
  TYPE Stack IS PRIVATE;                -- Copying allowed
  Stack_error: EXCEPTION;               -- When error

  PROCEDURE reset( the:IN OUT Stack);
  PROCEDURE push( the:IN OUT Stack; item:IN Integer );
  PROCEDURE pop(the:IN OUT Stack; item :OUT Integer );
PRIVATE

  MAX_STACK: CONSTANT := 3;
  TYPE    Stack_index IS RANGE 0 .. MAX_STACK;
  SUBTYPE Stack_range IS Stack_index RANGE 1 .. MAX_STACK;
  TYPE    Stack_array IS ARRAY ( Stack_range ) OF Integer;

  TYPE Stack IS RECORD
    elements: Stack_array;          -- Array of elements
    tos     : Stack_index := 0;     -- Index
  END RECORD;

END Class_stack;


--[class_stack.adb] Implementation
PACKAGE BODY Class_stack IS

  PROCEDURE push( the:IN OUT Stack; item:IN Integer ) IS
  BEGIN
    IF the.tos /= MAX_STACK THEN
      the.tos := the.tos + 1;               -- Next element
      the.elements( the.tos ) := item;      -- Move in
    ELSE
      RAISE Stack_error;                    -- Failed
    END IF;
  END push;

  PROCEDURE pop( the:IN OUT Stack; item :OUT Integer ) IS
  BEGIN
    IF the.tos > 0 THEN
      item := the.elements( the.tos );      -- Top element
      the.tos := the.tos - 1;               -- Move down
    ELSE
      RAISE Stack_error;                    -- Failed
    END IF;
  END pop;

  PROCEDURE reset( the:IN OUT Stack ) IS
  BEGIN
    the.tos := 0;  -- Set TOS to 0 (Non existing element)
  END reset;

END Class_stack;

--[main.adb] Procedure
WITH Simple_io, Class_stack;
USE  Simple_io, Class_stack;
PROCEDURE main090 IS
  number_stack : Stack;              -- Stack of numbers
  action       : Character;          -- Action
  number       : Integer;            -- Number processed
BEGIN
  WHILE NOT end_of_file LOOP
    WHILE NOT end_of_line LOOP
      BEGIN
        get( action );
        CASE action IS               -- Process action
          WHEN '+' =>
            get( number ); push(number_stack,number);
            put("push number = "); put(number); new_line;
          WHEN '-' =>
            pop(number_stack,number);
            put("Pop number  = "); put(number); new_line;
          WHEN OTHERS =>
            put("Invalid action"); new_line;
        END CASE;
      EXCEPTION
        WHEN Stack_error =>
          put("Stack_error"); new_line;
        WHEN Data_error  =>
          put("Not a number"); new_line;
        WHEN End_error   =>
          put("Unexpected end of file"); new_line; EXIT;
      END;
    END LOOP;
    skip_Line;
  END LOOP;
  reset( number_stack );
END main090;
