--[class_stack.ads] Specification
WITH Text_io; USE Text_io;

GENERIC
  TYPE Stack_element IS PRIVATE;  -- Can specify any type
  MAX_STACK:IN Positive := 3;     -- Has to be typed / not const
PACKAGE Class_stack110 IS
  TYPE Stack IS PRIVATE;
  Stack_error: EXCEPTION;

  PROCEDURE reset( the:IN OUT Stack);
  PROCEDURE push( the:IN OUT Stack; item:IN Stack_element );
  PROCEDURE pop( the:IN OUT Stack; item:OUT Stack_element );
PRIVATE

  TYPE    Stack_index IS NEW Integer RANGE 0 .. MAX_STACK;
  SUBTYPE Stack_range IS Stack_index
          RANGE 1 .. Stack_index(MAX_STACK);
  TYPE    Stack_array IS ARRAY ( Stack_range ) OF Stack_element;

  TYPE Stack IS RECORD
    elements: Stack_array;          -- Array of elements
    tos     : Stack_index := 0;     -- Index
  END RECORD;

END Class_stack110;

--[class_stack.adb] Implementation
PACKAGE BODY Class_stack110 IS

  PROCEDURE push( the:IN OUT Stack; item:IN Stack_element ) IS
  BEGIN
    IF the.tos /= Stack_index(MAX_STACK) THEN
      the.tos := the.tos + 1;               -- Next element
      the.elements( the.tos ) := item;      -- Move in
    ELSE
      RAISE Stack_error;                        -- Failed
    END IF;
  END push;

  PROCEDURE pop( the:IN OUT Stack; item :OUT Stack_element ) IS
  BEGIN
    IF the.tos > 0 THEN
      item := the.elements( the.tos );      -- Top element
      the.tos := the.tos - 1;               -- Move down
    ELSE
      RAISE Stack_error;                        -- Failed
    END IF;
  END pop;

  PROCEDURE reset( the:IN OUT Stack ) IS
  BEGIN
    the.tos := 0;  -- Set TOS to 0 (Non existing element)
  END reset;

END Class_stack110;

--[class_stack_char.ADS] Implementation Instantiation
WITH Class_stack110;
  PRAGMA ELABORATE_ALL( Class_stack110 );
  PACKAGE Class_stack_char IS NEW Class_stack110(Character,3);

--[class_stack_int.ADS] Implementation Instantiation
WITH Class_stack110;
  PRAGMA ELABORATE_ALL( Class_stack110 );
  PACKAGE Class_stack_int IS NEW Class_stack110(Integer);

--[class_stack_stack_int.adb] Implementation Instantiation
WITH Class_stack110, Class_stack_int;
  PACKAGE Class_stack_stack_int IS
    NEW Class_stack110(Class_stack_int.Stack);

--[main1.adb] Procedure
WITH Simple_io, Class_stack_int;
USE  Simple_io, Class_stack_int;
PROCEDURE main111 IS
  number_stack : Stack;              -- Stack of numbers
  action       : Character;          -- Action
  number       : Integer;            -- Number processed
BEGIN
  WHILE NOT end_of_file LOOP
    WHILE NOT end_of_line LOOP
      BEGIN
        get( action );
        CASE action IS               -- Process action
          WHEN '+' =>
            get( number ); push(number_stack,number);
            put("push number = "); put(number); new_line;
          WHEN '-' =>
            pop(number_stack,number);
            put("Pop number  = "); put(number); new_line;
          WHEN OTHERS =>
            put("Invalid action"); new_line;
        END CASE;
      EXCEPTION
        WHEN Stack_error =>
          put("Stack_error"); new_line;
        WHEN Data_error  =>
          put("Not a number"); new_line;
        WHEN End_error   =>
          put("Unexpected end of file"); new_line; EXIT;
      END;
    END LOOP;
    skip_Line;
  END LOOP;

  reset( number_stack );
END main111;

--[main2.adb] Procedure
WITH Simple_io, Class_stack_int,Class_stack_stack_int;
USE  Simple_io, Class_stack_int,Class_stack_stack_int;
PROCEDURE main112 IS
  number_stack1      : Class_stack_int.Stack;
  number_stack2      : Class_stack_int.Stack;
  stack_number_stack : Class_stack_stack_int.Stack;
  number             : Integer;
BEGIN
  push( number_stack1, 1 );
  push( number_stack1, 2 );
  push( stack_number_stack, number_stack1 );
  pop(  stack_number_stack, number_stack2 );
  pop(  number_stack2, number );
  IF number = 2 THEN put("ok "); ELSE put("Fail "); END IF;
  pop(  number_stack2, number );
  IF number = 1 THEN put("ok "); ELSE put("Fail "); END IF;
  new_line;
END main112;


--[main.adb] Procedure
WITH Simple_io, main111, main112;
USE  Simple_io;
PROCEDURE main110 IS
BEGIN
  put("Example 1"); new_line; main111;
  put("Example 2"); new_line; main112;
END main110;
