--[pack_types.ads] Specification
-- Is it possible to overload Integer in package Standard as
--  this does not work
WITH Text_io; USE  Text_io;
PACKAGE Pack_types IS
  TYPE Integer IS RANGE -2_000_000_000 .. 2_000_000_000;
  PROCEDURE put( i:IN Integer );
END Pack_types;

--[pack_types.adb] Implementation
WITH Text_io; USE  Text_io;
PACKAGE BODY Pack_types IS
  PACKAGE Class_integer_io IS NEW Text_io.Integer_io( Integer );
  PROCEDURE put( i:IN Integer ) IS
  BEGIN
    put("[Overloaded]");
    Class_integer_io.put( i );
  END put;
END Pack_types;

--[class_rational.ads] Specification
PACKAGE Class_rational IS
  TYPE Rational IS PRIVATE;

  FUNCTION "+" ( f:IN Rational; s:IN Rational ) RETURN Rational;
  FUNCTION "-" ( f:IN Rational; s:IN Rational ) RETURN Rational;
  FUNCTION "*" ( f:IN Rational; s:IN Rational ) RETURN Rational;
  FUNCTION "/" ( f:IN Rational; s:IN Rational ) RETURN Rational;

  FUNCTION rat_const( f:IN Integer;
                      s:IN Integer:=1 ) RETURN Rational;
  PROCEDURE put( the:IN Rational );
PRIVATE
  FUNCTION sign( the:IN Rational ) RETURN Rational;
  FUNCTION simplify( the:IN Rational ) RETURN Rational;
  TYPE Rational IS RECORD
    above : Integer := 0;      -- Numerator
    below : Integer := 1;      -- Denominator
  END RECORD;
END Class_rational;

--[class_rational.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_rational IS


  FUNCTION "+" (f:IN Rational; s:IN Rational) RETURN Rational IS
    res : Rational;
  BEGIN
    res.below := f.below * s.below;
    res.above := f.above * s.below + s.above * f.below;
    RETURN simplify(res);
  END "+";

  FUNCTION "-" (f:IN Rational; s:IN Rational) RETURN Rational IS
    res : Rational;
  BEGIN
    res.below := f.below * s.below;
    res.above := f.above * s.below - s.above * f.below;
    RETURN simplify(res);
  END "-";

  FUNCTION "*" (f:IN Rational; s:IN Rational) RETURN Rational IS
    res : Rational;
  BEGIN
    res.above := f.above * s.above;
    res.below := f.below * s.below;
    RETURN simplify(res);
  END "*";

  FUNCTION "/" (f:IN Rational; s:IN Rational) RETURN Rational IS
    res : Rational;
  BEGIN
    res.above := f.above * s.below;
    res.below := f.below * s.above;
    RETURN simplify(res);
  END "/";

  FUNCTION rat_const( f:IN Integer; s:IN Integer:=1 ) RETURN Rational IS
  BEGIN
    IF f = 0 THEN
      RETURN Rational'(0,1);
    ELSE
      RETURN simplify( sign( Rational'( f, s ) ) );
    END IF;
  END rat_const;

  PROCEDURE put( the:IN Rational ) IS
    above : Integer := the.above;
    below : Integer := the.below;
  BEGIN
    IF above = 0 THEN                           -- Rational 0
      put( "0" );
    ELSE
      IF above < 0 THEN
        put("-"); above := -above;              --  make +ve
      END IF;
      IF above >= below THEN                    -- Whole number
        put( above/below, width=>1 ); put(" ");
        above := above REM below;               -- Fraction
      END IF;
      IF above /= 0 THEN
        put( above, width=>1 ); put( "/" );
        put( below, width=>1 );                 -- Fraction
      END IF;
    END IF;
  END put;

  FUNCTION sign( the:IN Rational ) RETURN Rational IS
  BEGIN
    IF the.below >= 0 THEN            --   -a/b or a/b
      RETURN the;
    ELSE                              --   a/-b or -a/-b
      RETURN Rational'( -the.above, -the.below );
    END IF;
  END sign;

  FUNCTION simplify( the:IN Rational ) RETURN Rational IS
    res: Rational := the;
    d  : Positive;                    -- Divisor to reduce with
  BEGIN
    IF res.below = 0 THEN             -- Invalid treat as 0
      res.above := 0; res.below := 1;
    END IF;
    d := 2;                           -- Divide by 2, 3, 4 ...
    WHILE d < res.below LOOP
      WHILE res.below REM d = 0 AND THEN res.above REM d = 0 LOOP
        res.above := res.above / d;
        res.below := res.below / d;
      END LOOP;
      d := d + 1;
    END LOOP;
    RETURN res;
  END simplify;

END Class_rational;

--[pack_procedures.ads] Specification
PACKAGE Pack_procedures120 IS
  PROCEDURE main1;
  PROCEDURE main2;
  PROCEDURE main3;
  PROCEDURE main4;
  PROCEDURE main5;
END Pack_procedures120;

--[pack_procedures.adb] Implementation
WITH Simple_io, Class_rational;
USE  Simple_io, Class_rational;
PACKAGE BODY Pack_procedures120 IS

PROCEDURE main1 IS
  a,b : Rational;
BEGIN
  a := rat_const( 1, 2 );
  b := rat_const( 1, 3 );

  put( "a     = " ); put( a );   new_line;
  put( "b     = " ); put( b );   new_line;
  put( "a + b = " ); put( a+b ); new_line;
  put( "a - b = " ); put( a-b ); new_line;
  put( "b - a = " ); put( b-a ); new_line;
  put( "a * b = " ); put( a*b ); new_line;
  put( "a / b = " ); put( a/b ); new_line;
END main1;

PROCEDURE main2 IS
  a,b : Rational;
BEGIN
  a := rat_const(1, 2);
  b := rat_const(1, 3);

  put( "a     = " ); put( a ); put(" <1/2> "); new_line;
  put( "b     = " ); put( b ); put("  <1/3> "); new_line;
  put( "a + b = " ); put( a+b ); put(" <5/6> "); new_line;
  put( "a - b = " ); put( a-b ); put(" <1/6> "); new_line;
  put( "b - a = " ); put( b-a ); put(" <-1/6> "); new_line;
  put( "a * b = " ); put( a*b ); put(" <1/6> "); new_line;
  put( "a / b = " ); put( a/b ); put(" <1 1/2> "); new_line;

  new_line;
  a := rat_const(3, 2);
  b := rat_const(4, 5);
  put( "a     = " ); put( a ); put(" <3/2> "); new_line;
  put( "b     = " ); put( b ); put(" <4/5> "); new_line;
  put( "a + b = " ); put( a+b ); put(" <2 3/10> "); new_line;
  put( "a - b = " ); put( a-b ); put(" <7/10> "); new_line;
  put( "b - a = " ); put( b-a ); put(" <-7/10> "); new_line;
  put( "a * b = " ); put( a*b ); put(" <1 1/5> "); new_line;
  put( "a / b = " ); put( a/b ); put(" <1 7/8> "); new_line;
END main2;

PROCEDURE main3 IS
  a : Rational;
BEGIN
  -- put( 8/16 ) is ambiguous;
  new_line;
  put( " 0, 0      = " ); a := rat_const(0, 0);   put( a ); new_line;
  put( " 0, 5      = " ); a := rat_const(0, 5);   put( a ); new_line;
  put( " 5, 0      = " ); a := rat_const(5, 0);   put( a ); new_line;
  put( " 8, 16     = " ); a := rat_const(8, 16);  put( a ); new_line;
  put( " 8, 8      = " ); a := rat_const(8, 8);   put( a ); new_line;
  put( " -17, 8    = " ); a := rat_const(-17, 8); put( a ); new_line;
  put( " -17, -8   = " ); a := rat_const(-17, -8);put( a ); new_line;
  put( " 17, -8    = " ); a := rat_const(17, -8); put( a ); new_line;
  put( " 7, 97     = " ); a := rat_const(7, 97);  put( a ); new_line;
  put( " 40, 20    = " ); a := rat_const(40, 20); put( a ); new_line;
  put( " 20, 40    = " ); a := rat_const(20, 40); put( a ); new_line;
  put( " 648, 972  = " ); a := rat_const(648,972);put( a ); new_line;
END main3;

PROCEDURE main4 IS
  a : Rational;
BEGIN
  a := rat_const(0, 0);
  FOR i IN 1 .. 5 LOOP
    a := a + rat_const(1,i);
  END LOOP;
  put("Sum of 1/1 + 1/2 + ... 1/5 is "); put( a ); new_line;
END main4;

PROCEDURE main5 IS
  a : Rational;
  UP_TO : CONSTANT := 7;
BEGIN
  a := rat_const(1, 1);
  FOR i IN 1 .. UP_TO LOOP
    a := a + rat_const(1, i);
  END LOOP;
  FOR i IN REVERSE 1 .. UP_TO LOOP
    a := a - rat_const(1, i);
  END LOOP;
  put("Sum  check answer should be 1 <=> "); put( a ); new_line;
  a := rat_const(1);
  FOR i IN 1 .. UP_TO LOOP
    a := a * rat_const(1, i);
  END LOOP;
  FOR i IN REVERSE 1 .. UP_TO LOOP
    a := a / rat_const(1,i);
  END LOOP;
  put("Mult check answer should be 1 <=> "); put( a ); new_line;
END main5;

END Pack_procedures120;

WITH Simple_io, Class_rational;
USE  TYPE Class_rational.Rational;
PROCEDURE main126 IS
  a,b : Class_rational.Rational;
BEGIN
  a := Class_rational.rat_const( 1, 2 );
  b := Class_rational.rat_const( 1, 3 );

  Simple_io.put( "a + b = " ); Class_rational.put( a+b );
  Simple_io.new_line;
END main126;

--[main.adb] Procedure
WITH Text_io, Pack_procedures120, main126; 
USE  Text_io, Pack_procedures120;
PROCEDURE main120 IS
BEGIN
   put("Example 1"); new_line; main1;
   put("Example 2"); new_line; main2;
   put("Example 3"); new_line; main3;
   put("Example 4"); new_line; main4;
   put("Example 5"); new_line; main5;
   put("Example 6"); new_line; main126;
END main120;

