--[class_bounded_string_80.ADS] Implementation Instantiation
WITH Ada.Strings.Bounded; USE Ada.Strings.Bounded;
  PACKAGE B_String56 IS
    NEW Generic_Bounded_Length( 80 );

--[class_pic.ads] Specification
-- Note I/O errors on finalize initialize passed to user
WITH B_string56, Ada.Finalization; 
USE  B_string56, Ada.Finalization;
GENERIC
  TYPE Index IS PRIVATE;                  -- Index for record
  TYPE Data  IS PRIVATE;                  -- Data for record
  WITH FUNCTION  ">"( f:IN Index; s:IN Index ) RETURN Boolean;
PACKAGE Class_pic IS
  Not_there, Exists, Per_error : EXCEPTION; -- Raised Exceptions
  TYPE PIC  IS NEW Limited_controlled WITH PRIVATE;
  PROCEDURE initialize( the:IN OUT PIC );
  PROCEDURE initialize( the:IN OUT PIC; id:IN String );
  PROCEDURE finalize( the:IN OUT PIC );
  PROCEDURE discard( the:IN OUT PIC );
  PROCEDURE set_name( the:IN OUT PIC; id:IN String );
  FUNCTION  get_name( the:IN PIC ) RETURN String;

  PROCEDURE add( the:IN OUT PIC; i:IN Index; d:IN Data );
  PROCEDURE extract( the:IN OUT PIC; i:IN Index; d:IN OUT Data );
  PROCEDURE update( the:IN OUT PIC; i:IN Index; d:IN OUT Data );
PRIVATE
  TYPE Leaf;                     -- Index + Data
  TYPE Subtree IS ACCESS Leaf;   --
  TYPE PIC IS NEW Limited_controlled WITH RECORD
    tree   : Subtree := NULL;    --  Storage
    obj_id : Bounded_string;     --  Name of object
  END RECORD;

  FUNCTION  find( the:IN Subtree; i:IN Index) RETURN Subtree;
  PROCEDURE release_storage( the:IN OUT Subtree );

END Class_pic;

--[class_pic.adb] Implementation
WITH unchecked_deallocation, sequential_io;
PACKAGE BODY Class_pic IS

  TYPE Element IS RECORD    --
    s_index: Index;         -- The Index
    s_data : Data;          -- The Data
  END RECORD;

  TYPE Leaf IS RECORD       --
    left   : Subtree;       -- Possible left node
    rec    : Element;       -- Index + data
    right  : Subtree;       -- Possible right node;
  END RECORD;

  PACKAGE io IS NEW Sequential_io( Element );

  PROCEDURE initialize( the:IN OUT PIC ) IS
  BEGIN
    the.tree := NULL;    -- No storage
  END initialize;

  PROCEDURE initialize( the:IN OUT PIC; id:IN String ) IS
    per : io.File_type;    -- File descriptor
    cur : Element;         -- Persistent data record element
  BEGIN
    set_name( the, id );                  -- Name object
    io.open( per, io.IN_FILE, id );       -- Open saved state
    WHILE NOT io.end_of_file( per ) LOOP  -- Restore saved state
      io.read( per, cur );
      add( the, cur.s_index, cur.s_data );
    END LOOP;
    io.close( per );
  EXCEPTION                               -- Return real exception
     WHEN OTHERS => RAISE Per_error;      --  as sub code
  END initialize;

  PROCEDURE finalize( the:IN OUT PIC ) IS
    per : io.File_type;    -- File descriptor
    PROCEDURE rec_finalize( the:IN Subtree ) IS -- Save state
    BEGIN
      IF the /= NULL THEN                   -- Subtree save as
        io.write( per, the.rec );           --  Item
        rec_finalize( the.left );           --  LHS
        rec_finalize( the.right );          --  RHS
      END IF;
    END rec_finalize;
  BEGIN
    IF to_string(the.obj_id) /= "" THEN     -- If save state
      io.create( per, io.OUT_FILE,
                 to_string( the.obj_id ) );
      rec_finalize( the.tree );
      io.close( per );
    END IF;
    release_storage( the.tree );
  EXCEPTION                              -- Return real exception
     WHEN OTHERS => RAISE Per_error;     --  as sub code
  END finalize;

  PROCEDURE discard( the:IN OUT PIC ) IS
  BEGIN
    set_name( the, "" );                 -- No name
    release_storage( the.tree );         -- Release storage
  END discard;

  PROCEDURE set_name( the:IN OUT PIC; id:IN String ) IS
  BEGIN
    the.obj_id := to_bounded_string(id); -- Set object name
  END set_name;

  FUNCTION  get_name( the:IN PIC ) RETURN String IS
  BEGIN
      RETURN to_string( the.obj_id );     -- Name of object
  END get_name;

  PROCEDURE add( the:IN OUT PIC; i:IN Index; d:IN Data ) IS
    PROCEDURE add_s(the:IN OUT Subtree; i:IN Index; d:IN Data) IS
    BEGIN
      IF the = NULL THEN
        the := NEW Leaf'( NULL, Element'(i,d), NULL );
      ELSE
        IF i = the.rec.s_index THEN     -- Index all ready exists
          RAISE Exists;
        ELSIF i > the.rec.s_index THEN  -- Try on RHS
          add_s( the.right, i, d );
        ELSE                            -- LHS
          add_s( the.left, i, d );
        END IF;
      END IF;
    END add_s;
  BEGIN
    add_s( the.tree, i, d );
  END add;

  PROCEDURE extract(the:IN OUT PIC; i:IN Index; d:IN OUT Data) IS
    node_is : Subtree;
  BEGIN
    node_is := find( the.tree, i );     -- Find node with iey
    d := node_is.rec.s_data;            -- return data
  END extract;

  PROCEDURE update(the:IN OUT PIC; i:IN Index; d:IN OUT Data) IS
    node_is : Subtree;
  BEGIN
    node_is := find( the.tree, i );     -- Find node with iey
    node_is.rec.s_data := d;            -- Update data
  END update;


  FUNCTION find( the:IN Subtree; i:IN Index) RETURN Subtree IS
  BEGIN
    IF the = NULL THEN RAISE Not_there; END IF;
    IF i = the.rec.s_index THEN
      RETURN the;                           -- Found
    ELSE
      IF i > the.rec.s_index
        THEN RETURN find( the.right, i );   -- Try RHS
        ELSE RETURN find( the.left,  i );   -- Try LHS
      END IF;
    END IF;
  END find;

  PROCEDURE dispose IS
    NEW unchecked_deallocation( Leaf, Subtree );

  PROCEDURE release_storage( the:IN OUT Subtree ) IS
  BEGIN
    IF the /= NULL THEN             -- Not empty
      release_storage( the.left );  -- Free LHS
      release_storage( the.right ); -- Free RHS
      dispose( the );               -- Dispose of item
    END IF;
    the := NULL;                    -- Subtree root NULL
  END release_storage;

END Class_pic;

--[pack_types.ads] Specification
-- Subtype as get demands a String
PACKAGE Pack_types150 IS
  SUBTYPE  Country IS String( 1 .. 12 );
  SUBTYPE  IDC     IS String( 1 .. 6 );
END Pack_types150;

----[gt.adb] Procedure
--WITH Pack_types;
--USE  Pack_types;
--FUNCTION gt(f:IN Country; s:IN Country ) RETURN Boolean IS
--BEGIN
--  RETURN f > s;
--END gt;

--[class_tel_list.ADS] Implementation Instantiation
WITH Class_pic, Pack_types150;
USE  Pack_types150;
  PRAGMA ELABORATE_ALL( Class_pic );
  PACKAGE Class_tel_list IS NEW
    Class_pic( Country, IDC, ">" );

--[pack_procedures.ads] Specification
PACKAGE Pack_procedures150 IS
  PROCEDURE ex1;
  PROCEDURE ex2;
  PROCEDURE ex3;
  PROCEDURE ex4;
END Pack_procedures150;

--[pack_procedures.adb] Implementation
WITH Simple_io, Pack_types150, Class_tel_list;
USE  Simple_io, Pack_types150, Class_tel_list;
PACKAGE BODY Pack_procedures150 IS

PROCEDURE ex1 IS
  tel_list : PIC;
BEGIN
  put("Creating Telephone list"); new_line;
  set_name( tel_list, "tel_list.per" );
  add( tel_list, "Canada      ", "+1    " );
  add( tel_list, "USA         ", "+1    " );
  add( tel_list, "Netherlands ", "+31   " );
  add( tel_list, "Belgium     ", "+32   " );
  add( tel_list, "France      ", "+33   " );
  add( tel_list, "Gibraltar   ", "+350  " );
  add( tel_list, "Ireland     ", "+353  " );
  add( tel_list, "Switzerland ", "+41   " );
  add( tel_list, "UK          ", "+44   " );
  add( tel_list, "Denmark     ", "+45   " );
  add( tel_list, "Norway      ", "+47   " );
  add( tel_list, "Germany     ", "+49   " );
  add( tel_list, "Australia   ", "+61   " );
  add( tel_list, "Japan       ", "+81   " );
END ex1;

PROCEDURE ex2 IS
  tel_list : PIC;
  action   : Character;
  name     : Country;
  tel      : IDC;
BEGIN
  initialize( tel_list, "tel_list.per" );
  WHILE NOT end_of_file LOOP
    BEGIN
      get( action );                       -- Action to perform
      CASE action IS
        WHEN '+' =>                        -- Add
          get( name ); get( tel );
          add( tel_list, name, tel );
        WHEN '=' =>                        -- Extract
          get( name );
          extract( tel_list, name, tel );
          put( "IDC for " ); put( name );
          put( " is "); put( tel ); new_line;
        WHEN '*' =>                        -- Update
          get( name ); get( tel );
          update( tel_list, name, tel );
        WHEN OTHERS =>                     -- Invalid action
          NULL;
      END CASE;
    EXCEPTION
      WHEN Not_there =>                    -- Not there
        put("Name not in directory"); new_line;
      WHEN Exists =>                       -- Exists
        put("Name already in directory"); new_line;
    END;
    skip_line;
  END LOOP;
END ex2;

PROCEDURE ex3 IS
  tel_list : PIC;
  name     : Country;
  tel      : IDC;
BEGIN
  add( tel_list, "Canada      ", "+1    " );
  add( tel_list, "USA         ", "+1    " );
  add( tel_list, "Netherlands ", "+31   " );
  add( tel_list, "Belgium     ", "+32   " );
  add( tel_list, "France      ", "+33   " );
  add( tel_list, "Gibraltar   ", "+350  " );
  name := "France      ";
  extract( tel_list, name, tel );
  put( "IDC for " ); put( name ); put( " is "); put( tel ); new_line;
  name := "USA         ";
  extract( tel_list, name, tel );
  put( "IDC for " ); put( name ); put( " is "); put( tel ); new_line;
END ex3;

PROCEDURE ex4 IS
  tel_list : PIC;
  name     : Country;
  tel      : IDC;
BEGIN
  add( tel_list, "Canada      ", "+1    " );
  put("Object identity is ["); put( get_name(tel_list) ); put("]"); new_line;
  set_name( tel_list, "tel_list.per" );
  put("Object identity is ["); put( get_name(tel_list) ); put("]"); new_line;
  discard( tel_list );
END ex4;

END Pack_procedures150;

--[main.adb] Procedure
WITH Simple_io, Pack_procedures150;
USE  Simple_io, Pack_procedures150;
PROCEDURE main150 IS
BEGIN
  put("Example 1"); new_line; ex1;
  put("Example 2"); new_line; ex2;
  put("Example 3"); new_line; ex3;
  put("Example 4"); new_line; ex4;
END main150;
