WITH Ada.Text_io;   -- **** FIXED
USE  Ada.Text_io;   -- **** FIXED
PACKAGE Class_line IS
  TYPE Line_status IS ( TEXT_LINE, FILE_NAME, UNKNOWN );
  TYPE Line IS PRIVATE;
  PROCEDURE get_line( the:IN OUT Line; fd:IN Ada.Text_io.File_type );
  PROCEDURE put_line( the:IN Line; fd:IN Ada.Text_io.File_type );
  PROCEDURE get_fd( the:IN OUT Line; fd:IN OUT Ada.Text_io.File_type );
  FUNCTION  status( the:IN Line ) RETURN Line_status;
PRIVATE
  MAX_LINE : CONSTANT := 200;
  SUBTYPE Line_index IS Integer RANGE 0 .. MAX_LINE+1;
  SUBTYPE Line_range IS Line_index RANGE 1 .. MAX_LINE;
  SUBTYPE Line_array IS String( Line_range );
  TYPE Line IS RECORD
    chs : Line_array;       -- Characters of line
    len : Line_index;       -- Positions used
    open: Boolean := FALSE; -- Output file open
  END RECORD;
  name     : Line_array;    -- File name from file
  name_pos : Line_index;    -- Characters in name
END Class_line;

WITH Ada.Text_io, Ada.Characters.handling;
USE  Ada.Text_io, Ada.Characters.handling;
PACKAGE BODY Class_line IS
  PROCEDURE get_line( the:IN OUT Line; fd:IN Ada.Text_io.File_type ) IS
    pos : Line_index := 0;
    ch  : Character;
  BEGIN
    WHILE NOT end_of_line( fd ) LOOP
      get( fd, ch );
      IF pos < MAX_LINE THEN
        pos := pos + 1;
        the.chs(pos) := ch;
      END IF;
    END LOOP;
    the.len := pos;
    skip_line( fd );
  END get_line;
     
  PROCEDURE put_line( the:IN Line; fd:IN Ada.Text_io.File_type ) IS
  BEGIN
    IF the.open THEN
      FOR i IN 2 .. the.len LOOP
        put( fd, the.chs(i) );
      END LOOP;
      new_line( fd );
    END IF;
  END put_line;

  FUNCTION status( the:IN Line ) RETURN Line_status IS
    pos  : Line_index := 0;
  BEGIN
    IF the.len >= 1 AND THEN the.chs(1) = '+' THEN 
      RETURN TEXT_LINE;
    END IF;
    IF the.len >= 2 AND THEN the.chs(1..2) = "@@" THEN
      FOR i IN 3 .. the.len-2 LOOP
        IF is_upper( the.chs(i) ) OR is_lower( the.chs(i) ) OR
           is_digit( the.chs(i) ) OR the.chs(i) = '_' OR 
           the.chs(i) = '.' THEN 
          pos := pos + 1;
          name(pos) := the.chs(i);
        END IF;
      END LOOP;
      name_pos := pos;
      put( "Extracting file " & name(1..pos) ); new_line;
      RETURN FILE_NAME;
    END IF;
    RETURN UNKNOWN;
  END status;
      
  PROCEDURE get_fd( the:IN OUT Line; fd:IN OUT Ada.Text_io.File_type ) IS
  BEGIN
    IF the.open THEN                    -- Output file open
      close( fd ); the.open := FALSE;
    END IF;
    create( File=>fd, Mode=>OUT_FILE,   -- Create file
            Name=>name(1..name_pos) );
    the.open := TRUE;
  EXCEPTION
    WHEN Name_error =>
      put("Exp: Can not create file " & name(1..name_pos) ); 
      new_line;
    WHEN Status_error =>
      put("Exp: " & name(1..name_pos) & " all ready open" );
      new_line;
    WHEN OTHERS =>
      put("Exp: " & name(1..name_pos) & " unknown error" );
      new_line;
  END;

END Class_line;
      
    
WITH Ada.Text_io, Ada.Command_line, Class_line; -- **** FIXED
USE  Ada.Text_io, Ada.Command_line, Class_line;
PROCEDURE main280 IS
  i_fd   : Ada.Text_io.File_type;        -- File descriptor
  o_fd   : Ada.Text_io.File_type;        -- File descriptor
  a_line : Class_line.Line;
BEGIN
  IF argument_count >= 1 THEN
    FOR i IN 1 .. argument_count LOOP    -- Repeat for each file
      BEGIN
        open( File=>i_fd, Mode=>IN_FILE, -- Open file
              Name=>argument(i) );
        WHILE NOT end_of_file(i_fd) LOOP -- For each Line
          get_line( a_line, i_fd );
          CASE status(a_line) IS
            WHEN TEXT_LINE =>            -- Write to file
              put_line( a_line, o_fd );
            WHEN FILE_NAME =>            -- Get file name
              get_fd( a_line, o_fd );    
            WHEN UNKNOWN =>              -- Ignore
              NULL;
          END CASE;
        END LOOP;
        close(i_fd);                     -- Close file
      EXCEPTION
        WHEN Name_error =>
          put("Exp: " & argument(i) & " no such file" );
          new_line;
        WHEN Status_error =>
          put("Exp: " & argument(i) & " all ready open" );
          new_line;
        WHEN OTHERS =>
          put("Exp: " & argument(i) & " unknow error" );
          new_line;
      END;
    END LOOP;
  ELSE
    put("Usage: Exp file1 ... "); new_line;
  END IF;
END main280;
