with Ada.Unchecked_Conversion;
with Grafmenu;
with Interfaces.C.Strings;
with System;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;
--  use type Win32.Uint; 

   Display_Text          : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "DISPLAY");

   Help_Text             : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "Help not implimented yet");

   File_Menu_Text        : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "MenuFile");

   Edit_Menu_Text        : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "MenuEdit");

   File_Bitmap_Text      : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "BitmapFile");

   Edit_Bitmap_Text        : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "BitmapEdit");

   Font_Bitmap_Text        : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "BitmapFont");

   Help_Bitmap_Text        : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "BitmapHelp");

   Bitmap_Pop_Font_Handle_Array : array ( 0 .. 2) of Win32.Windef.Hbitmap;

   Face_Name_Array : constant array ( 0 .. 2 ) of Interfaces.C.Strings.Chars_Ptr :=
     (0 =>
       Interfaces.C.Strings.New_String
         (Str =>
           "Courier New"),
      1 =>
       Interfaces.C.Strings.New_String
         (Str =>
           "Arial"),
      2 =>
       Interfaces.C.Strings.New_String
         (Str =>
           "Times New Roman"));

   Current_Font          : Win32.Int := Grafmenu.Idm_Cour;

   Menu_Handle           : Win32.Windef.Hmenu;

   Popup_Menu_Handle     : Win32.Windef.Hmenu;

   Help_Bitmap_Handle    : Win32.Windef.Hbitmap; 

   File_Bitmap_Handle    : Win32.Windef.Hbitmap; 

   Edit_Bitmap_Handle    : Win32.Windef.Hbitmap; 

   Font_Bitmap_Handle    : Win32.Windef.Hbitmap;

   Size                  : aliased Win32.Windef.Size; 

   False                 : constant Win32.Bool := 0;

   Window_Class          : aliased Win32.Winuser.Wndclass;

   Message               : aliased Win32.Winuser.Msg;

   Log_Font              : aliased Win32.Wingdi.Logfont; 

   Integer_Result        : Win32.Int;
     
   Dword_Result          : Win32.Dword;

   Uint_Result           : Win32.Uint;

   Bool_Result           : Win32.Bool;
                     
   Long_Result           : Win32.Long;

   Hgdiobj_Result        : Win32.Windef.Hgdiobj;

   Atom_Result           : Win32.Windef.Atom;

   function Convert_Chars_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Integer_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Lpcstr);

   function Convert_Integer_To_Ac_Devmodea_T is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Wingdi.Ac_Devmodea_T);

   function Convert_Hbrush_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hbrush,
        Target => Win32.Lpcstr);

   function Convert_Hbitmap_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hbitmap,
        Target => Win32.Lpcstr);

   function Convert_Hmenu_To_Uint is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hmenu,
        Target => Win32.Uint);

   function Convert_Address_To_Lpvoid is new
     Ada.Unchecked_Conversion
       (Source => System.Address,
        Target => Win32.Lpvoid);

   function Convert_Integer_To_Lpvoid is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Lpvoid);

   function Convert_Address_To_Ac_Bitmap_T is new
     Ada.Unchecked_Conversion
       (Source => System.Address,
        Target => Win32.Wingdi.Ac_Bitmap_T);

   function Convert_Address_To_Ac_Textmetric_T is new
     Ada.Unchecked_Conversion
       (Source => System.Address,
        Target => Win32.Wingdi.Ac_Textmetric_T);

   function Convert_Hbitmap_To_Hgdiobj is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hbitmap,
        Target => Win32.Windef.Hgdiobj);

   function Convert_Character_To_Char is new
     Ada.Unchecked_Conversion
       (Source => Character,
        Target => Win32.Char);

   function Convert_Hfont_To_Integer is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hfont,
        Target => Integer);

   function Stretch_Bitmap
     (Bitmap_Handle : Win32.Windef.Hbitmap) return Win32.Windef.Hbitmap is

     Bitmap_One                       : Win32.Wingdi.Bitmap;

     Bitmap_Two                       : Win32.Wingdi.Bitmap;

     Bitmap_Two_Handle                : Win32.Windef.Hbitmap;

     Device_Context_Handle            : Win32.Windef.Hdc;

     Memory_One_Device_Context_Handle : Win32.Windef.Hdc;

     Memory_Two_Device_Context_Handle : Win32.Windef.Hdc;

     Text_Metric                      : Win32.Wingdi.Textmetric;

   begin

     Device_Context_Handle :=
       Win32.Wingdi.Createic
         (Lpszdriver  =>
           Convert_Chars_Ptr_To_Lpcstr
             (S =>
               Display_Text),
          Lpszdevice  =>
           Convert_Integer_To_Lpcstr
             (S =>
               0),
          Lpszoutput  =>
           Convert_Integer_To_Lpcstr
             (S =>
               0),
          Lpvinitdata =>
           Convert_Integer_To_Ac_Devmodea_T
             (S =>
               0));

     Bool_Result :=
       Win32.Wingdi.Gettextmetrics
         (Hdc  =>
           Device_Context_Handle,
          Lptm =>
           Convert_Address_To_Ac_Textmetric_T
             (S => Text_Metric'Address));

     Memory_One_Device_Context_Handle :=
       Win32.Wingdi.Createcompatibledc
         (Hdc =>
           Device_Context_Handle);

     Memory_Two_Device_Context_Handle :=
       Win32.Wingdi.Createcompatibledc
         (Hdc =>
           Device_Context_Handle);

     Bool_Result :=
       Win32.Wingdi.Deletedc
         (Hdc =>
           Device_Context_Handle);

     Integer_Result :=
       Win32.Wingdi.Getobject 
         (Hgdiobj =>
           Convert_Hbitmap_To_Hgdiobj
             (S =>
               Bitmap_Handle),
          Cbbuffer =>
           Win32.Int
             (Win32.Wingdi.Bitmap'Size / 8),
          Lpvobject =>
           Convert_Address_To_Lpvoid
             (S =>
               Bitmap_One'Address));

     Bitmap_Two :=
       Bitmap_One;

     Bitmap_Two.Bmwidth :=
       (Text_Metric.Tmavecharwidth * Bitmap_Two.Bmwidth) / 4;

     Bitmap_Two.Bmheight :=
       (Text_Metric.Tmheight * Bitmap_Two.Bmheight) / 8;

     Bitmap_Two.Bmwidthbytes :=
       ((Bitmap_Two.Bmwidthbytes + 15) / 16) * 2;

     Bitmap_Two_Handle :=
       Win32.Wingdi.Createbitmapindirect
         (Lpbm =>
           Convert_Address_To_Ac_Bitmap_T
             (S =>
               Bitmap_Two'Address));

     Hgdiobj_Result :=
       Win32.Wingdi.Selectobject
         (Hdc    => 
           Memory_One_Device_Context_Handle,
          Hgdiobj =>
           Bitmap_Handle);

     Hgdiobj_Result :=
       Win32.Wingdi.Selectobject
         (Hdc    => 
           Memory_Two_Device_Context_Handle,
          Hgdiobj =>
           Bitmap_Two_Handle);

     Bool_Result :=
       Win32.Wingdi.Stretchblt
         (Hdcdest     => 
           Memory_Two_Device_Context_Handle,
          Nxorigindes =>
           0,
          Nyorigindes =>
           0,
          Nwidthdest  =>
           Win32.Int
             (Bitmap_Two.Bmwidth),
          Nheightdest =>
           Win32.Int
             (Bitmap_Two.Bmheight),
          Hdcsrc      =>
           Memory_One_Device_Context_Handle,
          Nxoriginsrc =>
           0,
          Nyoriginsrc =>
           0,
          Nwidthsrc   =>
           Win32.Int
             (Bitmap_One.Bmwidth),
          Nheightsrc  =>
           Win32.Int
             (Bitmap_One.Bmheight),
          Dwrop       => 
           Win32.Wingdi.Srccopy);

     Bool_Result :=
       Win32.Wingdi.Deletedc
         (Hdc =>
           Memory_One_Device_Context_Handle);

     Bool_Result :=
       Win32.Wingdi.Deletedc
         (Hdc =>
           Memory_Two_Device_Context_Handle);

     Bool_Result :=
       Win32.Wingdi.Deleteobject
         (Hgdiobj =>
           Bitmap_Handle);

     return Bitmap_Two_Handle;

   end;

   function Get_Bitmap_Font
     (Index : Integer) return Win32.Windef.Hbitmap is

     Bitmap_Handle                : Win32.Windef.Hbitmap;

     Font_Handle                  : Win32.Windef.Hfont;

     Device_Context_Handle        : Win32.Windef.Hdc;

     Memory_Device_Context_Handle : Win32.Windef.Hdc;

     Text_Metric                  : Win32.Wingdi.Textmetric;

     Size                         : aliased Win32.Windef.Size; 

   begin

     Device_Context_Handle :=
       Win32.Wingdi.Createic
         (Lpszdriver  =>
           Convert_Chars_Ptr_To_Lpcstr
             (S =>
               Display_Text),
          Lpszdevice  =>
           Convert_Integer_To_Lpcstr
             (S =>
               0),
          Lpszoutput  =>
           Convert_Integer_To_Lpcstr
             (S =>
               0),
          Lpvinitdata =>
           Convert_Integer_To_Ac_Devmodea_T
             (S =>
               0));

     Bool_Result :=
       Win32.Wingdi.Gettextmetrics
         (Hdc  =>
           Device_Context_Handle,
          Lptm =>
           Convert_Address_To_Ac_Textmetric_T
             (S =>
               Text_Metric'Address));

     Log_Font.Lfheight :=
       2 * Text_Metric.Tmheight;


     for i in 1 .. integer(Interfaces.C.Strings.Strlen(Item => Face_Name_Array(Index))) + 1  loop

       if i <=  integer(Interfaces.C.Strings.Strlen(Item => Face_Name_Array(Index))) then

         Log_Font.lffacename(i - 1) :=
           Convert_Character_To_Char
             (S =>
               Interfaces.C.Strings.Value
                 (Item =>
                   Face_Name_Array(Index))(i));

       else

         Log_Font.lffacename(i - 1) :=
           Convert_Character_To_Char
             (S =>
               Character'Val(0));

       end if;

     end loop;

     Memory_Device_Context_Handle :=
       Win32.Wingdi.Createcompatibledc
         (Hdc =>
           Device_Context_Handle);

     Font_Handle :=
       Win32.Wingdi.Selectobject
         (Hdc     =>
           Memory_Device_Context_Handle,
          hgdiobj =>
           Win32.Wingdi.Createfontindirect
             (Lplf =>
               Convert_Address_To_Ac_Textmetric_T
                 (S =>
                   Log_Font'Address)));
     
     Bool_Result :=
       Win32.Wingdi.Gettextextentpoint
         (Hdc =>
           Memory_Device_Context_Handle,
          Lpstring =>
           Convert_Chars_Ptr_To_Lpcstr
             (S =>
               Face_Name_Array(Index)),
          Cbstring =>
           Win32.Int
             (Interfaces.C.Strings.Strlen
               (Item =>
                 Face_Name_Array(Index))),
          Lpsize =>
           Size'Unchecked_Access);

     Bitmap_Handle :=
       Win32.Wingdi.Createbitmap
         (Nwidth   =>
           Win32.Int
             (Size.Cx),
          Nheight  =>
           Win32.Int
             (Size.Cy),
          Cbplanes =>
           1,
          Cbbits   =>
           1,
          Lpvbits  =>
           Convert_Integer_To_Lpvoid
             (S =>
               0));

     Hgdiobj_Result :=
       Win32.Wingdi.Selectobject
         (Hdc     =>
           Memory_Device_Context_Handle,
          hgdiobj =>
           Bitmap_Handle);
    
     Bool_Result := 
       Win32.Wingdi.Textout
	 (Hdc   =>
           Memory_Device_Context_Handle,
          Nxstart  => 
           0,
          Nystart  => 
           0,
          Lpstring => 
           Convert_Chars_Ptr_To_Lpcstr
             (S => 
               Face_Name_Array(Index)),
          Cbstring => 
           Win32.Int
             (Interfaces.C.Strings.Strlen
               (Item => 
                 Face_Name_Array(Index))));

     Bool_Result :=
       Win32.Wingdi.Deleteobject
         (Hgdiobj =>
           Win32.Wingdi.Selectobject
             (Hdc     =>
               Memory_Device_Context_Handle,
              Hgdiobj =>
               Font_Handle));

     Bool_Result :=
       Win32.Wingdi.Deletedc
         (Hdc =>
           Memory_Device_Context_Handle);

     Bool_Result :=
       Win32.Wingdi.Deletedc
         (Hdc =>
           Device_Context_Handle);

     return Bitmap_Handle;

   end;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

           Dword_Result :=
             Win32.Winuser.Checkmenuitem 
               (Hmenu        =>
                 Win32.Winuser.Getmenu
                   (Hwnd =>
                     Window_Handle),
                Uidcheckitem =>
                 Win32.Uint
                   (Current_Font),
                Ucheck       =>
                 Win32.Winuser.Mf_Checked);

 	   return (0);

	 when Win32.Winuser.Wm_Syscommand =>

           case
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Wparam))) is

             when
               Grafmenu.Idm_Help            =>

               Integer_Result :=
                 Win32.Winuser.MessageBox
                   (Hwnd      => 
                     Window_Handle,
                    Lptext    => 
                     Convert_Chars_Ptr_To_Lpcstr
                       (S => 
                         Help_Text),
                    Lpcaption =>
                     Convert_Chars_Ptr_To_Lpcstr
                       (S => 
                         Windows.Data.Class_Name),
                    Utype     =>
                     Win32.Uint
                       (Win32.Winuser.Mb_Iconexclamation or
                        Win32.Winuser.Mb_Ok));

             when
               others              =>
             
               return
                 Win32.Winuser.Defwindowproc
	           (Hwnd   =>
                     Window_Handle, 
	            Msg    => 
                     Message, 
        	    Wparam => 
                     Wparam,
	            Lparam => 
                     Lparam);

           end case;
           
 	   return (0);

	 when Win32.Winuser.Wm_Command =>

           case
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Wparam))) is

             when
               Grafmenu.Idm_New    |
               Grafmenu.Idm_Open   |
               Grafmenu.Idm_Save   |
               Grafmenu.Idm_Saveas |
               Grafmenu.Idm_Undo   |
               Grafmenu.Idm_Cut    |
               Grafmenu.Idm_Copy   |
               Grafmenu.Idm_Paste  |
               Grafmenu.Idm_Del    =>

               Bool_Result :=
                 Win32.Winuser.Messagebeep
                   (Utype =>
                     0);

             when
               Grafmenu.Idm_Cour   |
               Grafmenu.Idm_Arial  |
               Grafmenu.Idm_Times  =>

               Menu_Handle :=
                 Win32.Winuser.Getmenu
                   (Hwnd =>
                     Window_Handle);

               Dword_Result :=
                 Win32.Winuser.Checkmenuitem
                   (Hmenu =>
                     Menu_Handle,
                    Uidcheckitem =>
                     Win32.Uint
                       (Current_Font),
                    Ucheck =>
                     Win32.Winuser.MF_Unchecked);

               Current_Font :=
                 Win32.Int
                   (Win32.Utils.Loword
                     (Win32.Dword
                       (Wparam)));

               Dword_Result :=
                 Win32.Winuser.Checkmenuitem
                   (Hmenu =>
                     Menu_Handle,
                    Uidcheckitem =>
                     Win32.Uint
                       (Current_Font),
                    Ucheck =>
                     Win32.Winuser.MF_Checked);

             when
               others              =>

         	 return
                   Win32.Winuser.Defwindowproc
	             (Hwnd   =>
                       Window_Handle, 
	              Msg    => 
                       Message, 
	              Wparam => 
                       Wparam,
	              Lparam => 
                       Lparam);

           end case;
           
 	   return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Menu_Handle :=
        Win32.Winuser.Createmenu;

      Popup_Menu_Handle :=
        Win32.Winuser.Loadmenu
          (Hinstance  =>
            Instance,
           Lpmenuname =>
            Convert_Chars_Ptr_To_Lpcstr
              (S =>
                File_Menu_Text));

      File_Bitmap_Handle :=
        Stretch_Bitmap
          (Bitmap_Handle =>
            Win32.Winuser.Loadbitmap
              (Hinstance =>
                Instance,
               Lpbitmapname =>
                Convert_Chars_Ptr_To_Lpcstr
                  (S =>
                    File_Bitmap_Text)));

      Bool_Result :=
        Win32.Winuser.Appendmenu
          (Hmenu      =>
            Menu_Handle,
           Uflags     =>
            Win32.Uint
              (Win32.Winuser.Mf_Bitmap or
               Win32.Winuser.Mf_Popup),
           Uidnewitem =>
             Convert_Hmenu_To_Uint
               (S =>
                 Popup_Menu_Handle),
           Lpnewitem  =>
            Convert_Hbitmap_To_Lpcstr
              (S =>
                File_Bitmap_Handle));

      Popup_Menu_Handle :=
        Win32.Winuser.Loadmenu
          (Hinstance  =>
            Instance,
           Lpmenuname =>
            Convert_Chars_Ptr_To_Lpcstr
              (S =>
                Edit_Menu_Text));

      Edit_Bitmap_Handle :=
        Stretch_Bitmap
          (Bitmap_Handle =>
            Win32.Winuser.Loadbitmap
              (Hinstance =>
                Instance,
               Lpbitmapname =>
                Convert_Chars_Ptr_To_Lpcstr
                  (S =>
                    Edit_Bitmap_Text)));
      Bool_Result :=
        Win32.Winuser.Appendmenu
          (Hmenu      =>
            Menu_Handle,
           Uflags     =>
            Win32.Uint
              (Win32.Winuser.Mf_Bitmap or
               Win32.Winuser.Mf_Popup),
           Uidnewitem =>
            Convert_Hmenu_To_Uint
              (Popup_Menu_Handle),
           Lpnewitem  =>
            Convert_Hbitmap_To_Lpcstr
              (S =>
                Edit_Bitmap_Handle));

      Popup_Menu_Handle :=
        Win32.Winuser.Createmenu;

      for i in 0 .. 2 loop

        Bitmap_Pop_Font_Handle_Array(i) :=
          Get_Bitmap_Font
            (Index       =>
              i);

        Bool_Result :=
          Win32.Winuser.Appendmenu
            (Hmenu      =>
              Popup_Menu_Handle,
             Uflags     =>
              Win32.Winuser.Mf_Bitmap,
             Uidnewitem =>
               Win32.Uint
                 (Grafmenu.Idm_Cour + Win32.Int(i)),
             Lpnewitem  =>
              Convert_Hbitmap_To_Lpcstr
                (S =>
                  Bitmap_Pop_Font_Handle_Array(i)));
        
      end loop;

      Font_Bitmap_Handle :=
        Stretch_Bitmap
          (Bitmap_Handle =>
            Win32.Winuser.Loadbitmap
              (Hinstance =>
                Instance,
               Lpbitmapname =>
                Convert_Chars_Ptr_To_Lpcstr
                  (S =>
                    Font_Bitmap_Text)));

      Bool_Result :=
        Win32.Winuser.Appendmenu
          (Hmenu      =>
            Menu_Handle,
           Uflags     =>
            Win32.Uint
              (Win32.Winuser.Mf_Bitmap or
               Win32.Winuser.Mf_Popup),
           Uidnewitem =>
            Convert_Hmenu_To_Uint
              (Popup_Menu_Handle),
           Lpnewitem  =>
            Convert_Hbitmap_To_Lpcstr
              (S =>
                Font_Bitmap_Handle));

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Menu_Handle,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

      Menu_Handle :=
        Win32.Winuser.Getsystemmenu
          (Hwnd =>
            Windows.Data.Window_Handle,
           Brevert =>
            False);

      Help_Bitmap_Handle :=
        Stretch_Bitmap
          (Bitmap_Handle =>
            Win32.Winuser.Loadbitmap
              (Hinstance =>
                Instance,
               Lpbitmapname =>
                Convert_Chars_Ptr_To_Lpcstr
                  (S =>
                    Help_Bitmap_Text)));
      Bool_Result :=
        Win32.Winuser.Appendmenu
          (Hmenu      =>
            Menu_Handle,
           Uflags     =>
            Win32.Winuser.Mf_Separator,
           Uidnewitem =>
            0,
           Lpnewitem  =>
            Convert_Integer_To_Lpcstr
              (S =>
                0));

      Bool_Result :=
        Win32.Winuser.Appendmenu
          (Hmenu      =>
            Menu_Handle,
           Uflags     =>
            Win32.Winuser.Mf_Bitmap,
           Uidnewitem =>
            Win32.Uint
              (Grafmenu.Idm_Help),
           Lpnewitem  =>
            Convert_Hbitmap_To_Lpcstr
              (S =>
                Help_Bitmap_Handle));

     Bool_Result := 
       Win32.Winuser.Showwindow
         (Hwnd     => 
           Windows.Data.Window_Handle, 
	  Ncmdshow =>
           Win32.Winmain.Get_Ncmdshow);

     Bool_Result := 
       Win32.Winuser.Updatewindow
         (Hwnd => 
           Windows.Data.Window_Handle);

     Windows.Routines.Main_Windows_Message_Loop;

     Bool_Result :=
       Win32.Wingdi.Deleteobject
         (Hgdiobj =>
           Help_Bitmap_Handle);

     Bool_Result :=
       Win32.Wingdi.Deleteobject
         (Hgdiobj =>
           Edit_Bitmap_Handle);

     Bool_Result :=
       Win32.Wingdi.Deleteobject
         (Hgdiobj =>
           File_Bitmap_Handle);

     Bool_Result :=
       Win32.Wingdi.Deleteobject
         (Hgdiobj =>
           Font_Bitmap_Handle);

     for i in 0 .. 2 loop

       Bool_Result :=
         Win32.Wingdi.Deleteobject
           (Hgdiobj =>
             Bitmap_Pop_Font_Handle_Array(i));

     end loop;
 
     return Message.Wparam;

  end Main;

end Windows.Routines;
