with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with Nopopups;
with System;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;
--  use type Win32.Uint; 

   Main_Menu_Text        : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "MenuMain");

   File_Menu_Text        : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "MenuFile");

   Edit_Menu_Text        : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "MenuEdit");

   Main_Menu_Handle      : Win32.Windef.Hmenu;

   File_Menu_Handle      : Win32.Windef.Hmenu;

   Edit_Menu_Handle      : Win32.Windef.Hmenu;

   Window_Class          : aliased Win32.Winuser.Wndclass;

   Message               : aliased Win32.Winuser.Msg;

   Instance              : Win32.Windef.Hinstance;

   Integer_Result        : Win32.Int;
     
   Dword_Result          : Win32.Dword;

   Uint_Result           : Win32.Uint;

   Bool_Result           : Win32.Bool;
                     
   Long_Result           : Win32.Long;

   Atom_Result           : Win32.Windef.Atom;

   function Convert_Chars_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Long_To_Hinstance is new
     Ada.Unchecked_Conversion
       (Source => Win32.Long,
        Target => Win32.Windef.Hinstance);

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

           Instance :=
             Convert_Long_To_HInstance
               (S =>
                 Win32.Winuser.Getwindowlong
                   (Hwnd   =>
                     Window_Handle,
                    Nindex =>
                     Win32.Winuser.Gwl_Hinstance));

           Main_Menu_Handle :=
             Win32.Winuser.Loadmenu
               (Hinstance  =>
                 Instance,
                Lpmenuname =>
                 Convert_Chars_Ptr_To_Lpcstr
                   (S =>
                     Main_Menu_Text));

           File_Menu_Handle :=
             Win32.Winuser.Loadmenu
               (Hinstance  =>
                 Instance,
                Lpmenuname =>
                 Convert_Chars_Ptr_To_Lpcstr
                   (S =>
                     File_Menu_Text));

           Edit_Menu_Handle :=
             Win32.Winuser.Loadmenu
               (Hinstance  =>
                 Instance,
                Lpmenuname =>
                 Convert_Chars_Ptr_To_Lpcstr
                   (S =>
                     Edit_Menu_Text));

           Bool_Result :=
             Win32.Winuser.Setmenu
               (Hwnd  =>
                 Window_Handle,
                Hmenu =>
                 Main_Menu_Handle);

 	   return (0);

	 when Win32.Winuser.Wm_Command =>

           case
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Wparam))) is

             when
               Nopopups.Idm_Main   =>

               Bool_Result :=
                 Win32.Winuser.Setmenu
                   (Hwnd  =>
                     Window_Handle,
                    Hmenu =>
                     Main_Menu_Handle);

               return (0);

             when
               Nopopups.Idm_File   =>

               Bool_Result :=
                 Win32.Winuser.Setmenu
                   (Hwnd  =>
                     Window_Handle,
                    Hmenu =>
                     File_Menu_Handle);

             when
               Nopopups.Idm_Edit   =>

               Bool_Result :=
                 Win32.Winuser.Setmenu
                   (Hwnd  =>
                     Window_Handle,
                    Hmenu =>
                     Edit_Menu_Handle);

             when
               Nopopups.Idm_New    |
               Nopopups.Idm_Open   |
               Nopopups.Idm_Save   |
               Nopopups.Idm_Saveas |
               Nopopups.Idm_Undo   |
               Nopopups.Idm_Cut    |
               Nopopups.Idm_Copy   |
               Nopopups.Idm_Paste  |
               Nopopups.Idm_Del    =>

               Bool_Result :=
                 Win32.Winuser.Messagebeep
                   (Utype =>
                     0);

             when
               others              =>

         	 return
                   Win32.Winuser.Defwindowproc
	             (Hwnd   =>
                       Window_Handle, 
	              Msg    => 
                       Message, 
	              Wparam => 
                       Wparam,
	              Lparam => 
                       Lparam);

           end case;
           
 	   return (0);

	 when Win32.WinUser.Wm_Destroy =>

            Bool_Result :=
              Win32.Winuser.Setmenu
                (Hwnd  =>
                  Window_Handle,
                 Hmenu =>
                  Edit_Menu_Handle);

            Bool_Result :=
              Win32.Winuser.Destroymenu
                (Hmenu =>
                  File_Menu_Handle);

            Bool_Result :=
              Win32.Winuser.Destroymenu
                (Hmenu =>
                  Edit_Menu_Handle);

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
