with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with Popmenu;
with System;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;
--  use type Win32.Uint; 

   Help_Message_Text     : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "Help not yet implimented");

   True                  : constant Win32.Bool := 1;

   About_Message_Text     : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "Popup Menu Demonstration Program");

   Menu_Handle           : Win32.Windef.Hmenu;

   Paint_Record          : aliased win32.Winuser.Paintstruct;

   Window_Class          : aliased Win32.Winuser.Wndclass;

   Color                 : constant array(0 .. 4) of Win32.Int :=
     (0 =>
       Win32.Wingdi.White_Brush,
      1 => 
       Win32.Wingdi.Ltgray_Brush,
      2 => 
       Win32.Wingdi.Gray_Brush,
      3 => 
       Win32.Wingdi.Dkgray_Brush,
      4 => 
       Win32.Wingdi.Black_Brush); 

   Device_Context_Handle : aliased Win32.Windef.Hdc;

   Message               : aliased Win32.Winuser.Msg;

   Point                 : aliased Win32.Windef.Point;

   Selection             : Win32.Int := Popmenu.Idm_White;

   Integer_Result        : Win32.Int;
     
   Dword_Result          : Win32.Dword;

   Uint_Result           : Win32.Uint;

   Bool_Result           : Win32.Bool;
                     
   Long_Result           : Win32.Long;

   Atom_Result           : Win32.Windef.Atom;

   function Convert_Hicon_To_Integer is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hbrush,
        Target => Integer);

   function Convert_Integer_To_Ac_Rect_T is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Wingdi.Ac_Rect_T);

   function Convert_Hinstance_To_Integer is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hbrush,
        Target => Integer);

   function Convert_Hgdiobj_To_Long is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hgdiobj,
        Target => Win32.Long);

   function Convert_Chars_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

           Menu_Handle :=
             Win32.Winuser.Loadmenu
               (Hinstance  =>
                 Window_Class.Hinstance,
                Lpmenuname =>
                 Convert_Chars_Ptr_To_Lpcstr
                   (S =>
                     Windows.Data.Class_Name));

           Menu_Handle :=
             Win32.Winuser.Getsubmenu
               (Hmenu =>
                 Menu_Handle,
                Npos  =>
                 0);

 	   return (0);

	 when Win32.Winuser.Wm_Rbuttondown =>

           Point.X :=
             Win32.Long
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)));

           Point.Y :=
             Win32.Long
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam)));

           Bool_Result :=
             Win32.Winuser.Clienttoscreen
               (Hwnd    =>
                 Window_Handle,
                Lppoint =>
                 Point'Access);

           Bool_Result :=
             Win32.Winuser.Trackpopupmenu
               (Hmenu     =>
                 Menu_Handle,
                Uflags    =>
                 0,
                X         =>
                 Win32.Int
                   (Point.X),
                Y         =>
                 Win32.Int
                   (Point.Y),
                Nreserved =>
                 0,
                Hwnd      =>
                 Window_Handle,
                Prcrect   =>
                 Convert_Integer_To_Ac_Rect_T
                   (S =>
                     0));
  
 	   return (0);

	 when Win32.Winuser.Wm_Command =>

           case
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Wparam))) is

             when
               Popmenu.IDM_New    |
               Popmenu.IDM_Open   |
               Popmenu.IDM_Save   |
               Popmenu.IDM_Saveas |
               Popmenu.IDM_Undo   |
               Popmenu.IDM_Cut    |
               Popmenu.IDM_Copy   |
               Popmenu.IDM_Paste  |
               Popmenu.IDM_Del    =>

               Bool_Result :=
                 Win32.Winuser.Messagebeep
                   (Utype =>
                     0);

             when
               Popmenu.IDM_Exit   =>

               Long_Result :=
                 Win32.Winuser.Sendmessage
                   (Hwnd   =>
                     Window_Handle,
                    Msg    =>
                     Win32.Winuser.Wm_Close,
                    Lparam =>
                     0,
                    Wparam =>
                     0);

             when
               Popmenu.IDM_White  |
               Popmenu.IDM_Ltgray |
               Popmenu.IDM_Gray   |
               Popmenu.IDM_Dkgray |
               Popmenu.IDM_Black  =>

               Dword_Result :=
                 Win32.Winuser.Checkmenuitem
                   (Hmenu =>
                     Menu_Handle,
                    Uidcheckitem =>
                     Win32.Uint
                       (Selection),
                    Ucheck =>
                     Win32.Winuser.MF_Unchecked);

               Selection :=
                 Win32.Int
                   (Win32.Utils.Loword
                     (Win32.Dword
                       (Wparam)));

               Dword_Result :=
                 Win32.Winuser.Checkmenuitem
                   (Hmenu =>
                     Menu_Handle,
                    Uidcheckitem =>
                     Win32.Uint
                       (Selection),
                    Ucheck =>
                     Win32.Winuser.MF_Checked);

                 Dword_Result :=
                   Win32.Winuser.Setclasslong
                     (Hwnd      =>
                       Window_Handle,
                      Nindex    =>
                       Win32.Winuser.Gcl_Hbrbackground,
                      Dwnewlong =>
                       Convert_Hgdiobj_To_Long
                         (Win32.Wingdi.Getstockobject
                            (Fnobject =>
                              Color
                                ((Integer
                                  (Win32.Utils.Loword
                                     (Win32.Dword
                                        (Wparam)))) -
                                Integer
                                  (Popmenu.Idm_White))))); 

               Bool_Result :=
                 Win32.Winuser.Invalidaterect
                   (Hwnd   =>
                     Window_Handle,
                    Lprect =>
                     Convert_Integer_To_Ac_Rect_T
                       (S =>
                         0),
                    Berase => 
                     True);

             when
               Popmenu.Idm_Help   =>

               Integer_Result :=
                 Win32.Winuser.MessageBox
                   (Hwnd      => 
                     Window_Handle,
                    Lptext    => 
                     Convert_Chars_Ptr_To_Lpcstr
                       (S => 
                         Help_Message_Text),
                    Lpcaption =>
                     Convert_Chars_Ptr_To_Lpcstr
                       (S => 
                         Windows.Data.Class_Name),
                    Utype     =>
                     Win32.Uint
                       (Win32.Winuser.Mb_Iconexclamation or
                        Win32.Winuser.Mb_Ok));

             when
               Popmenu.Idm_About   =>

               Integer_Result :=
                 Win32.Winuser.MessageBox
                   (Hwnd      => 
                     Window_Handle,
                    Lptext    => 
                     Convert_Chars_Ptr_To_Lpcstr
                       (S => 
                         About_Message_Text),
                    Lpcaption =>
                     Convert_Chars_Ptr_To_Lpcstr
                       (S => 
                         Windows.Data.Class_Name),
                    Utype     =>
                     Win32.Uint
                       (Win32.Winuser.Mb_Iconexclamation or
                        Win32.Winuser.Mb_Ok));

             when
               others              =>

         	 return
                   Win32.Winuser.Defwindowproc
	             (Hwnd   =>
                       Window_Handle, 
	              Msg    => 
                       Message, 
	              Wparam => 
                       Wparam,
	              Lparam => 
                       Lparam);

           end case;
           
 	   return (0);

	 when Win32.Winuser.Wm_Timer =>

           Bool_Result :=
             Win32.Winuser.Messagebeep
               (Utype =>
                 0);

	   return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
