with Interfaces.C.Strings;
With Ada.Unchecked_Conversion;
with Win32.Wingdi;
with Win32.Utils;
with Windows.Data;
package body Windows.Child_Routines is
  use type Win32.Int;
  use type Win32.Uint;
  use type Win32.Wparam;

  True                        : constant Win32.Bool := 1;

  False                       : constant Win32.Bool := 0;

  Rectangle                   : aliased Win32.Windef.Rect;

  Bool_Result                 : Win32.Bool;

  Integer_Result              : Win32.Int;

  Window_Handle_Result        : Win32.Windef.Hwnd;

  Device_Context_Handle       : Win32.Windef.Hdc;

  Brush_Handle                : Win32.Windef.Hbrush;

  Paint_Record                : aliased Win32.Winuser.Paintstruct;

  Colorref_Result             : Win32.Windef.Colorref;

  Long_Result                 : Win32.Long;

  function About_Dialog
    (Dialog_Handle : Win32.Windef.Hwnd;
     Message       : Win32.Uint;
     Wparam        : Win32.Wparam;
     Lparam        : Win32.Lparam) return Win32.Bool is
  begin

    case
      Message is

      when
        Win32.Winuser.Wm_Initdialog =>
       
        return True;

      when
        Win32.Winuser.Wm_Command    =>

        case
          Win32.Int
            (Win32.Utils.Loword
              (Win32.Dword
                (Wparam))) is
         
          when
            Win32.Winuser.Idok     =>
       
            Bool_Result :=
              Win32.Winuser.Enddialog
                (Hdlg    =>
                  Dialog_Handle,
                 Nresult =>
                  1);

            return True;

          when
            others =>

            return False;

        end case;

      when
        others =>

        return False;

    end case;      

  end About_Dialog;

  function Ellipse_Push
    (Window_Handle : Win32.Windef.Hwnd;
     Message       : Win32.Uint;
     Wparam        : Win32.Wparam;
     Lparam        : Win32.Lparam) return Win32.Lresult is

     Text_String   : Interfaces.C.Strings.Chars_Ptr :=
       Interfaces.C.Strings.New_String
         (Str =>
           "                                         ");
     function Convert_Chars_Ptr_To_Lpcstr is new
       Ada.Unchecked_Conversion
         (Source =>
           Interfaces.C.Strings.Chars_Ptr,
          Target =>
           Win32.Lpcstr);

     function Convert_Chars_Ptr_To_Lpstr is new
       Ada.Unchecked_Conversion
         (Source =>
           Interfaces.C.Strings.Chars_Ptr,
          Target =>
           Win32.Lpstr);

     function Convert_Long_To_Wparam is new
       Ada.Unchecked_Conversion
         (Source =>
           Win32.Long,
          Target =>
           Win32.Wparam);

     function Convert_Hwnd_To_Lparam is new
       Ada.Unchecked_Conversion
         (Source =>
           Win32.Windef.Hwnd,
          Target =>
           Win32.Lparam);

  begin

    case
      Message is

      when
        Win32.Winuser.Wm_Paint      =>

        Bool_Result :=
          Win32.Winuser.Getclientrect
            (Hwnd   =>
              Window_Handle,
             Lprect =>
              Rectangle'Access);
       
        Integer_Result :=
          Win32.Winuser.Getwindowtext
            (Hwnd      =>
              Window_Handle,
             Lpstring  =>
              Convert_Chars_Ptr_To_Lpstr
                (S =>
                  Text_String),
             Nmaxcount =>
              Win32.Int
                (Interfaces.C.Strings.Strlen
                  (Item =>
                    Text_String)));

        Device_Context_Handle :=
          Win32.Winuser.Beginpaint
            (Hwnd    =>
              Window_Handle,
             Lppaint =>
              Paint_Record'Access);

        Brush_Handle :=
          Win32.Wingdi.Createsolidbrush
            (Clrref =>
              Win32.Windef.Colorref
                (Win32.Winuser.Getsyscolor
                  (Win32.Winuser.Color_Window)));

        Brush_Handle :=
          Win32.Wingdi.Selectobject
            (Hdc     =>
              Device_Context_Handle,
             Hgdiobj =>
              Brush_Handle);

        Colorref_Result :=
            (Win32.Wingdi.Setbkcolor
              (Hdc    =>
                Device_Context_Handle,
               Clrref =>
                Win32.Windef.Colorref
                  (Win32.Winuser.Getsyscolor
                    (Win32.Winuser.Color_Window))));

        Colorref_Result :=
          Win32.Wingdi.Settextcolor
            (Hdccolor =>
              Device_Context_Handle,
             Crcolor  =>
              Win32.Windef.Colorref
                (Win32.Winuser.Getsyscolor
                  (Win32.Winuser.Color_Windowtext)));
             
        Bool_Result :=
          Win32.Wingdi.Ellipse
            (Hdc         =>
              Device_Context_Handle,
             Nleftrect   =>
              Win32.Int
                (Rectangle.Left),
             Ntoprect    =>
              Win32.Int
                (Rectangle.Top),
             Nrightrect  =>
              Win32.Int
                (Rectangle.Right),
             Nbottomrect =>
              Win32.Int
                (Rectangle.Bottom));      

        Integer_Result :=
          Win32.Winuser.Drawtext
            (Hdc      =>
              Device_Context_Handle,
             Lpstring =>
              Convert_Chars_Ptr_To_Lpcstr
                (S =>
                  Text_String),
             Ncount   =>
              -1,
             Lprect   =>
              Rectangle'Access,
             Uformat  =>         
              Win32.Winuser.Dt_Singleline or
              Win32.Winuser.Dt_Center     or
              Win32.Winuser.Dt_Vcenter);

        Bool_Result :=
          Win32.Wingdi.Deleteobject
            (Hgdiobj =>
              Win32.Wingdi.Selectobject
                (Hdc     =>
                  Device_Context_Handle,
                 Hgdiobj =>
                  Brush_Handle));

        Bool_Result :=
          Win32.Winuser.Endpaint
            (Hwnd =>
              Window_Handle,
             Lppaint =>
              Paint_Record'Access);

        return (0);

      when
        Win32.Winuser.Wm_Keyup      =>

        if
          Wparam /= 
          Win32.Winuser.Vk_Space then

          return
            Win32.Winuser.Defwindowproc
              (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
               Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

        else

          Long_Result :=
            Win32.Winuser.Sendmessage
              (Hwnd   =>
                Win32.Winuser.Getparent
                  (Hwnd =>
                    Window_Handle),
               Msg    =>
                Win32.Winuser.Wm_Command,
               Wparam =>
                Convert_Long_To_Wparam
                  (S =>
                    Win32.Winuser.Getwindowlong
                      (Hwnd =>
                        Window_Handle,
                       Nindex =>
                        Win32.Winuser.Gwl_id)),
               Lparam =>
                Convert_Hwnd_To_Lparam
                  (S =>
                    Window_Handle));
 
          return (0);

        end if;

      when
        Win32.Winuser.Wm_Lbuttonup  =>

        Long_Result :=
          Win32.Winuser.Sendmessage
            (Hwnd   =>
              Win32.Winuser.Getparent
                (Hwnd =>
                  Window_Handle),
             Msg    =>
              Win32.Winuser.Wm_Command,
             Wparam =>
              Convert_Long_To_Wparam
                (S =>
                  Win32.Winuser.Getwindowlong
                    (Hwnd =>
                      Window_Handle,
                     Nindex =>
                      Win32.Winuser.Gwl_id)),
             Lparam =>
              Convert_Hwnd_To_Lparam
                (S =>
                  Window_Handle));
 
          return (0);

      when
        others =>

        null;

    end case;      

      return
        Win32.Winuser.Defwindowproc
	  (Hwnd   =>
            Window_Handle, 
	   Msg    => 
            Message, 
           Wparam => 
            Wparam,
	   Lparam => 
            Lparam);

  end;

end Windows.Child_Routines;

