with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Child_Routines;
with Windows.Data;
package body Windows.Routines is
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;
  use type Win32.Windef.Hwnd;

   Window_Class          : aliased Win32.Winuser.Wndclass;

   Message               : aliased Win32.Winuser.Msg;

   Modeless_Dialog_Handle : Win32.Windef.Hwnd;

   Dialog_Text            : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "ColorScrDlg");

   False                 : constant Win32.Bool := 0;
     
   Integer_Result        : Win32.Int;
     
   Dword_Result          : Win32.Dword;

   Uint_Result           : Win32.Uint;

   Bool_Result           : Win32.Bool;
                     
   Long_Result           : Win32.Long;

   Idm_About             : constant Win32.Int := 1;

   Atom_Result           : Win32.Windef.Atom;


   function Convert_Chars_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source =>
         Interfaces.C.Strings.Chars_Ptr,
        Target =>
         Win32.Lpcstr);

   procedure Main_Windows_Message_Loop is

     function Convert_Integer_To_Hwnd is new
       Ada.Unchecked_Conversion
         (Source =>
           Integer,
          Target =>
           Win32.Windef.Hwnd);

  begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
         if
           Modeless_Dialog_Handle =
             Convert_Integer_To_Hwnd
               (S =>
                 0) or
           Win32.Winuser.Isdialogmessage
             (Hdlg  =>
               Modeless_Dialog_Handle,
              Lpmsg =>
               Message'Access) = False then

           Bool_Result := 
             Win32.Winuser.Translatemessage
	       (Lpmsg => 
                 Message'Access);
	 
   	   Long_Result := 
             Win32.Winuser.Dispatchmessage
	       (Lpmsg => 
                 Message'Access);
	 
        end if;

      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

      function Convert_Dword_To_Hgdiobj is new
        Ada.Unchecked_Conversion
         (Source =>
           Win32.Dword,
          Target =>
           Win32.Windef.Hgdiobj);

      function Convert_Hgdiobj_To_Long is new
        Ada.Unchecked_Conversion
         (Source =>
           Win32.Windef.Hgdiobj,
          Target =>
           Win32.Long);

   begin

      case
        Message is

	 when
           Win32.WinUser.Wm_Destroy =>

           Bool_Result :=
            Win32.Wingdi.Deleteobject
              (Hgdiobj =>
                Convert_Dword_To_Hgdiobj
                  (S =>
                    Win32.Winuser.Setclasslong
                     (Hwnd      =>
                       Window_Handle,
                      Nindex    =>
                       Win32.Winuser.Gcl_Hbrbackground,
                      Dwnewlong =>
                       Convert_Hgdiobj_To_Long
                         (S =>
                           Win32.Wingdi.Getstockobject
                             (Fnobject =>
                                Win32.Wingdi.White_Brush)))));
              
	   Win32.WinUser.Postquitmessage
	     (Nexitcode => 
              0);

	   return (0);

	 when
           others =>

	   return
             Win32.Winuser.Defwindowproc
	       (Hwnd   =>
                 Window_Handle, 
	        Msg    => 
                 Message, 
	        Wparam => 
                 Wparam,
	        Lparam => 
                 Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is

     function Convert_Integer_To_Hbrush is new
       Ada.Unchecked_Conversion
         (Source =>
           Integer,
          Target =>
           Win32.Windef.Hbrush);

   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             WindowS.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Modeless_Dialog_Handle :=
      Win32.Winuser.Createdialog
        (Hinstance      =>
          Instance,
         Lptemplatename =>
          Convert_Chars_Ptr_To_Lpcstr
            (S =>
              Dialog_Text),
         Hwndparent     =>
          Windows.Data.Window_Handle,
         Lpdialogfunc   =>
          Windows.Child_Routines.Color_Scroll'Access);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
