with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.crt.Ctype;
with Win32.crt.Stdlib;
with Win32.crt.Strings;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;
  use type Win32.Windef.Hwnd;

   type Bit is (Off,On);
   for Bit use (Off =>0, On => 1);
   for Bit'Size use 1; 

   type Bit_Array is array (Integer range 0 .. 31) of Bit;
   pragma pack(Bit_Array);

   Window_Class          : aliased Win32.Winuser.Wndclass;

   Message               : aliased Win32.Winuser.Msg;

   Modeless_Dialog_Handle : Win32.Windef.Hwnd;

   Dialog_Text            : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "ColorScrDlg");

   Button_Handle         : Win32.Windef.Hwnd;

   False                 : constant Win32.Bool := 0;
     
   True                  : constant Win32.Bool := 1;
     
   New_Number            : Win32.Bool := True;
     
   Number                : Win32.Int;

   First                 : Win32.Int;

   Operation             : Win32.Int := Character'Pos('=');

   Integer_Result        : Win32.Int;
     
   Dword_Result          : Win32.Dword;

   Uint_Result           : Win32.Uint;

   Bool_Result           : Win32.Bool;
                     
   Long_Result           : Win32.Long;

   Idm_About             : constant Win32.Int := 1;

   Atom_Result           : Win32.Windef.Atom;

   Window_Handle_Result  : Win32.Windef.Hwnd;

   function Convert_Chars_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source =>
         Interfaces.C.Strings.Chars_Ptr,
        Target =>
         Win32.Lpcstr);

   function Convert_Int_To_Bit_Array is new
     Ada.Unchecked_Conversion
       (Source =>
         Win32.Int,
        Target =>
         Bit_Array);

   function Convert_Bit_Array_To_Int is new
     Ada.Unchecked_Conversion
       (Source =>
         Bit_Array,
        Target =>
         Win32.Int);

   procedure Main_Windows_Message_Loop is

     function Convert_Integer_To_Hwnd is new
       Ada.Unchecked_Conversion
         (Source =>
           Integer,
          Target =>
           Win32.Windef.Hwnd);

  begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
         Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Bit_And
     (L,R : Win32.Int) return Win32.Int is

     Return_Value : Bit_Array;
 
     Temp_Left    : constant Bit_Array :=
       Convert_Int_To_Bit_Array
         (S =>
           L);

     Temp_Right   : constant Bit_Array :=
       Convert_Int_To_Bit_Array
         (S =>
           R);

   begin

     for i in 0 .. 31 loop

       if
         Temp_Left(i) = 
         Temp_Right(i) then

         Return_Value(i) :=
           Temp_Left(i);

       else

         Return_Value(i) :=
           Off;

       end if;

     end loop;

     return
       Convert_Bit_Array_To_Int
         (S =>
           Return_Value);

   end;

   function Bit_Or
     (L,R : Win32.Int) return Win32.Int is

     Return_Value : Bit_Array;
 
     Temp_Left    : constant Bit_Array :=
       Convert_Int_To_Bit_Array
         (S =>
           L);

     Temp_Right   : constant Bit_Array :=
       Convert_Int_To_Bit_Array
         (S =>
           R);

   begin

     for i in 0 .. 31 loop

       if
         (Temp_Left(i) = On or
         Temp_Right(i) = On) then

         Return_Value(i) :=
           On;

       else

         Return_Value(i) :=
           Off;

       end if;

     end loop;

     return
       Convert_Bit_Array_To_Int
         (S =>
           Return_Value);

   end;

   function Bit_Xor
     (L,R : Win32.Int) return Win32.Int is

     Return_Value : Bit_Array;
 
     Temp_Left    : constant Bit_Array :=
       Convert_Int_To_Bit_Array
         (S =>
           L);

     Temp_Right   : constant Bit_Array :=
       Convert_Int_To_Bit_Array
         (S =>
           R);

   begin

     for i in 0 .. 31 loop

       if
         Temp_Left(i) =
         Temp_Right(i) then

         Return_Value(i) :=
           Off;

       else

         Return_Value(i) :=
           On;

       end if;

     end loop;

     return
       Convert_Bit_Array_To_Int
         (S =>
           Return_Value);

   end;

   procedure Show_Number
     (Window_Handle : Win32.Windef.Hwnd;
      Number        : Win32.Int) is

     Text_String : Interfaces.C.Strings.Chars_Ptr :=
       Interfaces.C.Strings.New_String
         (Str =>
           "                    ");

     function Convert_Chars_Ptr_To_Pstr is new
       Ada.Unchecked_Conversion
         (Source =>
           Interfaces.C.Strings.Chars_Ptr,
          Target =>
           Win32.Pstr);

   begin

     Bool_Result :=
       Win32.Winuser.Setdlgitemtext
         (Hdlg       =>
           Window_Handle,
          Niddlgitem =>
           Win32.Winuser.Vk_Escape,
          Lpstring   =>
           Win32.Lpcstr
             (Win32.crt.Strings.Strupr
               (String1 =>
                 Win32.crt.Stdlib.Ltoa
                   (Value =>
                     Number,
                    String1 =>
                      Convert_Chars_Ptr_To_Pstr
                        (S =>
                          Text_String),
                    Radix   =>
                     16))));

     Interfaces.C.Strings.Free
       (Item =>
         Text_String);

   end;

   function Calculate
     (First,Operation,Number : Win32.Int) return Win32.int is
   begin

     case
       Character'Val
         (Integer
           (Operation)) is

       when 
         '=' =>

         return
           Number; 

       when 
         '+' =>

         return
           First + Number; 

       when 
         '-' =>

         return
           First - Number; 

       when 
         '*' =>

         return
           First * Number; 

       when 
         '&' =>

         return
           Bit_And
             (L =>
               First,
              R =>
               Number); 

       when 
         '|' =>

         return
           Bit_Or
             (L =>
               First,
              R =>
               Number); 

       when 
         '^' =>

         return
           Bit_Xor
             (L =>
               First,
              R =>
               Number); 

       when 
         '<' =>

         return
           Win32.Int
             (Interfaces.Shift_Left
               (Value =>
                 Interfaces.Unsigned_32
                   (First),
                Amount =>
                 Natural
                   (Number))); 

       when 
         '>' =>

         return
           Win32.Int
             (Interfaces.Shift_Right
               (Value =>
                 Interfaces.Unsigned_32
                   (First),
                Amount =>
                 Natural
                   (Number))); 

       when 
         '/' =>

         if
           Number /= 0 then

           return
             First / Number;

         else

           return

             Win32.Int'Last;

         end if; 

       when 
         '%' =>

         if
           Number /= 0 then

           return
             First rem Number;

         else

           return

             Win32.Int'Last;

         end if; 

       when 
         others =>

         return
           0; 

     end case;

   end;

   procedure Char
     (Wparam        : in out Win32.Wparam;
      Window_Handle : In     Win32.Windef.Hwnd) is

     function Convert_Character_To_Uchar is new
       Ada.Unchecked_Conversion
         (Source =>
           Character,
          Target =>
           Win32.Uchar);

     function Convert_Uchar_To_Character is new
       Ada.Unchecked_Conversion
         (Source =>
           Win32.Uchar,
          Target =>
           Character);

   begin

     Wparam :=
       Win32.Wparam
         (Character'Pos
           (Convert_Uchar_To_Character
             (Win32.crt.Ctype.Toupper
               (C =>
                 Convert_Character_To_Uchar
                   (Character'Val
                     (Integer
                       (Wparam)))))));
       
     if
       Integer
         (Wparam) =
       Integer
         (Win32.Winuser.Vk_Return) then

       Wparam :=
         Character'Pos('=');

     end if;

     Button_Handle :=
       Win32.Winuser.Getdlgitem
         (Hdlg       =>
           Window_Handle,
          Niddlgitem =>
           Win32.Int
             (Wparam));

     if
       Button_Handle /= null then

       Long_Result := 
         Win32.Winuser.Sendmessage
           (Hwnd   =>
             Button_Handle,
            Msg    =>
             Win32.Winuser.Bm_Setstate,
            Wparam =>
             1,
            Lparam =>
             0);

       Long_Result := 
         Win32.Winuser.Sendmessage
           (Hwnd   =>
             Button_Handle,
            Msg    =>
             Win32.Winuser.Bm_Setstate,
            Wparam =>
             0,
            Lparam =>
             0);

     else

       Bool_Result :=
         Win32.Winuser.Messagebeep
           (Utype =>
             0);

     end if; 

   end;

   procedure Command
     (Number        : in out Win32.Int;
      First         : in out Win32.Int;
      New_Number    : in out Win32.Bool;
      Wparam        : in out Win32.Wparam;
      Window_Handle : In     Win32.Windef.Hwnd) is

     function Convert_Character_To_Uchar is new
       Ada.Unchecked_Conversion
         (Source =>
           Character,
          Target =>
           Win32.Uchar);

   begin

     Window_Handle_Result :=  
       Win32.Winuser.Setfocus
         (Hwnd => Window_Handle);

     if
       Integer
         (Win32.Utils.Loword
           (Win32.Dword
             (Wparam))) = 
       Integer
         (Win32.Winuser.Vk_Back) then

       Number :=
         Number / 16;

       Show_Number
         (Window_Handle =>
           Window_Handle,
          Number        =>
           Number);
              
     elsif
             
       Integer
         (Win32.Utils.Loword
           (Win32.Dword
             (Wparam))) = 
       Integer
         (Win32.Winuser.Vk_Escape) then

       Number :=
        0;

       Show_Number
         (Window_Handle =>
           Window_Handle,
          Number        =>
           Number);
              
     elsif

       Integer
         (Win32.crt.Ctype.Isxdigit
           (Convert_Character_To_Uchar
             (Character'Val
               (Integer
                 (Win32.Utils.Loword
                   (Win32.Dword
                     (Wparam))))))) /= 0 then

       if
         New_Number = 
         True then

         First :=
           Number;
               
         Number :=
           0;

       end if;

       New_Number :=
         False;

       if 
         Number <=
         Win32.Int
           (Interfaces.Shift_Right
             (Value  =>
               Interfaces.Unsigned_32
                 (Win32.Uint'Last),
              Amount =>
               4)) then

         if
           Integer
             (Win32.crt.Ctype.Isdigit
               (Convert_Character_To_Uchar
                 (Character'Val
                   (Integer
                     (Wparam))))) /= 0 then


           Number :=
             16 * Number + (Win32.Int(Wparam) - 48);
                   
         else

           Number :=
             16 * Number + (Win32.Int(Wparam) - 
               (Character'Pos('A') - 10));
                   
         end if;
          
         Show_Number
           (Window_Handle =>
             Window_Handle,
            Number        =>
             Number);
                  
       else

         Bool_Result :=
           Win32.Winuser.Messagebeep
             (Utype =>
               0);

       end if;

     else

       if
         New_Number =
         False then
       
         Number :=
           Calculate
             (First     =>
               First,
              Operation =>
               Operation,
              Number    =>
               Number);

         Show_Number
           (Window_Handle =>
             Window_Handle,
            Number        =>
             Number);

       end if;

       New_Number :=
         True;

       Operation :=
         Win32.Int
           (Win32.Utils.Loword
              (Win32.Dword
                (Wparam)));

     end if;

   end;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

      function Convert_Dword_To_Hgdiobj is new
        Ada.Unchecked_Conversion
         (Source =>
           Win32.Dword,
          Target =>
           Win32.Windef.Hgdiobj);

      function Convert_Hgdiobj_To_Long is new
        Ada.Unchecked_Conversion
         (Source =>
           Win32.Windef.Hgdiobj,
          Target =>
           Win32.Long);

     Temp_Wparam  : Win32.Wparam := Wparam;

   begin

      case
        Message is

	 when
           Win32.WinUser.Wm_Keydown =>

           if
             Integer
               (Temp_Wparam) = 
             Integer
               (Win32.Winuser.Vk_Left) then

             Temp_Wparam :=
               Win32.Winuser.Vk_Back;

             Char
               (Wparam         =>
                 Temp_Wparam,
                Window_Handle =>
                 Window_Handle);

             Command
               (Number        =>
                 Number,
                First         =>
                 First,
                New_Number    =>
                 New_Number,
                Wparam        =>
                 Temp_Wparam,
                Window_Handle =>
                 Window_Handle);

             return (0);
          
           else

  	     return
               Win32.Winuser.Defwindowproc
	         (Hwnd   =>
                   Window_Handle, 
	          Msg    => 
                   Message, 
	          Wparam => 
                   Wparam,
	          Lparam => 
                   Lparam);

           end if;

	 when
           Win32.WinUser.Wm_Char    =>

           Char
             (Wparam        =>
               Temp_Wparam,
              Window_Handle =>
              Window_Handle);

           Command
             (Number        =>
               Number,
              First         =>
               First,
              New_Number    =>
               New_Number,
              Wparam        =>
               Temp_Wparam,
              Window_Handle =>
               Window_Handle);
          
	 when
           Win32.WinUser.Wm_Command =>

           Command
             (Number        =>
               Number,
              First         =>
               First,
              New_Number    =>
               New_Number,
              Wparam        =>
               Temp_Wparam,
              Window_Handle =>
               Window_Handle);
          
           return (0);

	 when
           Win32.WinUser.Wm_Destroy =>

	   Win32.WinUser.Postquitmessage
	     (Nexitcode => 
              0);

	   return (0);

	 when
           others =>

	   return
             Win32.Winuser.Defwindowproc
	       (Hwnd   =>
                 Window_Handle, 
	        Msg    => 
                 Message, 
	        Wparam => 
                 Wparam,
	        Lparam => 
                 Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is

     Window_Handle : Win32.Windef.Hwnd;

     function Convert_Integer_To_Hwnd is new
       Ada.Unchecked_Conversion
        (Source =>
          Integer,
         Target =>
          Win32.Windef.Hwnd);

     function Convert_Integer_To_Dlgproc is new
       Ada.Unchecked_Conversion
        (Source =>
          Integer,
         Target =>
          Win32.Winuser.Dlgproc);
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Win32.Winuser.Loadicon
               (Hinstance  =>
                 Instance,
                Lpiconname =>
                 Convert_Chars_Ptr_To_Lpcstr
                   (S =>
                     Windows.Data.Class_Name)),
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             WindowS.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

    Window_Handle :=
      Win32.Winuser.Createdialog
        (Hinstance      =>
          Instance,
         Lptemplatename =>
          Convert_Chars_Ptr_To_Lpcstr
            (S =>
              Windows.Data.Class_Name),
         Hwndparent     =>
          null,
         Lpdialogfunc   =>
          null);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
