with Ada.Unchecked_Conversion;
with System;
with Win32.Commdlg;
with Win32.Stdio;
with Win32.Stdlib;
with Win32.Winuser;
package body Windows.Pop.File is
  use type Interfaces.C.Strings.Chars_Ptr;
  use type Interfaces.C.Size_T;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Long;
  use type Win32.Stdio.File_Access;
 
  Filter         : Interfaces.C.Strings.Chars_Ptr_Array(0 .. 6) :=
    (0 =>
      Interfaces.C.Strings.New_String
        (Str =>
          "Text_Files (*.TXT)"),
     1 =>
      Interfaces.C.Strings.New_String
        (Str =>
          "*.txt"),
     2 =>
      Interfaces.C.Strings.New_String
        (Str =>
          "ASCII Files (*.ASC)"),
     3 =>
      Interfaces.C.Strings.New_String
        (Str =>
          "*.asc"),
     4 =>
      Interfaces.C.Strings.New_String
        (Str =>
          "All Files (*.ASC)"),
     5 =>
      Interfaces.C.Strings.New_String
        (Str =>
          "*.*"),
     6 =>
      Interfaces.C.Strings.Null_Ptr);

  Ofn            : aliased Win32.Commdlg.Openfilename;

  Bool_Result    : Win32.Bool;

  Integer_Result : Win32.Int;

  function
    Convert_Chars_Ptr_To_Lpvoid is new
      Ada.Unchecked_Conversion
        (Source =>
          Interfaces.C.Strings.Chars_Ptr,
         Target =>
          Win32.Lpvoid);

  function
    Convert_Long_To_Size_T is new
      Ada.Unchecked_Conversion
        (Source =>
          Win32.Long,
         Target =>
          Interfaces.C.Size_T);

  function
    Convert_Chars_Ptr_To_Lpcstr is new
      Ada.Unchecked_Conversion
        (Source =>
          Interfaces.C.Strings.Chars_Ptr,
         Target =>
          Win32.Lpcstr);

  function
    Convert_Chars_Ptr_To_Lpstr is new
      Ada.Unchecked_Conversion
        (Source =>
          Interfaces.C.Strings.Chars_Ptr,
         Target =>
          Win32.Lpstr);

  function
    Create_Blank_String
      (Length :
        Win32.Long) return Interfaces.C.Strings.Chars_Ptr is
      
    Temp_String      : String(1 .. Integer(Length) + 1) := (others => ' ');

  begin

    return
      Interfaces.C.Strings.New_String
        (Str =>
          Temp_String);

  exception

    when Storage_Error =>

      return
        Interfaces.C.Strings.Null_Ptr;

  end;

  procedure
    Initialize
      (Window_Handle :
        Win32.Windef.Hwnd) is

    function
      Convert_Address_To_Lpcstr is new
        Ada.Unchecked_Conversion
          (Source =>
            System.Address,
           Target =>
            Win32.Lpcstr);

  begin

    Ofn.Lstructsize       :=
      Win32.Dword
        (Win32.Commdlg.Openfilename'Size/8);

    Ofn.Hwndowner         :=
      Window_Handle;

    Ofn.Hinstance         :=
      null;

    Ofn.Lpstrfilter       :=
      Convert_Address_To_Lpcstr
        (S =>
          Filter'Address);
    
    Ofn.Lpstrcustomfilter :=
      null;

    Ofn.Nmaxcustfilter    :=
      0;
    
    Ofn.Nfilterindex      :=
      0;
    
    Ofn.Lpstrfile         :=
      null;

    Ofn.Nmaxfile         :=
      Win32.Stdlib.Max_Path;

    Ofn.Lpstrfiletitle   :=
      null;

    Ofn.Nmaxfiletitle    :=
      Win32.Dword
        (Win32.Stdlib.Max_Fname) +
      Win32.Dword
        (Win32.Stdlib.Max_Ext);

    Ofn.Lpstrinitialdir   :=
      null;

    Ofn.Lpstrtitle       :=
      null;

    Ofn.Flags            :=
      0;

    Ofn.Nfileoffset      :=
      0;

    Ofn.Nfileextension   :=
      0;

    Ofn.Lpstrdefext      :=
      Convert_Chars_Ptr_To_Lpcstr
        (S =>
          Interfaces.C.Strings.New_String
            (Str =>
              "txt"));

    Ofn.Lcustdata        :=
      0;

    Ofn.Lpfnhook         :=
      null;

    Ofn.Lptemplatename   :=
      null;

  end;

  function
    Open
      (Window_Handle :
        Win32.Windef.Hwnd;
       File_Name     :
        Interfaces.C.Strings.Chars_Ptr;
       Title_Name    :
        Interfaces.C.Strings.Chars_Ptr)
      return Boolean is

  begin

    Ofn.Hwndowner         :=
      Window_Handle;

    Ofn.Lpstrfile         :=
      Convert_Chars_Ptr_To_Lpstr
        (S =>
          File_Name);

    Ofn.Lpstrfiletitle   :=
      Convert_Chars_Ptr_To_Lpstr
        (S =>
          Title_Name);

    Ofn.Flags            :=
      Win32.Dword
        (Win32.Commdlg.Ofn_Hidereadonly) or
      Win32.Dword
        (Win32.Commdlg.Ofn_Createprompt);

    Bool_Result :=
      Win32.Commdlg.Getopenfilename
        (Lpofn =>
          Ofn'Access);
 
    if 
      Bool_Result = 
      Win32.False then

      return
        False;

    else
      
      return
        True;

    end if; 

  end;

  function
    Save
      (Window_Handle :
        Win32.Windef.Hwnd;
       File_Name     :
        Interfaces.C.Strings.Chars_Ptr;
       Title_Name    :
        Interfaces.C.Strings.Chars_Ptr)
      return Boolean is

  begin

    Ofn.Hwndowner         :=
      Window_Handle;

    Ofn.Lpstrfile         :=
      Convert_Chars_Ptr_To_Lpstr
        (S =>
          File_Name);

    Ofn.Lpstrfiletitle   :=
      Convert_Chars_Ptr_To_Lpstr
        (S =>
          Title_Name);

    Ofn.Flags            :=
      Win32.Commdlg.Ofn_Overwriteprompt;

    Bool_Result :=
      Win32.Commdlg.Getopenfilename
        (Lpofn =>
          Ofn'Access);
 
    if 
      Bool_Result = 
      Win32.False then

      return
        False;

    else
      
      return
        True;

    end if; 

  end;

  function
    Length
      (File :
        Win32.Stdio.File_Access)
      return Win32.Long is

    File_Length      : Win32.Long;

    Current_Position : Win32.Long;

  begin

    Current_Position :=
      Win32.Stdio.Ftell
        (Stream =>
          File); 

    Integer_Result :=
      Win32.Stdio.Fseek
        (Stream =>
          File,
         Offset =>
          0,
         Origin =>
          Win32.Stdio.Seek_End);
    
    File_Length :=
      Win32.Stdio.Ftell
        (Stream =>
          File); 

    Integer_Result :=
      Win32.Stdio.Fseek
        (Stream =>
          File,
         Offset =>
          Current_Position,
         Origin =>
          Win32.Stdio.Seek_Set);

    return
      File_Length;

  end;

  function
    Read
      (Window_Handle :
        Win32.Windef.Hwnd;
       File_Name     :
        Interfaces.C.Strings.Chars_Ptr)
      return Boolean is

    File           : Win32.Stdio.File_Access;

    Len            : Win32.Long;

    Buffer_Pointer : Interfaces.C.Strings.Chars_Ptr;

    Rb_Text        : constant Interfaces.C.Strings.Chars_Ptr :=
      Interfaces.C.Strings.New_String
        (Str =>
          "rb");

    Size_T_Result    : Interfaces.C.Size_T;

  begin

    File :=
      Win32.Stdio.Fopen
        (Filename =>
          Convert_Chars_Ptr_To_Lpcstr
            (S =>
              File_Name),
         Mode     =>
          Convert_Chars_Ptr_To_Lpcstr
            (S =>
              Rb_Text));
       
    if
      File =
      null then

      return 
        False;

    end if;

    Len :=
      Length
        (File =>
          File);

    Buffer_Pointer :=
      Create_Blank_String
        (Length =>
          Len);

    if 
      Buffer_Pointer =
      Interfaces.C.Strings.Null_Ptr then

      Integer_Result :=
        Win32.Stdio.Fclose
          (Stream =>
            FIle);

      return
        False;

    end if;

    Size_T_Result :=
      Win32.Stdio.Fread
        (Buffer =>
          Convert_Chars_Ptr_To_Lpvoid
            (S =>
              Buffer_Pointer),
         Size   =>
          1,
         Count  =>
          Convert_Long_To_Size_T
            (S =>
              Len),
         Stream =>
           File);

    Integer_Result :=
      Win32.Stdio.Fclose
        (Stream =>
          FIle);

    Bool_Result :=
      Win32.Winuser.Setwindowtext
        (Hwnd     =>
          Window_Handle,
         Lpstring =>
          Convert_Chars_Ptr_To_Lpcstr
            (S =>
              Buffer_Pointer));

    Interfaces.C.Strings.Free
      (Item =>
        Buffer_Pointer);

    return
      True;

  end;

  function
    Write
      (Window_Handle :
        Win32.Windef.Hwnd;
       File_Name     :
        Interfaces.C.Strings.Chars_Ptr)
      return Boolean is

    File           : Win32.Stdio.File_Access;

    Len            : Win32.Long;

    Buffer_Pointer : Interfaces.C.Strings.Chars_Ptr;

    Wb_Text        : constant Interfaces.C.Strings.Chars_Ptr :=
      Interfaces.C.Strings.New_String
        (Str =>
          "wb");

  begin

    File :=
      Win32.Stdio.Fopen
        (Filename =>
          Convert_Chars_Ptr_To_Lpcstr
            (S =>
              File_Name),
         Mode     =>
          Convert_Chars_Ptr_To_Lpcstr
            (S =>
              Wb_Text));
       
    if
      File =
      null then

      return 
        False;

    end if;

    Len :=
      Win32.Long
        (Win32.Winuser.Getwindowtextlength
          (Hwnd =>
            Window_Handle));

    Buffer_Pointer :=
      Create_Blank_String
        (Length =>
          Len + 1);

    if 
      Buffer_Pointer =
      Interfaces.C.Strings.Null_Ptr then

      Integer_Result :=
        Win32.Stdio.Fclose
          (Stream =>
            FIle);

      return
        False;

    end if;

    Integer_Result :=
      Win32.Winuser.Getwindowtext
        (Hwnd      =>
          Window_Handle,
         Lpstring  =>
          Convert_Chars_Ptr_To_Lpstr
            (S =>
              Buffer_Pointer),
         Nmaxcount =>
          Win32.Int
            (Len + 1));
    
    if
      Convert_Long_To_Size_T
        (Len) /=
      Win32.Stdio.Fwrite
        (Buffer =>
          Convert_Chars_Ptr_To_Lpvoid
            (S =>
              Buffer_Pointer),
         Size   =>
          1,
         Count  =>
          Convert_Long_To_Size_T
            (S =>
              Len + 1),
         Stream =>
           File) then

      Integer_Result :=
        Win32.Stdio.Fclose
          (Stream =>
            FIle);

      Interfaces.C.Strings.Free
        (Item =>
          Buffer_Pointer);

      return
        False;

    end if;

    Integer_Result :=
      Win32.Stdio.Fclose
        (Stream =>
          FIle);

    Interfaces.C.Strings.Free
      (Item =>
        Buffer_Pointer);

    return
      True;

  end;

end;
        
