with Ada.Unchecked_Conversion;
With Interfaces.C.Strings;
with System;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
   use type System.Address;
   use type Win32.Dword;
   use type Win32.Int;
   use type Win32.Bool;
   use type Win32.Uint;

   Paint_Record  : aliased win32.Winuser.Paintstruct;

   Rectangle     : aliased win32.WinDef.Rect;

   Hello_Windows : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str =>"Hello Windows, from ObjectAda 7.1");

   Device_Context_Handle : aliased Win32.Windef.Hdc;

   Integer_Result : Win32.Int;
   Bool_Result    : Win32.Bool;
   Long_Result    : Win32.Long;
   Atom_Result    : Win32.Windef.Atom;

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Windows.Data.Message'Access,
	      Hwnd  =>
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Windows.Data.Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Windows.Data.Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin
      case Message is
	 when Win32.Winuser.Wm_Paint =>

	    Device_Context_Handle := 
              Win32.Winuser.Beginpaint
	        (Hwnd    => 
                  Window_Handle,
	         Lppaint => 
                  Paint_Record'Access);

	    Bool_Result := 
              Win32.Winuser.Getclientrect
	        (Hwnd   => 
                  Window_Handle,
	         Lprect => 
                  Rectangle'Access);

	    Integer_Result := 
              Win32.Winuser.Drawtext
	        (Hdc =>
                  Device_Context_Handle,
	         Lpstring => 
                  Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      Hello_Windows),
	         Ncount   => 
                  - 1,
	         Lprect   => 
                  Rectangle'Access,
	         Uformat  => 
                  Win32.Winuser.Dt_Singleline + 
	          Win32.Winuser.Dt_Center +
	          Win32.Winuser.Dt_Vcenter);

	    Bool_Result := 
              Win32.Winuser.Endpaint
	        (Hwnd    => 
                  Window_Handle,
	         Lppaint => 
                  Paint_Record'Access);

	    return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return 
              Win32.Winuser.Defwindowproc
	        (Hwnd   => 
                  Window_Handle, 
	         Msg    => 
                  Message, 
	         Wparam => 
                  Wparam,
	         Lparam => 
                  Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_Instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = 
        System.Null_Address then

	 Windows.Data.Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Windows.Data.Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.CreateWindow
	  (Lpclassname  => 
            Convert_Char_Ptr_To_Lpcstr
              (S => 
                Windows.Data.Class_Name),
           Lpwindowname => 
            Convert_Char_Ptr_To_Lpcstr
              (S => 
                Windows.Data.Caption),
           Dwstyle      => 
            Windows.Data.Window_Style,
           X            => 
            Windows.Data.Window_Horizontal_Start, 
           Y            => 
            Windows.Data.Window_Vertical_Start,
           Nwidth       => 
            Windows.Data.Window_Width,
           Nheight      => 
            Windows.Data.Window_Height,
           Hwndparent   => 
            Windows.Data.Window_Parent,
           Hmenu        => 
            Windows.Data.Window_Menu,
           Hinstance    => 
            Instance,
           Lpparam      => 
            Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow => 
          Win32.Winmain.Get_Ncmdshow);

    Bool_result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Windows.Data.Message.Wparam;

   end Main;

end Windows.Routines;
