-- Filename: device_capabilities.ads
-- 
-- This is the conversion of Petzold's Win 95
-- Chapter 4 program devcaps1.  This file contains
-- data needed exclusively for the devcaps1 program.
--
with Interfaces.C.Strings;
with Win32;
with Win32.Wingdi;

package Device_Capabilities is
   use type Win32.Char_Array;

   type Dc is
      (Horzsize,
       Vertsize,
       Horzres,
       Vertres,
       Bitspixel,
       Planes,
       Numbrushes,
       Numpens,
       Nummarkers,
       Numfonts,
       Numcolors,
       Pdevicesize,
       Aspectx,
       Aspecty,
       Aspectxy,
       Logpixelsx,
       Logpixelsy,
       Sizepalette,
       Numreserved,
       Colorres);

   for Dc use                                            -- correct numbers
                                                         -- at time of writing
      (Horzsize            => Win32.Wingdi.Horzsize,     --4,
       Vertsize            => Win32.Wingdi.VERTSIZE,     --6,
       Horzres             => Win32.Wingdi.Horzres,      --8,
       Vertres             => Win32.Wingdi.Vertres,      --10,
       Bitspixel           => Win32.Wingdi.Bitspixel,    --12,
       Planes              => Win32.Wingdi.Planes,       --14,
       Numbrushes          => Win32.Wingdi.Numbrushes,   --16,
       Numpens             => Win32.Wingdi.Numpens,      --18,
       Nummarkers          => Win32.Wingdi.Nummarkers,   --20,
       Numfonts            => Win32.Wingdi.Numfonts,     --22,
       Numcolors           => Win32.Wingdi.Numcolors,    --24,
       Pdevicesize         => Win32.Wingdi.Pdevicesize,  --26,
       Aspectx             => Win32.Wingdi.Aspectx,      --40,
       Aspecty             => Win32.Wingdi.Aspecty,      --42,
       Aspectxy            => Win32.Wingdi.Aspectxy,     --44,
       Logpixelsx          => Win32.Wingdi.Logpixelsx,   --88,
       Logpixelsy          => Win32.Wingdi.Logpixelsy,   --90,
       Sizepalette         => Win32.Wingdi.Sizepalette,  --104,
       Numreserved         => Win32.Wingdi.Numreserved,  --106,
       Colorres            => Win32.Wingdi.Colorres);    --108);

   type Dc_Record is
      record
         Label : Interfaces.C.Strings.Chars_Ptr;
         Desc  : Interfaces.C.Strings.Chars_Ptr;
      end record;

   type Dc_Record_Array is array ( Dc'First .. Dc'Last ) of Dc_Record;

Devcaps1_Record_Array : constant Dc_Record_Array :=
   (Horzsize    =>
       (Label => Interfaces.C.Strings.New_String("Horzsize"),
        Desc  => Interfaces.C.Strings.New_String("Width in millimeters:")),
    Vertsize    => 
       (Label => Interfaces.C.Strings.New_String("Vertsize"),
        Desc  => Interfaces.C.Strings.New_String("Height in millimeters:")),
    Horzres     => 
       (Label => Interfaces.C.Strings.New_String("Horzres"),
        Desc  => Interfaces.C.Strings.New_String("Width in pixels:")),
    Vertres     => 
       (Label => Interfaces.C.Strings.New_String("Vertres"),
        Desc  => Interfaces.C.Strings.New_String("Height in raster lines:")),
    Bitspixel   => 
       (Label => Interfaces.C.Strings.New_String("Bitspixel"),
        Desc  => Interfaces.C.Strings.New_String("Color bits per pixel")),
    Planes      => 
       (Label => Interfaces.C.Strings.New_String("Planes"),
        Desc  => Interfaces.C.Strings.New_String("Number of color planes")),
    Numbrushes  => 
       (Label => Interfaces.C.Strings.New_String("Numbrushes"),
        Desc  => Interfaces.C.Strings.New_String("Number of device brushes")),
    Numpens     => 
       (Label => Interfaces.C.Strings.New_String("Numpens"),
        Desc  => Interfaces.C.Strings.New_String("Number of device pens")),
    Nummarkers  => 
       (Label => Interfaces.C.Strings.New_String("Nummarkers"),
        Desc  => Interfaces.C.Strings.New_String("Number of device markers")),
    Numfonts    => 
       (Label => Interfaces.C.Strings.New_String("Numfonts"),
        Desc  => Interfaces.C.Strings.New_String("Number of device fonts")),
    Numcolors   => 
       (Label => Interfaces.C.Strings.New_String("Numcolors"),
        Desc  => Interfaces.C.Strings.New_String("Number of device colors")),
    Pdevicesize => 
       (Label => Interfaces.C.Strings.New_String("Pdevicesize"),
        Desc  => Interfaces.C.Strings.New_String("Size of device structure")),
    Aspectx     => 
       (Label => Interfaces.C.Strings.New_String("Aspectx"),
        Desc  => Interfaces.C.Strings.New_String("Relative width of pixel")),
    Aspecty     => 
       (Label => Interfaces.C.Strings.New_String("Aspecty"),
        Desc  => Interfaces.C.Strings.New_String("Relative height of pixel")),
    Aspectxy    => 
       (Label => Interfaces.C.Strings.New_String("Aspectxy"),
        Desc  => Interfaces.C.Strings.New_String("Relative diagonal of pixel")),
    Logpixelsx  => 
       (Label => Interfaces.C.Strings.New_String("Logpixelsx"),
        Desc  => Interfaces.C.Strings.New_String("Horizontal dots per inch")),
    Logpixelsy  => 
       (Label => Interfaces.C.Strings.New_String("Logpixelsy"),
        Desc  => Interfaces.C.Strings.New_String("Vertical dots per inch")),
    Sizepalette => 
       (Label => Interfaces.C.Strings.New_String("Sizepalette"),
        Desc  => Interfaces.C.Strings.New_String("Number of palette entries")),
    Numreserved => 
       (Label => Interfaces.C.Strings.New_String("Numreserved"),
        Desc  => Interfaces.C.Strings.New_String("Reserved palette entries")),
    Colorres    => 
       (Label => Interfaces.C.Strings.New_String("Colorres"),
        Desc  => Interfaces.C.Strings.New_String("Actual color resolution")));
   
end Device_Capabilities;

