-- Filename: routines.adb
--
-- This is the conversion of Petzold's Win 95
-- Chapter 4 program devcaps1.
-- 
-- Contains : 1. main_windows_message_loop
--            2. procedure Convert_Integer_To_Char_Array
--            3. function Main_Windows  (wndproc)
--            4. function main          (winmain)
--            
with System;
with Ada.Unchecked_Conversion;

with Interfaces.C.Strings;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winuser;

with Device_Capabilities;
with Windows.Data;

package body Windows.Routines is
   use type System.Address;
   use type Win32.Dword;
   use type Win32.Bool;
   use type Win32.Byte;
   use type Win32.Int;
   use type Win32.Long;
   use type Win32.Uint;

   Hdc : aliased Win32.Windef.Hdc;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => Windows.Data.Message'Access,
	      Hwnd =>System.Null_Address, 
	      Wmsgfiltermin => Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Windows.Data.Bresult := Win32.Winuser.Translatemessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
   	 Windows.Data.Lresult := Win32.Winuser.Dispatchmessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
      end loop;
   end Main_Windows_Message_Loop;



   procedure Convert_Integer_To_Char_Array
     (Int : in Integer;
      Char_Array : in out Win32.Char_Array) is
     Temporary_String : string := Integer'Image(int);
   begin
     for i in 1 .. (Char_Array'Length - 1) loop
       Char_Array(i) := ' ';
     end loop;
     for i in 1 .. Temporary_String'Length loop
       Char_Array(((Char_Array'Length - 1) - Temporary_String'Length) + i) := 
         Win32.Char(Temporary_String(i));
     end loop;
   end Convert_Integer_To_Char_Array;


   function Dc_to_Win32int is new Ada.Unchecked_Conversion
                                  (Device_Capabilities.Dc, Win32.int);

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message : Win32.Uint;
      Wparam : Win32.Wparam;
      Lparam : Win32.Lparam) return Win32.Lresult is

      function Convert_Char_Ptr_To_Lpcstr is new
        Ada.Unchecked_Conversion
          (Source => Interfaces.C.Strings.Chars_Ptr,
           Target => Win32.Lpcstr);

   begin
      case Message is

	 when Win32.Winuser.Wm_Create =>
	    Hdc := Win32.Winuser.Getdc
               (Hwnd => Window_Handle);
	    Windows.Data.Bresult := Win32.Wingdi.Gettextmetrics
               (Hdc => Hdc,
                Lptm => Windows.Data.Text_Metric'Access);
            Windows.Data.Character_Width := Windows.Data.Text_Metric.
                                               Tmavecharwidth;
            if Windows.Data.Text_Metric.Tmpitchandfamily mod 2 > 0 then
              windows.Data.Cap_width :=
                (3 * Windows.Data.CHaracter_Width)/2;
            else
              windows.Data.Cap_width :=
                (2 * Windows.Data.CHaracter_Width)/2;
            end if;
            Windows.Data.Character_Height := 
              Windows.Data.Text_Metric.TmHeight +
              Windows.Data.Text_Metric.Tmexternalleading;
            Windows.Data.Iresult := Win32.Winuser.Releasedc
              (Hwnd => Window_Handle,
               Hdc  => Hdc);
            return (0);

	 when Win32.Winuser.Wm_Paint =>
	    Hdc := Win32.Winuser.Beginpaint
	      (Hwnd    => Window_Handle,
	       Lppaint => Windows.Data.Paint_Record'Access);
            for i in Device_Capabilities.Dc'First .. 
               Device_Capabilities.Dc'Last loop
	      Windows.Data.Bresult := Win32.Wingdi.Textout
                (Hdc   => Hdc,
                 Nxstart  => Win32.Int(Windows.Data.Character_Width),
                 Nystart  => win32.Int(Windows.Data.Character_Height *
                             ( 1 + Device_Capabilities.Dc'POS(i))),
                 Lpstring => Convert_Char_Ptr_To_Lpcstr
                     (Device_Capabilities.Devcaps1_Record_Array(i).Label),
                 Cbstring => Win32.Int(Interfaces.C.Strings.Strlen
                     (Device_Capabilities.Devcaps1_Record_Array(i).Label))
                );

	      Windows.Data.Bresult := Win32.Wingdi.Textout
                (Hdc   => Hdc,
                 Nxstart  => Win32.Int(Windows.Data.Character_Width +
                                       22 * Windows.Data.Cap_Width),
                 Nystart  => win32.Int(Windows.Data.Character_Height *
                                       (1 + Device_Capabilities.Dc'POS(i))),
                 Lpstring => Convert_Char_Ptr_To_Lpcstr
                     (Device_Capabilities.Devcaps1_Record_Array(i).Desc),
                 Cbstring => Win32.Int(Interfaces.C.Strings.Strlen
                     (Device_Capabilities.Devcaps1_Record_Array(i).Desc))
                );

              Windows.Data.Uresult := Win32.Wingdi.Settextalign
                (Hdc => Hdc,
                 Fmode => Windows.Data.Align_Top_Right);

	      Convert_Integer_To_Char_Array
                (Int => 
                  integer(Win32.Wingdi.GetDeviceCaps
                    (Hdc, Dc_to_Win32int(Device_Capabilities.Dc(i)))),
                 Char_Array => Windows.Data.Char_Array);

              Windows.Data.Bresult := Win32.Wingdi.Textout
	        (Hdc   => Hdc,
                 Nxstart  => Win32.Int(Windows.Data.Character_Width +
                                       22 * Windows.Data.Cap_Width +
                                       60 * Windows.Data.Character_Width),
                 Nystart  => Win32.Int(Windows.Data.Character_Height *
                                       (1 + Device_Capabilities.Dc'POS(i))),
                 Lpstring => Windows.Data.Char_Array
                                (Windows.Data.Char_Array'First)'Access,
                 Cbstring => Windows.Data.Char_Array'Length);

              Windows.Data.Uresult := Win32.Wingdi.Settextalign
                (Hdc => Hdc,
                 Fmode => Windows.Data.Align_Top_Left);
            end loop;

	    Windows.Data.Bresult := Win32.Winuser.Endpaint
               (hwnd   => Window_Handle,
                Lppaint => Windows.Data.Paint_Record'Access);
	    return (0);

	 when Win32.WinUser.Wm_Destroy =>
	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 0);
	    return (0);

	 when others =>
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => Window_Handle, 
	       Msg    => Message, 
	       Wparam => Wparam,
	       Lparam => Lparam);
      end case;
      return (0);
   end Main_Windows;



   function Main
     (Instance : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line : Win32.Pcstr;
      Show_Command : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Windows.Data.Window_Class :=      
	   (Style         => Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => Instance,
	    Hicon         => Windows.Data.Window_Class_Icon,
	    Hcursor       => Windows.Data.Window_Class_Cursor,
	    Hbrbackground => Windows.Data.Window_Class_Background,
	    Lpszmenuname  => Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => Windows.Data.Class_Name
                                (Windows.Data.Class_Name'First)'Access);

	 Windows.Data.Aresult :=
	   win32.winuser.Registerclass
	   (Lpwndclass => Windows.Data.Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := Win32.WinUser.CreateWindow
	(Lpclassname => Windows.Data.Class_Name
                           (Windows.Data.Class_Name'First)'Access,
            Lpwindowname => Windows.Data.Caption
                               (Windows.Data.Caption'First)'Access,
            Dwstyle      => Windows.Data.Window_Style,
            X            => Windows.Data.Window_Horizontal_Start, 
            Y            => Windows.Data.Window_Vertical_Start,
            Nwidth       => Windows.Data.Window_Width,
            Nheight      => Windows.Data.Window_Height,
            Hwndparent   => Windows.Data.Window_Parent,
            Hmenu        => Windows.Data.Window_Menu,
            Hinstance    => Instance,
            Lpparam      => Windows.Data.Window_Parameters);

    Windows.Data.Bresult := Win32.Winuser.Showwindow
      (Hwnd => Windows.Data.Window_Handle, 
	 Ncmdshow =>win32.Winmain.Get_Ncmdshow);

    Windows.Data.Bresult := Win32.Winuser.Updatewindow
      (Hwnd => Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Windows.Data.Message.Wparam;

   end Main;

end Windows.Routines;
