-- Filename: routines.adb
--
-- This is the conversion of Petzold's Win 95
-- Chapter 4 program Linedemo.
-- 
-- Contains : 1. main_windows_message_loop
--            2. function Main_Windows  (wndproc)
--            3. function main          (winmain)
--            
with System;
with Unchecked_Conversion;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winuser;

with Linedemo;
with Windows.Data;

package body Windows.Routines is
   use type System.Address;
   use type Win32.Bool;
   use type Win32.Byte;
   use type Win32.Dword;
   use type Win32.Float;
   use type Win32.Int;
   use type Win32.Long;
   use type Win32.Uint;
   use type Win32.Word;

   Hdc : aliased Win32.Windef.Hdc;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => Windows.Data.Message'Access,
	      Hwnd =>System.Null_Address, 
	      Wmsgfiltermin => Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Windows.Data.Bresult := Win32.Winuser.Translatemessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
   	 Windows.Data.Lresult := Win32.Winuser.Dispatchmessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
      end loop;
   end Main_Windows_Message_Loop;



   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message : Win32.Uint;
      Wparam : Win32.Wparam;
      Lparam : Win32.Lparam) return Win32.Lresult is

   begin
      case Message is
         when Win32.Winuser.Wm_Size =>
            -- get client area width and height
            Linedemo.Client_Width  := Win32.Int(Win32.Windef.LoWord
                                                (Win32.Dword(Lparam)));
            Linedemo.Client_Height := Win32.Int(Win32.Windef.HiWord
                                                (Win32.Dword(Lparam)));
            return (0);

	 when Win32.Winuser.Wm_Paint =>
	    Hdc := Win32.Winuser.Beginpaint -- BEGINPAINT get device context
	      (Hwnd    => Window_Handle,
	       Lppaint => Windows.Data.Paint_Record'Access);

            -- Call Rectangle GDI to draw rectangle
            Windows.Data.Bresult := Win32.Wingdi.Rectangle
            (Hdc => Hdc,
             nLeftRect => Win32.Int(Linedemo.Client_Width / 8),
             nTopRect => Win32.Int(Linedemo.Client_Height / 8),
             nRightRect => Win32.Int(7 * Linedemo.Client_Width / 8),
             nBottomRect => Win32.Int(7 * Linedemo.Client_Height / 8)
            );

            -- Call MoveToEx and LineTo GDI functions twice for diagonal lines
            Windows.Data.Bresult := Win32.Wingdi.MoveToEx
                              (Hdc => Hdc,
                               X => Win32.Int(0),
                               Y => Win32.Int(0),
                               Lppoint => Null);
            Windows.Data.Bresult := Win32.Wingdi.LineTo
                              (Hdc => Hdc,
                               Xend => Win32.Int(Linedemo.Client_Width),
                               Yend => Win32.Int(Linedemo.Client_Height));

            Windows.Data.Bresult := Win32.Wingdi.MoveToEx
                              (Hdc => Hdc,
                               X => Win32.Int(0),
                               Y => Win32.Int(Linedemo.Client_Height),
                               Lppoint => Null);
            Windows.Data.Bresult := Win32.Wingdi.LineTo
                              (Hdc => Hdc,
                               Xend => Win32.Int(Linedemo.Client_Width),
                               Yend => Win32.Int(0));

            -- Call Ellipse GDI to draw ellipse
            Windows.Data.Bresult := Win32.Wingdi.Ellipse
            (Hdc => Hdc,
             nLeftRect => Win32.Int(Linedemo.Client_Width / 8),
             nTopRect => Win32.Int(Linedemo.Client_Height / 8),
             nRightRect => Win32.Int(7 * Linedemo.Client_Width / 8),
             nBottomRect => Win32.Int(7 * Linedemo.Client_Height / 8)
            );

            -- Call RoundRect GDI to draw round rectangle
            Windows.Data.Bresult := Win32.Wingdi.Roundrect
            (Hdc => Hdc,
             nLeftRect => Win32.Int(Linedemo.Client_Width / 4),
             nTopRect => Win32.Int(Linedemo.Client_Height / 4),
             nRightRect => Win32.Int(3 * Linedemo.Client_Width / 4),
             nBottomRect => Win32.Int(3 * Linedemo.Client_Height / 4),
             nEllipseWidth => Win32.Int(Linedemo.Client_Width / 4),
             nEllipseHeight => Win32.Int(Linedemo.Client_Height / 4)
            );

            Windows.Data.Bresult := Win32.Winuser.EndPaint  -- ENDPAINT
                    (Hwnd => Window_Handle,
                     Lppaint => Windows.Data.Paint_Record'Access);
	    return (0);

	 when Win32.WinUser.Wm_Destroy =>
	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 0);
	    return (0);

	 when others =>
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => Window_Handle, 
	       Msg    => Message, 
	       Wparam => Wparam,
	       Lparam => Lparam);
      end case;

      return (0);
   end Main_Windows;



   function Main
     (Instance : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line : Win32.Pcstr;
      Show_Command : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Windows.Data.Window_Class :=      
	   (Style         => Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => Instance,
	    Hicon         => Windows.Data.Window_Class_Icon,
	    Hcursor       => Windows.Data.Window_Class_Cursor,
	    Hbrbackground => Windows.Data.Window_Class_Background,
	    Lpszmenuname  => Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => Windows.Data.Class_Name
                                (Windows.Data.Class_Name'First)'Access);

	 Windows.Data.Aresult :=
	   win32.winuser.Registerclass
	   (Lpwndclass => Windows.Data.Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := Win32.WinUser.CreateWindow
	(Lpclassname => Windows.Data.Class_Name
                           (Windows.Data.Class_Name'First)'Access,
            Lpwindowname => Windows.Data.Caption
                           (Windows.Data.Caption'First)'Access,
            Dwstyle      => Windows.Data.Window_Style,
            X            => Windows.Data.Window_Horizontal_Start, 
            Y            => Windows.Data.Window_Vertical_Start,
            Nwidth       => Windows.Data.Window_Width,
            Nheight      => Windows.Data.Window_Height,
            Hwndparent   => Windows.Data.Window_Parent,
            Hmenu        => Windows.Data.Window_Menu,
            Hinstance    => Instance,
            Lpparam      => Windows.Data.Window_Parameters);

    Windows.Data.Bresult := Win32.Winuser.Showwindow
      (Hwnd => Windows.Data.Window_Handle, 
	 Ncmdshow =>win32.Winmain.Get_Ncmdshow);

    Windows.Data.Bresult := Win32.Winuser.Updatewindow
      (Hwnd => Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Windows.Data.Message.Wparam;

   end Main;

end Windows.Routines;

