-- Filename: routines.adb
--
-- This is the conversion of Petzold's Win 95
-- Chapter 4 program RandRect.
-- 
-- Contains : 1. main_windows_message_loop
--            2. function Main_Windows  (wndproc)
--            3. function main          (winmain)
--            4. Draw_Rectangle
--            



with Ada.Numerics.Float_Random;
with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;

with Win32;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winnt;
with Win32.Winuser;

with Randrect;
with Windows.Data;



package body Windows.Routines is
--   use type System.Address;
--use Win32;  -- put in after all dot notation completed
             -- needed to see "and" and "or" for wparam
   use type Win32.Bool;
--   use type Win32.Byte;
--   use type Win32.Dword;
--   use type Win32.Float;
   use type Win32.Int;
   use type Win32.Long;
   use type Win32.Uint;
   use type Win32.Windef.Hinstance;
   use type Win32.Winuser.Msg;
--   use type Win32.Word;
--   use type Win32.Wparam;



   procedure Draw_Rectangle (Window_Handle : in Win32.Windef.Hwnd) is

      Brush_Handle  : Win32.Windef.Hbrush;
      Device_Handle : aliased Win32.Windef.Hdc;
      Xleft, XRight  : Win32.Int;
      Ytop, Ybottom  : Win32.Int;

      Rcolor, Gcolor, Bcolor : Win32.Byte;
      RGB_Color : Win32.Windef.Colorref;

      Width, Height : Win32.Int;

   begin

      Width  := Randrect.Client_Width;
      Height := Randrect.Client_Height;

      if (not ((Width = 0) or
               (Height = 0) ) ) then

         Xleft   := Win32.Int
                    (Ada.Numerics.Float_Random.Random
                     (Gen => Randrect.Rand_Gen) * 10000.0)
                     mod Width;
         Ytop    := Win32.Int
                    (Ada.Numerics.Float_Random.Random
                     (Gen => Randrect.Rand_Gen) * 10000.0)
                     mod Height;
         XRight  := Win32.Int
                    (Ada.Numerics.Float_Random.Random
                     (Gen => Randrect.Rand_Gen) * 10000.0)
                     mod Width;
         Ybottom := Win32.Int
                    (Ada.Numerics.Float_Random.Random
                     (Gen => Randrect.Rand_Gen) * 10000.0)
                     mod Height;
         Windows.Data.Bresult := Win32.Winuser.Setrect (
                                      Lprc    => Randrect.Rectangle'Access,
                                      Xleft   => Xleft,
                                      Ytop    => Ytop,
                                      Xright  => Xright,
                                      Ybottom => Ybottom);

         Rcolor := Win32.Byte (Win32.Int (
                   (Ada.Numerics.Float_Random.Random
                    (Gen => Randrect.Rand_Gen) * 1000.0))
                    mod 256);
         Gcolor := Win32.Byte (Win32.Int (
                   (Ada.Numerics.Float_Random.Random
                    (Gen => Randrect.Rand_Gen) * 1000.0))
                    mod 256);
         Bcolor := Win32.Byte (Win32.Int (
                   (Ada.Numerics.Float_Random.Random
                    (Gen => Randrect.Rand_Gen) * 1000.0))
                    mod 256);
         RGB_Color := Win32.Wingdi.RGB (Bred   => Rcolor,
                                        Bgreen => Gcolor,
                                        Bblue  => Bcolor);
         Brush_Handle := Win32.Wingdi.CreateSolidBrush
                             (Clrref => RGB_Color);

         Device_Handle := Win32.Winuser.Getdc (Hwnd => Window_Handle);
         Windows.Data.Iresult := Win32.Winuser.FillRect
                                    (Hdc  => Device_Handle,
                                     Lprc => Randrect.Rectangle'Access,
                                     Hbr  => Brush_Handle);

         Windows.Data.Iresult := Win32.Winuser.Releasedc
                                    (Hwnd => Window_Handle,
                                     Hdc  => Device_Handle);
         Windows.Data.Bresult := Win32.Wingdi.DeleteObject
                    (Hgdiobj => Win32.Windef.Hgdiobj(Brush_Handle));
      end if;

   end Draw_Rectangle;



   procedure Main_Windows_Message_Loop is
   begin
      Peek_Loop:
      while (Win32.True = Win32.True) loop
         if (Win32.Winuser.Peekmessage
	     (Lpmsg => Windows.Data.Message'Access,
	      Hwnd =>System.Null_Address, 
	      Wmsgfiltermin => 0,
	      Wmsgfiltermax => 0,
              Wremovemsg => Win32.Winuser.Pm_Remove) = Win32.True) then
            exit Peek_Loop when (Windows.Data.Message.Message =
                                 Win32.Winuser.Wm_Quit);
	 
      	    Windows.Data.Bresult := Win32.Winuser.Translatemessage
	                   (Lpmsg => Windows.Data.Message'Access);
	 
   	    Windows.Data.Lresult := Win32.Winuser.Dispatchmessage
	                   (Lpmsg => Windows.Data.Message'Access);
         else
            Draw_Rectangle (Windows.Data.Window_Handle);
         end if;
      end loop Peek_Loop;
   end Main_Windows_Message_Loop;



   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message : Win32.Uint;
      Wparam : Win32.Wparam;
      Lparam : Win32.Lparam) return Win32.Lresult is

      Hgdi_Object : Win32.Windef.Hgdiobj;

   begin

      case Message is
         when Win32.WinUser.Wm_Size =>
            Randrect.Client_Width  := Win32.Int(Win32.Windef.Loword
                                         (L => Win32.Dword(Lparam)));
            Randrect.Client_Height := Win32.Int(Win32.Windef.Hiword
                                         (L => Win32.Dword(Lparam)));
            return (0);

	 when Win32.WinUser.Wm_Destroy =>
	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 0);
	    return (0);

	 when others =>
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => Window_Handle, 
	       Msg    => Message, 
	       Wparam => Wparam,
	       Lparam => Lparam);
      end case;

      return (0);
   end Main_Windows;



   function Main
     (Instance : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line : Win32.Pcstr;
      Show_Command : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Windows.Data.Window_Class :=      
	   (Style         => Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => Instance,
	    Hicon         => Windows.Data.Window_Class_Icon,
	    Hcursor       => Windows.Data.Window_Class_Cursor,
	    Hbrbackground => Windows.Data.Window_Class_Background,
	    Lpszmenuname  => Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => Windows.Data.Class_Name
                                (Windows.Data.Class_Name'First)'Access);

	 Windows.Data.Aresult :=
	   win32.winuser.Registerclass
	   (Lpwndclass => Windows.Data.Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := Win32.WinUser.CreateWindow
	(Lpclassname => Windows.Data.Class_Name
                           (Windows.Data.Class_Name'First)'Access,
            Lpwindowname => Windows.Data.Caption
                           (Windows.Data.Caption'First)'Access,
            Dwstyle      => Windows.Data.Window_Style,
            X            => Windows.Data.Window_Horizontal_Start, 
            Y            => Windows.Data.Window_Vertical_Start,
            Nwidth       => Windows.Data.Window_Width,
            Nheight      => Windows.Data.Window_Height,
            Hwndparent   => Windows.Data.Window_Parent,
            Hmenu        => Windows.Data.Window_Menu,
            Hinstance    => Instance,
            Lpparam      => Windows.Data.Window_Parameters);

    Windows.Data.Bresult := Win32.Winuser.Showwindow
      (Hwnd => Windows.Data.Window_Handle, 
	 Ncmdshow =>win32.Winmain.Get_Ncmdshow);

    Windows.Data.Bresult := Win32.Winuser.Updatewindow
      (Hwnd => Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Windows.Data.Message.Wparam;

   end Main;

end Windows.Routines;
