-- Filename: Routines.adb
--
-- This is the conversion of Petzold's Win 95
-- Chapter 4 program Scramble.
-- 
-- Contains : 1. main_windows_message_loop
--            
--            2. function main          (winmain)
--
--            


-- with Ada.Numerics.Elementary_Functions;
with Ada.Unchecked_Conversion;
-- with Interfaces.C.Strings;
with System;

-- needed for all programs so far
with Win32;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winuser;

with Scramble;
with Windows.Data;


package body Windows.Routines is
--use Win32;  -- put in after all dot notation completed
             -- needed to see "and" and "or" for wparam
   use type Win32.Bool;
--   use type Win32.Char_Array;
   use type Win32.DWord;

   use type Win32.Int;
--   use type Win32.Uint;
   use type Win32.Windef.Hrgn;



   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message : Win32.Uint;
      Wparam : Win32.Wparam;
      Lparam : Win32.Lparam) return Win32.Lresult is
   begin
      return (0);
   end Main_Windows;



   function Main
     (Instance : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line : Win32.Pcstr;
      Show_Command : Win32.Int ) return Win32.Uint is


      Bit_Map_Handle : Win32.Windef.HBitMap;
      Device_Context_Handle : Win32.Windef.Hdc;
      Hgd_Iobj : Win32.Windef.Hgdiobj;
      Memory_Context_Handle : Win32.Windef.Hdc;
      X1, X2, Y1, Y2 : Win32.Int;

      function Convert_Sys_Adr_To_LpCstr is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.LpCstr);
         Null_LpCstr : Win32.LpCstr :=
                       Convert_Sys_Adr_To_LpCstr(System.Null_Address);

      function Convert_Sys_Adr_To_Ac_Devmodea_T is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Wingdi.Ac_Devmodea_T);
         Null_Ac_Devmodea_T : Win32.Wingdi.Ac_Devmodea_T :=
                       Convert_Sys_Adr_To_Ac_Devmodea_T(System.Null_Address);

      function Convert_Sys_Adr_To_Hwnd is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Windef.Hwnd);
         Null_Hwnd : Win32.Windef.Hwnd :=
                       Convert_Sys_Adr_To_Hwnd(System.Null_Address);

   begin

      if (Win32.Winuser.LockWindowUpdate
               (HWndLock =>Win32.Winuser.GetDesktopWindow) =
            Win32.True) then

         -- get device context, memory device context,
         -- screen width/height, and bit map handle
         Device_Context_Handle := Win32.Wingdi.CreateDc
                  (lpszDriver => Scramble.Driver_Name
                                 (Scramble.Driver_Name'First)'Access,
                   lpszDevice => Null_LpCstr,
                   lpszOutput => Null_LpCstr,
                   lpvInitData => Null_Ac_Devmodea_T);
         Memory_Context_Handle := Win32.Wingdi.CreateCompatibleDc
                               (Device_Context_Handle);
         Scramble.Screen_Width  := Win32.Winuser.GetSystemMetrics
                                (Win32.Winuser.Sm_CxScreen) / 10;
         Scramble.Screen_Height := Win32.Winuser.GetSystemMetrics
                                   (Win32.Winuser.Sm_CyScreen) / 10;
         Bit_Map_Handle := Win32.Wingdi.CreateCompatibleBitmap
                        (Hdc => Device_Context_Handle,
                         nWidth => Scramble.Screen_Width,
                         nHeight => Scramble.Screen_Height);

         -- get handle to graphic device - bit map
         Hgd_Iobj := Win32.Wingdi.SelectObject
                       (Hdc => Memory_Context_Handle,
                        Hgdiobj => Bit_Map_Handle);

         -- scramble the screen
         for I in 0 .. 1 loop
            for J in 0 .. (Scramble.Num - 1) loop
               if I = 0 then
                  X1 := Scramble.Screen_Width *
                        (Scramble.Random_Win32Int.Random (Scramble.Gen) mod 10);
                  Scramble.IKeep (J, 0) := X1;
                  Y1 := Scramble.Screen_Height *
                        (Scramble.Random_Win32Int.Random (Scramble.Gen) mod 10);
                  Scramble.IKeep (J, 1) := Y1;
                  X2 := Scramble.Screen_Width *
                        (Scramble.Random_Win32Int.Random (Scramble.Gen) mod 10);
                  Scramble.IKeep (J, 2) := X2;
                  Y2 := Scramble.Screen_Height *
                        (Scramble.Random_Win32Int.Random (Scramble.Gen) mod 10);
                  Scramble.IKeep (J, 3) := Y2;
               else
                  X1 := Scramble.IKeep (Scramble.Num - 1 - J, 0);
                  Y1 := Scramble.IKeep (Scramble.Num - 1 - J, 1);
                  X2 := Scramble.IKeep (Scramble.Num - 1 - J, 2);
                  Y2 := Scramble.IKeep (Scramble.Num - 1 - J, 3);
               end if;

               -- use bit blits to swap rectangular blocks
               Windows.Data.Bresult := Win32.Wingdi.Bitblt
                   (hdcDest => Memory_Context_Handle,
                    nXDest => 0,
                    nYDest => 0, 
                    nWidth => Scramble.Screen_Width,
                    nHeight=> Scramble.Screen_Height,
                    hdcSrc=>  Device_Context_Handle,
                    nXSrc =>  X1,
                    nYSrc =>  Y1,
                    dwRop =>  Win32.Wingdi.SrcCopy);
               Windows.Data.Bresult := Win32.Wingdi.Bitblt
                   (hdcDest => Device_Context_Handle,
                    nXDest => X1,
                    nYDest => Y1, 
                    nWidth => Scramble.Screen_Width,
                    nHeight=> Scramble.Screen_Height,
                    hdcSrc=>  Device_Context_Handle,
                    nXSrc =>  X2,
                    nYSrc =>  Y2,
                    dwRop =>  Win32.Wingdi.SrcCopy);
               Windows.Data.Bresult := Win32.Wingdi.Bitblt
                   (hdcDest => Device_Context_Handle,
                    nXDest => X2,
                    nYDest => Y2, 
                    nWidth => Scramble.Screen_Width,
                    nHeight=> Scramble.Screen_Height,
                    hdcSrc=>  Memory_Context_Handle,
                    nXSrc =>  0,
                    nYSrc =>  0,
                    dwRop =>  Win32.Wingdi.SrcCopy);

               Win32.Winbase.Sleep (dwMilliseconds => Win32.DWord(10));
            end loop;
         end loop;

         -- Delete device contexts and object
         Windows.Data.Bresult := Win32.Wingdi.DeleteDc
                                 (Hdc => Memory_Context_Handle);
         Windows.Data.Bresult := Win32.Wingdi.DeleteDc
                                 (Hdc => Device_Context_Handle);
         Windows.Data.Bresult := Win32.Wingdi.DeleteObject
                                 (Hgdiobj => Bit_Map_Handle);

         -- enable drawing in the locked window
         Windows.Data.Bresult := Win32.Winuser.LockWindowUpdate
                                 (HWndLock => Null_Hwnd);
      end if;

      return (Win32.False);
   end Main;

end Windows.Routines;
