with Ada.Unchecked_Conversion;
with Ada.Unchecked_Deallocation;
with Interfaces.C.Strings;
with System;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
   use type Interfaces.C.Strings.Chars_Ptr;
   use type System.Address;
   use type Win32.Dword;
   use type Win32.Bool;
   use type Win32.Byte;
   use type Win32.Int;
   use type Win32.Long;
   use type Win32.Uint;

   Text_Metric           : aliased win32.Wingdi.Textmetric;

   Rectangle             : aliased Win32.Windef.Rect;

   Character_Width       : Win32.Long;

   Character_Height      : Win32.Long;

   Client_Width          : Win32.Long;

   Caret_Height          : Win32.Long;

   Caret_Width           : Win32.Long;

   Client_Height         : Win32.Long;

   Buffer_Width          : Win32.Long;

   Buffer_Height         : Win32.Long;

   Bitmap_Pointer        : constant Win32.Windef.Hbitmap := null;

   Width                 : Win32.Long;

   Height                : Win32.Long;

   True                  : constant Win32.bool := 1;

   type Buffer_Pointer_Type  is access String;

   Buffer_Pointer        : Buffer_Pointer_Type := null;

   Paint_Record          : aliased win32.Winuser.Paintstruct;

   Window_Class          : aliased Win32.Winuser.Wndclass;

   Message               : aliased Win32.Winuser.Msg;

   Message_Box_Text      : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       ("Window Too Large. Cannot allocate enough memory");

   Device_Context_Handle : aliased Win32.Windef.Hdc;

   -- define dummy variables to hold results of function calls
   Integer_Result : Win32.Int;

   Bool_Result     : Win32.Bool;

   Uint_Result     : Win32.Uint;

   Long_Result     : Win32.Long;

   Atom_Result     : Win32.Windef.Atom;

   Object_Result   : Win32.Windef.Hgdiobj;

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => System.Address,
        Target => Win32.Lpcstr);

   procedure free is new Ada.Unchecked_Deallocation
     (Object => 
        String,
       Name  =>
        Buffer_Pointer_Type);

   function Min(l,r : integer) return integer is
   begin
     if l < r then
       return l;
     else
       return r;
     end if;
   end Min;

   function Max(l,r : integer) return integer is
   begin
     if l > r then
       return l;
     else
       return r;
     end if;
   end Max;

   function Create
     (Window_Handle : in Win32.Windef.Hwnd;
      Width         : in Win32.Long;
      Height        : in Win32.Long) return 
        Buffer_Pointer_Type  is

     Temporary_Array_Pointer : Buffer_Pointer_Type := 
           new String(1 .. integer(Width * Height));
   begin

     for i in 1 .. temporary_array_Pointer'Length loop
 
      temporary_array_Pointer(i) := ' ';

     end loop;

     return temporary_Array_Pointer;

   exception

     when others =>

       Integer_Result :=
         Win32.Winuser.MessageBox
           (Hwnd      => Window_Handle,
            Lptext    => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Message_Box_Text),
            Lpcaption =>
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Utype     =>
             Win32.Winuser.Mb_Iconexclamation or
             Win32.Winuser.Mb_Ok);

   end Create;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               System.Null_Address, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

	    Device_Context_Handle := 
              Win32.Winuser.Getdc
               (Hwnd =>
                 Window_Handle);

            Object_Result :=
              Win32.Wingdi.SelectObject
                (Hdc    =>
                  Device_Context_Handle,
                 Hgdiobj => 
                  Win32.Wingdi.Getstockobject
                    (Fnobject => 
                      Win32.Wingdi.System_Fixed_Font));

	    Bool_Result := 
              Win32.Wingdi.Gettextmetrics
                (Hdc =>
                  Device_Context_Handle,
                 Lptm =>
                  Text_Metric'Access);

            Character_Width :=
              Text_Metric.Tmavecharwidth;

            Character_Height := 
              Text_Metric.Tmheight;

            Integer_Result := 
              Win32.Winuser.Releasedc
                (Hwnd =>
                  Window_Handle,
                 Hdc  =>
                  Device_Context_Handle);

            return (0);

	 when Win32.Winuser.Wm_Size =>

            Client_Width :=
              Win32.Long
                (Win32.Utils.Loword
                   (Win32.Dword
                     (Lparam)));
                                         
            Client_Height :=
              Win32.Long
                (Win32.Utils.Hiword
                   (Win32.Dword
                     (Lparam)));

            Buffer_Width :=
              Win32.Long
                (Max
                  (L => 
                    1,
                   R => 
                    Integer
                      (Client_Width /
                       Character_Width)));

            Buffer_Height :=
              Win32.Long
                (Max
                  (L => 
                    1,
                   R => 
                    Integer
                      (Client_Height /
                       Character_Height)));

            if Buffer_Pointer /= null then

              Free
                (X => 
                  Buffer_Pointer);

            end if;

            Buffer_Pointer := 
              Create
                (Window_Handle => 
                  Window_Handle,
                 Width         => 
                  Buffer_Width,
                 Height => 
                  Buffer_Height);

            Caret_Width := 
              0;

            Caret_Height := 
              0;

           if Window_Handle = 
                Win32.Winuser.Getfocus then
             
             Bool_Result :=
               Win32.Winuser.SetCaretPos
                 (X =>
                   Win32.Int
                     (Caret_Width *
                      Character_Width),
                  Y => 
                   Win32.Int
                     (Caret_Height *
                      Character_Height));

           end if;
 
           return (0);

	 when Win32.Winuser.Wm_Setfocus =>

           Bool_Result :=
             Win32.Winuser.CreateCaret
               (Hwnd    =>
                 Window_Handle,
                Hbitmap =>
                 Bitmap_Pointer,
                Nwidth  => 
                 Win32.Int
                   (Character_Width),
                Nheight => 
                 Win32.Int
                   (Character_Width));

             Bool_Result :=
               Win32.Winuser.SetCaretPos
                 (X =>
                   Win32.Int
                     (Caret_Width *
                      Character_Width),
                  Y => 
                   Win32.Int
                     (Caret_Height *
                      Character_Height));

             Bool_Result :=
               Win32.Winuser.ShowCaret
                 (Hwnd =>
                   Window_Handle);

           return (0);

	 when Win32.Winuser.Wm_Killfocus =>
           
           Bool_Result :=
             Win32.Winuser.Hidecaret
               (Hwnd =>
                 Window_Handle);
    
           Bool_Result :=
             Win32.Winuser.Destroycaret;

           return (0);

	 when Win32.Winuser.Wm_Keydown =>

           case Wparam is

             when Win32.Winuser.Vk_Home =>

               Caret_Width := 
                 0;

             when Win32.Winuser.Vk_End =>

               Caret_Width :=
                 Buffer_Width - 1;

             when Win32.Winuser.Vk_Prior =>

               Caret_Height := 
                 0;

             when Win32.Winuser.Vk_Next =>

               Caret_Height :=
                 Buffer_Height - 1;

             when Win32.Winuser.Vk_Left =>

               Caret_Width :=
                 Win32.Long
                   (Max
                     (L => 
                       Integer
                         (Caret_Width - 1),
                      R => 
                       0 ));

             when Win32.Winuser.Vk_Right =>

               Caret_Width :=
                 Win32.Long
                   (Min
                     (L => 
                       Integer
                         (Caret_Width + 1),
                      R => 
                       Integer
                       (Buffer_Width  - 1)));

             when Win32.Winuser.Vk_Up =>

               Caret_Height :=
                 Win32.Long
                   (Max
                     (L => 
                       Integer
                         (Caret_Height - 1),
                      R => 
                       0 ));

             when Win32.Winuser.Vk_Down =>

               Caret_Height :=
                 Win32.Long
                   (Min
                     (L => 
                       Integer
                         (Caret_Height + 1),
                      R => 
                       Integer
                         (Buffer_Height  - 1)));

           
             when Win32.Winuser.Vk_Delete =>

               for i in Caret_Width ..
                        Buffer_Width -1 loop
               
                 Buffer_Pointer
                   (Integer
                     (Caret_Height * buffer_width + i) + 1) :=
                   Buffer_Pointer
                     (Integer
                       (Caret_Height * buffer_width + 
                         (i + 1)) + 1);

               end loop;
                  
                 Buffer_Pointer
                   (Integer
                     (Caret_Height * buffer_width + 
                       (Buffer_Width -1)) + 1) := 
                         ' ';

               Bool_Result :=
                 Win32.Winuser.Hidecaret
                  (Hwnd =>
                    Window_Handle);

 	       Device_Context_Handle := 
                 Win32.Winuser.Getdc
                   (Hwnd =>
                      Window_Handle);
    
               Object_Result :=
                 Win32.Wingdi.Selectobject
                   (Hdc    => 
                     Device_Context_Handle,
                    Hgdiobj => 
                     Win32.Wingdi.Getstockobject
                      (Fnobject => 
                        Win32.Wingdi.System_Fixed_Font));

               Bool_Result := 
                 Win32.Wingdi.Textout
	           (Hdc   =>
                     Device_Context_Handle,
                    Nxstart  => 
                     Win32.Int
                       (Caret_Width *
                        Character_Width),
                    Nystart  => 
                     Win32.Int
                       (Caret_Height *
                        Character_Height),
                    Lpstring => 
                     Convert_Char_Ptr_To_Lpcstr
                       (S =>
                         Buffer_Pointer
                           (Integer
                             (Caret_Height * 
                              Buffer_Width + 
                              Caret_Width) + 1)'Address),
                    Cbstring => 
                     Win32.Int
                       (Buffer_Width - 
                        Caret_Width));

               Bool_Result :=
                 Win32.Winuser.ShowCaret
                   (Hwnd =>
                     Window_Handle);

               Integer_Result := 
                 Win32.Winuser.Releasedc
                   (Hwnd =>
                     Window_Handle,
                    Hdc  =>
                     Device_Context_Handle);

           when others => 

             null;
 
           end case;

           Bool_Result :=
             Win32.Winuser.Setcaretpos
               (X =>
                 Win32.Int
                   (Caret_Width *
                    Character_Width),
                Y => 
                 Win32.Int
                   (Caret_Height *
                    Character_Height));

           return (0);

	 when Win32.Winuser.Wm_Char =>
            
           for i in 1 .. Integer
                           (Win32.Utils.Loword
                             (Win32.Dword
                               (Lparam))) loop

             case Wparam is


               when 8 =>

                 if Caret_Width > 0 then

                   Caret_Width :=
                     Caret_Width - 1;

                   Long_Result :=
                     Win32.Winuser.Sendmessage
                       (Hwnd   => 
                         Window_Handle,
                        Msg    => 
                         Win32.Winuser.Wm_Keydown,
                        Wparam => 
                         Win32.Winuser.Vk_Delete,
                        Lparam => 
                         1);

                 end if;

               when  9 =>
 
                 while Caret_Width mod 8 /= 0 loop

                   Long_Result :=
                     Win32.Winuser.Sendmessage
                       (Hwnd   => 
                         Window_Handle,
                        Msg    => 
                         Win32.Winuser.Wm_Char,
                        Wparam => 
                         Win32.Winuser.Vk_Space,
                        Lparam => 
                         1);

                 end loop;                    

               when  12 =>

                 Caret_Height :=
                   Caret_Height + 1;
     
                 if Buffer_Height = 
                      Caret_Height then

                    Caret_Height := 
                      0;

                 end if;

               when  13 =>

                 Caret_Width :=
                   0 ;
     
                 Caret_Height :=
                   Caret_Height + 1;
     
                 if Buffer_Height = 
                      Caret_Height then

                    Caret_Height := 
                      0;

                 end if;

               when  27 =>

                 for y in 0 .. Buffer_Height - 1 loop

                   for x in 0 .. Buffer_Width - 1 loop

                     Buffer_Pointer
                       (Integer
                         (Y * Buffer_Width + X) + 1) :=
                       ' ';

                   end loop; 

                 end loop;

                 Caret_Width := 
                   0;

                 Caret_Height := 
                   0;

                 Bool_Result :=
                   Win32.Winuser.Invalidaterect
                     (Hwnd   =>
                       Window_Handle,
                      Lprect => 
                       null,
                      Berase =>
                       True);

               when others =>

                 Buffer_Pointer
                   (Integer
                     (Caret_Height * Buffer_Width + Caret_Width) + 1) :=
                       Character'Val
                         (Integer
                           (Wparam));

                 Bool_Result :=
                   Win32.Winuser.Hidecaret
                    (Hwnd => 
                      Window_Handle);

 	         Device_Context_Handle := 
                   Win32.Winuser.Getdc
                     (Hwnd =>
                       Window_Handle);

                 Object_Result :=
                   Win32.Wingdi.Selectobject
                     (Hdc    => 
                       Device_Context_Handle,
                      Hgdiobj => 
                       Win32.Wingdi.Getstockobject
                        (Fnobject => 
                          Win32.Wingdi.System_Fixed_Font));

                Bool_Result := 
                  Win32.Wingdi.Textout
	            (Hdc   =>
                      Device_Context_Handle,
                     Nxstart  => 
                      Win32.Int
                        (Caret_Width *
                         Character_Width),
                     Nystart  => 
                      Win32.Int
                        (Caret_Height *
                         Character_Height),
                     Lpstring => 
                      Convert_Char_Ptr_To_Lpcstr
                        (S =>
                          Buffer_Pointer
                            (Integer
                              (Caret_Height * Buffer_Width + Caret_Width) + 1)'Address),
                     Cbstring => 
                      1);

                Caret_Width :=
                  Caret_Width + 1;

                if Caret_Width = 
                     Buffer_Width then

                  Caret_Width := 
                    0;

                  Caret_Height :=
                    Caret_Height + 1;

                  if Caret_Height = 
                       Buffer_Height then

                    Caret_Height := 
                     0;

                  end if;

                end if;

                Bool_Result :=
                  Win32.Winuser.Showcaret
                    (Hwnd =>
                      Window_Handle);

                Integer_Result := 
                  Win32.Winuser.Releasedc
                   (Hwnd =>
                     Window_Handle,
                    Hdc  =>
                     Device_Context_Handle);

             end case;

           end loop;

           Bool_Result :=
             Win32.Winuser.Setcaretpos
               (X =>
                 Win32.Int
                   (Caret_Width *
                    Character_Width),
                Y => 
                 Win32.Int
                   (Caret_Height *
                    Character_Height));

            return (0);

	 when Win32.Winuser.Wm_Paint =>

	   Device_Context_Handle :=
             Win32.Winuser.Beginpaint
	       (Hwnd    =>
                 Window_Handle,
	        Lppaint =>
                 Paint_Record'Access);

           Object_Result :=
             Win32.Wingdi.Selectobject
               (Hdc    => 
                 Device_Context_Handle,
                Hgdiobj => 
                 Win32.Wingdi.Getstockobject
                   (Fnobject => 
                     Win32.Wingdi.System_Fixed_Font));

             for y in 0 .. Buffer_Height - 1 loop   

                Bool_Result := 
                  Win32.Wingdi.Textout
	            (Hdc   => 
                      Device_Context_Handle,
                     Nxstart  => 
                      0, 
                     Nystart  => 
                      Win32.Int
                        (y *
                         Character_Height),
                     Lpstring => 
                      Convert_Char_Ptr_To_Lpcstr
                        (S =>
                          Buffer_Pointer
                            (integer(Y * Buffer_Width)+ 1)'Address),
                     Cbstring => 
                      Win32.Int
                        (Buffer_Width));
             
             end loop;
    
	     Bool_Result :=
               Win32.Winuser.Endpaint
                 (Hwnd    =>
                   Window_Handle,
                  Lppaint =>
                   Paint_Record'Access);

	    return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
                0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
