with Ada.Numerics.Generic_Elementary_Functions;
with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Crt.Time;
with Win32.Crt.Types;
with Win32.Utils;
with Win32.Windef;
with Win32.Winbase;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type Interfaces.C.Unsigned;
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Float;
  use type Win32.Int;
  use type Win32.Long;

   package Math is new Ada.Numerics.Generic_Elementary_Functions(Win32.Float);

   Two_Pi : constant Win32.Float :=
     Win32.Float( 2.0 * Ada.Numerics.Pi);
  
   type Point_Array_Type is array(integer range <>) of Win32.Windef.Point;

   type Point_Array_Array_Type is array(integer range <>) of Point_Array_Type(0 .. 4);

   Message_Box_Text         : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str =>
         "Too many clocks or timers!");

   Date_And_Time            : Win32.Crt.Time.A_Tm_T;

   Id_Timer                 : constant Win32.Uint := 1;

   Previous_Date_And_Time   : Win32.Crt.Time.A_Tm_T := new Win32.Crt.Time.Tm'(0,0,0,0,0,0,0,0,0);

   True                     : constant Win32.Bool := 1;

   False                    : constant Win32.Bool := 0;

   Change_Time              : Win32.Bool := False;

   Client_Width             : Win32.Int;

   Client_Height            : Win32.Int;

   Temp_Time : aliased Win32.Crt.Types.Time_T;

   Time_T_Result : aliased Win32.Crt.Types.Time_T;

   Paint_Record             : aliased win32.Winuser.Paintstruct;

   Rectangle                : aliased win32.Windef.Rect;

   Window_Class             : aliased Win32.Winuser.Wndclass;

   Message                  : aliased Win32.Winuser.Msg;

   Device_Context_Handle    : aliased Win32.Windef.Hdc;

   Integer_Time             : Win32.Uint;

   Integer_Date             : Win32.Uint;

   Integer_Result           : Win32.Int;

   Uint_Result              : Win32.Uint;

   Dword_Result             : Win32.Dword;

   Bool_Result              : Win32.Bool;

   Long_Result              : Win32.Long;

   Atom_Result              : Win32.Windef.Atom;

   Object_Result   : Win32.Windef.Hgdiobj;

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

--   function Convert_Address_To_Ac_Tm_t is new
--     Ada.Unchecked_Conversion
--       (Source => System.Address,
--        Target => Win32.Crt.Time.Ac_Tm_T);

   function Convert_Address_To_Ac_Time_T is new
     Ada.Unchecked_Conversion
       (Source => System.Address,
        Target => Win32.Crt.Time.Ac_Time_T);

   function Convert_Address_To_Ac_Point_T is new
     Ada.Unchecked_Conversion
       (Source => System.Address,
        Target => Win32.Wingdi.Ac_Point_T);

   function Convert_Integer_To_Psize is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Windef.Psize);

   function Convert_Address_To_Ppoint is new
     Ada.Unchecked_Conversion
       (Source => System.Address,
        Target => Win32.Windef.Ppoint);

  procedure Set_Isotropic
     (Device_Context_Handle : Win32.Windef.Hdc;
      Client_Width          : Win32.Int;
      Client_Height         : Win32.Int) is 
                                           
   Null_Psize : Win32.Windef.Psize; 

   Null_Ppoint : Win32.Windef.Ppoint;  

   function Setwindowextex(hdc     : Win32.Windef.HDC;
                  nXExtent: Win32.INT;
                  nYExtent: Win32.INT;
                  lpSize  : Win32.Windef.PSIZE)
                                return Win32.BOOL;          
    pragma Import(Stdcall, SetWindowExtEx, "SetWindowExtEx");

    function SetViewportExtEx(hdc     : Win32.Windef.HDC;
                              nXExtent: Win32.INT;
                              nYExtent: Win32.INT;
                              lpSize  : Win32.Windef.PSIZE)
                                  return Win32.BOOL;
    pragma Import(Stdcall, SetViewportExtEx, "SetViewportExtEx");

    function SetViewportOrgEx(hdc    : Win32.Windef.HDC;
                              X      : Win32.INT;
                              Y      : Win32.INT;
                              lpPoint: Win32.Windef.PPOINT)
                                  return Win32.BOOL;
    pragma Import(Stdcall, SetViewportOrgEx, "SetViewportOrgEx");

   begin

   Null_Psize := 
     Convert_Integer_To_Psize
      (S => 
        0);

   Null_Ppoint :=  
     Convert_Address_To_Ppoint
      (S => 
        System.Null_Address);

     Integer_Result :=
       Win32.Wingdi.Setmapmode
         (Hdc =>
           Device_Context_Handle,
          Fnmapmode =>
           Win32.Wingdi.Mm_Isotropic);
 
     Bool_Result :=
--       Win32.Wingdi.Setwindowextex
       Setwindowextex
         (Hdc =>
           Device_Context_Handle,
          Nxextent =>
           1000,
          Nyextent =>
           1000,
          Lpsize   =>
           --Convert_Address_To_Psize
             --(S => 
               --Temp_Null));
           Null_Psize);

     Bool_Result :=
--       Win32.Wingdi.Setviewportextex
       Setviewportextex
         (Hdc =>
           Device_Context_Handle,
          Nxextent =>
           Client_Width / 2,
          Nyextent =>
           - Client_Height / 2,
          Lpsize   =>
           --Convert_Address_To_Psize
             --(S =>
                --Temp_Null));
           Null_Psize);

     Bool_Result :=
--       Win32.Wingdi.Setviewportorgex
       Setviewportorgex
         (Hdc =>
           Device_Context_Handle,
          X =>
           Client_Width / 2,
          Y =>
           Client_Height / 2,
          Lppoint   =>
           --Convert_Address_To_Ppoint
             --(S => 
               --Temp_Null));
           Null_Ppoint);

   end Set_Isotropic;

   procedure Rotate_Point
     (Point_Array : in out Point_Array_Type;
      Number      : Integer;
      Angle       : Integer) is

     Temporary_Point : Win32.Windef.Point;

   begin

     for i in 0 .. Number - 1 loop

       Temporary_Point.X :=
         Win32.Long
           (Win32.Float
              (Point_Array(i).X) * 
            Math.Cos
              (Two_Pi *
                Win32.Float
                  (Angle) / 
                360.0) +
            Win32.Float
              (Point_Array(i).Y) * 
            Math.Sin
             (Two_Pi *
               Win32.Float
                 (Angle) / 
               360.0));

       Temporary_Point.Y :=
         Win32.Long
           (Win32.Float
             (Point_Array(i).Y) * 
            Math.Cos
              (Two_Pi *
               Win32.Float
                 (Angle) / 
               360.0) -
            Win32.Float
              (Point_Array(i).X) * 
            Math.Sin
             (Two_Pi *
              Win32.Float
                (Angle) / 
              360.0));

         Point_Array(i) := Temporary_Point;

     end loop;     

   end Rotate_Point;

   procedure Draw_Clock
     (Device_Context_Handle : Win32.Windef.Hdc) is

     Point_Array : Point_Array_Type(0 .. 2);
   
     Angle : Win32.Int := 0;

   begin

     while Angle < 360 loop

      Point_Array(0).X :=
         0;

       Point_Array(0).Y :=
         900;
   
       Rotate_Point
         (Point_Array =>
           Point_Array,
          Number =>
           1,
          Angle =>
           Integer
             (Angle));
       
       if Angle mod 5 > 0 then

         Point_Array(2).X :=
           33;

       else

         Point_Array(2).X :=
           100;

       end if;

       Point_Array(2).Y :=
         Point_Array(2).X;

       Point_Array(0).X :=
         Point_Array(0).X - Point_Array(2).X / 2;

       Point_Array(0).Y :=
         Point_Array(0).Y - Point_Array(2).Y / 2;

       Point_Array(1).X :=
         Point_Array(0).X +
         Point_Array(2).X;

       Point_Array(1).Y :=
         Point_Array(0).Y +
         Point_Array(2).Y;

       Object_Result :=
         Win32.Wingdi.Selectobject
           (Hdc    => 
             Device_Context_Handle,
            Hgdiobj => 
             Win32.Wingdi.Getstockobject
               (Fnobject => 
                 Win32.Wingdi.Black_Brush));

       Bool_Result :=
         Win32.Wingdi.Ellipse
           (Hdc =>
             Device_Context_Handle,
            Nleftrect =>
             Win32.Int
               (Point_Array(0).X),
            Ntoprect  =>
             Win32.Int
               (Point_Array(0).Y),
            Nrightrect  =>
             Win32.Int
               (Point_Array(1).X),
            Nbottomrect =>
             Win32.Int
               (Point_Array(1).Y));

       Angle :=
         Angle + 6;
           
     end loop;

   end Draw_Clock;

   procedure Draw_Hands
     (Device_Context_Handle : Win32.Windef.Hdc;
      Date_And_Time         : Win32.Crt.Time.A_Tm_T;
      Change_Time           : Win32.Bool) is

      Points                : Point_Array_Array_Type(0 .. 2) :=
       (0 => (0 => (0,-150),1 => (100,0),2 => (0,600),3 => (-100,0),4=> (0,-150)),
        1 => (0 => (0,-200),1 => ( 50,0),2 => (0,800),3 => (- 50,0),4=> (0,-200)),
        2 => (0 => (0,   0),1 => (  0,0),2 => (0,  0),3 => (   0,0),4=> (0, 800)));

--      Temporary_Points      : Point_Array_Array_Type(0 .. 2,0 .. 4);

      Angle_Array           : array(0 .. 2) of integer;

      Temporary_Points : Point_Array_Array_Type(0 .. 2) := Points;

      Counter : integer := 0 ;     

   begin
 
     Angle_Array(0) :=
       Integer
         ((Date_And_Time.Tm_Hour * 30) mod 360 + Date_And_Time.Tm_Min / 2);

     Angle_Array(1) :=
       Integer
         (Date_And_Time.Tm_Min * 6);

     Angle_Array(2) :=
       Integer
         (Date_And_Time.Tm_Sec * 6);

     if Change_Time > 0 then

       Counter :=
         0;

     else

       Counter :=
         2;

     end if;

     while Counter < 3 loop

       Rotate_Point
         (Point_Array =>
           Temporary_Points
             (Counter),
          Number      =>
           5,
          Angle       =>
           Angle_Array
             (Counter));

       Bool_Result :=
         Win32.Wingdi.Polyline
           (Hdc     =>
             Device_Context_Handle,
            Lppt    =>
             Convert_Address_To_Ac_Point_T
               (Temporary_Points
                 (Counter)'Address),
            Cpoints =>
             5);

       Counter :=
         Counter + 1;

     end loop;
     
   end Draw_Hands;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.WinUser.Wm_Create =>

           Time_T_Result :=
             Win32.Crt.Time.Time
               (Temp_Time'Access);
               
               
--       function localtime(timer: ac_time_t) return a_tm_t;     -- time.h:142
           Date_And_Time :=
             Win32.Crt.Time.Localtime
               (Timer =>
                 Convert_Address_To_Ac_Time_T
                   (S =>
                     Temp_Time'Address));

           Previous_Date_And_Time.all :=
             Date_And_Time.all;

	   return (0);

	 when Win32.WinUser.Wm_Size =>

           Client_Width :=
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)));

           Client_Height :=
             Win32.Int
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam)));

	   return (0);

	 when Win32.WinUser.Wm_Timer =>

           Time_T_Result :=
             Win32.Crt.Time.Time
               (Temp_Time'Access);

           Date_And_Time :=
             Win32.Crt.Time.Localtime
               (Timer =>
                 Convert_Address_To_Ac_Time_T
                   (S =>
                     Temp_Time'Address));
         
           if Previous_Date_And_Time.Tm_Hour /= Date_And_Time.Tm_Hour or
              Previous_Date_And_Time.Tm_Sec  /= Date_And_Time.Tm_Sec then 

             Change_Time :=
               True;

           else

             Change_Time :=
               False;

           end if;

	   Device_Context_Handle := 
             Win32.Winuser.Getdc
               (Hwnd =>
                 Window_Handle);

           Set_Isotropic
             (Device_Context_Handle =>
               Device_Context_Handle,
              Client_Width          =>
               Client_Width,
              Client_Height         =>
               Client_Height);

           Object_Result :=
             Win32.Wingdi.Selectobject
               (Hdc    => 
                 Device_Context_Handle,
                Hgdiobj => 
                 Win32.Wingdi.Getstockobject
                   (Fnobject => 
                     Win32.Wingdi.White_Pen));

           Draw_Hands
             (Device_Context_Handle =>
               Device_Context_Handle,
              Date_And_Time         =>
               Previous_Date_And_Time,
              Change_Time           =>
               Change_Time);

           Object_Result :=
             Win32.Wingdi.Selectobject
               (Hdc    => 
                 Device_Context_Handle,
                Hgdiobj => 
                 Win32.Wingdi.Getstockobject
                   (Fnobject => 
                     Win32.Wingdi.Black_Pen));

           Draw_Hands
             (Device_Context_Handle =>
               Device_Context_Handle,
              Date_And_Time         =>
               Date_And_Time,
              Change_Time           =>
               Change_Time);

           Integer_Result := 
             Win32.Winuser.Releasedc
               (Hwnd =>
                 Window_Handle,
                Hdc  =>
                 Device_Context_Handle);

           Previous_Date_And_Time.all :=
             Date_And_Time.all;

	   return (0);

	 when Win32.WinUser.Wm_Paint =>

	   Device_Context_Handle :=
             Win32.Winuser.Beginpaint
	       (Hwnd    =>                                                       
                 Window_Handle,
	        Lppaint =>
                 Paint_Record'Access);

           Set_Isotropic
            (Device_Context_Handle =>
              Device_Context_Handle,
              Client_Width          =>
               Client_Width,
              Client_Height         =>
               Client_Height);

           Draw_Clock
             (Device_Context_Handle =>
               Device_Context_Handle);

           Draw_Hands
             (Device_Context_Handle =>
               Device_Context_Handle,
              Date_And_Time         =>
               Previous_Date_And_Time,
              Change_Time           =>
               Change_Time);

    	   Bool_Result :=
             Win32.Winuser.Endpaint
               (Hwnd    =>
                 Window_Handle,
                Lppaint =>
                 Paint_Record'Access);

	   return (0);

	 when Win32.WinUser.Wm_Destroy =>

            Bool_Result :=
              Win32.Winuser.Killtimer
                (Hwnd     =>
                  Window_Handle,
                 Uidevent =>
                  Id_Timer);

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
                0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Uint_Result :=
     0;

    while Uint_Result = 0 loop 

      Uint_Result :=
        Win32.Winuser.Settimer
          (Hwnd        =>
            Windows.Data.Window_Handle,
           Nidevent    =>
            Id_Timer,
           Uelapse     =>
            1_000,
           Lptimerfunc =>
            null);

      if Uint_Result = 0 then

        Integer_Result :=
          Win32.Winuser.Messagebox
             (Hwnd      => 
               Windows.Data.Window_Handle,
              Lptext    => 
               Convert_Char_Ptr_To_Lpcstr
                 (S => 
                   Message_Box_Text),
              Lpcaption =>
               Convert_Char_Ptr_To_Lpcstr
                 (S => 
                   Windows.Data.Class_Name),
              Utype     =>
               Win32.Winuser.Mb_Iconexclamation or
               Win32.Winuser.Mb_Ok);

        return(0);     

      end if;

    end loop;

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
