with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Crt.Time;
with Win32.Crt.Types;
with Win32.Windef;
with Win32.Winbase;
with Win32.Wingdi;
with Win32.Winmain;
--with Win32.Time;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type Interfaces.C.Unsigned;
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Int;


   Message_Box_Text         : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str =>
         "Too many clocks or timers!");

   Date_Separator            : Interfaces.C.Strings.Chars_Ptr :=
       Interfaces.C.Strings.New_String
         (Str =>
           "  ");

   Time_Separator           : Interfaces.C.Strings.Chars_Ptr :=
       Interfaces.C.Strings.New_String
         (Str =>
           "  ");

   AMPM : array ( 0 .. 1) of Interfaces.C.Strings.Chars_Ptr :=
     (0 =>
       Interfaces.C.Strings.New_String
         (Str =>
           "     "),
      1 =>
       Interfaces.C.Strings.New_String
         (Str =>
           "     "));

   True                     : constant Win32.bool := 1;

   Paint_Record             : aliased win32.Winuser.Paintstruct;

   Rectangle                : aliased win32.Windef.Rect;

   Window_Class             : aliased Win32.Winuser.Wndclass;

   Message                  : aliased Win32.Winuser.Msg;

   Device_Context_Handle    : aliased Win32.Windef.Hdc;

   Id_Timer                 : constant Win32.Uint := 1;

   Integer_Time             : Win32.Uint;

   Integer_Date             : Win32.Uint;

   Integer_Result           : Win32.Int;

   Uint_Result              : Win32.Uint;

   Dword_Result             : Win32.Dword;

   Bool_Result              : Win32.Bool;

   Long_Result              : Win32.Long;

   Atom_Result              : Win32.Windef.Atom;

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Char_Ptr_To_Lpstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpstr);

   procedure Size_The_Window
     (Starting_X    : in out Win32.Int;
      Starting_Y    : in out Win32.Int;
      Client_Width  : in out Win32.Int;
      Client_Height : in out Win32.Int) is

      Device_Context_Handle    : aliased Win32.Windef.Hdc;

      Text_Metric              : aliased win32.Wingdi.Textmetric;

   begin

     Device_Context_Handle :=
       Win32.Wingdi.Createic
         (Lpszdriver  =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "Display")),
          Lpszdevice =>
           null,
          Lpszoutput =>
           null,
          Lpvinitdata =>
           null);

     Bool_Result :=
       Win32.Wingdi.Gettextmetrics
         (Hdc  => 
           Device_Context_Handle,
          Lptm =>
           Text_Metric'Access);

     Client_Width :=
       2 * 
       Win32.Winuser.Getsystemmetrics
         (Nindex =>
           Win32.Winuser.Sm_Cxdlgframe) +
       16 * Win32.Int(Text_Metric.Tmavecharwidth);

     Starting_X :=
       Win32.Winuser.Getsystemmetrics
         (Nindex =>
           Win32.Winuser.Sm_Cxscreen) -
       Client_Width;

     Client_Height :=
       2 * 
       Win32.Winuser.Getsystemmetrics
         (Nindex =>
           Win32.Winuser.Sm_Cydlgframe) +
       2 * Win32.Int(Text_Metric.Tmheight);

     Starting_Y :=
       0;

     Bool_Result := 
       Win32.Wingdi.Deletedc
         (Hdc  =>
           Device_Context_Handle);

   end Size_The_Window;

   procedure Set_International is

     Application_Name  : constant Interfaces.C.Strings.Chars_Ptr :=
       Interfaces.C.Strings.New_String
         (Str =>
           "intl");

     foo : Win32.Lpcstr;

     two : win32.dword := 2;
 
   begin

     Integer_Date :=
       Win32.Winbase.Getprofileint
         (Lpappname =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Application_Name),

          Lpkeyname =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "iDate")),
          Ndefault  =>
           0);

     Integer_Time :=
       Win32.Winbase.Getprofileint
         (Lpappname =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Application_Name),

          Lpkeyname =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "iTime")),
          Ndefault  =>
           0);

    Dword_Result :=
      Win32.Winbase.Getprofilestring
        (Lpappname        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Application_Name),
         Lpkeyname        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "sDate")),
         Lpdefault        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "/")),
         Lpreturnedstring =>
           Convert_Char_Ptr_To_Lpstr
             (S =>
               Date_Separator),
         Nsize            =>
          2);

    Dword_Result :=
      Win32.Winbase.Getprofilestring
        (Lpappname        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Application_Name),
         Lpkeyname        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "sTime")),
         Lpdefault        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "/")),
         Lpreturnedstring =>
           Convert_Char_Ptr_To_Lpstr
             (S =>
               Time_Separator),
         Nsize            =>
          2);

    Dword_Result :=
      Win32.Winbase.Getprofilestring
        (Lpappname        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Application_Name),
         Lpkeyname        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "s1159")),
         Lpdefault        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "AM")),
         Lpreturnedstring =>
           Convert_Char_Ptr_To_Lpstr
             (S =>
               AMPM(0)),
         Nsize            =>
          5);

    Dword_Result :=
      Win32.Winbase.Getprofilestring
        (Lpappname        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Application_Name),
         Lpkeyname        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "s2359")),
         Lpdefault        =>
           Convert_Char_Ptr_To_Lpcstr
             (S =>
               Interfaces.C.Strings.New_String
                 (Str =>
                   "PM")),
         Lpreturnedstring =>
           Convert_Char_Ptr_To_Lpstr
             (S =>
               AMPM(1)),
         Nsize            =>
          Win32.Dword(5));

   end Set_International;

   procedure Paint_Window
     (Window_Handle         : Win32.Windef.Hwnd;
      Device_Context_Handle : Win32.Windef.Hdc) is

     Date_And_Time            : Win32.Crt.Time.A_Tm_T;

     Scratch_Line_Pointer : Interfaces.C.Strings.Chars_Ptr;

     Scratch_Line : String :=
       ("f                                       ");

     Scratch_Line_Index : integer :=1;

     Day_String_Array : array(0 .. 6) of string(1 ..3) :=
       (0 => "Sun",
        1 => "Mon",
        2 => "Tue",
        3 => "Wed",
        4 => "Thr",
        5 => "Fri",
        6 => "Sat");

   Temp_Time : aliased Win32.Crt.Types.Time_T;

   Time_T_Result : aliased Win32.Crt.Types.Time_T;

--   function Convert_Address_To_Ac_Tm_t is new
--     Ada.Unchecked_Conversion
--       (Source => System.Address,
--        Target => Win32.Crt.Time.Ac_Tm_T);
 
   function Convert_Address_To_Ac_Time_T is new
     Ada.Unchecked_Conversion
       (Source => System.Address,
        Target => Win32.Crt.Time.Ac_Time_T);

   function Trim_Leading
     (Input : String) return String is

   begin

     for i in 1 .. Input'Length loop

       if Input(i) /= ' ' then

         return Input(i .. Input'Length);

       end if;

     end loop;

   end Trim_Leading;

   begin

     Time_T_Result :=
       Win32.Crt.Time.Time
         (Temp_Time'Access);

     Date_And_Time :=
       Win32.Crt.Time.Localtime
         (Timer =>
           Convert_Address_To_Ac_Time_T
             (S =>
               Temp_Time'Address));

     Scratch_Line(1 .. 3) :=
       Day_String_Array
         (Integer
            (Date_And_Time.Tm_Wday));

     Scratch_Line(4 .. 4) :=
       " ";

     Scratch_Line_Index := 
       5; 

     If Integer_Date = 1 then

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Mday))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Mday));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Mday))'Length;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index) :=
           Interfaces.C.Strings.Value
             (Date_Separator); 

       Scratch_Line_Index :=
         Scratch_Line_Index + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Mon + 1))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Mon + 1));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Mon + 1))'Length ;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index) :=
           Interfaces.C.Strings.Value
             (Date_Separator); 

       Scratch_Line_Index :=
         Scratch_Line_Index + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Year))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Year));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Year))'Length;

     elsif Integer_Date = 2 then
 
       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Year))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Year));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Year))'Length;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index) :=
           Interfaces.C.Strings.Value
             (Date_Separator); 

       Scratch_Line_Index :=
         Scratch_Line_Index + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Mon + 1))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Mon + 1));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Mon + 1))'Length ;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index) :=
           Interfaces.C.Strings.Value
             (Date_Separator); 

       Scratch_Line_Index :=
         Scratch_Line_Index + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Mday))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Mday));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Mday))'Length;

     else

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Mon + 1))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Mon + 1));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Mon + 1))'Length ;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index) :=
           Interfaces.C.Strings.Value
             (Date_Separator); 

       Scratch_Line_Index :=
         Scratch_Line_Index + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Mday))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Mday));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Mday))'Length;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index) :=
           Interfaces.C.Strings.Value
             (Date_Separator); 

       Scratch_Line_Index :=
         Scratch_Line_Index + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Year))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Year));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Year))'Length;

     end if;

     Scratch_Line
       (Scratch_Line_Index) :=
       Character'Val(13); 

     Scratch_Line_Index :=
       Scratch_Line_Index + 1;

     If Integer_Time = 1 then

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Hour))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Hour));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Hour))'Length ;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index) :=
           Interfaces.C.Strings.Value
             (Time_Separator); 

       Scratch_Line_Index :=
         Scratch_Line_Index + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Min))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Min));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Min))'Length;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index) :=
           Interfaces.C.Strings.Value
             (Time_Separator); 

       Scratch_Line_Index :=
         Scratch_Line_Index + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Sec))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Sec));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Sec))'Length;

     else

       if Date_And_Time.Tm_Hour mod 12 > 0 then

         Scratch_Line
           (Scratch_Line_Index .. 
            Scratch_Line_Index + 
              Trim_Leading
                (Win32.Int'Image
                  (Date_And_Time.Tm_Hour mod 12))'Length - 1) :=
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Hour mod 12));
         
         Scratch_Line_Index := 
           Scratch_Line_Index + 
             Trim_Leading
               (Win32.Int'Image
                 (Date_And_Time.Tm_Hour mod 12))'Length ;

       else

         Scratch_Line
           (Scratch_Line_Index .. 
            Scratch_Line_Index + 
              Trim_Leading
                (Win32.Int'Image
                  (Date_And_Time.Tm_Hour))'Length - 1) :=
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Hour));
         
         Scratch_Line_Index := 
           Scratch_Line_Index + 
             Trim_Leading
               (Win32.Int'Image
                 (Date_And_Time.Tm_Hour))'Length ;

       end if;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index) :=
           Interfaces.C.Strings.Value
             (Time_Separator); 

       Scratch_Line_Index :=
         Scratch_Line_Index + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Min))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Min));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Min))'Length;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index) :=
           Interfaces.C.Strings.Value
             (Time_Separator); 

       Scratch_Line_Index :=
         Scratch_Line_Index + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index + 
            Trim_Leading
              (Win32.Int'Image
                (Date_And_Time.Tm_Sec))'Length - 1) :=
         Trim_Leading
           (Win32.Int'Image
             (Date_And_Time.Tm_Sec));
         
       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Trim_Leading
             (Win32.Int'Image
               (Date_And_Time.Tm_Sec))'Length + 1;

       Scratch_Line
         (Scratch_Line_Index .. 
          Scratch_Line_Index +
           Integer
             (Interfaces.C.Strings.Strlen
               (AMPM
                 (Integer
                   (Date_And_Time.Tm_Hour/12)))) - 1) :=
           Interfaces.C.Strings.Value
             (AMPM
               (Integer
                 (Date_And_Time.Tm_Hour/12))); 

       Scratch_Line_Index := 
         Scratch_Line_Index + 
           Integer
             (Interfaces.C.Strings.Strlen
               (AMPM
                 (Integer
                   (Date_And_Time.Tm_Hour/12))));

     end if;

     Scratch_Line_Pointer :=
       Interfaces.C.Strings.New_String
         (Str =>
             Scratch_Line(1 .. Scratch_Line_Index));

     Bool_Result := 
       Win32.Winuser.Getclientrect
	 (Hwnd   => 
           Window_Handle,
	  Lprect => 
           Rectangle'Access);

     Integer_Result := 
       Win32.Winuser.Drawtext
	 (Hdc =>
           Device_Context_Handle,
	  Lpstring => 
           Convert_Char_Ptr_To_Lpcstr
             (S => 
              Scratch_Line_Pointer),
	  Ncount   => 
           - 1,
	  Lprect   => 
           Rectangle'Access,
	  Uformat  => 
           Win32.Winuser.Dt_Center or 
	   Win32.Winuser.Dt_Noclip);

        
     Interfaces.C.Strings.Free
       (Item =>
          Scratch_Line_Pointer);

   end Paint_Window;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.WinUser.Wm_Create =>

           Set_International;

	    return (0);

	 when Win32.WinUser.Wm_Timer =>

           Bool_Result := 
             Win32.Winuser.Invalidaterect
               (Hwnd   => 
                 Window_Handle,
                Lprect => 
                 null,
                Berase  => 
                 True);

	    return (0);

	 when Win32.WinUser.Wm_Paint =>

	   Device_Context_Handle :=
             Win32.Winuser.Beginpaint
	       (Hwnd    =>                                                       
                 Window_Handle,
	        Lppaint =>
                 Paint_Record'Access);

           Paint_Window
             (Window_Handle         =>
               Window_Handle,
              Device_Context_Handle =>
               Device_Context_Handle);

    	   Bool_Result :=
             Win32.Winuser.Endpaint
               (Hwnd    =>
                 Window_Handle,
                Lppaint =>
                 Paint_Record'Access);

	    return (0);

	 when Win32.WinUser.Wm_Wininichange =>

           Bool_Result := 
             Win32.Winuser.InvalidateRect
               (Hwnd   => 
                 Window_Handle,
                Lprect => 
                 null,
                Berase  => 
                 True);

           Set_International;

	    return (0);

	 when Win32.WinUser.Wm_Destroy =>

            Bool_Result :=
              Win32.Winuser.Killtimer
                (Hwnd     =>
                  Window_Handle,
                 Uidevent =>
                  Id_Timer);

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
                0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Size_The_Window
        (Starting_X    =>
          Windows.Data.Window_Horizontal_Start,
         Starting_Y    =>
          Windows.Data.Window_Vertical_Start,
         Client_Width  =>
          Windows.Data.Window_Width,
         Client_Height =>
          Windows.Data.Window_Height);

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Uint_Result :=
     0;

    while Uint_Result = 0 loop 

      Uint_Result :=
        Win32.Winuser.Settimer
          (Hwnd        =>
            Windows.Data.Window_Handle,
           Nidevent    =>
            Id_Timer,
           Uelapse     =>
            1_000,
           Lptimerfunc =>
            null);

      if Uint_Result = 0 then

        Integer_Result :=
          Win32.Winuser.Messagebox
             (Hwnd      => 
               Windows.Data.Window_Handle,
              Lptext    => 
               Convert_Char_Ptr_To_Lpcstr
                 (S => 
                   Message_Box_Text),
              Lpcaption =>
               Convert_Char_Ptr_To_Lpcstr
                 (S => 
                   Windows.Data.Class_Name),
              Utype     =>
               Win32.Winuser.Mb_Iconexclamation or
               Win32.Winuser.Mb_Ok);

        return(0);     

      end if;

    end loop;

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winuser.Sw_Shownoactivate);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
