with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type Interfaces.C.Unsigned;
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int;
  use type Win32.Long;

   type Button_Record is
     record
       Style : Win32.Dword;
       Text  : Interfaces.C.Strings.Chars_Ptr;
     end record;

   Button_Array : array(0 .. 9) of Button_Record :=
     (0 => 
       (Style =>
         Win32.Winuser.Bs_Pushbutton,
        Text  =>
         Interfaces.C.Strings.New_String
           (Str =>
             "Pushbutton")),
      1 => 
       (Style =>
         Win32.Winuser.Bs_Defpushbutton,
        Text  =>
         Interfaces.C.Strings.New_String
           (Str =>
             "Default Pushbutton")),
      2 => 
       (Style =>
         Win32.Winuser.Bs_Checkbox,
        Text  =>
         Interfaces.C.Strings.New_String
           (Str =>
             "Check Box")),
      3 => 
       (Style =>
         Win32.Winuser.Bs_Autocheckbox,
        Text  =>
         Interfaces.C.Strings.New_String
           (Str =>
             "Automatic Check Box")),
      4 => 
       (Style =>
         Win32.Winuser.Bs_Radiobutton,
        Text  =>
         Interfaces.C.Strings.New_String
           (Str =>
             "Radio Button")),
      5 => 
       (Style =>
         Win32.Winuser.Bs_3state,
        Text  =>
         Interfaces.C.Strings.New_String
           (Str =>
             "Three State")),
      6 => 
       (Style =>
         Win32.Winuser.Bs_Auto3state,
        Text  =>
         Interfaces.C.Strings.New_String
           (Str =>
             "Automatic Three State")),
      7 => 
       (Style =>
         Win32.Winuser.Bs_Groupbox,
        Text  =>
         Interfaces.C.Strings.New_String
           (Str =>
             "Group Box")),
      8 => 
       (Style =>
         Win32.Winuser.Bs_Autoradiobutton,
        Text  =>
         Interfaces.C.Strings.New_String
           (Str =>
             "Automatic Radio Button")),
      9 => 
       (Style =>
         Win32.Winuser.Bs_Ownerdraw,
        Text  =>
         Interfaces.C.Strings.New_String
           (Str =>
             "Owner Draw")));

   Button_Text              : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "Button");

   Top_Line : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       ("iMsg           wParam         LParam");
 
   Underline_Line : constant Interfaces.C.STrings.Chars_Ptr  :=
     Interfaces.C.Strings.New_String
       ("_______        ______         ______");

   Number_Of_Buttons        : constant Win32.Int := 10;

   Window_Handle_Array      : array (0 .. Number_Of_Buttons -1) of Win32.Windef.Hwnd;

   Character_Width          : Win32.Long;

   Character_Height         : Win32.Long;

   True                     : constant Win32.Bool := 1;

   Paint_Record             : aliased win32.Winuser.Paintstruct;

   Rectangle                : aliased win32.Windef.Rect;

   Text_Metric              : aliased Win32.Wingdi.Textmetric;

   Window_Class             : aliased Win32.Winuser.Wndclass;

   Message                  : aliased Win32.Winuser.Msg;

   Device_Context_Handle    : aliased Win32.Windef.Hdc;

   Integer_Result           : Win32.Int;

   Bool_Result              : Win32.Bool;

   Long_Result              : Win32.Long;

   Atom_Result              : Win32.Windef.Atom;

   Object_Result            : Win32.Windef.Hgdiobj;

   function Convert_Integer_To_Hmenu is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Windef.Hmenu);

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Lparam_To_Lpcreatestruct is new
     Ada.Unchecked_Conversion
       (Source => Win32.Lparam,
        Target => Win32.Winuser.Lpcreatestruct);

   procedure Set_Output_Text
     (Message          : Win32.Uint;
      Wparam           : Win32.Wparam;
      Lparam           : Win32.Lparam;
      Character_Width  : win32.Long;
      Character_Height : Win32.Long;
      Rectangle_Bottom : Win32.Long) is

     Scratch_Line_Pointer : Interfaces.C.Strings.Chars_Ptr;

     Scratch_Line : String :=
       ("                                                      ");

     Temporary_Unsigned : Interfaces.Unsigned_32 :=
       Interfaces.Unsigned_32(Message);

     Temporary_String : String(1 .. 9) := "         ";

     Counter : integer := 0;

     Index_Into_String : integer := 0;

   begin

     If Message = Win32.Winuser.Wm_Drawitem then

       Scratch_Line(1 ..11) := "WM_DRAWITEM";

     else

       Scratch_Line(1 ..10) := "WM_COMMAND";

     end if; 

     while Counter < 32 loop 

       Temporary_Unsigned :=
         Interfaces.Shift_Right
           (Interfaces.Shift_Left
              (Interfaces.Unsigned_32
                 (Wparam),
               (32 + Counter) - 32),
            28);       

       if Counter = 16 then

         Index_Into_String :=
           1;

         Temporary_String(5) :=
           '-';
       end if;

       if integer(Temporary_Unsigned) > 9 then

         case Temporary_Unsigned is

           when 10    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'A';

           when 11    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'B';

           when 12    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'C';

           when 13    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'D';

           when 14    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'E';

           when 15    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'F';

           when others =>
           
             null;

         end case;

       else

         Temporary_String((Counter/4 + 1) + Index_Into_String) :=
           Character'Val(48 + Integer(Temporary_Unsigned));

       end if;

        Counter :=
          Counter + 4;

     end loop;

     Scratch_Line(16 .. (16 + Temporary_String'Length) - 1) := 
       Temporary_String;

     Counter :=
       0;

     Temporary_String := "         ";

     while Counter < 32 loop 

       Temporary_Unsigned :=
         Interfaces.Shift_Right
           (Interfaces.Shift_Left
              (Interfaces.Unsigned_32
                 (Lparam),
               (32 + Counter) - 32),
            28);       

       if Counter = 16 then

         Index_Into_String :=
           1;

         Temporary_String(5) :=
           '-';
       end if;

       if integer(Temporary_Unsigned) > 9 then

         case Temporary_Unsigned is

           when 10    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'A';

           when 11    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'B';

           when 12    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'C';

           when 13    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'D';

           when 14    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'E';

           when 15    =>

             Temporary_String((Counter/4 + 1) + Index_Into_String) :=
               'F';

           when others =>
           
             null;

         end case;

       else

         Temporary_String((Counter/4 + 1) + Index_Into_String) :=
           Character'Val(48 + Integer(Temporary_Unsigned));

       end if;

        Counter :=
          Counter + 4;

     end loop;

     Scratch_Line(30 .. (30 + Temporary_String'Length) - 1) := 
       Temporary_String;

     Scratch_Line_Pointer := 
      Interfaces.C.Strings.New_String
        (Str => Scratch_Line);

     Bool_Result := 
       Win32.Wingdi.Textout
         (Hdc   => 
           Device_Context_Handle,
          Nxstart  => 
           Win32.Int
             (30 * Character_Width),
          Nystart  => 
           Win32.Int
             (Character_Height * 
               (Win32.Long
                 (Rectangle_Bottom)/ Character_Height -1)),
          Lpstring => 
           Convert_Char_Ptr_To_Lpcstr
            (S => 
              Scratch_Line_Pointer),
          Cbstring => 
           Win32.Int
            (Interfaces.C.Strings.Strlen
              (Item => 
                Scratch_Line_Pointer)));

        Interfaces.C.Strings.Free
          (Item =>
            Scratch_Line_Pointer);

   end Set_Output_Text;  

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

	   Device_Context_Handle := 
             Win32.Winuser.Getdc
               (Hwnd =>
                 Window_Handle);

           Object_Result :=
             Win32.Wingdi.Selectobject
               (Hdc    => 
                 Device_Context_Handle,
                Hgdiobj => 
                 Win32.Wingdi.Getstockobject
                   (Fnobject => 
                     Win32.Wingdi.System_Fixed_Font));

           Bool_Result :=
             Win32.Wingdi.Gettextmetrics
               (Hdc  => 
                 Device_Context_Handle,
                Lptm =>
                 Text_Metric'Access);

           Character_Width :=
             Text_Metric.Tmavecharwidth;

           Character_Height :=
             Text_Metric.Tmheight + Text_Metric.Tmexternalleading;

           Integer_Result := 
             Win32.Winuser.Releasedc
               (Hwnd =>
                 Window_Handle,
                Hdc  =>
                 Device_Context_Handle);

           for i in 0 .. Number_Of_Buttons -1 loop

             Window_Handle_Array(i) :=
               Win32.Winuser.Createwindow
            	 (Lpclassname => 
                   Convert_Char_Ptr_To_Lpcstr
                     (S => 
                       Button_Text),
                  Lpwindowname => 
                   Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      Button_Array(integer(i)).Text),
                  Dwstyle      => 
                   Win32.Winuser.Ws_Childwindow or 
                   Win32.Winuser.Ws_Visible or 
                   Button_Array(integer(i)).Style,
                  X            => 
                   Win32.Int
                    (Character_Width), 
                  Y            => 
                   Win32.Int
                    (Character_Height * 
                      Win32.Long( 1 + 2 * i)),
                  Nwidth       => 
                   Win32.Int
                    (Character_Width * 23),
                  Nheight      => 
                   Win32.Int
                    (7 * Character_Height / 4),
                  Hwndparent   => 
                   Window_Handle,
                  Hmenu        =>  
                   Convert_Integer_To_Hmenu
                     (S =>
                       integer
                         (i)),
                  Hinstance    =>
                   Convert_Lparam_To_lpcreatestruct
                     (S =>
                        Lparam).Hinstance,
                  Lpparam      => 
                   null);
                 
           end loop;

	   return (0);

	 when Win32.Winuser.Wm_Size =>

           Rectangle.Left :=
             24 * Character_Width;

           Rectangle.Top :=
             2 * Character_Height;

           Rectangle.Right :=
             Win32.Long
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)));
         
           Rectangle.Bottom :=
             Win32.Long
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam)));

	   return (0);

	 when Win32.Winuser.Wm_Paint =>

            Bool_Result :=
              Win32.Winuser.Invalidaterect
                (Hwnd   => 
                  Window_Handle,
                 Lprect => 
                  null,
                 Berase =>
                  True);

	    Device_Context_Handle := 
              Win32.Winuser.Beginpaint
	        (Hwnd    => 
                  Window_Handle,
	         Lppaint => 
                  Paint_Record'Access);

           Object_Result :=
             Win32.Wingdi.Selectobject
               (Hdc    => 
                 Device_Context_Handle,
                Hgdiobj => 
                 Win32.Wingdi.Getstockobject
                   (Fnobject => 
                     Win32.Wingdi.System_Fixed_Font));

           Integer_Result :=
             Win32.Wingdi.Setbkmode
               (Hdc      =>
                 Device_Context_Handle,
                Fnbkmode => 
                 Win32.Wingdi.Transparent);

	    Bool_Result := 
              Win32.Wingdi.Textout
	        (Hdc   => 
                  Device_Context_Handle,
                 Nxstart  => 
                  Win32.Int
                     (30 * Character_Width),
                 Nystart  => 
                   Win32.Int
                     (Character_Height),
                 Lpstring => 
                  Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      Top_Line),
                 Cbstring => 
                  Win32.Int
                    (Interfaces.C.Strings.Strlen
                      (Item => 
                        Top_Line)));

	    Bool_Result := 
              Win32.Wingdi.Textout
	        (Hdc   => 
                  Device_Context_Handle,
                 Nxstart  => 
                  Win32.Int
                     (30 * Character_Width),
                 Nystart  => 
                   Win32.Int
                     (Character_Height),
                 Lpstring => 
                  Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      Underline_Line),
                 Cbstring => 
                  Win32.Int
                    (Interfaces.C.Strings.Strlen
                      (Item => 
                        Underline_Line)));

	   Bool_Result := 
             Win32.Winuser.Endpaint
	       (Hwnd    => 
                 Window_Handle,
	        Lppaint => 
                 Paint_Record'Access);

	   return (0);

	 when Win32.WinUser.Wm_Drawitem |
              Win32.WinUser.Wm_Command =>

           Bool_Result := 
             Win32.Winuser.Scrollwindow
              (Hwnd       => 
                Window_Handle,
               Xamount    => 
                0,
               Yamount    => 
                Win32.Int
                  (- Character_Height),
               Lprect     => 
                Rectangle'Access,
               Lpcliprect =>
                Rectangle'Access);

	   Device_Context_Handle := 
             Win32.Winuser.Getdc
               (Hwnd =>
                 Window_Handle);

           Object_Result :=
             Win32.Wingdi.Selectobject
               (Hdc    => 
                 Device_Context_Handle,
                Hgdiobj => 
                 Win32.Wingdi.Getstockobject
                   (Fnobject => 
                     Win32.Wingdi.System_Fixed_Font));

           Set_Output_Text
             (Message          => 
               Message,
              Wparam           => 
               Wparam,
              Lparam           =>
               Lparam,
              Character_Width  =>
               Character_Width,
              Character_Height =>
               Character_Height,
              Rectangle_Bottom =>
               Rectangle.Bottom);

           Integer_Result := 
             Win32.Winuser.Releasedc
               (Hwnd =>
                 Window_Handle,
                Hdc  =>
                 Device_Context_Handle);

           Bool_Result :=
             Win32.Winuser.ValidateRect
               (Hwnd   =>
                 Window_Handle,
                Lprect => 
                 Rectangle'Access);

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

	 when Win32.WinUser.Wm_Destroy =>

	   Win32.WinUser.Postquitmessage
	     (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
