with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
with Windows.Child_Routines;
package body Windows.Routines is
  use type Interfaces.C.Unsigned;
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int;
  use type Win32.Long;

  Primary_Color_Array              : Array(0 .. 2) of Win32.Windef.Colorref :=
    (0 =>
       Win32.Wingdi.Rgb
         (Bred  =>
           255,
          Bgreen =>
           0,
          Bblue =>
           0),
     1 =>
       Win32.Wingdi.Rgb
         (Bred  =>
           0,
          Bgreen =>
           255,
          Bblue =>
           0),
     2 =>
       Win32.Wingdi.Rgb
         (Bred  =>
           0,
          Bgreen =>
           0,
          Bblue =>
           255));

  Brush_Handle_Array               : array(0 .. 2) of Win32.Windef.Hbrush;

  Brush_Handle                     : Win32.Windef.Hbrush;

  Old_Scroll_Array                 : array(0 .. 2) of Win32.Winuser.Wndproc;

  Color_Label_Array                : constant array(0 ..2) of Interfaces.C.Strings.Chars_Ptr :=
    (0 => 
      Interfaces.C.Strings.New_String
        (Str =>
          "Red"),
     1 => 
      Interfaces.C.Strings.New_String
        (Str =>
          "Green"),
     2 => 
      Interfaces.C.Strings.New_String
        (Str =>
          "Red"));

   Label_Window_Handle_Array       : array(0 .. 2) of Win32.Windef.Hwnd;

   Value_Window_Handle_Array       : array(0 .. 2) of Win32.Windef.Hwnd;

   Rectangle_Window_Handle         : Win32.Windef.Hwnd;

   Color_Array              : array ( 0 .. 2) of Integer :=
     ( 0 => 0,
       1 => 0,
       2 => 0); 

   Temporary_Integer        : Integer ;

   Zero_Text              : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "0");

   Static_Text              : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "static");

   Scroll_Bar_Text          : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "Scrollbar");

   Temporary_String_Pointer : Interfaces.C.Strings.Chars_Ptr;

   Character_Width          : Win32.Int;

   Character_Height         : Win32.Int;

   Client_Width             : Win32.Int;

   Client_Height            : Win32.Int;

   True                     : constant Win32.Bool := 1;

   False                    : constant Win32.Bool := 0;

   Color_Rectangle          : aliased win32.Windef.Rect;

   Window_Class             : aliased Win32.Winuser.Wndclass;

   Message                  : aliased Win32.Winuser.Msg;

   Device_Context_Handle    : aliased Win32.Windef.Hdc;

   Integer_Result           : Win32.Int;

   Dword_Result             : Win32.Dword;

   Bool_Result              : Win32.Bool;

   Long_Result              : Win32.Long;

   Atom_Result              : Win32.Windef.Atom;

   Object_Result            : Win32.Windef.Hgdiobj;

   Colorref_Result          : Win32.Windef.Colorref;

   Window_Handle_Result     : Win32.Windef.Hwnd;
 
   function Convert_Integer_To_Hmenu is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Windef.Hmenu);

   function Convert_Hwnd_To_Integer is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hwnd,
        Target => Integer);

   function Convert_Long_To_Wndproc is new
     Ada.Unchecked_Conversion
       (Source => Win32.Long,
        Target => Win32.Winuser.Wndproc);

   function Convert_Wndproc_To_Long is new
     Ada.Unchecked_Conversion
       (Source => Win32.Winuser.Wndproc,
        Target => Win32.Long);

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Hwnd_To_Lresult is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hwnd,
        Target => Win32.Lresult);

   function Convert_Lparam_To_Hwnd is new
     Ada.Unchecked_Conversion
       (Source => Win32.Lparam,
        Target => Win32.Windef.Hwnd);

   function Convert_Wparam_To_Hdc is new
     Ada.Unchecked_Conversion
       (Source => Win32.Wparam,
        Target => Win32.Windef.Hdc);

   function Convert_Dword_To_Hgdiobj is new
     Ada.Unchecked_Conversion
       (Source => Win32.Dword,
        Target => Win32.Windef.Hgdiobj);

   function Convert_Hgdiobj_To_Long is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hgdiobj,
        Target => Win32.Long);

   function Convert_Dword_To_Colorref is new
     Ada.Unchecked_Conversion
       (Source => Win32.Dword,
        Target => Win32.Windef.Colorref);

   function Min(L,R : integer) return Integer is

   begin

     if L < R then

       return L;

     else

       return R;

     end if;

   end Min;

   function Max(L,R : Integer) return Integer is

   begin

     if L > R then

       return L;

     else

       return R;

     end if;

   end Max;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

           for i in 0 .. 2 loop

             Brush_Handle_Array(i) :=
               Win32.Wingdi.Createsolidbrush
                 (Clrref =>
                   Primary_Color_Array(i));

           end loop;

           Brush_Handle :=
             Win32.Wingdi.Createsolidbrush
               (Clrref =>
                 Win32.Windef.Colorref
                   (Win32.Winuser.Getsyscolor
                     (Nindex =>
                       Win32.Winuser.Color_Btnhighlight)));      

           Character_Height :=
             Win32.Int
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Win32.Winuser.Getdialogbaseunits)));

	   return (0);

	 when Win32.Winuser.Wm_Size =>

           Client_Width :=
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)));

           Client_Height :=
             Win32.Int
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam)));

           Bool_Result :=
             Win32.Winuser.Setrect
               (Lprc    =>
                 Color_Rectangle'Access,
                Xleft   =>
                 Client_Width / 2,
                Ytop    =>
                 0,
                Xright  =>
                 Client_Width,
                Ybottom =>
                 Client_Height);

           Bool_Result :=
             Win32.Winuser.Movewindow
               (Hwnd     =>
                 Rectangle_Window_Handle,
                X        =>
                 0,
                Y        =>
                 0,
                Nwidth   =>
                 Client_Width / 2,
                NHeight  =>
                 Client_Height,
                Brepaint =>
                 True);

           for i in 0 .. 2 loop

             Bool_Result :=
               Win32.Winuser.Movewindow
                 (Hwnd     =>
                   Windows.Child_Routines.Scroll_Window_Handle_Array(i),
                  X        =>
                   Win32.Int
                     (2 * i + 1) *
                   Client_Width / 14 ,
                  Y        =>
                   2 * Character_Height,
                  Nwidth   =>
                   Client_Width / 14,
                  NHeight  =>
                   Client_Height - 4 * Character_Height,
                  Brepaint =>
                   True);
             
             Bool_Result :=
               Win32.Winuser.Movewindow
                 (Hwnd     =>
                   Label_Window_Handle_Array(i),
                  X        =>
                   Win32.Int
                     (4 * i + 1) *
                   Client_Width / 28 ,
                  Y        =>
                   Character_Height / 2,
                  Nwidth   =>
                   Client_Width / 7,
                  NHeight  =>
                   Character_Height,
                  Brepaint =>
                   True);
             
             Bool_Result :=
               Win32.Winuser.Movewindow
                 (Hwnd     =>
                   Value_Window_Handle_Array(i),
                  X        =>
                   Win32.Int
                     (4 * i + 1) *
                    Client_Width / 28 ,
                  Y        =>
                   Client_Height - 3 * Character_Height / 2,
                  Nwidth   =>
                   Client_Width / 7,
                  NHeight  =>
                   Character_Height,
                  Brepaint =>
                   True);
             
             end loop;

           Window_Handle_Result :=
             Win32.Winuser.Setfocus
               (Hwnd =>
                 Window_Handle);
           
	   return (0);

	 when Win32.Winuser.Wm_Setfocus =>

           Window_Handle_Result :=
             Win32.Winuser.Setfocus
               (Hwnd =>
                 Windows.Child_Routines.Scroll_Window_Handle_Array
                   (Windows.Child_Routines.Focus));
           
	   return (0);

	 when Win32.Winuser.Wm_Vscroll =>

           Temporary_Integer :=
             Integer
               (Win32.Winuser.Getwindowlong
                 (Hwnd =>
                   Convert_Lparam_To_Hwnd
                     (S =>
                       Lparam),
                  Nindex =>
                   Win32.Winuser.Gwl_Id));

           case Integer
             (Win32.Utils.Loword
               (Win32.Dword
                 (Wparam))) is

             when Win32.Winuser.Sb_Pagedown     =>

               Color_Array(Temporary_Integer) :=
                 Color_Array(Temporary_Integer) + 15;

               Color_Array(Temporary_Integer) :=
                 Min
                   (L =>
                     255,
                    R =>
                     Color_Array(Temporary_Integer) + 1);
                
             when Win32.Winuser.Sb_Linedown     =>

               Color_Array(Temporary_Integer) :=
                 Min
                   (L =>
                     255,
                    R =>
                     Color_Array(Temporary_Integer) + 1);
                
             when Win32.Winuser.Sb_Pageup       =>

               Color_Array(Temporary_Integer) :=
                 Color_Array(Temporary_Integer) - 15;

               Color_Array(Temporary_Integer) :=
                 Max
                   (L =>
                     0,
                    R =>
                     Color_Array(Temporary_Integer) - 1);
                
             when Win32.Winuser.Sb_Lineup       =>

               Color_Array(Temporary_Integer) :=
                 Max
                   (L =>
                     0,
                    R =>
                     Color_Array(Temporary_Integer) - 1);
                
             when Win32.Winuser.Sb_Top          =>

               Color_Array(Temporary_Integer) :=
                 0;
   
             when Win32.Winuser.Sb_Bottom       =>

               Color_Array(Temporary_Integer) :=
                 255;
   
             when Win32.Winuser.Sb_Thumbposition |
                  Win32.Winuser.Sb_Thumbtrack    =>

               Color_Array(Temporary_Integer) :=
                 Integer
                   (Win32.Utils.Hiword
                     (Win32.Dword
                       (Wparam)));

             when others                        =>

               null;

           end case;

           Integer_Result := 
             Win32.Winuser.Setscrollpos
               (Hwnd    => 
                 Windows.Child_Routines.Scroll_Window_Handle_Array(Temporary_Integer),
                Nbar    => 
                 Win32.Winuser.Sb_Ctl,
                Npos    => 
                 Win32.Int
                   (Color_Array(Temporary_Integer)),
                Bredraw => 
                 True);

           Temporary_String_Pointer :=
             Interfaces.C.Strings.New_String
               (Integer'Image
                 (Color_Array(Temporary_Integer)));

           Bool_result :=
             Win32.Winuser.Setwindowtext
               (Hwnd     =>
                 Value_Window_Handle_Array(Temporary_Integer),
                Lpstring =>
                 Convert_Char_Ptr_To_Lpcstr
                   (Temporary_String_Pointer));

           Interfaces.C.Strings.Free
             (Item =>
               Temporary_String_Pointer);

           Bool_Result :=
             Win32.Wingdi.Deleteobject
               (Hgdiobj =>
                 Convert_Dword_To_Hgdiobj
                   (Win32.Winuser.Setclasslong
                     (Hwnd      =>
                       Window_Handle,
                      Nindex    =>
                       Win32.Winuser.Gcl_Hbrbackground,
                      Dwnewlong =>
                        Convert_Hgdiobj_To_Long
                         (Win32.Wingdi.Createsolidbrush
                           (Clrref =>
                             Win32.Wingdi.Rgb
                               (Bred =>
                                 Win32.Byte
                                   (Color_Array(0)),
                                Bgreen =>
                                 Win32.Byte
                                   (Color_Array(1)),
                                Bblue  =>
                                 Win32.Byte
                                   (Color_Array(2)))))))); 
      
           Bool_Result :=
             Win32.winuser.Invalidaterect
               (Hwnd   =>
                 Window_Handle,
                Lprect =>
                 Color_Rectangle'Access,
                Berase =>
                 True);

	   return (0);

	 when Win32.WinUser.Wm_Ctlcolorscrollbar =>

           Temporary_Integer :=
             Integer
               (Win32.Winuser.Getwindowlong
                 (Hwnd =>
                   Convert_Lparam_To_Hwnd
                     (S =>
                       Lparam),
                  Nindex =>
                   Win32.Winuser.Gwl_Id));

	   return 
             Convert_Hwnd_To_Lresult
               (Brush_Handle_Array
                 (Temporary_Integer));

	 when Win32.WinUser.Wm_Ctlcolorstatic =>

           Temporary_Integer :=
             Integer
               (Win32.Winuser.Getwindowlong
                 (Hwnd =>
                   Convert_Lparam_To_Hwnd
                     (S =>
                       Lparam),
                  Nindex =>
                   Win32.Winuser.Gwl_Id));
        
           if Temporary_Integer >= 3 and
              Temporary_Integer <= 8 then

             Colorref_Result :=
               Win32.Wingdi.Settextcolor
                 (Hdccolor =>
                   Convert_Wparam_To_Hdc
                     (S =>
                       Wparam),
                  Crcolor  =>
                   Primary_Color_Array(Temporary_Integer mod 3));

             Colorref_Result :=
               Win32.Wingdi.Setbkcolor
                 (Hdc    =>
                   Convert_Wparam_To_Hdc
                     (S =>
                       Wparam),
                  Clrref =>
                   Convert_Dword_To_Colorref
                     (Win32.Winuser.Getsyscolor
                       (Nindex =>
                         Win32.Winuser.Color_Btnhighlight)));

           
             return 
               Convert_Hwnd_To_Lresult
                 (Brush_Handle);
         
           else

	    return 
              Win32.Winuser.Defwindowproc
	        (Hwnd   =>
                  Window_Handle, 
	         Msg    => 
                  Message, 
         	 Wparam => 
                  Wparam,
	         Lparam => 
                  Lparam);

           end if;

	 when Win32.WinUser.Wm_Syscolorchange =>

           Bool_Result :=
             Win32.Wingdi.Deleteobject
               (Hgdiobj =>
                 Brush_Handle); 

           Brush_Handle :=
             Win32.Wingdi.Createsolidbrush
               (Clrref =>
                 Win32.Windef.Colorref
                   (Win32.Winuser.Getsyscolor
                     (Nindex =>
                       Win32.Winuser.Color_Btnhighlight)));

	    return (0);

	 when Win32.WinUser.Wm_Destroy =>

           Bool_Result :=
             Win32.Wingdi.Deleteobject
               (Hgdiobj =>
                 Convert_Dword_To_Hgdiobj
                   (Win32.Winuser.Setclasslong
                     (Hwnd      =>
                       Window_Handle,
                      Nindex    =>
                       Win32.Winuser.Gcl_Hbrbackground,
                      Dwnewlong =>
                       Convert_Hgdiobj_To_Long
                         (Win32.Wingdi.Getstockobject
                           (Fnobject =>
                             Win32.Wingdi.White_Brush))))); 

           for i in 0 .. 2 loop

             Bool_Result :=
               Win32.Wingdi.Deleteobject
                 (Hgdiobj =>
                   Brush_Handle_Array(i)); 

           end loop;

           Bool_Result :=
             Win32.Wingdi.Deleteobject
               (Hgdiobj =>
                 Brush_Handle); 

	   Win32.WinUser.Postquitmessage
	     (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Win32.Wingdi.Createsolidbrush
               (Clrref =>
                 Win32.Wingdi.Rgb
                   (Bred  =>
                     0,
                    Bgreen =>
                     0,
                    Bblue  =>
                     0)),
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

      Rectangle_Window_Handle :=
        Win32.Winuser.Createwindow
          (Lpclassname => 
            Convert_Char_Ptr_To_Lpcstr
              (S => 
                Static_Text),
           Lpwindowname => 
            null,
           Dwstyle      => 
            Win32.Winuser.Ws_Child or 
            Win32.Winuser.Ws_Visible or 
            Win32.Winuser.Ss_Whiterect,
           X            => 
            0, 
           Y            => 
            0,
           Nwidth       => 
            0,
           Nheight      => 
            0,
           Hwndparent   => 
            Windows.Data.Window_Handle,
           Hmenu        =>  
            Convert_Integer_To_Hmenu
             (S =>
               9),
           Hinstance    =>
            Window_Class.Hinstance,
           Lpparam      => 
            null);

    for i in 0 .. 2 loop

      Windows.Child_Routines.Scroll_Window_Handle_Array(i) :=
        Win32.Winuser.Createwindow
          (Lpclassname => 
            Convert_Char_Ptr_To_Lpcstr
              (S => 
                Scroll_Bar_Text),
           Lpwindowname => 
            null,
           Dwstyle      => 
            Win32.Winuser.Ws_Child or 
            Win32.Winuser.Ws_Visible or 
            Win32.Winuser.Ws_Tabstop or
            Win32.Winuser.Sbs_Vert,
           X            => 
            0, 
           Y            => 
            0,
           Nwidth       => 
            0,
           Nheight      => 
            0,
           Hwndparent   => 
            Windows.Data.Window_Handle,
           Hmenu        =>  
            Convert_Integer_To_Hmenu
             (S =>
               i),
           Hinstance    =>
            Instance,
           Lpparam      => 
            null);

      Label_Window_Handle_Array(i) :=
        Win32.Winuser.Createwindow
          (Lpclassname => 
            Convert_Char_Ptr_To_Lpcstr
              (S => 
                Static_Text),
           Lpwindowname => 
            Convert_Char_Ptr_To_Lpcstr
              (S => 
                Color_Label_Array(i)),
           Dwstyle      => 
            Win32.Winuser.Ws_Child or 
            Win32.Winuser.Ws_Visible or 
            Win32.Winuser.SS_Center,
           X            => 
            0, 
           Y            => 
            0,
           Nwidth       => 
            0,
           Nheight      => 
            0,
           Hwndparent   => 
            Windows.Data.Window_Handle,
           Hmenu        =>  
            Convert_Integer_To_Hmenu
             (S =>
               i + 3),
           Hinstance    =>
            Instance,
           Lpparam      => 
            null);

      Value_Window_Handle_Array(i) :=
        Win32.Winuser.Createwindow
          (Lpclassname => 
            Convert_Char_Ptr_To_Lpcstr
              (S => 
                Static_Text),
           Lpwindowname => 
            Convert_Char_Ptr_To_Lpcstr
              (S => 
                Zero_Text),
           Dwstyle      => 
            Win32.Winuser.Ws_Child or 
            Win32.Winuser.Ws_Visible or 
            Win32.Winuser.SS_Center,
           X            => 
            0, 
           Y            => 
            0,
           Nwidth       => 
            0,
           Nheight      => 
            0,
           Hwndparent   => 
            Windows.Data.Window_Handle,
           Hmenu        =>  
            Convert_Integer_To_Hmenu
             (S =>
               i + 6),
           Hinstance    =>
            Instance,
           Lpparam      => 
            null);

    Windows.Child_Routines.Old_Scroll_Array(i) :=
      Convert_Long_To_Wndproc
        (S =>
          Win32.Winuser.Setwindowlong
            (Hwnd      =>
              Windows.Child_Routines.Scroll_Window_Handle_Array(i),
             Nindex    =>
              Win32.Winuser.Gwl_Wndproc,
             Dwnewlong =>
              Convert_Wndproc_To_Long
                (S =>
                  Windows.Child_Routines.Scroll'Access)));

    Bool_Result := 
      Win32.Winuser.Setscrollrange
        (Hwnd => 
          WIndows.Child_Routines.Scroll_Window_Handle_Array(i),
         Nbar => 
          Win32.Winuser.Sb_Ctl,
         Nminpos => 
          0,
         Nmaxpos => 
          255,
         Bredraw => 
          False);

    Integer_Result := 
      Win32.Winuser.Setscrollpos
        (Hwnd    => 
          Windows.Data.Window_Handle,
         Nbar    => 
          Win32.Winuser.Sb_Ctl,
         Npos    => 
          0,
         Bredraw => 
          False);

  end loop;

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
