with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Crt.Direct;
with Win32.Crt.Stdlib;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Child_Routines;
with Windows.Data;
package body Windows.Routines is
  use type Interfaces.C.Unsigned;
  use type Interfaces.C.Size_T;
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;
  use type Win32.Windef.Hfile;

   Text_Metric                : aliased win32.Wingdi.Textmetric;

   List_Window_Handle         : Win32.Windef.Hwnd;
                           
   Text_Window_Handle         : Win32.Windef.Hwnd;
                           
   Listbox_Text               : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "listbox");

   Static_Text               : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "static");

   Star_Dot_Star            : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
          "*.*");

   True                     : constant Win32.Bool := 1;

   False                    : constant Win32.Bool := 0;

   Is_A_Valid_File          : Win32.Bool;

   Rectangle                : aliased Win32.Windef.Rect;

   Paint_Record             : aliased win32.Winuser.Paintstruct;

   Window_Class             : aliased Win32.Winuser.Wndclass;

   Open_file                : aliased Win32.Winbase.Ofstruct;

   Message                  : aliased Win32.Winuser.Msg;

   Device_Context_Handle    : aliased Win32.Windef.Hdc;

   Maximum_Path             : constant Win32.Int := 256;

   Number_Of_Bytes_To_Read  : constant Win32.Int := 8192;

   Old_Function_List        : Win32.Winuser.Wndproc;

   Read_Buffer_String        : String(1 .. Integer(Number_Of_Bytes_To_Read)) :=
      (others => 
        Character'Val(32));

   Temporary_String         : String(1 .. Integer(Maximum_Path) + 1) :=
      (others => 
        Character'Val(32));

   Read_Buffer              : Interfaces.C.Strings.Chars_Ptr;
  
   Temporary_Read_Buffer    : Interfaces.C.Strings.Chars_Ptr;
  
   Temporary_String_Ptr     : Interfaces.C.Strings.Chars_Ptr;

   File_String_Ptr          : Interfaces.C.Strings.Chars_Ptr;

   File_Handle              : Win32.Windef.Hfile;

   Integer_Result           : Win32.Int;

   Dword_Result             : Win32.Dword;

   Bool_Result              : Win32.Bool;
                     
   Long_Result              : Win32.Long;

   Uint_Result              : Win32.Uint;

   Atom_Result              : Win32.Windef.Atom;

   Pstr_Result              : Win32.Pstr;

   Object_Result            : Win32.Windef.Hgdiobj;

   Colorref_Result          : Win32.Windef.Colorref;

   Hfile_Result             : Win32.Windef.Hfile;

   Window_Handle_Result     : Win32.Windef.Hwnd;
 
   function Convert_Integer_To_Hmenu is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Windef.Hmenu);

   function Convert_Integer_To_Hbrush is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Windef.Hbrush);

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Pstr_To_Chars_Ptr is new
     Ada.Unchecked_Conversion
       (Source => Win32.Pstr,
        Target => Interfaces.C.Strings.Chars_Ptr);

   function Convert_Long_To_Hinstance is new
     Ada.Unchecked_Conversion
       (Source => Win32.Long,
        Target => Win32.Windef.Hinstance);

   function Convert_String_Array_To_Integer is new
     Ada.Unchecked_Conversion
       (Source => Win32.Crt.Stdlib.String_Array,
        Target => Integer);

   function Convert_Integer_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Lpcstr);

   function Convert_Lpcstr_To_Chars_Ptr is new
     Ada.Unchecked_Conversion
       (Source => Win32.Lpcstr,
        Target => Interfaces.C.Strings.Chars_Ptr);

   function Convert_Chars_Ptr_To_Lparam is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lparam);

   function Convert_Chars_Ptr_To_Lpvoid is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpvoid);

   function Convert_Chars_Ptr_To_Pcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Pcstr);

   function Convert_Chars_Ptr_To_Pstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Pstr);

   function Convert_Long_To_Wparam is new
     Ada.Unchecked_Conversion
       (Source => Win32.Long,
        Target => Win32.Wparam);

   function Convert_Long_To_Wndproc is new
     Ada.Unchecked_Conversion
       (Source => Win32.Long,
        Target => Win32.Winuser.Wndproc);

   function Convert_Wndproc_To_Long is new
     Ada.Unchecked_Conversion
       (Source => Win32.Winuser.Wndproc,
        Target => Win32.Long);

   function Convert_Dword_To_Colorref is new
     Ada.Unchecked_Conversion
       (Source => Win32.Dword,
        Target => Win32.Windef.Colorref);

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

   Long_Result              : Win32.Long;

   Integer_Result           : Win32.Int;

   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

	    Device_Context_Handle := 
              Win32.Winuser.Getdc
               (Hwnd =>
                 Window_Handle);

            Object_Result :=
              Win32.Wingdi.Selectobject
                (Hdc     =>
                  Device_Context_Handle,
                 Hgdiobj =>
                  Win32.Wingdi.Getstockobject
                    (Fnobject =>
                      Win32.Wingdi.System_Fixed_Font));

	    Bool_Result := Win32.Wingdi.Gettextmetrics
               (Hdc => 
                 Device_Context_Handle,
                Lptm => 
                 Text_Metric'Access);

            Integer_Result := 
              Win32.Winuser.Releasedc
                (Hwnd =>
                  Window_Handle,
                 Hdc  =>
                  Device_Context_Handle);

            Rectangle.Left :=
              20 * Text_Metric.Tmavecharwidth;

            Rectangle.Top :=
              3 * Text_Metric.Tmheight;

            List_Window_Handle := 
              Win32.Winuser.Createwindow
        	(Lpclassname => 
                  Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      Listbox_Text),
                 Lpwindowname => 
                  null,
                 Dwstyle      => 
                  Win32.Winuser.Ws_Childwindow or
                  Win32.Winuser.Ws_Visible or
                  Win32.Winuser.Lbs_Standard,
                X            => 
                 Win32.Int
                   (Text_Metric.Tmavecharwidth), 
                Y            => 
                 Win32.Int
                   (Text_Metric.Tmheight * 3),
                Nwidth       => 
                 Win32.Int
                   (Text_Metric.Tmavecharwidth) * 13 +
                   Win32.Winuser.Getsystemmetrics
                     (Nindex =>
                       Win32.Winuser.Sm_cxvscroll),
                Nheight      => 
                 Win32.Int
                   (Text_Metric.Tmheight * 10),
                Hwndparent   => 
                 Window_Handle,
                Hmenu        => 
                 Convert_Integer_To_Hmenu
                   (S =>
                     1),
                Hinstance    => 
                 Convert_Long_To_Hinstance
                   (S =>
                     Win32.Winuser.Getwindowlong
                       (Hwnd   =>
                         Window_Handle,
                        Nindex =>
                         Win32.Winuser.Gwl_Hinstance)), 
                Lpparam      =>
                 Null);

            Temporary_String_Ptr     :=
              Interfaces.C.Strings.New_String
               (Str =>
                 Temporary_String);

            Text_Window_Handle := 
              Win32.Winuser.Createwindow
        	(Lpclassname => 
                  Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      Static_Text),
                 Lpwindowname => 
                  Win32.Lpcstr
                    (Win32.Crt.Direct.Getcwd
                      (Buffer =>
                        Convert_Chars_Ptr_To_Pstr
                          (S =>
                            Temporary_String_Ptr),
                       Maxlen =>
                        Maximum_Path)),
                 Dwstyle      => 
                  Win32.Winuser.Ws_Childwindow or
                  Win32.Winuser.Ws_Visible or
                  Win32.Winuser.Ss_Left,
                X            => 
                 Win32.Int
                   (Text_Metric.Tmavecharwidth), 
                Y            => 
                 Win32.Int
                   (Text_Metric.Tmheight),
                Nwidth       => 
                 Win32.Int
                   (Text_Metric.Tmavecharwidth) *
                    Maximum_Path,
                Nheight      => 
                 Win32.Int
                   (Text_Metric.Tmheight),
                Hwndparent   => 
                 Window_Handle,
                Hmenu        => 
                 Convert_Integer_To_Hmenu
                   (S =>
                     2),
                Hinstance    => 
                 Convert_Long_To_Hinstance
                   (S =>
                     Win32.Winuser.Getwindowlong
                       (Hwnd   =>
                         Window_Handle,
                        Nindex =>
                         Win32.Winuser.Gwl_Hinstance)), 
                Lpparam      =>
                 Null);

             Windows.Child_Routines.Old_Function_List :=
               Convert_Long_To_Wndproc
                 (Win32.Winuser.Setwindowlong
                   (Hwnd      =>
                     List_Window_Handle,
                    Nindex    =>
                     Win32.Winuser.Gwl_Wndproc,
                    Dwnewlong =>
                     Convert_Wndproc_To_Long
                       (S =>
                          Windows.Child_Routines.List'Access)));

             Long_Result :=
               Win32.Winuser.SendMessage
                 (Hwnd   =>
                   List_Window_Handle,
                  Msg    => 
                   Win32.Winuser.Lb_Dir,
                  Wparam => 
                   16#37#,
                  Lparam => 
                   Convert_Chars_Ptr_To_Lparam
                     (Star_Dot_Star));
                
 	   return (0);

	 when Win32.Winuser.Wm_Size =>

           Rectangle.Right :=
             Win32.Long
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)));

           Rectangle.Bottom :=
             Win32.Long
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Wparam)));

	   return (0);

	 when Win32.Winuser.Wm_Setfocus =>

           Window_Handle_Result :=
             Win32.Winuser.Setfocus
               (Hwnd =>
                 List_Window_Handle);
           
	   return (0);

	 when Win32.Winuser.Wm_Command =>

           if  
             Integer
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Wparam))) = 1 or
             Integer
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Wparam))) = Win32.Winuser.Lbn_Dblclk then

             Long_Result :=
               Win32.Winuser.SendMessage
                (Hwnd   =>
                  List_Window_Handle,
                 Msg    => 
                  Win32.Winuser.Lb_Getcursel,
                 Wparam => 
                  0,
                 Lparam => 
                  0);

             if Long_Result /= Win32.Winuser.Lb_Err then

               Long_Result :=
                 Win32.Winuser.SendMessage
                  (Hwnd   =>
                    List_Window_Handle,
                   Msg    => 
                    Win32.Winuser.Lb_Gettext,
                   Wparam => 
                    Convert_Long_To_Wparam
                      (S =>
                        Long_Result),
                   Lparam => 
                    Convert_Chars_Ptr_To_Lparam
                      (S =>
                        Temporary_String_Ptr));

               if
                 Win32.Winbase.Openfile
                   (Lpfilename =>
                     Convert_Char_Ptr_To_Lpcstr
                       (Temporary_String_Ptr),
                    Lpreopenbuff =>
                     Open_File'Access,
                    Ustyle       =>
                     Win32.Winbase.Of_Exist or
                     Win32.Winbase.Of_Read) /= -1 then

                 Is_A_Valid_File :=
                   True;

                 File_String_Ptr :=
                   Interfaces.C.Strings.New_String
                     (Str =>
                       Interfaces.C.Strings.Value
                         (Item =>
                           Temporary_String_Ptr));

                 Pstr_Result :=
                   Win32.Crt.Direct.Getcwd
                     (Buffer =>
                       Convert_Chars_Ptr_To_Pstr
                         (S =>
                           Temporary_String_Ptr),
                      Maxlen =>
                       Maximum_Path);

                Temporary_String_Ptr :=
                  Interfaces.C.Strings.New_String
                    (Str =>
                      Interfaces.C.Strings.Value
                        (Item =>
                          Temporary_String_Ptr) &
                      Interfaces.C.Strings.Value
                        (Item =>
                          File_String_Ptr));

                 Bool_Result :=
                   Win32.Winuser.Setwindowtext
                     (Hwnd     =>
                       List_Window_Handle,
                      Lpstring =>
                       Convert_Char_Ptr_To_Lpcstr
                        (S =>
                          Temporary_String_Ptr));
 
               else
                                
                 Is_A_Valid_File :=
                   False;

                 File_String_Ptr :=
                   Interfaces.C.Strings.New_String
                     (Str =>
                       Interfaces.C.Strings.Value
                         (Item =>Temporary_String_Ptr)(2 .. Integer(Interfaces.C.Strings.Strlen(Temporary_String_Ptr)) -1));

                 Integer_Result :=
                   Win32.Crt.Direct.Chdir
                     (Dirname =>
                       Convert_Chars_Ptr_To_Pcstr
                         (S =>
                           File_String_Ptr));

                 Pstr_Result :=
                   Win32.Crt.Direct.Getcwd
                     (Buffer =>
                       Convert_Chars_Ptr_To_Pstr
                         (S =>
                           Temporary_String_Ptr),
                      Maxlen =>
                       Maximum_Path);

                 Bool_Result :=
                   Win32.Winuser.Setwindowtext
                     (Hwnd     =>
                       Text_Window_Handle,
                      Lpstring =>
                       Convert_Char_Ptr_To_Lpcstr
                        (S =>
                          Temporary_String_Ptr));
 
                Long_Result :=
                  Win32.Winuser.SendMessage
                    (Hwnd   =>
                      List_Window_Handle,
                     Msg    => 
                      Win32.Winuser.Lb_Resetcontent,
                     Wparam => 
                      0,
                     Lparam => 
                      0);
                
                Long_Result :=
                  Win32.Winuser.SendMessage
                    (Hwnd   =>
                      List_Window_Handle,
                     Msg    => 
                      Win32.Winuser.Lb_Dir,
                     Wparam => 
                      16#37#,
                     Lparam => 
                      Convert_Chars_Ptr_To_Lparam
                        (Star_Dot_Star));
                
               end if;
               
               Bool_Result :=
                 Win32.Winuser.Invalidaterect
                   (Hwnd   =>
                     Window_Handle,
                    Lprect =>
                     null,
                    Berase =>
                     True);
             end if;

           end if;

	   return (0);

         when Win32.Winuser.Wm_Paint =>

	   Device_Context_Handle := 
             Win32.Winuser.Beginpaint
	       (Hwnd    => 
                 Window_Handle,
	        Lppaint => 
                 Paint_Record'Access);

           Object_Result :=
             Win32.Wingdi.Selectobject
               (Hdc     =>
                 Device_Context_Handle,
                Hgdiobj =>
                 Win32.Wingdi.Getstockobject
                   (Fnobject =>
                     Win32.Wingdi.System_Fixed_Font));

           Colorref_Result :=
             Win32.Wingdi.Settextcolor
               (Hdccolor =>
                 Device_Context_Handle,
                Crcolor  =>
                 Convert_Dword_To_Colorref
                   (S =>
                     Win32.Winuser.Getsyscolor
                       (Nindex =>
                         Win32.Winuser.Color_Btntext)));

           Colorref_Result :=
             Win32.Wingdi.Setbkcolor
               (Hdc     =>
                 Device_Context_Handle,
                Clrref  =>
                 Convert_Dword_To_Colorref
                   (S =>
                     Win32.Winuser.Getsyscolor
                       (Nindex =>
                         Win32.Winuser.Color_Btnface)));

           File_Handle :=
            Win32.Winbase.Openfile
              (Lpfilename =>
                Convert_Char_Ptr_To_Lpcstr
                 (File_String_Ptr),
               Lpreopenbuff =>
                Open_File'Access,
               Ustyle       =>
                Win32.Winbase.Of_Reopen or
               Win32.Winbase.Of_Read);

           if Integer(File_Handle) /= Integer(Is_A_Valid_File) and 
              Integer(File_Handle) /= -1 then

            Read_Buffer :=
              Interfaces.C.Strings.New_String
                (Str =>
                  Read_Buffer_String);

             Uint_Result := 
               Win32.Winbase.Lread
                 (Hfile    =>
                   File_Handle,
                  Lpbuffer =>
                   Convert_Chars_Ptr_To_Lpvoid
                     (Read_Buffer),
                  Ubytes   =>
                   Win32.Uint
                     (Number_Of_Bytes_To_Read));

             Hfile_Result :=
              Win32.Winbase.Lclose
                (Hfile =>
                  File_Handle);

	    Integer_Result := 
              Win32.Winuser.Drawtext
	        (Hdc =>
                  Device_Context_Handle,
	         Lpstring => 
                  Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      Read_Buffer),
	         Ncount   => 
                  Win32.Int
                    (Uint_Result),
	         Lprect   => 
                  Rectangle'Access,
	         Uformat  => 
                  Win32.Winuser.Dt_Wordbreak  or 
	          Win32.Winuser.Dt_Expandtabs or 
	          Win32.Winuser.Dt_Noclip     or
	          Win32.Winuser.Dt_Noprefix);

             Interfaces.C.Strings.Free
               (Item =>
                 Read_Buffer);

           else

             Is_A_Valid_File :=
               False;  

           end if;
    
 	   Bool_Result := 
             Win32.Winuser.Endpaint
	       (Hwnd    => 
                 Window_Handle,
	        Lppaint => 
                 Paint_Record'Access);

	   return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
