With Ada.Text_Io;
with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type Interfaces.C.Unsigned;
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int;
  use type Win32.Long;

   type Point_Array_Type is array(integer range <>) of Win32.Windef.Point;

   Point_Array              : Point_Array_Type(0 .. 2);

   Button_Text              : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "Button");

   Button_Text_2            : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "");

   Smaller_Window_Handle    : Win32.Windef.Hwnd;

   Larger_Window_Handle     : Win32.Windef.Hwnd;

   Character_Width          : Win32.Int;

   Character_Height         : Win32.Int;

   Client_Width             : Win32.Int;

   Client_Height            : Win32.Int;

   Window_Client_Width      : Win32.Int;
                          
   Window_Client_Height     : Win32.Int;

   Draw_Item_Record         : Win32.Winuser.Lpdrawitemstruct;

   True                     : constant Win32.Bool := 1;

   Smaller_Idc              : constant integer    := 1;

   Larger_Idc               : constant integer    := 2;

   Rectangle                : aliased win32.Windef.Rect;

   Window_Class             : aliased Win32.Winuser.Wndclass;

   Message                  : aliased Win32.Winuser.Msg;

   Device_Context_Handle    : aliased Win32.Windef.Hdc;

   Integer_Result           : Win32.Int;

   Bool_Result              : Win32.Bool;

   Long_Result              : Win32.Long;

   Atom_Result              : Win32.Windef.Atom;

   Object_Result            : Win32.Windef.Hgdiobj;

   function Convert_Integer_To_Hmenu is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Windef.Hmenu);

   function Convert_Address_To_Ac_Point_T is new
     Ada.Unchecked_Conversion
       (Source => System.Address,
        Target => Win32.Wingdi.Ac_Point_T);

   function Convert_Address_To_Ac_Rect_T is new
     Ada.Unchecked_Conversion
       (Source => System.Address,
        Target => Win32.Wingdi.Ac_Rect_T);

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Lparam_To_Lpdrawitemstruct is new
     Ada.Unchecked_Conversion
       (Source => Win32.Lparam,
        Target => Win32.Winuser.Lpdrawitemstruct);

   procedure Triangle
     (Device_Context_Handle : Win32.Windef.Hdc;
      Point_Array           : Point_Array_Type) is

   begin

     Object_Result :=
       Win32.Wingdi.Selectobject
         (Hdc    => 
           Device_Context_Handle,
          Hgdiobj => 
           Win32.Wingdi.Getstockobject
             (Fnobject => 
               Win32.Wingdi.Black_Brush));

     Bool_Result :=
       Win32.Wingdi.Polygon
         (Hdc     => 
           Device_Context_Handle,
          Lppoints => 
           Convert_Address_To_Ac_Point_T
             (Point_Array(0)'Address),
          Ncount   =>
           3);
              
     Object_Result :=
       Win32.Wingdi.Selectobject
         (Hdc    => 
           Device_Context_Handle,
          Hgdiobj => 
           Win32.Wingdi.Getstockobject
             (Fnobject => 
               Win32.Wingdi.White_Brush));

   end Triangle;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

           Character_Width :=
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Win32.Winuser.Getdialogbaseunits)));

           Character_Height :=
             Win32.Int
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Win32.Winuser.Getdialogbaseunits)));

           Smaller_Window_Handle :=
             Win32.Winuser.Createwindow
               (Lpclassname => 
                 Convert_Char_Ptr_To_Lpcstr
                   (S => 
                     Button_Text),
                Lpwindowname => 
                 Convert_Char_Ptr_To_Lpcstr
                   (S => 
                     Button_Text_2),
                Dwstyle      => 
                 Win32.Winuser.Ws_Child or 
                 Win32.Winuser.Ws_Visible or 
                 Win32.Winuser.Bs_Ownerdraw,
                X            => 
                 0, 
                Y            => 
                 0,
                Nwidth       => 
                 Character_Width * 8,
                Nheight      => 
                 Character_Height * 4,
                Hwndparent   => 
                 Window_Handle,
                Hmenu        =>  
                 Convert_Integer_To_Hmenu
                  (S =>
                    Smaller_Idc),
                Hinstance    =>
                 Window_Class.Hinstance,
                Lpparam      => 
                 null);
                 
           Larger_Window_Handle :=
             Win32.Winuser.Createwindow
               (Lpclassname => 
                 Convert_Char_Ptr_To_Lpcstr
                   (S => 
                     Button_Text),
                Lpwindowname => 
                 Convert_Char_Ptr_To_Lpcstr
                   (S => 
                     Button_Text_2),
                Dwstyle      => 
                 Win32.Winuser.Ws_Child or 
                 Win32.Winuser.Ws_Visible or 
                 Win32.Winuser.Bs_Ownerdraw,
                X            => 
                 0, 
                Y            => 
                 0,
                Nwidth       => 
                  Character_Width * 8,
                Nheight      => 
                  Character_Height * 4,
                Hwndparent   => 
                 Window_Handle,
                Hmenu        =>  
                 Convert_Integer_To_Hmenu
                  (S =>
                    Larger_Idc),
                Hinstance    =>
                 Window_Class.Hinstance,
                Lpparam      => 
                 null);
                 
	   return (0);

	 when Win32.Winuser.Wm_Size =>

           Client_Width :=
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)));

           Client_Height :=
             Win32.Int
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam)));

           Bool_Result :=
             Win32.Winuser.Movewindow
               (Hwnd     =>
                 Smaller_Window_Handle,
                X        =>
                 Client_Width / 2 - 3 *
                  (Character_Width * 8) / 2,
                Y        =>
                 Client_Height / 2 - 
                  (Character_Height * 4) / 2,
                Nwidth   =>
                 Character_Width * 8,
                NHeight  =>
                 Character_Height * 4,
                Brepaint =>
                 True);

           Bool_Result :=
             Win32.Winuser.Movewindow
               (Hwnd     =>
                 Larger_Window_Handle,
                X        =>
                 Client_Width / 2 +
                  (Character_Width * 8) / 2,
                Y        =>
                 Client_Height / 2 - 
                  (Character_Height * 4) / 2,
                Nwidth   =>
                 Character_Width * 8,
                NHeight  =>
                 Character_Height * 4,
                Brepaint =>
                 True);

	   return (0);

	 when Win32.Winuser.Wm_Command =>

           Bool_Result :=
             Win32.Winuser.Getwindowrect
               (Hwnd   =>
                 Window_Handle,
                Lprect =>
                 Rectangle'Access);

           case Integer(Wparam) is
  
             when Smaller_Idc =>

               Rectangle.Left :=
                 Rectangle.Left + 
                   Win32.Long
                    (Client_Width / 20);

               Rectangle.Right :=
                 Rectangle.Right - 
                   Win32.Long
                     (Client_Width / 20);

               Rectangle.Top :=
                 Rectangle.Top + 
                   Win32.Long
                     (Client_Height / 20);

               Rectangle.Bottom :=
                 Rectangle.Bottom - 
                   Win32.Long
                     (Client_Height / 20);

             when Larger_Idc  =>

               Rectangle.Left :=
                 Rectangle.Left - 
                   Win32.Long
                    (Client_Width / 20);

               Rectangle.Right :=
                 Rectangle.Right + 
                   Win32.Long
                     (Client_Width / 20);

               Rectangle.Top :=
                 Rectangle.Top - 
                   Win32.Long
                     (Client_Height / 20);

               Rectangle.Bottom :=
                 Rectangle.Bottom + 
                   Win32.Long
                     (Client_Height / 20);

             when others      =>

               null;


           end case;

           Bool_Result :=
             Win32.Winuser.Movewindow
               (Hwnd     =>
                 Window_Handle,
                X        =>
                 Win32.Int
                   (Rectangle.Left),
                Y        =>
                 Win32.Int
                   (Rectangle.Top),
                Nwidth   =>
                 Win32.Int
                   (Rectangle.Right - Rectangle.Left),
                NHeight  =>
                 Win32.Int
                   (Rectangle.Bottom - Rectangle.Top),
                Brepaint =>
                 True);

	   return (0);

	 when Win32.Winuser.Wm_Drawitem =>

           Draw_Item_Record :=
             Convert_Lparam_To_Lpdrawitemstruct
               (S =>
                 Lparam);

           Integer_Result :=
             Win32.Winuser.Fillrect
               (Hdc  =>
                 Draw_Item_Record.Hdc,
                Lprc =>
                 Convert_Address_To_Ac_Rect_T
                   (Draw_Item_Record.Rcitem'Address),
                Hbr  =>
                 Win32.Windef.Hbrush
                   (Win32.Wingdi.Getstockobject
                     (Fnobject => 
                       Win32.Wingdi.White_Brush)));
          
           Integer_Result :=
             Win32.Winuser.Framerect
               (Hdc  =>
                 Draw_Item_Record.Hdc,
                Lprc =>
                 Convert_Address_To_Ac_Rect_T
                   (Draw_Item_Record.Rcitem'Address),
                Hbr  =>
                 Win32.Windef.Hbrush
                   (Win32.Wingdi.Getstockobject
                     (Fnobject => 
                       Win32.Wingdi.Black_Brush)));
          
           Window_Client_Width :=
             Win32.Int
               (Draw_Item_Record.Rcitem.Right - 
                Draw_Item_Record.Rcitem.Left);

           Window_Client_Height :=
             Win32.Int
               (Draw_Item_Record.Rcitem.Bottom - 
                Draw_Item_Record.Rcitem.Top);

           case Integer(Draw_Item_Record.Ctlid) is

             when Smaller_Idc =>

               Point_Array(0).X :=
                 Win32.Long
                   (3 * Window_Client_Width / 8);

               Point_Array(0).Y :=
                 Win32.Long
                   (1 * Window_Client_Height / 8);

               Point_Array(1).X :=
                 Win32.Long
                   (5 * Window_Client_Width / 8);

               Point_Array(1).Y :=
                 Win32.Long
                   (1 * Window_Client_Height / 8);

               Point_Array(2).X :=
                 Win32.Long
                   (4 * Window_Client_Width / 8);

               Point_Array(2).Y :=
                 Win32.Long
                   (3 * Window_Client_Height / 8);

               Triangle
                 (Device_Context_Handle =>
                   Draw_Item_Record.Hdc,
                  Point_Array           =>
                   Point_Array);

               Point_Array(0).X :=
                 Win32.Long
                   (7 * Window_Client_Width / 8);

               Point_Array(0).Y :=
                 Win32.Long
                   (3 * Window_Client_Height / 8);

               Point_Array(1).X :=
                 Win32.Long
                   (7 * Window_Client_Width / 8);

               Point_Array(1).Y :=
                 Win32.Long
                   (5 * Window_Client_Height / 8);

               Point_Array(2).X :=
                 Win32.Long
                   (5 * Window_Client_Width / 8);

               Point_Array(2).Y :=
                 Win32.Long
                   (4 * Window_Client_Height / 8);

               Triangle
                 (Device_Context_Handle =>
                   Draw_Item_Record.Hdc,
                  Point_Array           =>
                   Point_Array);

               Point_Array(0).X :=
                 Win32.Long
                   (5 * Window_Client_Width / 8);

               Point_Array(0).Y :=
                 Win32.Long
                   (7 * Window_Client_Height / 8);

               Point_Array(1).X :=
                 Win32.Long
                   (3 * Window_Client_Width / 8);

               Point_Array(1).Y :=
                 Win32.Long
                   (7 * Window_Client_Height / 8);

               Point_Array(2).X :=
                 Win32.Long
                   (4 * Window_Client_Width / 8);

               Point_Array(2).Y :=
                 Win32.Long
                   (5 * Window_Client_Height / 8);

               Triangle
                 (Device_Context_Handle =>
                   Draw_Item_Record.Hdc,
                  Point_Array           =>
                   Point_Array);

               Point_Array(0).X :=
                 Win32.Long
                   (1 * Window_Client_Width / 8);

               Point_Array(0).Y :=
                 Win32.Long
                   (5 * Window_Client_Height / 8);

               Point_Array(1).X :=
                 Win32.Long
                   (1 * Window_Client_Width / 8);

               Point_Array(1).Y :=
                 Win32.Long
                   (3 * Window_Client_Height / 8);

               Point_Array(2).X :=
                 Win32.Long
                   (3 * Window_Client_Width / 8);

               Point_Array(2).Y :=
                 Win32.Long
                   (4 * Window_Client_Height / 8);

               Triangle
                 (Device_Context_Handle =>
                   Draw_Item_Record.Hdc,
                  Point_Array           =>
                   Point_Array);

             when Larger_Idc  =>

               Point_Array(0).X :=
                 Win32.Long
                   (5 * Window_Client_Width / 8);

               Point_Array(0).Y :=
                 Win32.Long
                   (3 * Window_Client_Height / 8);

               Point_Array(1).X :=
                 Win32.Long
                   (3 * Window_Client_Width / 8);

               Point_Array(1).Y :=
                 Win32.Long
                   (3 * Window_Client_Height / 8);

               Point_Array(2).X :=
                 Win32.Long
                   (4 * Window_Client_Width / 8);

               Point_Array(2).Y :=
                 Win32.Long
                   (1 * Window_Client_Height / 8);

               Triangle
                 (Device_Context_Handle =>
                   Draw_Item_Record.Hdc,
                  Point_Array           =>
                   Point_Array);

               Point_Array(0).X :=
                 Win32.Long
                   (5 * Window_Client_Width / 8);

               Point_Array(0).Y :=
                 Win32.Long
                   (5 * Window_Client_Height / 8);

               Point_Array(1).X :=
                 Win32.Long
                   (5 * Window_Client_Width / 8);

               Point_Array(1).Y :=
                 Win32.Long
                   (3 * Window_Client_Height / 8);

               Point_Array(2).X :=
                 Win32.Long
                   (7 * Window_Client_Width / 8);

               Point_Array(2).Y :=
                 Win32.Long
                   (4 * Window_Client_Height / 8);

               Triangle
                 (Device_Context_Handle =>
                   Draw_Item_Record.Hdc,
                  Point_Array           =>
                   Point_Array);

               Point_Array(0).X :=
                 Win32.Long
                   (3 * Window_Client_Width / 8);

               Point_Array(0).Y :=
                 Win32.Long
                   (5 * Window_Client_Height / 8);

               Point_Array(1).X :=
                 Win32.Long
                   (5 * Window_Client_Width / 8);

               Point_Array(1).Y :=
                 Win32.Long
                   (5 * Window_Client_Height / 8);

               Point_Array(2).X :=
                 Win32.Long
                   (4 * Window_Client_Width / 8);

               Point_Array(2).Y :=
                 Win32.Long
                   (7 * Window_Client_Height / 8);

               Triangle
                 (Device_Context_Handle =>
                   Draw_Item_Record.Hdc,
                  Point_Array           =>
                   Point_Array);

               Point_Array(0).X :=
                 Win32.Long
                   (3 * Window_Client_Width / 8);

               Point_Array(0).Y :=
                 Win32.Long
                   (3 * Window_Client_Height / 8);

               Point_Array(1).X :=
                 Win32.Long
                   (3 * Window_Client_Width / 8);

               Point_Array(1).Y :=
                 Win32.Long
                   (5 * Window_Client_Height / 8);

               Point_Array(2).X :=
                 Win32.Long
                   (1 * Window_Client_Width / 8);

               Point_Array(2).Y :=
                 Win32.Long
                   (4 * Window_Client_Height / 8);

               Triangle
                 (Device_Context_Handle =>
                   Draw_Item_Record.Hdc,
                  Point_Array           =>
                   Point_Array);

             when others      =>

               null;

           end case;

           if Draw_Item_Record.Itemstate mod 2 = 
              Win32.Winuser.Ods_Selected then

             Bool_Result :=
               Win32.Winuser.Invertrect
                 (Hdc  =>
                   Draw_Item_Record.Hdc,
                  Lprc =>
                   Convert_Address_To_Ac_Rect_T
                     (Draw_Item_Record.Rcitem'Address));

           end if;

           if (Draw_Item_Record.Itemstate mod 32) > 15 then 

             Draw_Item_Record.Rcitem.Left :=
               Draw_Item_Record.Rcitem.Left + 
                 Win32.Long
                   (Window_Client_Width / 16);

             Draw_Item_Record.Rcitem.Top :=
               Draw_Item_Record.Rcitem.Top + 
                 Win32.Long
                   (Window_Client_Height / 16);

             Draw_Item_Record.Rcitem.Right :=
               Draw_Item_Record.Rcitem.Right - 
                 Win32.Long
                   (Window_Client_Width / 16);

             Draw_Item_Record.Rcitem.Bottom :=
               Draw_Item_Record.Rcitem.Bottom - 
                 Win32.Long
                   (Window_Client_Height / 16);

             Bool_Result :=
               Win32.Winuser.Drawfocusrect
                 (Hdc  =>
                   Draw_Item_Record.Hdc,
                  Lprc =>
                   Convert_Address_To_Ac_Rect_T
                     (Draw_Item_Record.Rcitem'Address));

           end if;

	   return (0);
      
	 when Win32.WinUser.Wm_Destroy =>

	   Win32.WinUser.Postquitmessage
	     (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
