with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with Poepoem;
with System;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;

   Text_Metric           : aliased win32.Wingdi.Textmetric;

   Rectangle             : aliased win32.Windef.Rect;

   Paint_Record          : aliased win32.Winuser.Paintstruct;

   Window_Class          : aliased Win32.Winuser.Wndclass;

   Device_Context_Handle : aliased Win32.Windef.Hdc;

   Scrollbar_Text        : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "scrollbar");

   Type_Text             : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "TEXT");

   Text_Pointer           : Interfaces.C.Strings.Chars_Ptr;

   Poem_Resource          : Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "              ");

   Null_Character_String : constant String(1..1) :=(others => Character'Val(0));

   Temporary_String_Pointer : Interfaces.C.Strings.Chars_Ptr;

   Message               : aliased Win32.Winuser.Msg;

   Icon_Handle           : Win32.Windef.Hicon;

   Character_Width       : Win32.Int;

   Character_Height      : Win32.Int;

   Scroll_Width          : Win32.Int;

   Client_Height         : Win32.Int;

   Number_Of_Lines       : Win32.Int;

   Position              : Win32.Int;

   Scroll_Window_Handle  : Win32.Windef.Hwnd;

   Resource_Handle       : Win32.Windef.Hglobal;

   True                  : constant Win32.Bool := 1;

   False                 : constant Win32.Bool := 0;

   Integer_Result        : Win32.Int;
     
   Dword_Result          : Win32.Dword;

   Uint_Result           : Win32.Uint;

   Bool_Result           : Win32.Bool;
                     
   Long_Result           : Win32.Long;

   Atom_Result           : Win32.Windef.Atom;

   Window_Handle_Result  : Win32.Windef.Hwnd;

   function Convert_Hrsrc_To_Integer is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hrsrc,
        Target => Integer);

   function Min(L,R : Win32.Int) return Win32.Int is
   begin
     if L < R then
       return L;
     else
       return R;
     end if;
   end Min;

   function Max(L,R : Win32.Int) return Win32.Int is
   begin
     if L > R then
       return L;
     else
       return R;
     end if;
   end Max;

   function Convert_Integer_To_Hmenu is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Windef.Hmenu);

   function Convert_Lpvoid_To_Chars_Ptr is new
     Ada.Unchecked_Conversion
       (Source => Win32.Lpvoid,
        Target => Interfaces.C.Strings.Chars_Ptr);

   function Convert_Chars_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Chars_Ptr_To_Lpstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpstr);

   function Convert_Integer_To_Ac_Rect_T is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Wingdi.Ac_Rect_T);

   procedure Process_Resource_Handle
     (Text_Pointer    : in out Interfaces.C.Strings.Chars_Ptr;
      Number_Of_Lines : in out Win32.Int) is

      Index_Into_String : Integer;

      Temporary_String : String(1 .. Integer(Interfaces.C.Strings.Strlen(Item =>Text_Pointer))) :=
       Interfaces.C.Strings.Value
          (Item =>
             Text_Pointer);

   begin

     for i in 1 .. Temporary_String'Length loop

       if 
          Temporary_String(i) = '\' or             
          Temporary_String(i) = Character'Val(0) then

          Index_Into_String :=
            i - 1;

          exit;

       end if;

       if 
         Temporary_String(i) = Character'Val(10) then             

         Number_Of_Lines :=
         Number_Of_Lines + 1;

       end if;

     end loop;

     Text_Pointer :=
       Interfaces.C.Strings.New_String
         (Str =>
           Temporary_String(1 .. Index_Into_String));

   end;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

   Hrsrc_Result          : Win32.Windef.Hrsrc;

   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

 	   Device_Context_Handle := 
             Win32.Winuser.Getdc
              (Hwnd =>
                Window_Handle);

           Bool_Result := 
             Win32.Wingdi.Gettextmetrics
              (Hdc => 
                Device_Context_Handle,
               Lptm => 
                Text_Metric'Access);

           Character_Width :=
             Win32.Int
               (Text_Metric.Tmavecharwidth);

           Character_Height :=
             Win32.Int
               (Text_Metric.Tmheight +
                Text_Metric.Tmexternalleading);

           Integer_Result := 
             Win32.Winuser.Releasedc
               (Hwnd =>
                 Window_Handle,
                Hdc  =>
                 Device_Context_Handle);

           Scroll_Width :=
             Win32.Winuser.Getsystemmetrics
               (Nindex =>
                 Win32.Winuser.Sm_Cxvscroll);

           Scroll_Window_Handle := 
             Win32.Winuser.Createwindow
               (Lpclassname => 
                 Convert_Chars_Ptr_To_Lpcstr
                   (S => 
                     Scrollbar_Text),
                Lpwindowname => 
                 null,
                Dwstyle      => 
                 Win32.Winuser.Ws_Child   or
                 Win32.Winuser.Ws_Visible or
                 Win32.Winuser.Sbs_Vert,
                X            => 
                 0, 
                Y            => 
                 0,
                Nwidth       => 
                 0,
                Nheight      => 
                 0,
                Hwndparent   => 
                 Window_Handle,
                Hmenu        => 
                 Convert_Integer_To_Hmenu
                   (S =>
                     1),
                Hinstance    => 
                 Window_Class.Hinstance, 
                Lpparam      =>
                 Null);

           Integer_Result :=
             Win32.Winuser.Loadstring
               (Hinstance  =>
                 Window_Class.Hinstance,
                Uid        =>
                 Poepoem.Ids_Poemres,
                Lpbuffer   =>
                 Convert_Chars_Ptr_To_Lpstr
                   (Poem_Resource),
                Nbuffermax =>
                 Win32.Int
                   (Interfaces.C.Strings.Strlen
                     (Item =>
                       Poem_Resource)));

           Resource_Handle :=
             Win32.Winbase.Loadresource
               (Hmodule  =>
                 Window_Class.Hinstance,
                Hresinfo =>
                 Win32.Winbase.Findresource
                  (Hmodule =>
                    Window_Class.Hinstance,
                   Lpname  =>
                    Convert_Chars_Ptr_To_Lpcstr
                      (S =>
                        Poem_Resource),
                   Lptype  =>
                    Convert_Chars_Ptr_To_Lpcstr
                      (S =>
                        Type_Text)));

           Text_Pointer :=
             Convert_Lpvoid_To_Chars_Ptr
               (S =>
                 Win32.Winbase.Lockresource
                   (Hresdata =>
                     Resource_Handle));

           Number_Of_Lines :=
             0;

           Process_Resource_Handle
             (Text_Pointer =>
               Text_Pointer,
              Number_Of_Lines =>
               Number_Of_Lines);

           Bool_Result :=
             Win32.Winuser.Setscrollrange
               (Hwnd =>
                 Scroll_Window_Handle,
                Nbar =>
                 Win32.Winuser.Sb_Ctl,
                Nminpos =>
                 0,
                Nmaxpos =>
                 Number_Of_Lines,
                Bredraw =>
                 False);

           Integer_Result :=
             Win32.Winuser.Setscrollpos
               (Hwnd =>
                 Scroll_Window_Handle,
                Nbar =>
                 Win32.Winuser.Sb_Ctl,
                Npos =>
                 0,
                Bredraw =>
                 False);

 	   return (0);

	 when Win32.Winuser.Wm_Size =>

           Client_Height :=
             Win32.Int
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam)));

           Bool_Result :=
             Win32.Winuser.Movewindow
               (Hwnd     =>
                 Scroll_Window_Handle,
                X        =>
                 Win32.Int
                   (Win32.Utils.Loword
                     (Win32.Dword
                       (Lparam))) - 
                 Scroll_Width,
                Y        =>
                 0,
                Nwidth   =>
                 Scroll_Width,
                Nheight  =>
                 Client_Height,
                Brepaint =>
                 True);

           Window_Handle_Result :=
             Win32.Winuser.Setfocus
               (Hwnd =>
                 Window_Handle);

	   return (0);

	 when Win32.Winuser.Wm_Setfocus =>

           Window_Handle_Result :=
             Win32.Winuser.Setfocus
               (Hwnd =>
                 Scroll_Window_Handle);

	   return (0);

	 when Win32.Winuser.Wm_Vscroll =>

           case Wparam is

             when Win32.Winuser.Sb_Top =>

               Position :=
                 0;

             when Win32.Winuser.Sb_Bottom =>

               Position :=
                 Number_Of_Lines;

             when Win32.Winuser.Sb_Lineup =>

               Position :=
                 Position - 1;

             when Win32.Winuser.Sb_Linedown =>

               Position :=
                 Position + 1;

             when Win32.Winuser.Sb_Pageup =>

               Position :=
                 Position - (Client_Height / Character_Height);

             when Win32.Winuser.Sb_Pagedown =>

               Position :=
                 Position + (Client_Height / Character_Height);

             when Win32.Winuser.Sb_Thumbposition =>

               Position :=
                 Win32.Int
                   (Win32.Utils.Loword
                      (Win32.Dword
                         (Lparam)));

             when others =>

             null;

           end case;

           Position :=
             Max
               (L =>
                 0,
                R =>
                 Min
                   (L =>
                     Position,
                    R =>
                     Number_Of_Lines));

           if 
             Position /=
             Win32.Winuser.Getscrollpos
               (Hwnd =>
                 Scroll_Window_Handle,
                Nbar =>
                 Win32.Winuser.Sb_Ctl) then

             Integer_Result :=
               Win32.Winuser.Setscrollpos
                 (Hwnd =>
                   Scroll_Window_Handle,
                  Nbar =>
                   Win32.Winuser.Sb_Ctl,
                  Npos =>
                   Position,
                  Bredraw =>
                   True);              

             Bool_Result :=
               Win32.Winuser.Invalidaterect
                 (Hwnd   =>
                   Window_Handle,
                  Lprect =>
                   Convert_Integer_To_Ac_Rect_T
                     (S =>
                       0),
                  Berase => 
                   True);

           end if;

	   return (0);

         when Win32.Winuser.Wm_Paint =>

	   Device_Context_Handle := 
             Win32.Winuser.Beginpaint
	       (Hwnd    => 
                 Window_Handle,
	        Lppaint => 
                 Paint_Record'Access);

           Bool_Result :=
             Win32.Winuser.Getclientrect
               (Hwnd   =>
                 Window_Handle,
                Lprect =>
                 Rectangle'Access);

           Rectangle.Left :=
             Rectangle.Left + 
               Win32.Long
                 (Character_Width);

           Rectangle.Top :=
             Rectangle.Top +
               Win32.Long 
                 (Character_Height * (1 - Position));

	   Integer_Result := 
             Win32.Winuser.Drawtext
	       (Hdc =>
                 Device_Context_Handle,
	        Lpstring => 
                 Convert_Chars_Ptr_To_Lpcstr
                   (S => 
                     Text_Pointer),
	        Ncount   => 
                 - 1,
	        Lprect   => 
                 Rectangle'Access,
	        Uformat  => 
                 Win32.Winuser.Dt_Externalleading);

 	   Bool_Result := 
             Win32.Winuser.Endpaint
	       (Hwnd    => 
                 Window_Handle,
	        Lppaint => 
                 Paint_Record'Access);

	   return (0);

	 when Win32.WinUser.Wm_Destroy =>

           Bool_Result :=
             Win32.Winbase.Freeresource
               (Hresdata =>
                 Resource_Handle);

	   Win32.WinUser.Postquitmessage
	     (Nexitcode => 
               0);

	   return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
  
   begin

      Integer_Result :=
        Win32.Winuser.Loadstring
          (Hinstance  =>
            Instance,
           Uid        =>
            Poepoem.Ids_Appname,
           Lpbuffer   =>
            Convert_Chars_Ptr_To_Lpstr
              (Windows.Data.Class_Name),
           Nbuffermax =>
            Win32.Int
              (Interfaces.C.Strings.Strlen
                (Item =>
                  Windows.Data.Class_Name)));

      Integer_Result :=
        Win32.Winuser.Loadstring
          (Hinstance  =>
            Instance,
           Uid        =>
            Poepoem.Ids_Caption,
           Lpbuffer   =>
            Convert_Chars_Ptr_To_Lpstr
              (Windows.Data.Caption),
           Nbuffermax =>
            Win32.Int
              (Interfaces.C.Strings.Strlen
                (Item =>
                  Windows.Data.Caption)));

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Win32.Winuser.Loadicon
               (Hinstance => 
                 Instance,
                Lpiconname =>
                 Convert_Chars_Ptr_To_Lpcstr
                  (Windows.Data.Class_Name)),
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
