with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;

   Paint_Record          : aliased win32.Winuser.Paintstruct;

   Window_Class          : aliased Win32.Winuser.Wndclass;

   Device_Context_Handle : aliased Win32.Windef.Hdc;

   Message               : aliased Win32.Winuser.Msg;

   Icon_Handle           : Win32.Windef.Hicon;

   Icon_Width            : Win32.Int;

   Icon_Height           : Win32.Int;

   Client_Width          : Win32.Int;

   Client_Height         : Win32.Int;

   Width                 : Win32.Int;

   Height                : Win32.Int;

   Integer_Result        : Win32.Int;
     
   Dword_Result          : Win32.Dword;

   Uint_Result           : Win32.Uint;

   Bool_Result           : Win32.Bool;
                     
   Long_Result           : Win32.Long;

   Atom_Result           : Win32.Windef.Atom;

   function Convert_Hicon_To_Integer is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hbrush,
        Target => Integer);

   function Convert_Hinstance_To_Integer is new
     Ada.Unchecked_Conversion
       (Source => Win32.Windef.Hbrush,
        Target => Integer);

   function Convert_Chars_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

           Icon_Handle :=
             Win32.Winuser.Loadicon
               (Hinstance => 
                 Window_Class.Hinstance,
                Lpiconname =>
                 Convert_Chars_Ptr_To_Lpcstr
                  (Windows.Data.Class_Name));

           Icon_Width :=
             Win32.Winuser.Getsystemmetrics
               (Nindex =>
                 Win32.Winuser.Sm_Cxicon);

           Icon_Height :=
             Win32.Winuser.Getsystemmetrics
               (Nindex =>
                 Win32.Winuser.Sm_Cyicon);

 	   return (0);

	 when Win32.Winuser.Wm_Size =>

           Client_Width :=
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)));

           Client_Height :=
             Win32.Int
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam)));

	   return (0);

         when Win32.Winuser.Wm_Paint =>

	   Device_Context_Handle := 
             Win32.Winuser.Beginpaint
	       (Hwnd    => 
                 Window_Handle,
	        Lppaint => 
                 Paint_Record'Access);

           Height :=
             Icon_Height;

           while Height < Client_Height loop

             Width :=
               Icon_Width;

             while Width < Client_Width loop

               Bool_Result :=
                 Win32.Winuser.Drawicon
                   (Hdc   =>
                     Device_Context_Handle,
                    X     =>
                     Width,
                    Y     =>
                     Height,
                    Hicon =>
                     Icon_Handle);

             Width :=
               Width +
               2 * Icon_Width;

             end loop;

             Height :=
               Height +
               2 * Icon_Height;

           end loop;

 	   Bool_Result := 
             Win32.Winuser.Endpaint
	       (Hwnd    => 
                 Window_Handle,
	        Lppaint => 
                 Paint_Record'Access);

	   return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Win32.Winuser.Loadicon
               (Hinstance => 
                 Instance,
                Lpiconname =>
                 Convert_Chars_Ptr_To_Lpcstr
                  (Windows.Data.Class_Name)),
	    Hcursor       => 
             Win32.Winuser.Loadcursor
               (Hinstance    => 
                 Instance,
                Lpcursorname =>
                 Convert_Chars_Ptr_To_Lpcstr
                  (Windows.Data.Class_Name)),
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
