
--
-- Author : Pascal Obry
--
-- This is a sample program to show some OpenGL features.
--
-- It compile fine under GNAT 3.04a. You must add the following
-- libraries : OPENGL32.LIB GLU32.LIB GLAUX.LIB
--

with Win32.Gl;
with Win32.Glu;
with Win32.Glaux;
with Interfaces.C;
with Interfaces.C.Strings;
with System;
with Unchecked_Conversion;

procedure OGLMaterial is

   use Win32.Gl;
   use Win32.Glu;
   use Win32.Glaux;
   use Interfaces;

   function Conv is new Unchecked_Conversion (System.Address, AUXRESHAPEPROC);
   function Conv is new Unchecked_Conversion (System.Address, AUXMAINPROC);

   type Glfloat_4 is array (1 .. 4) of aliased Glfloat;
   type Glfloat_8 is array (1 .. 8) of aliased Glfloat;

   Window_Name : aliased C.Char_Array := C.To_C ("Lighting");
   Res : Glenum;

   CtlPoints : array (1 .. 4, 1 .. 4, 1 .. 3) of aliased Glfloat;
   TheNurb   : PGLUnurbsObj;

   --  callbacks must be defined with the Stdcall Convention
   procedure MyReshape (W, H : in Glsizei);
   pragma Convention (Stdcall, MyReshape);

   procedure Display;
   pragma Convention (Stdcall, Display);

   --  here start the definition of the gl scene.
   procedure MyInit is

      Ambient        : Glfloat_4 := ( 0.0, 0.0, 0.0, 1.0 );
      Diffuse        : Glfloat_4 := ( 1.0, 1.0, 1.0, 1.0 );
      Specular       : Glfloat_4 := ( 1.0, 1.0, 1.0, 1.0 );
      Position       : Glfloat_4 := ( 0.0, 3.0, 2.0, 0.0 );
      Lmodel_Ambient : Glfloat_4 := ( 0.4, 0.4, 0.4, 1.0 );
      local_view     : aliased Glfloat   := 0.0 ;

   begin

      GlEnable (GL_DEPTH_TEST);
      GlDepthFunc (GL_LESS);

      GlLightfv (GL_LIGHT0, GL_AMBIENT, Ambient (1)'Unchecked_Access);
      GlLightfv (GL_LIGHT0, GL_DIFFUSE, Diffuse (1)'Unchecked_Access);
      GlLightfv (GL_LIGHT0, GL_POSITION, Position (1)'Unchecked_Access);
      GlLightModelfv (GL_LIGHT_MODEL_AMBIENT,
                      Lmodel_Ambient (1)'Unchecked_Access);
      GlLightModelfv (GL_LIGHT_MODEL_LOCAL_VIEWER,
                      Local_View'Unchecked_Access);

      GlEnable (GL_LIGHTING);
      GlEnable (GL_LIGHT0);

      GlClearColor (0.0, 0.1, 0.1, 0.0);
   end MyInit;


   procedure Display is
      No_Mat            : Glfloat_4 := ( 0.0, 0.0, 0.0, 1.0 );
      Mat_Ambient       : Glfloat_4 := ( 0.7, 0.7, 0.7, 1.0 );
      Mat_Ambient_Color : Glfloat_4 := ( 0.8, 0.8, 0.2, 1.0 );
      Mat_Diffuse       : Glfloat_4 := ( 0.1, 0.5, 0.8, 1.0 );
      Mat_Specular      : Glfloat_4 := ( 1.0, 1.0, 1.0, 1.0 );
      No_Shininess      : aliased Glfloat := 0.0;
      Low_Shininess     : aliased Glfloat := 5.0;
      High_Shininess    : aliased Glfloat := 100.0;
      Mat_Emission      : Glfloat_4 := (0.3, 0.2, 0.2, 0.0);
   begin

      GlClear (GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT);

      glPushMatrix;
      glTranslatef (-3.75, 3.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT, No_Mat (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, Mat_Diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, No_Mat (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    No_Shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, No_Mat (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      glPopMatrix;

      glPushMatrix;
      glTranslatef (-1.25, 3.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT, no_mat (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    low_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, no_mat (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      GlPopMatrix;

      GlPushMatrix;
      glTranslatef (1.25, 3.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT, no_mat (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    high_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, no_mat (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      GlPopMatrix;

      GlPushMatrix;
      glTranslatef (3.75, 3.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT, no_mat (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, no_mat (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    no_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, mat_emission (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      GlPopMatrix;

      GlPushMatrix;
      glTranslatef (-3.75, 0.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT, mat_ambient (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, no_mat (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    no_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, no_mat (1)'Unchecked_Access);
      auxSolidSphere(1.0);
      GlPopMatrix;

      GlPushMatrix;
      glTranslatef (-1.25, 0.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT, mat_ambient (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    low_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, no_mat (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      GlPopMatrix;

      GlPushMatrix;
      glTranslatef (1.25, 0.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT, mat_ambient (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    high_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, no_mat (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      GlPopMatrix;

      GlPushMatrix;
      glTranslatef (3.75, 0.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT, mat_ambient (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, no_mat (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    no_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, mat_emission (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      GlPopMatrix;

      GlPushMatrix;
      glTranslatef (-3.75, -3.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT,
                    mat_ambient_color (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, no_mat (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    no_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, no_mat (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      GlPopMatrix;

      GlPushMatrix;
      glTranslatef (-1.25, -3.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT,
                    mat_ambient_color (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    low_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, no_mat (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      GlPopMatrix;

      GlPushMatrix;
      glTranslatef (1.25, -3.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT,
                    mat_ambient_color (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    high_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION, no_mat (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      GlPopMatrix;

      GlPushMatrix;
      glTranslatef (3.75, -3.0, 0.0);
      GlMaterialfv (GL_FRONT, GL_AMBIENT,
                    mat_ambient_color (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, no_mat (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SHININESS,
                    no_shininess'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_EMISSION,
                    mat_emission (1)'Unchecked_Access);
      AuxSolidSphere (1.0);
      GlPopMatrix;

      glFlush;
   end Display;

   procedure MyReshape (W, H : GLsizei) is
   begin
      GlViewport (0, 0, w, h);
      GlMatrixMode (GL_PROJECTION);
      glLoadIdentity;
      if W <= (H * 2) then
        glOrtho (-6.0, 6.0, -3.0 * (GLdouble(H)*2.0)/GLdouble(W),
                 3.0 * (GLdouble(H)*2.0)/GLdouble(W), -10.0, 10.0);
      else
         glOrtho (-6.0 * GLdouble(W)/(GLdouble(H) * 2.0),
                  6.0 * GLdouble(W)/(GLdouble(H)*2.0), -3.0, 3.0,
                  -10.0, 10.0);
         GlMatrixMode (GL_MODELVIEW);
      end if;
   end MyReshape;

begin
   AuxInitDisplayMode (AUX_SINGLE + AUX_RGB + AUX_DEPTH16);
   AuxInitPosition (0, 0, 600, 600);
   Res := AuxInitWindow (Window_Name (Window_Name'First)'Unchecked_Access);
   MyInit;
   AuxReshapeFunc (Conv(MyReshape'Address));
   Auxmainloop (Conv(Display'Address));
end OGLMaterial;
