
--
-- Author : Pascal Obry
--
-- This is a sample program to show some OpenGL features.
--
-- It compile fine under GNAT 3.04a. You must add the following
-- libraries : OPENGL32.LIB GLU32.LIB GLAUX.LIB
--

with Win32.Gl;
with Win32.Glu;
with Win32.Glaux;
with Interfaces.C;
with Interfaces.C.Strings;
with System;
with Unchecked_Conversion;

procedure OGLStencil is

   use Win32.Gl;
   use Win32.Glu;
   use Win32.Glaux;
   use Interfaces;

   function Conv is new Unchecked_Conversion (System.Address, AUXRESHAPEPROC);
   function Conv is new Unchecked_Conversion (System.Address, AUXMAINPROC);

   type Glfloat_4 is array (1 .. 4) of aliased Glfloat;

   Window_Name : aliased C.Char_Array := C.To_C ("Stencil Test");
   Res : Glenum;

   YELLOWMAT : constant := 1;
   BLUEMAT   : constant := 2;

   --  callbacks must be defined with the Stdcall Convention
   procedure MyReshape (W, H : in Glsizei);
   pragma Convention (Stdcall, MyReshape);

   procedure Display;
   pragma Convention (Stdcall, Display);

   procedure myinit is

      Yellow_Diffuse  : Glfloat_4 := ( 0.7, 0.7, 0.0, 1.0 );
      Yellow_Specular : Glfloat_4 := ( 1.0, 1.0, 1.0, 1.0 );

      Blue_Diffuse    : Glfloat_4 := ( 0.1, 0.1, 0.7, 1.0 );
      Blue_Specular   : Glfloat_4 := ( 0.1, 1.0, 1.0, 1.0 );

      Position_One    : Glfloat_4 := ( 1.0, 1.0, 1.0, 0.0 );

   begin

      GlNewList (YELLOWMAT, GL_COMPILE);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE,
                    Yellow_Diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR,
                    Yellow_Specular (1)'Unchecked_Access);
      GlMaterialf (GL_FRONT, GL_SHININESS, 64.0);
      glEndList;

      GlNewList (BLUEMAT, GL_COMPILE);
      GlMaterialfv (GL_FRONT, GL_DIFFUSE,
                    Blue_Diffuse (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR,
                    Blue_Specular (1)'Unchecked_Access);
      GlMaterialf (GL_FRONT, GL_SHININESS, 45.0);
      glEndList;

      GlLightfv (GL_LIGHT0, GL_POSITION,
                 Position_One (1)'Unchecked_Access);

      GlEnable (GL_LIGHT0);
      GlEnable (GL_LIGHTING);
      GlDepthFunc (GL_LESS);
      GlEnable (GL_DEPTH_TEST);

      GlClearStencil (16#0#);
      GlEnable (GL_STENCIL_TEST);
   end MyInit;

   procedure Display is
   begin
      GlClear (GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT);

      glStencilFunc (GL_EQUAL, 16#1#, 16#1#);
      glCallList (BLUEMAT);
      auxSolidSphere (0.5);

      glStencilFunc (GL_NOTEQUAL, 16#1#, 16#1#);
      glStencilOp (GL_KEEP, GL_KEEP, GL_KEEP);
      glPushMatrix;
      glRotatef (45.0, 0.0, 0.0, 1.0);
      glRotatef (45.0, 0.0, 1.0, 0.0);
      glCallList (YELLOWMAT);
      auxSolidTorus (0.275, 0.85);

      glPushMatrix;
      glRotatef (90.0, 1.0, 0.0, 0.0);
      auxSolidTorus (0.275, 0.85);
      glPopMatrix;

      glPopMatrix;
      glFlush;
   end Display;

   procedure MyReshape (W, H : in GLsizei) is
   begin

      GlViewport (0, 0, w, h);

      GlClear (GL_STENCIL_BUFFER_BIT);

      GlMatrixMode (GL_PROJECTION);
      glLoadIdentity;
      GlOrtho (-3.0, 3.0, -3.0, 3.0, -1.0, 1.0);
      GlMatrixMode (GL_MODELVIEW);
      glLoadIdentity;

      glStencilFunc (GL_ALWAYS, 16#1#, 16#1#);
      glStencilOp (GL_REPLACE, GL_REPLACE, GL_REPLACE);
      GlBegin (GL_QUADS);
      glVertex3f (-1.0, 0.0, 0.0);
      glVertex3f (0.0, 1.0, 0.0);
      glVertex3f (1.0, 0.0, 0.0);
      glVertex3f (0.0, -1.0, 0.0);
      glEnd;

      GlMatrixMode (GL_PROJECTION);
      glLoadIdentity;
      GluPerspective (45.0, GLdouble (W) / GLdouble (H), 3.0, 7.0);
      GlMatrixMode (GL_MODELVIEW);
      glLoadIdentity;
      GlTranslatef (0.0, 0.0, -5.0);
   end MyReshape;

begin
   auxInitDisplayMode (AUX_SINGLE + AUX_RGB + AUX_DEPTH16 + AUX_STENCIL);
   auxInitPosition (0, 0, 400, 400);
   Res := AuxInitWindow (Window_Name (Window_Name'First)'Unchecked_Access);
   myinit;
   auxReshapeFunc (Conv(MyReshape'Address));
   AuxMainLoop (Conv(Display'Address));
end OGLStencil;
