
--
-- Author : Pascal Obry
--
-- This is a sample program to show some OpenGL features.
--
-- It compile fine under GNAT 3.04a. You must add the following
-- libraries : OPENGL32.LIB GLU32.LIB GLAUX.LIB
--

with Win32.Gl;
with Win32.Glu;
with Win32.Glaux;
with Interfaces.C;
with Interfaces.C.Strings;
with System;
with Unchecked_Conversion;

procedure OGLTest is

   use Win32.Gl;
   use Win32.Glu;
   use Win32.Glaux;
   use Interfaces;

   function Conv is new Unchecked_Conversion (System.Address, AUXRESHAPEPROC);
   function Conv is new Unchecked_Conversion (System.Address, AUXMAINPROC);

   Window_Name : aliased C.Char_Array := C.To_C ("Perspective 3-D Cube");
   Res : Glenum;

   type Glfloat_4 is array (1 .. 4) of aliased Glfloat;

   --  callbacks must be defined with the Stdcall Convention
   procedure MyReshape (W, H : in Glsizei);
   pragma Convention (Stdcall, MyReshape);

   procedure Display;
   pragma Convention (Stdcall, Display);

   --  here start the definition of the gl scene.
   procedure MyInit is

      Mat_Ambient  : Glfloat_4 := (0.0, 0.0, 1.0, 1.0);
      Mat_Specular : Glfloat_4 := (1.0, 1.0, 1.0, 1.0);

      Light_Position : Glfloat_4 := (0.5, 0.8, 0.0, 1.0);
      Lm_Ambient     : Glfloat_4 := (0.1, 0.1, 0.1, 1.0);

   begin
      GlMaterialfv (GL_FRONT, GL_AMBIENT, Mat_Ambient (1)'Unchecked_Access);
      GlMaterialfv (GL_FRONT, GL_SPECULAR, Mat_Specular (1)'Unchecked_Access);
      GlMaterialf  (GL_FRONT, GL_SHININESS, 90.0);

      GlLightfv      (GL_LIGHT0, GL_POSITION,
                      Light_Position (1)'Unchecked_Access);
      GlLightModelfv (GL_LIGHT_MODEL_AMBIENT,
                      Lm_Ambient (1)'Unchecked_Access);

      GlEnable (GL_LIGHTING);
      GlEnable (GL_LIGHT0);
      GlDepthFunc (GL_LESS);
      GlEnable (GL_DEPTH_TEST);
      GlShadeModel (GL_FLAT);

      GlClearColor (0.0, 0.0, 0.0, 0.0);
   end MyInit;

   procedure MyReshape (W, H : in Glsizei) is
   begin
      GlMatrixMode (GL_PROJECTION);
      GlLoadIdentity;
      GlFrustum (-1.0, 1.0, -1.0, 1.0, 1.5, 60.0);
      GlMatrixMode (GL_MODELVIEW);
      GlViewPort (0, 0, W, H);
   end MyReshape;

   procedure Display is

      procedure Display_Objects is

         Cube_Diffuse : Glfloat_4 := (1.0, 0.1, 0.1, 0.4);

      begin
         GlMaterialfv (GL_FRONT, GL_DIFFUSE,
                       Cube_Diffuse (1)'Unchecked_Access);
         AuxSolidCube (1.0);
      end Display_Objects;

   begin
      GlClear (GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT);
      GlLoadIdentity;
      GlScalef (1.0, 2.0, 1.0);

      for I in reverse 3 .. 50 loop
         GlClear (GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT);
         GlPushMatrix;
         GlTranslatef (0.0, 0.0, -GlFloat (I));
         Glrotatef (25.0, 0.3, 1.0, 0.0);
         Display_Objects;
         AuxSwapBuffers;
         GlPopMatrix;
      end loop;

      for I in reverse 1 .. 360 loop
         GlClear (GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT);
         GlPushMatrix;
         GlTranslatef (0.0, 0.0, -3.0);
         Glrotatef (Glfloat (I*2 mod 360), 0.3, 1.0, 0.0);
         Display_Objects;
         AuxSwapBuffers;
         GlPopMatrix;
      end loop;

      GlFlush;
   end Display;

begin
   AuxInitDisplayMode (AUX_DOUBLE + AUX_RGB + AUX_DEPTH);
   AuxInitPosition (0, 0, 500, 500);
   Res := AuxInitWindow (Window_Name (Window_Name'First)'Unchecked_Access);
   MyInit;
   AuxReshapeFunc (Conv(MyReshape'Address));
   Auxmainloop (Conv(Display'Address));
end OGLTest;
