-- $Source: /home/harp/1/proto/monoBANK/winnt/wxform_ada.adb,v $ 
-- $Revision: 1.5 $ $Date: 96/09/04 07:29:23 $ $Author: mg $ 
-- 
-- Create the window and start message loop.  This is based on code found in
-- wxform.c.  The other routines that made up the original c program can be
-- found in wxform_util package.


with Ada.Unchecked_Conversion;
with Interfaces.C;
with Interfaces.C.Strings;  use Interfaces.C.Strings;
with System;
with Win32;   use Win32;
with Win32.WinDef;  use Win32.WinDef;
with Win32.WinMain;
with Win32.WinUser;
with Win32.WinGdi;
with Win32.WinNT;
with Wxform_Pkg;
with Wxform_Util;

procedure Wxform_Ada is

  package WU renames Wxform_Util;

  use type Interfaces.C.INT;
  use type Win32.DWORD;

  MSG    : WinUser.LPMSG;
  hicon  : Win32.DWORD;  -- WinDef.HICON_T;

  -- dummy function return values
  bResult : Win32.BOOL;
  lResult : Win32.LONG;
  dResult : Win32.DWORD;

  NULL_HANDLE: constant WinNT.HANDLE := System.Null_Address;


	--  FUNCTION: InitApplication(HINSTANCE)
	--
	--  PURPOSE: Initializes window data and registers window class
	--
	--  COMMENTS:
	--
	--  This function is called at initialization time only if no other
	--  instances of the application are running.  This function performs
	--  initialization tasks that can be done once for any number of running
	--  instances.
	function InitApplication(hInstance : Windef.HINSTANCE) return Win32.BOOL is

	  WC_Ptr : Winuser.PWNDCLASSA;
	  use type Win32.UINT;
	begin
          Wxform_Pkg.WC :=(    
                    style       => Winuser.CS_HREDRAW or Winuser.CS_VREDRAW,
		    lpfnWndProc => WU.MainWndProc'Access,
		    cbClsExtra  => 0,								
		    cbWndExtra  => 0,
		    hInstance   => hInstance, 
		    hIcon       => WinUser.LoadIcon(hInstance,
					     WU.New_LPCSTR("TransformIcon")),
		    hCursor     => WinUser.LoadCursor(System.Null_Address,
					 Win32.LPCSTR(WinUser.IDC_ARROW)),
		    hbrBackground=>WinDef.HBRUSH (WinGdi.GetStockObject(
				     WinGdi.LTGRAY_BRUSH)),
		    lpszMenuName  => null,
		    lpszClassName => WU.New_LPCSTR("wxform"));
	  WC_Ptr := Wxform_Pkg.WC'Access;
	  --  Register the window class and return success/failure code.
	  return Win32.BOOL (Winuser.RegisterClass
	                           (Winuser.ac_WNDCLASSA_t(WC_Ptr)));

	end InitApplication;

begin
  -- Get the hInst passed in to WinMain by windows and store it in a global
  WU.hInst := WinMain.Get_hInstance; 

  bResult := InitApplication(WU.hInst);

  WU.hwndMain := WinUser.CreateWindow (
         lpClassName => WU.New_LPCSTR("wxform"),
         lpWindowName=> WU.New_LPCSTR("World Transform Demonstration"),
         dwStyle     => WinUser.WS_OVERLAPPEDWINDOW or WinUser.WS_CLIPCHILDREN,
         X           => WinUser.CW_USEDEFAULT, 
         Y           => WinUser.CW_USEDEFAULT,
         nWidth      => WinUser.CW_USEDEFAULT,
         nHeight     => WinUser.CW_USEDEFAULT,
         hWndParent  => System.Null_Address,
         hMenu       => System.Null_Address,
	 hInstance   => WU.hInst,
         lpParam     => System.Null_Address);

  bResult := WinUser.ShowWindow  (WU.hwndMain, WinMain.Get_nCmdShow);
  bResult := WinUser.UpdateWindow(WU.hwndMain);

  -- Get the track object and paint it for the first time.
  WU.doTrackObject (WU.TROB_NEW,   WU.hwndMain, 0);
  WU.doTrackObject (WU.TROB_PAINT, WU.hwndMain, 0);

  -- load and display dialog for the world transform matrix.
  -- then fill its entry fields.  Also, get the HICON from the
  -- main window and fill it into the dialog's class structure
  -- for this application.
  WU.hwndTransform := WinUser.CreateDialog(WU.hInst, 
                                           WU.New_LPCSTR("TransformDlg"),
                                           WU.hwndMain,
                                           WU.TransformDlgProc'Access);
  hicon   := WinUser.GetClassLong (WU.hwndMain, WinUser.GCL_HICON);
  dResult := WinUser.SetClassLong (WU.hwndTransform,
                                   WinUser.GCL_HICON,
                                   Win32.LONG (hicon));
  WU.showTransform := Win32.TRUE;
  lResult := WinUser.SendMessage(WU.hwndTransform, WU.WM_PUTUPFLOATS,0,0);

  -- load and display the dialog for the mouse position.
  -- minimize it initially.
  WU.hwndMouse := WinUser.CreateDialog(WU.hInst, 
                                       WU.New_LPCSTR("MouseDlg"),
                                       WU.hwndMain,
                                       WU.MouseDlgProc'Access);
  bResult := WinUser.ShowWindow (WU.hwndMouse, WinUser.SW_SHOWMINIMIZED);
  WU.showMouse := Win32.FALSE;

  -- load and display the dialog with the direct manipulation help.
  -- minimize it initially.   (Don't need a unique window procedure.)
  WU.hwndHelp := WinUser.CreateDialog(WU.hInst, 
                                      WU.New_LPCSTR ("helpDlg"), 
                                      WU.hwndMain, 
                                      WU.Null_Dlgproc'Access);
  bResult := WinUser.ShowWindow (WU.hwndHelp, WinUser.SW_SHOWMINIMIZED);

  --  /* Loop getting messages and dispatching them. */
  MSG := new WinUser.MSG;
  while (WinUser.GetMessage (MSG,System.Null_Address,0,0) /= 0) loop
      --if (!IsDialogMessage (hwndTransform, &msg))
      --if (!IsDialogMessage (hwndMouse, &msg))
      --if (!IsDialogMessage (hwndHelp, &msg)){
    if WinUser.IsDialogMessage (WU.hwndTransform, MSG) = Win32.FALSE and then
       WinUser.IsDialogMessage (WU.hwndMouse, MSG)     = Win32.FALSE and then
         WinUser.IsDialogMessage (WU.hwndHelp, MSG)    = Win32.FALSE then
      bResult := WinUser.TranslateMessage(WinUser.ac_MSG_t(MSG));
      lResult := WinUser.DispatchMessage(WinUser.ac_MSG_t(MSG));
    end if;
  end loop;

-------------------------------------------------------------------------------
--
-- THIS FILE AND ANY ASSOCIATED DOCUMENTATION IS PROVIDED "AS IS" WITHOUT 
-- WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT 
-- LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR 
-- A PARTICULAR PURPOSE.  The user assumes the entire risk as to the accuracy 
-- and the use of this file.  This file may be used only by licensees of 
-- Microsoft Corporation's WIN32 Software Development Kit in accordance with 
-- the terms of the licensee's End-User License Agreement for Microsoft 
-- Software for the WIN32 Development Kit.
--
-- Copyright (c) Intermetrics, Inc. 1995
-- Portions (c) 1985-1994 Microsoft Corporation with permission.
-- Microsoft is a registered trademark and Windows and Windows NT are 
-- trademarks of Microsoft Corporation.
--
-------------------------------------------------------------------------------

end Wxform_Ada;
