-- $Source: /home/harp/1/proto/monoBANK/winnt/ObjectAdawin32-crt-largeint-ops.adb,v $ 
-- $Revision: 1.1 $ $Date: 96/09/13 11:31:58 $ $Author: mg $ 
with System;

-- Academic Ada body, where native 64 bit integers are not supported.
-- Wrapper C routines are needed instead of going directly to Win32 API,
-- since return by value of the LONGLONG record is not supported.

package body Win32.crt.Largeint.Ops is

    Unsupported_Operation : exception;

    MAX_IMAGE_LENGTH : constant := 50; --?
    subtype Image_Result is Interfaces.C.char_array(0 .. MAX_IMAGE_LENGTH);

    -- Type Ptr_64 is used for an ironic situation: we want to avoid
    -- an extra pass-by copy into the wrapper C routines.
    subtype Ptr_64 is System.Address;

    use type System.Bit_Order;
    High_Order_First : constant Boolean := 
        System.Default_Bit_Order = System.High_Order_First;

    -- visible

    function To_64(Low : DWORD; High: DWORD := 0)
        return DWORDLONG is
	Res: DWORDLONG;
    begin
	if High_Order_First then
	    Res.A := UINT(High);
	    Res.B := UINT(Low);
	else
	    Res.B := UINT(High);
	    Res.A := UINT(Low);
	end if;
	return Res;
    end;

    function To_64(Low : DWORD; High: LONG := 0)
        return LONGLONG is
	Res: LONGLONG;
    begin
	if High_Order_First then
	    Res.A := INT(High);
	    Res.B := INT(To_Signed(Low));
	else
	    Res.B := INT(High);
	    Res.A := INT(To_Signed(Low));
	end if;
	return Res;
    end;

    procedure Split (Value: DWORDLONG;
                     Low,
                     High : out DWORD) is
    begin
	if High_Order_First then
	    High := DWORD(Value.A);
	    Low  := DWORD(Value.B);
	else
	    Low  := DWORD(Value.A);
	    High := DWORD(Value.B);
	end if;
    end;


    -- Arithmetic
    function "+" (Left, Right: LONGLONG)  return LONGLONG is
	function wrap_add(Left, Right : Ptr_64) return LONGLONG;
	pragma Import(C, wrap_add, "wrap_add");
    begin
	return wrap_add(Left'Address, Right'Address);
    end;

    function "+" (Left, Right: DWORDLONG) return DWORDLONG is
	function wrap_add(Left, Right : Ptr_64) return DWORDLONG;
	pragma Import(C, wrap_add, "wrap_add");
    begin
	return wrap_add(Left'Address, Right'Address);
    end;

    function "-" (Left, Right: LONGLONG)  return LONGLONG is
	function wrap_sub(Left, Right : Ptr_64) return LONGLONG;
	pragma Import(C, wrap_sub, "wrap_sub");
    begin
	return wrap_sub(Left'Address, Right'Address);
    end;

    function "-" (Left, Right: DWORDLONG) return DWORDLONG is
	function wrap_sub(Left, Right : Ptr_64) return DWORDLONG;
	pragma Import(C, wrap_sub, "wrap_sub");
    begin
	return wrap_sub(Left'Address, Right'Address);
    end;

    function "*" (Left, Right: LONG)  return LONGLONG is
	function wrap_imul(Left, Right : LONG) return LONGLONG;
	pragma Import(C, wrap_imul, "wrap_imul");
    begin
	return wrap_imul(Left, Right);
    end;

    function "*" (Left, Right: DWORD) return DWORDLONG is
	function wrap_umul(Left, Right : DWORD) return DWORDLONG;
	pragma Import(C, wrap_umul, "wrap_umul");
    begin
	return wrap_umul(Left, Right);
    end;

    function "*" (Left : LONGLONG;
                  Right: LONG) return LONGLONG is
	function wrap_imul64x32
	    (Left : Ptr_64;
	     Right : LONG) return LONGLONG;
	pragma Import(C, wrap_imul64x32, "wrap_imul64x32");
    begin
	return wrap_imul64x32(Left'Address, Right);
    end;

    function "-" (Left       : LONGLONG) return LONGLONG is
	function wrap_neg(Left : Ptr_64) return LONGLONG;
	pragma Import(C, wrap_neg, "wrap_neg");
    begin
	return wrap_neg(Left'Address);
    end;

    function ExtendedMagicDivide
        (Dividend    : LONGLONG;
         MagicDivisor: LONGLONG;
         ShiftCount  : Natural) return LONGLONG is
	function wrap_ExtendedMagicDivide
	    (Dividend    : Ptr_64;
	     MagicDivisor: Ptr_64;
	     ShiftCount  : Natural) return LONGLONG;
	pragma Import(C, wrap_ExtendedMagicDivide, 
		      External_Name => "wrap_ExtendedMagicDivide");
    begin
	return wrap_ExtendedMagicDivide(Dividend'Address,
	    MagicDivisor'Address, ShiftCount);
    end;

    -- Large Integer divide - 64-bits / 32-bits -> 64-bits

    function ExtendedLargeIntegerDivide
        (Dividend : LONGLONG;
         Divisor  : DWORD;
         Remainder: access DWORD) return LONGLONG is
	function wrap_ExtendedLargeIntegerDivide
	    (Dividend : Ptr_64;
	     Divisor  : DWORD;
	     Remainder: access DWORD) return LONGLONG;
	pragma Import(C, wrap_ExtendedLargeIntegerDivide, 
		      External_Name => "wrap_ExtendedLargeIntegerDivide");
    begin
	return wrap_ExtendedLargeIntegerDivide(Dividend'Address, Divisor, Remainder);
    end;
 
    -- Large Integer divide - 64-bits / 32-bits -> 64-bits
    
    function LargeIntegerDivide
        (Dividend : LONGLONG;
         Divisor  : LONGLONG;
         Remainder: access LONGLONG) 
        return LONGLONG is
	function wrap_LargeIntegerDivide
	    (Dividend : Ptr_64;
	     Divisor  : Ptr_64;
	     Remainder: access LONGLONG) 
	    return LONGLONG;
	pragma Import(C, wrap_LargeIntegerDivide, 
		      External_Name => "wrap_LargeIntegerDivide");
    begin
	return wrap_LargeIntegerDivide(Dividend'Address, Divisor'Address, Remainder);
    end;

    -- Bitwise operations
    function "and" (Left, Right: DWORDLONG) return DWORDLONG is
	Result : DWORDLONG;
	use type Win32.UINT;
    begin
	Result.A := Left.A and Right.A;
	Result.B := Left.B and Right.B;
	return Result;
    end;

    function Shift_Left  (Value: DWORDLONG; Amount: Natural)
        return DWORDLONG is
	function wrap_Shift_Left  (Value: Ptr_64; Amount: Natural)
	    return DWORDLONG;
	pragma Import(C, wrap_Shift_Left, 
		      External_Name => "wrap_Shift_Left");
    begin
	return wrap_Shift_Left(Value'Address, Amount);
    end;

    function Shift_Right (Value: DWORDLONG; Amount: Natural)
        return DWORDLONG is
	function wrap_Shift_Right  (Value: Ptr_64; Amount: Natural)
	    return DWORDLONG;
	pragma Import(C, wrap_Shift_Right, 
		      External_Name => "wrap_Shift_Right");
    begin
	return wrap_Shift_Right(Value'Address, Amount);
    end;

    function Shift_Right_Arithmetic (Value: DWORDLONG; Amount: Natural)
        return DWORDLONG is
	function wrap_Shift_Right_Arithmetic  (Value: Ptr_64; Amount: Natural)
	    return DWORDLONG;
	pragma Import(C, wrap_Shift_Right_Arithmetic, 
		      External_Name => "wrap_Shift_Right_Arithmetic");
    begin
	return wrap_Shift_Right_Arithmetic(Value'Address, Amount);
    end;

    -- Printable images, hex or decimal

    function Image (Value: LONGLONG;  Hex: Boolean := Standard.False) 
        return String is
	function wrap_sImage (Value: Ptr_64;  Hex: Boolean) 
	    return Image_Result;
	pragma Import(C, wrap_sImage, 
		      External_Name => "wrap_sImage");
	Result : Image_Result := wrap_sImage(Value'Address, Hex);
	use type Interfaces.C.char;
    begin
	-- If C wrapper is unimplemented, raise exception
	if Result(0) = nul then
	    raise Unsupported_Operation;
	end if;
	return Interfaces.C.To_Ada(Result);
    end;

    function Image (Value: DWORDLONG;  Hex: Boolean := Standard.True) 
        return String is
	function wrap_uImage (Value: Ptr_64;  Hex: Boolean) 
	    return Image_Result;
	pragma Import(C, wrap_uImage, 
		      External_Name => "wrap_uImage");
	Result : Image_Result := wrap_uImage(Value'Address, Hex);
	use type Interfaces.C.char;
    begin
	-- If C wrapper is unimplemented, raise exception
	if Result(0) = nul then
	    raise Unsupported_Operation;
	end if;
	return Interfaces.C.To_Ada(Result);
    end;

    -- winnt.h
    function Int32x32To64(A, B: LONG) return LONGLONG is
    begin
        return Win32.crt.Largeint.Ops."*"(
	    Win32.crt.Largeint.Ops.LONG(A),
	    Win32.crt.Largeint.Ops.LONG(B));
    end Int32x32To64;

    function UInt32x32To64(A, B: DWORD) return DWORDLONG is
    begin
        return Win32.crt.Largeint.Ops."*"(
	    Win32.crt.Largeint.Ops.DWORD(A),
	    Win32.crt.Largeint.Ops.DWORD(B));
    end UInt32x32To64;

    function Int64ShllMod32 (Val: DWORDLONG; By : Shift_Count) return DWORDLONG is
    begin
        return DWORDLONG(Win32.crt.Largeint.Ops.Shift_Left(
                     Win32.crt.Largeint.Ops.DWORDLONG(Val), Natural(By)));
    end Int64ShllMod32;

    function Int64ShraMod32 (Val: LONGLONG;
                             By : Shift_Count) return LONGLONG is
    begin
        return Win32.crt.Largeint.Ops.To_Signed (
            Win32.crt.Largeint.Ops.Shift_Right_Arithmetic(
	        Win32.crt.Largeint.Ops.To_Unsigned(Val), Natural(By)));
    end Int64ShraMod32;

    function Int64ShrlMod32 (Val: DWORDLONG;
                             By : Shift_Count) return DWORDLONG is
    begin
        return DWORDLONG(Win32.crt.Largeint.Ops.Shift_Right(
                     Win32.crt.Largeint.Ops.DWORDLONG(Val), Natural(By)));
    end Int64ShrlMod32;

end Win32.crt.Largeint.Ops;
